package p12f.exe.holdercert.utils;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;

import com.ejie.r01f.log.R01FLog;
import com.ejie.r01f.servlet.upload.FilePart;
import com.ejie.r01f.servlet.upload.MultipartParser;
import com.ejie.r01f.servlet.upload.ParamPart;
import com.ejie.r01f.servlet.upload.Part;

/**
 * @author IOLABARO
 *
 */
public class HolderCertParamParser {
	
	public static final String PROTOCOL_DATA = "protocolData";
	public static final String HOLDERCERT_DATA = "holderCertData";
	public static final String HOLDERCERT_CERTIFIED_DATA = "holderCertCertifiedData";
	public static final String HOLDERCERT_DATA_LIST = "holderCertDataList";
	public static final String HOLDERCERT_CERTIFIED_DATA_LIST = "holderCertCertifiedDataList";
	public static final String FINANTIALORGCODE = "finantialORGCode";
	public static final String FUNCTION = "function";
	public static final String MODULE = "module";
	
	public static final String HOLDERCERT_FUNCTION = "holderCertified";
	public static final String HOLDERCERT_MULTIPLE_FUNCTION = "multipleHolderCertified";
	public static final String HOLDERCERT_MULTIPLE_BATCH_FUNCTION = "batchMultipleHolderCertified";
	public static final String HOLDER_CERT_INTERNAL_ERROR = "holderCertInternalError";
	
	public static final String HOLDERCERT_FUNCTION_RECEIVE = "holderCert";
	public static final String HOLDERCERT_MULTIPLE_FUNCTION_RECEIVE = "multipleHolderCert";
	public static final String HOLDERCERT_MULTIPLE_BATCH_FUNCTION_RECEIVE = "multipleHolderCertBatch";
	
	public Map parameters = new HashMap();
	
	/**
	 * Parseo de parmetros.
	 * @param request
	 * @param response
	 */
	public void parse (HttpServletRequest request, HttpServletResponse response){
		parse (request, response, true);
	}
	
	public void parse (HttpServletRequest request, HttpServletResponse response, boolean unzip){
		if (_isMultipartContent(request)){
			_parseMultiPartRequest(request,response, unzip);
		} else {
			_parseSimpleRequest(request,response);
		}
	}
	
	/**
	 * Obtener el parmetro de clave key.
	 * @param key
	 * @return
	 */
	public Object getParam(String key ){
		return parameters.get(key);
	}
	
	/* ********************************************************************** */
	/* 							METODOS PRIVADOS							  */
	/* ********************************************************************** */
	
	private void _parseSimpleRequest(HttpServletRequest request, HttpServletResponse response) {
		Enumeration  enum = request.getParameterNames();
		while(enum.hasMoreElements()){
			String paramName = (String)enum.nextElement();
			String paramValue = (String) request.getParameter(paramName);
			parameters.put(paramName, paramValue);
		}
	}
	
	private void _parseMultiPartRequest(HttpServletRequest request, HttpServletResponse response, boolean unzip) {
		try {
			R01FLog.to("p12f.holderCertParamParser").info("==============================================================");
			R01FLog.to("p12f.holderCertParamParser").info("Multipart.Parser");
			R01FLog.to("p12f.holderCertParamParser").info("==============================================================");
			
			int maxFileSize = 10 * 1024 * 1024;  //Tamanio maximo del fichero a subir: 10MB
			Part parte = null;
			MultipartParser parser = new MultipartParser(request, maxFileSize);
			parser.setEncoding("ISO-8859-1");
			R01FLog.to("p12f.holderCertParamParser").info("MultiParser Creado");
			
			while( (parte = parser.readNextPart()) != null ) { 
				R01FLog.to("p12f.holderCertParamParser").info(".PARTE:"+ parte.getName());
				R01FLog.to("p12f.holderCertParamParser").info(".parte.isParam()-->"+parte.isParam());
				if( parte.isParam() ) {
					R01FLog.to("p12f.holderCertParamParser").info(".Tratamiento de parmetro tipo standard : [ "+ parte.getName() + "]");
					ParamPart paramPart = (ParamPart) parte;
					parameters.put(parte.getName(), paramPart.getStringValue());
				} else {
					R01FLog.to("p12f.holderCertParamParser").info(".Tratamiento de parmetro tipo multipart : [ "+ parte.getName() + "]");
					if ( (parte.isFile()) || (parte.getName().equalsIgnoreCase("file")) ) {
						FilePart fp = (FilePart) parte;
						InputStream is = fp.getInputStream();
						try {
							if (unzip) {
								byte[] unzippedBytes = HolderCertDataListZipper.unzip(is);
								R01FLog.to("p12f.holderCertParamParser").info(" Descomprimir : " + parte.getName());
								parameters.put(parte.getName(), new ByteArrayInputStream(unzippedBytes));
							} else {
								parameters.put(parte.getName(), IOUtils.toByteArray(is));
							}
						} catch(Exception ex) {
							ex.printStackTrace();
						}
						is.close();
						R01FLog.to("p12f.holderCertParamParser").info(" Tratamiento multipart realizado ");
					}
				}
			} // fin del while
			R01FLog.to("p12f.holderCertParamParser").info("Tratamiento Parametros POST Finalizado");
			Enumeration en = request.getParameterNames();
			while (en.hasMoreElements()) {
				String key = (String)en.nextElement();
				String value = (String)request.getParameter(key);
				parameters.put(key, value);
			}
			R01FLog.to("p12f.holderCertParamParser").info("Tratamiento Parametros GET Finalizado");
		}catch(Exception ex){
			ex.printStackTrace();
		}
	}
	
	private static final boolean _isMultipartContent(HttpServletRequest request) {
		if (!"post".equals(request.getMethod().toLowerCase())) {
			return false;
		}
		String contentType = request.getContentType();
		if (contentType == null) {
			return false;
		}
		return contentType.toLowerCase().startsWith("multipart/");
		
	}
	
}
