package p12f.exe.otp.objects;

import java.io.Serializable;

import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

public class OTPError implements Serializable {
	
	public static final int NO_ERROR = 0;
	
	// Errores de generacin de OTP
	public static final int GENERATE_NO_APLICATION_ID = 100;   // No viene relleno el campo AplicationId
	public static final int GENERATE_NO_SERVICE_ID = 101;      // No viene relleno el campo ServiceId
	public static final int GENERATE_NO_OID = 102;             // No viene relleno el campo Oid
	public static final int GENERATE_OTHER = 900;              // Otro error no recogido producido durante la generacin del OTP
	
	// Errores de validacin de OTP
	public static final int VALIDATE_NO_OTP = 100;             // El OTP ya no existe y no se puede validar
	public static final int VALIDATE_NO_APLICATION_ID = 101;   // No se ha enviado el campo AplicationId del OTP a validar
	public static final int VALIDATE_NO_SERVICE_ID = 102;      // No se ha enviado el campo ServiceId del OTP a validar
	public static final int VALIDATE_NO_OID = 103;             // No se ha enviado el campo Oid del OTP a validar
	public static final int VALIDATE_NO_OTP_ID = 104;          // No se ha enviado el campo OtpId del OTP a validar
	public static final int VALIDATE_NO_RETRIES = 200;         // Se han gastado todos los reintentos para validar el OTP
	public static final int VALIDATE_EXPIRED = 300;            // El OTP ha expirado
	public static final int VALIDATE_WRONG_OTP = 400;          // El OTP no es correcto
	public static final int VALIDATE_OTHER = 900;              // Otro error no recogido producido durante la validacin del OTP
	
	// Errores de envo de OTP
	public static final int SEND_NO_OTP = 100;                         // No existe el OTP que se quiere enviar
	public static final int SEND_NO_APLICATION_ID = 101;               // No se ha enviado el campo AplicationId del OTP a enviar
	public static final int SEND_NO_SERVICE_ID = 102;                  // No se ha enviado el campo ServiceId del OTP a enviar
	public static final int SEND_NO_OID = 103;                         // No se ha enviado el campo Oid del OTP a enviar
	public static final int SEND_NO_OTP_ID = 104;                      // No se ha enviado el campo OtpId del OTP a enviar
	public static final int SEND_NO_PRESENTATION_DATA_TYPE = 200;      // No se ha expecificado un tipo de envo existente
	public static final int SEND_MAIL = 300;                           // Error no recogido en el envio del OTP va mail
	public static final int SEND_MAIL_NO_MAIL_TO_SEND = 301;           // No se ha expecificado el mail al que se quiere enviar el OTP
	public static final int SEND_MAIL_MALFORMED_MAIL_TO_SEND = 302;    // El mail expecificado para enviar el OTP est mal formado
	public static final int SEND_MAIL_NO_TEMPLATE = 303;               // El template expecificado para el envio del OTP no existe
	public static final int SEND_MAIL_NO_OTP_IN_MSG = 304;             // No se ha enviado el OTP en el texto del mensaje
	public static final int SEND_SMS = 400;                            // Error no recogido en el envio del OTP va sms
	public static final int SEND_SMS_NO_PHONE_NUMBER = 401;            // No se ha expecificado el nmero de telfono al que se quiere enviar el OTP
	public static final int SEND_SMS_MALFORMED_PHONE_NUMBER = 402;     // El nmero expecificado para enviar el OTP est mal formado
	public static final int SEND_SMS_NO_OTP_IN_MSG = 403;              // No se ha enviado el OTP en el texto del mensaje
	public static final int SEND_OTHER = 900;                          // Error no recogido en el envo del OTP
	
	
	public int otpErrorCode = NO_ERROR;
	
	public String otpErrorMsg = null;
	
	public OTPError() {
		
	}
	
	public OTPError(int otpErrorCode) {
		this.otpErrorCode = otpErrorCode;
	}
	
	public OTPError(int otpErrorCode, String otpErrorMsg) {
		this(otpErrorCode);
		this.otpErrorMsg = otpErrorMsg;
	}
	
	/**
	 * Convierte un Objeto OTPError a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto OTPError.
	 * @return Objeto OTPError construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static OTPError getObject(String xml) throws XOMarshallerException {
		return (OTPError)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
	
}
