package p12f.exe.otp.objects;

import java.io.Serializable;

import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

public class OTPRequest implements Serializable {
	
	public static final int OTP_TYPE_DIGITS_ONLY      = 0;
	public static final int OTP_TYPE_LETTERS_ONLY     = 1;
	public static final int OTP_TYPE_LETTERS_ONLY_MAY = 2;
	public static final int OTP_TYPE_LETTERS_ONLY_MIN = 3;
	public static final int OTP_TYPE_ALPHANUMERIC     = 4;
	public static final int OTP_TYPE_ALPHANUMERIC_MAY = 5;
	public static final int OTP_TYPE_ALPHANUMERIC_MIN = 6;
	
	/**
	 * Identificador de la aplicacin solicitante del OTP.
	 */
	public String aplicationId;
	/**
	 * Identificador del servicio solicitante del OTP.
	 */
	public String serviceId;

	/**
	 * Identificador del tercero solicitante del OTP.
	 */
	public String oid;
	
	/**
	 * Longitud del OTP.
	 */
	public int otpLength = 5;
	
	/**
	 * Tiempo de vida, en segundos, del OTP. -1 ilimitado.
	 */
	public int otpLifeSeconds = -1;
	
	/**
	 * Reintentos permitidos para validar el OTP. -1 ilimitado.
	 */
	public int otpMaxRetries = -1;
	
	/**
	 * Tipo de OTP.
	 */
	public int otpType = OTP_TYPE_ALPHANUMERIC;
	
	public OTPRequest() {
		
	}
	
	/**
	 * Convierte un Objeto OTPRequest a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto OTPRequest.
	 * @return Objeto OTPRequest construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static OTPRequest getObject(String xml) throws XOMarshallerException {
		return (OTPRequest)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}

}
