package p12f.exe.pasarelapagos.helpers;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.apache.pdfbox.util.PDFMergerUtility;

import p12f.exe.pasarelapagos.objects.ProtocolData;
import p12f.exe.pasarelapagos.objects.data.DataFileConstants;
import p12f.exe.pasarelapagos.objects.data.DataFileResponse;
import p12f.exe.pasarelapagos.paymentrequest.PaymentRequestData;
import p12f.exe.pasarelapagos.paymentrequest.PeticionPago;
import p12f.exe.pasarelapagos.paymentrequest.PresentationRequestData;
import p12f.exe.pasarelapagos.transformer.ObjectTransformer;
import p12f.exe.pasarelapagos.transformer.ObjectTransformerException;
import p12f.exe.pasarelapagos.utils.fop.XMLTransformer;

import com.ejie.r01f.log.R01FLog;
import com.ejie.r01f.rpcdispatcher.RPCException;
import com.ejie.r01f.util.encoders.BASE64Decoder;
import com.ejie.r01f.util.encoders.BASE64Encoder;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;

public class DataFileUtils {
	
	private DataFileUtils() {
		
	}
	
	/**
	 * Devuelve un array de bytes que representa al fichero/directorio pasado
	 * como parmetro.
	 * En el caso que sea un directorio, los bytes representarn a un ZIP con
	 * todos los ficheros que este contenga.
	 * @param f
	 * @return
	 * @throws IOException
	 */
	public static byte[] getFileBytes(File f) throws IOException {
		if (f.isFile()) {
			return org.apache.commons.io.FileUtils.readFileToByteArray(f);
		} else {
			File[] files = f.listFiles();
			if (DataFileUtils.getFileSize(f)==0){
				return null;
			}
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			ZipOutputStream zipfile = new ZipOutputStream(bos);
			ZipEntry zipentry = null;
			for (int i=0; i<files.length; i++) {
				File dirEntry = files[i];
				if (dirEntry.isDirectory()) {
					String dirName = dirEntry.getName();
					_addDirectoryToZip(zipfile, dirEntry, dirName);
					continue;
				}
				zipentry = new ZipEntry(dirEntry.getName());
				zipfile.putNextEntry(zipentry);
				zipfile.write(DataFileUtils.getFileBytes(dirEntry));
			}
			zipfile.close();
			return bos.toByteArray();
		}
	}
	
	/**
	 * Devuelve un zip con todos los ficheros que se le hayan pasado por parmetro.
	 * @param zipName
	 * @param filesPath
	 * @return
	 */
	public static byte[] getFileListBytes(String[] filesPath) throws IOException {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		ZipOutputStream zipfile = new ZipOutputStream(bos);
		ZipEntry zipentry = null;
		long zipSize = 0;
		for (int i=0; i<filesPath.length; i++) {
			File dirEntry = new File(filesPath[i]);
			zipSize += DataFileUtils.getFileSize(dirEntry);
			if (dirEntry.isDirectory()) {
				String dirName = dirEntry.getName();
				_addDirectoryToZip(zipfile, dirEntry, dirName);
				continue;
			}
			zipentry = new ZipEntry(dirEntry.getName());
			zipfile.putNextEntry(zipentry);
			zipfile.write(DataFileUtils.getFileBytes(dirEntry));
		}
		zipfile.close();
		if (zipSize == 0) {
			return null;
		}
		return bos.toByteArray();
	}
	
	/**
	 * Devuelve el tamao de un fichero/directorio.
	 * @param f
	 * @return
	 * @throws IOException
	 */
	public static long getFileSize(File f) throws IOException {
		if (f.isFile()) {
			return f.length();
		} else {
			return org.apache.commons.io.FileUtils.sizeOfDirectory(f);
		}
	}
	
	/**
	 * Renombre un fichero.
	 * @param f
	 * @param newName
	 * @return
	 */
	public static DataFileResponse renameFile(String oldNamePath, String newName) {
		DataFileResponse response = new DataFileResponse();
		File f = new File(oldNamePath);
		String path = f.getParent();
		File fNew = new File(path+File.separator+newName);
		if (fNew.exists()) {
			response.setResponseCode(DataFileConstants.RESULT_EXISTS_FILE);
			return response;
		}
		try {
			f.renameTo(fNew);
			response.setResponseCode(DataFileConstants.RESULT_OK);
		} catch (Exception e) {
			response.setResponseCode(DataFileConstants.RESULT_ERROR);
			response.setResponse(e.getMessage());
		}
		return response;
	}
	
	public static DataFileResponse uploadFile(String path, String base64content, boolean rewriteFiles) {
		DataFileResponse response = new DataFileResponse();
		try {
			File f = new File(path);
			if (f.exists() && !rewriteFiles) {
				response.setResponseCode(DataFileConstants.RESULT_EXISTS_FILE);
			} else {
				BASE64Decoder base64Decoder = new BASE64Decoder();
				byte[] data = base64Decoder.decodeBuffer(base64content);
				org.apache.commons.io.FileUtils.writeByteArrayToFile(f, data);
				response.setResponseCode(DataFileConstants.RESULT_OK);
			}
		} catch (IOException e) {
			e.printStackTrace();
			response.setResponseCode(DataFileConstants.RESULT_ERROR);
			response.setResponse(e.getMessage());
		}
		return response;
	}
	
	public static DataFileResponse uploadAndUnzipFile(String path, String base64content, boolean rewriteFiles) {
		DataFileResponse response = new DataFileResponse();
		StringBuffer noRewritedFiles = new StringBuffer("");
		boolean anyFileRewrite = false;
		BASE64Decoder base64Decoder = new BASE64Decoder();
		ZipInputStream zin = null;
		FileOutputStream fout = null;
		try {
			ByteArrayInputStream bis = new ByteArrayInputStream(base64Decoder.decodeBuffer(base64content));
			zin = new ZipInputStream(bis);
			ZipEntry ze = null;
			while ((ze = zin.getNextEntry()) != null) {
				R01FLog.to("p12f.data").info("Unzipping " + ze.getName());
				File f = new File(path + File.separator + ze.getName());
				boolean isDirectory = ze.getName().endsWith("/");
				boolean rewriteFile = true;
				if (isDirectory) {
					if (!f.exists()) {
						R01FLog.to("p12f.data").info("Crear directorio " + ze.getName());
						f.mkdir();
					}
				} else {
					if (!rewriteFiles && f.exists()) {
						R01FLog.to("p12f.data").info("No se sobreescribe " + ze.getName());
						rewriteFile = false;
						anyFileRewrite = true;
						noRewritedFiles.append(ze.getName()).append(File.pathSeparator);
					}
				}
				if (!isDirectory && rewriteFile) {
					fout = new FileOutputStream(f);
					R01FLog.to("p12f.data").info("Extraer " + ze.getName());
					for (int c = zin.read(); c != -1; c = zin.read()) {
						fout.write(c);
					}
					fout.close();
				} else {
					for (int c = zin.read(); c != -1; c = zin.read()) { }
				}
				zin.closeEntry();
			}
			zin.close();
			if (anyFileRewrite) {
				response.setResponseCode(DataFileConstants.RESULT_EXISTS_FILE);
				response.setResponse(noRewritedFiles.toString());
			} else {
				response.setResponseCode(DataFileConstants.RESULT_OK);
			}
		} catch (IOException e) {
			e.printStackTrace();
			response.setResponseCode(DataFileConstants.RESULT_ERROR);
			response.setResponse(e.getMessage());
		} finally {
			if (zin!=null) {
				try {
					zin.close();
				} catch (IOException e) { }
			}
			if (fout !=null) {
				try {
					fout.close();
				} catch (IOException e) { }
			}
		}
		return response;
	}
	
	public static boolean changePermissions(String completeFileName, int permissions) {
		File f = new File(completeFileName);
		if (f.exists() && _checkPermissions(permissions)){
			try {
				R01FLog.to("p12f.data").info("chmod "+permissions+" "+completeFileName);
				Runtime.getRuntime().exec( "chmod "+permissions+" "+completeFileName );
				return true;
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return false;
	}
	
	private static void _addDirectoryToZip(ZipOutputStream zipfile, File dir, String basePath) {
		try {
			File[] files = dir.listFiles();
			ZipEntry zipentry = null;
			for (int i=0; i<files.length; i++) {
				File dirEntry = files[i];
				if (dirEntry.isDirectory()) {
					_addDirectoryToZip(zipfile, dirEntry, basePath+File.separator+dirEntry.getName());
					continue;
				}
				zipentry = new ZipEntry(basePath+File.separator+dirEntry.getName());
				zipfile.putNextEntry(zipentry);
				zipfile.write(DataFileUtils.getFileBytes(dirEntry));
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private static boolean _checkPermissions(int permissions) {
		if (permissions <= 777) {
			return true;
		}
		return false;
	}

	public static String doBuildPaymentReceiptFile(PaymentRequestData paymentRequestData,
												   PresentationRequestData presentationRequestData,
												   ProtocolData protocolData) {
		
		String xsl;
		String paymentGatewayVersion;
		PeticionPago peticionPago = null;
		
		try {
			
			if (paymentRequestData.peticionesPago.size()==1) {
				
				///////////////////////////////////////// LIQUIDACIONES SIMPLES////////////////////////////////////////////////
	
				Iterator it = paymentRequestData.peticionesPago.keySet().iterator();
				peticionPago = (PeticionPago)paymentRequestData.peticionesPago.get(it.next());
				
				paymentGatewayVersion = peticionPago.liquidacion.paymentGatewayVersion;
				xsl = peticionPago.liquidacion.urlPlantilla;
				R01FLog.to("p12f.liquidacion").info("> urlPlantilla: " + xsl);
				
				
				java.io.Reader xmlReader = null;
				File plantillaXSL = new File(xsl);
			
				if(paymentGatewayVersion.equals("1")){
					p12f.exe.pasarelapagos.v1.objects.PeticionPago peticionPagoVieja = 
							_getPeticionPagoV1(paymentRequestData.toXML(),
											   presentationRequestData.toXML(),
											   protocolData.toXML());
					R01FLog.to("p12f.liquidacion").info("[P12F -DATA][Peticion de Pago V1:\n" + peticionPagoVieja.toXML() + "\n\n\n");	
					xmlReader = new BufferedReader(new StringReader(peticionPagoVieja.toXML()));
				}else if(paymentGatewayVersion.equals("2")){
					// Caso abierto a mltiples pagos, se pasa un objeto PaymentRequestData
					R01FLog.to("p12f.liquidacion").info("[P12F - DATA][doBuildPaymentReceiptFile] PaymentRequestData V2:\n" + 
							paymentRequestData.toXML() + "\n\n\n");	
					xmlReader = new BufferedReader(new StringReader(paymentRequestData.toXML()));
				}
				
				byte[] pdf = XMLTransformer.getPDF(xmlReader,plantillaXSL);
		
				BASE64Encoder base64Decoder = new BASE64Encoder();
				String data = base64Decoder.encodeBuffer(pdf);					
				
				return  data;
			
			} else {
				///////////////////////////////////////// LIQUIDACIONES  MULTIPLES ////////////////////////////////////////////////
			
				String resultStream = null;
							
				java.io.Reader xmlReader = null;
			
				ByteArrayOutputStream out = new ByteArrayOutputStream();
				PDFMergerUtility mergepdf = new PDFMergerUtility();
				mergepdf.setDestinationStream(out);
				List pdfs = new ArrayList();
				
				Iterator it = paymentRequestData.peticionesPago.keySet().iterator();
				while (it.hasNext()) {
					peticionPago = (PeticionPago)paymentRequestData.peticionesPago.get(it.next());
					paymentGatewayVersion = peticionPago.liquidacion.paymentGatewayVersion;
					PaymentRequestData paymentRequestDataAux = new PaymentRequestData();
					paymentRequestDataAux.peticionesPago.put(peticionPago.id, peticionPago);
					paymentRequestDataAux.datosAdicionales = paymentRequestData.datosAdicionales;
					String paymentRequestDataAuxXML = paymentRequestDataAux.toXML();
					xsl = peticionPago.liquidacion.urlPlantilla;
					R01FLog.to("p12f.liquidacion").info("> urlPlantilla: " + xsl);
			
					// Se imprime Justificante para el ciudadano.
					File plantillaXSL = new File(xsl);
					if(paymentGatewayVersion.equals("1")){
						p12f.exe.pasarelapagos.v1.objects.PeticionPago peticionPagoVieja = 
								_getPeticionPagoV1(paymentRequestData.toXML(),
												   presentationRequestData.toXML(),
												   protocolData.toXML());
						R01FLog.to("p12f.liquidacion").info("[P12f][P12F - DATA] Peticion de Pago V1:\n" + peticionPagoVieja.toXML() + "\n\n\n");	
						xmlReader = new BufferedReader(new StringReader(peticionPagoVieja.toXML()));
					}else if(paymentGatewayVersion.equals("2")){
						// Caso abierto a mltiples pagos, se pasa un objeto PaymentRequestData
						R01FLog.to("p12f.liquidacion").info("[P12f][P12F - DATA] PaymentRequestData V2:\n" + paymentRequestDataAuxXML + "\n\n\n");	
						xmlReader = new BufferedReader(new StringReader(paymentRequestDataAuxXML));
					}
					
					byte[] pdfAux = XMLTransformer.getPDF(xmlReader,plantillaXSL);
					InputStream pdf = new ByteArrayInputStream(pdfAux);
					// recorrer childern con cada fichero y mergeamos
					pdfs.add(pdf);
				}
				
				mergepdf.addSources(pdfs);
				mergepdf.mergeDocuments();
				
				
				BASE64Encoder base64Decoder = new BASE64Encoder();
				resultStream  = base64Decoder.encodeBuffer(out.toByteArray());		
			
				mergepdf.getDestinationStream().flush();
				mergepdf.getDestinationStream().close();				
				
				return resultStream;
			}
			
			
			
		}catch (Exception ex) {
			ex.printStackTrace();
			return null;
		}catch (Throwable  ex) {
			ex.printStackTrace();
			return null;
		}
			
				
		
		
	}
	
	
	/******************************************************************************************************************************/
	/* 	METODOS PRIVADOS 																										  */
	/******************************************************************************************************************************/
	
	/** Convierte objet
	 * @param paymentRequestDataXML
	 * @throws XOMarshallerException 
	 * @throws ObjectTransformerException 
	 */
	
	private static p12f.exe.pasarelapagos.v1.objects.PeticionPago
			_getPeticionPagoV1(String paymentRequestDataXML, 
			String presentationRequestDataXML, 
			String protocolDataXML) throws RPCException, XOMarshallerException, ObjectTransformerException {
		
		// Instanciar Object Transformer.
		ObjectTransformer transformer = new ObjectTransformer();	
		p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago = transformer.transformToOldPaymentData(paymentRequestDataXML,presentationRequestDataXML,protocolDataXML);
		return oldPeticionPago;

	}
}
