package p12f.exe.pasarelapagos.helpers;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import sun.misc.BASE64Decoder;

import com.ejie.r01f.log.R01FLog;
import com.ejie.r01f.servlet.upload.FilePart;
import com.ejie.r01f.servlet.upload.MultipartParser;
import com.ejie.r01f.servlet.upload.Part;
import com.ejie.r01f.servlet.upload.PartDigester;
import com.ejie.r01f.servlet.upload.PartDigesterException;
import com.ejie.r01f.util.encoders.BASE64Encoder;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Clase de utilidad para facilitar la subida de ficheros al servidor desde formularios HTML o el envo
 * de ficheros a travs de HTTP con codificacin Base64, etc.
 *
 */
public class FileUploadHelper {
	/**
	 * Sube al servidor los ficheros incluidos en la peticin, enviados como parmetros
	 * de tipo file en un formulario codificado como multipart/form-data.
	 * @param appCode Cdigo de la aplicacin con la configuracin de la subida de ficheros
	 * @param partDigesterID Identificativo del digester empleado.
	 * @param request Peticin con los ficheros.
	 * @return Lista de los ficheros subidos con el path completo de su ubicacin en el servidor
	 * @throws ServletException Si se produce cualquier error al tratar de subir los ficheros
	 */
	public String[] uploadRequestFiles(String appCode, String partDigesterID, HttpServletRequest request) throws ServletException
	{
		R01FLog.to("p12f.FileUploadHelper").fine("\n\n\n\n>>>>>>>>>>uploadRequestFiles.................");
		PartDigester partDigester = null;	// Clase "digestora" de los parts
											// (por ejemplo guarda los ficheros en disco o base de datos)
		ArrayList uploadedFiles = new ArrayList();
		
		try {
			// Digerir ahora los parametros que vienen como multipart 
			// Obtener el tamao mximo de los ficheros si se tiene el codigo de aplicacin
			// (bien porque se ha parametrizado al registra el servlet, bien porque se ha pasado como parametro de la request) 
			int maxFileSize = 1024 * 1024;		// Tamao mximo del fichero en bytes (por defecto 1M)
			if (appCode != null) maxFileSize = XMLProperties.getInt(appCode,"uploadServlet/maxFileSize");
			MultipartParser parser = new MultipartParser(request, maxFileSize);
			Part currPart = null;				// Parte actual de la request
			String serverUploadPath = XMLProperties.get(appCode, "uploadServlet/digesters/digester[@id='" + partDigesterID + "']/config/dir");
			while ((currPart = parser.readNextPart()) != null) {
				if ((currPart.isFile()) || (currPart.getName().equalsIgnoreCase("file"))) {
					FilePart filePart = (FilePart)currPart;
					
					String fileName = (filePart.getFileName() == null ? "" : filePart.getFileName());
					String mimeType = (filePart.getContentType() == null ? "" : filePart.getContentType());
					R01FLog.to("p12f.FileUploadHelper").info(">>> Recibido Fichero tipo '" + mimeType + "': '" + fileName + "'");
					// Solo procesar los filePart que "suban" algo.
					// Aquellos filePart "vacios" se distinguen porque mimeType=application/octet-stream
					// y el nombre del fichero es null
					if ( fileName != null && fileName.length() > 0 ) { 
						// Comprobar si el tipo mime esta entre los admitidos...
						List allowedMimeTypes = XMLProperties.getPropertyList(appCode,"uploadServlet/digesters/digester[@id='" + partDigesterID + "']/allowedMimeTypes/mimeType");
						if (allowedMimeTypes != null) {
							boolean mimeTypeAllowed = false;
							for(Iterator mit = allowedMimeTypes.iterator(); mit.hasNext(); ) {
								if ( ((String)mit.next()).equals(mimeType) ) {
									mimeTypeAllowed = true;
									break;
								}
							}
							if (!mimeTypeAllowed) {
								throw new ServletException("El mime-type: " + mimeType + " NO esta permitido. Revisa la configuracion del digester");
							}
						} else {
							R01FLog.to("p12f.FileUploadHelper").info(">>NO se hace comprobacion del mime-type ya que no se ha configurado en el digester!");
						}
						
						
						// Llamar a la clase digestora de parts
						if (partDigester == null) {
							partDigester = _getPartDigesterInstance(appCode,partDigesterID);
						}
						partDigester.digestFilePart(filePart);
						
						uploadedFiles.add(serverUploadPath + File.separator + fileName);
					}
				} else {
					R01FLog.to("p12f.FileUploadHelper").info(">>> Part NO reconocida. No es un parametro ni un fichero!!!!");
				} // fin del if principal
			} // fin del while
			
			R01FLog.to("p12f.FileUploadHelper").info("<<<<<<<<<<uploadRequestFiles.................\n\n\n\n");
		} catch (IOException ioEx) {
			throw new ServletException(ioEx);
		} catch (PartDigesterException partDigEx) {
			throw new ServletException(partDigEx);
		}
		return (String[])uploadedFiles.toArray(new String[uploadedFiles.size()]);
	}
	
	/**
	 * Sube al servidor los ficheros incluidos en la peticin, enviados como parmetros
	 * de tipo file en un formulario codificado como multipart/form-data.
	 * @param appCode Cdigo de la aplicacin con la configuracin de la subida de ficheros
	 * @param partDigesterID Identificativo del digester empleado.
	 * @param request Peticin con los ficheros.
	 * @return Lista de los ficheros subidos con el path completo de su ubicacin en el servidor
	 * @throws ServletException Si se produce cualquier error al tratar de subir los ficheros
	 */
	public String[] uploadRequestFiles(String appCode, String partDigesterID, HttpServletRequest request, String tmpId) throws ServletException {
		if (tmpId == null || "".equals(tmpId)) return uploadRequestFiles(appCode, partDigesterID, request);
		R01FLog.to("p12f.FileUploadHelper").fine("\n\n\n\n>>>>>>>>>>uploadRequestFiles.................");
		PartDigester partDigester = null;	// Clase "digestora" de los parts
											// (por ejemplo guarda los ficheros en disco o base de datos)
		ArrayList uploadedFiles = new ArrayList();
		
		try {
			// Digerir ahora los parametros que vienen como multipart 
			// Obtener el tamao mximo de los ficheros si se tiene el codigo de aplicacin
			// (bien porque se ha parametrizado al registra el servlet, bien porque se ha pasado como parametro de la request)
			int maxFileSize = 1024 * 1024;		// Tamao mximo del fichero en bytes (por defecto 1M)
			if (appCode != null) maxFileSize = XMLProperties.getInt(appCode,"uploadServlet/maxFileSize");
			MultipartParser parser = new MultipartParser(request, maxFileSize);
			Part currPart = null;				// Parte actual de la request
			String serverUploadPath = XMLProperties.get(appCode, "uploadServlet/digesters/digester[@id='" + partDigesterID + "']/config/dir")+'/'+tmpId;
			File fAux = new File(serverUploadPath);
			if (!fAux.exists()){
				fAux.mkdir();
			}
			while ((currPart = parser.readNextPart()) != null) {
				if ((currPart.isFile()) || (currPart.getName().equalsIgnoreCase("file"))) {
					FilePart filePart = (FilePart)currPart;
					
					String fileName = (filePart.getFileName() == null ? "" : filePart.getFileName());
					String mimeType = (filePart.getContentType() == null ? "" : filePart.getContentType());
					R01FLog.to("p12f.FileUploadHelper").info(">>> Recibido Fichero tipo '" + mimeType + "': '" + fileName + "'");
					
					// Solo procesar los filePart que "suban" algo.
					// Aquellos filePart "vacios" se distinguen porque mimeType=application/octet-stream
					// y el nombre del fichero es null
					if ( fileName != null && fileName.length() > 0 ) {
						// Comprobar si el tipo mime esta entre los admitidos...
						List allowedMimeTypes = XMLProperties.getPropertyList(appCode,"uploadServlet/digesters/digester[@id='" + partDigesterID + "']/allowedMimeTypes/mimeType");
						if (allowedMimeTypes != null) {
							boolean mimeTypeAllowed = false;
							for(Iterator mit = allowedMimeTypes.iterator(); mit.hasNext(); ) {
								if ( ((String)mit.next()).equals(mimeType) ) {
									mimeTypeAllowed = true;
									break;
								}
							}
							if (!mimeTypeAllowed) {
								throw new ServletException("El mime-type: " + mimeType + " NO esta permitido. Revisa la configuracion del digester");
							}
						} else {
							R01FLog.to("p12f.FileUploadHelper").info(">>NO se hace comprobacion del mime-type ya que no se ha configurado en el digester!");
						}
						
						// Llamar a la clase digestora de parts
						if (partDigester == null) {
							partDigester = _getPartDigesterInstance(appCode,partDigesterID,tmpId);
						}
						partDigester.digestFilePart(filePart);
						
						uploadedFiles.add(serverUploadPath + File.separator + fileName);
					}
				} else {
					R01FLog.to("p12f.FileUploadHelper").info(">>> Part NO reconocida. No es un parametro ni un fichero!!!!");
				} // fin del if principal
				
			} // fin del while
			
			R01FLog.to("p12f.FileUploadHelper").info("<<<<<<<<<<uploadRequestFiles.................\n\n\n\n");
		} catch (IOException ioEx) {
			throw new ServletException(ioEx);
		} catch (PartDigesterException partDigEx) {
			throw new ServletException(partDigEx);
		}
		return (String[])uploadedFiles.toArray(new String[uploadedFiles.size()]);
	}

	/**
	 * Obtiene una instancia de la clase digestora de parts.
	 * @param appCode cdigo de la aplicacin.
	 * @param partDigesterID identificador del digester.
	 * @return instancia de la clase que implementa el digester.
	 * @throws ServletException si la clase no puede ser instanciada.
	 * @throws PartDigesterException error al establecer los valores de configuracin del partDigester.
	 */
	private PartDigester _getPartDigesterInstance(String appCode,String partDigesterID) throws ServletException,PartDigesterException {	
		if (appCode == null) {
			throw new ServletException("UploadServlet configuration error: No se puede obtener la configuracion ya que no se ha especificado la propiedad appCode en el fichero web.xml o no se ha recibido el parametro de la request appCode");
		}
		if (partDigesterID == null) {
			throw new ServletException("UploadServlet configuration error: No se ha recibido el parametro digesterID");
		}
		try {
			PartDigester partDigester = null;	// Clase "digestora" de los parts
			
			// Acceder al fichero properties para obtener la clase del digester que hay
			// que instanciar y las propiedades del digester
			String partDigesterClassName = XMLProperties.getString(appCode,"uploadServlet/digesters/digester[@id='" + partDigesterID + "']/class");
			Properties partDigesterProperties = XMLProperties.getProperties(appCode,"uploadServlet/digesters/digester[@id='" + partDigesterID + "']/config");
			if (partDigesterProperties == null) partDigesterProperties = new Properties();
			partDigesterProperties.setProperty("appCode",appCode);
			
			if (partDigesterClassName != null) {
				R01FLog.to("p12f.FileUploadHelper").fine(">>> Instanciando clase digester de los parts: '" + partDigesterClassName + "'");
				ClassLoader cl = Thread.currentThread().getContextClassLoader();
				Class partDigesterClass = cl.loadClass(partDigesterClassName);
				//Class partDigesterClass = Class.forName(partDigesterClassName);   // No utilizar Class.forName... problemas con los EAR
				partDigester = (PartDigester)partDigesterClass.newInstance();
				partDigester.setConfig(partDigesterProperties);
			} else {
				throw new ServletException("UploadServlet configuration error: uploadServlet/digesters/digester[@id='" + partDigesterID + "']/class not found in " + appCode + ".properties file");
			}
			return partDigester;
		} catch (ClassNotFoundException cnfEx) {
			throw new ServletException("ClassNotFoundException in UploadServlet: " + cnfEx.toString(),cnfEx);
		} catch (IllegalAccessException illAccEx) {
			throw new ServletException("IllegalAccessException in UploadServlet: " + illAccEx.toString(),illAccEx);
		} catch (InstantiationException instEx) {
			throw new ServletException("InstantiationException in UploadServlet: " + instEx.toString(),instEx);
		}
	}
	
	/**
	 * Obtiene una instancia de la clase digestora de parts.
	 * @param appCode cdigo de la aplicacin.
	 * @param partDigesterID identificador del digester.
	 * @return instancia de la clase que implementa el digester.
	 * @throws ServletException si la clase no puede ser instanciada.
	 * @throws PartDigesterException error al establecer los valores de configuracin del partDigester.
	 */
	private PartDigester _getPartDigesterInstance(String appCode,String partDigesterID, String tmpId) throws ServletException,PartDigesterException {
		if (tmpId==null || "".equals(tmpId)) {
			return _getPartDigesterInstance(appCode, partDigesterID);
		}
		if (appCode == null) {
			throw new ServletException("UploadServlet configuration error: No se puede obtener la configuracion ya que no se ha especificado la propiedad appCode en el fichero web.xml o no se ha recibido el parametro de la request appCode");
		}
		if (partDigesterID == null) {
			throw new ServletException("UploadServlet configuration error: No se ha recibido el parametro digesterID");
		}
		try {
			PartDigester partDigester = null;	// Clase "digestora" de los parts
			
			// Acceder al fichero properties para obtener la clase del digester que hay
			// que instanciar y las propiedades del digester
			String partDigesterClassName = XMLProperties.getString(appCode,"uploadServlet/digesters/digester[@id='" + partDigesterID + "']/class");
			Properties partDigesterProperties = XMLProperties.getProperties(appCode,"uploadServlet/digesters/digester[@id='" + partDigesterID + "']/config");			
			if (partDigesterProperties == null) {
				partDigesterProperties = new Properties();
			}
			partDigesterProperties.setProperty("appCode",appCode);
			if (partDigesterProperties.containsKey("dir")) {
				partDigesterProperties.setProperty("dir", partDigesterProperties.getProperty("dir")+'/'+tmpId);
			}
			if (partDigesterClassName != null) {
				R01FLog.to("p12f.FileUploadHelper").fine(">>> Instanciando clase digester de los parts: '" + partDigesterClassName + "'");
				ClassLoader cl = Thread.currentThread().getContextClassLoader();
				Class partDigesterClass = cl.loadClass(partDigesterClassName);
				//Class partDigesterClass = Class.forName(partDigesterClassName);   // No utilizar Class.forName... problemas con los EAR
				partDigester = (PartDigester)partDigesterClass.newInstance();
				partDigester.setConfig(partDigesterProperties);
			} else {
				throw new ServletException("UploadServlet configuration error: uploadServlet/digesters/digester[@id='" + partDigesterID + "']/class not found in " + appCode + ".properties file");
			}
			return partDigester;
		} catch (ClassNotFoundException cnfEx) {
			throw new ServletException("ClassNotFoundException in UploadServlet: " + cnfEx.toString(),cnfEx);
		} catch (IllegalAccessException illAccEx) {
			throw new ServletException("IllegalAccessException in UploadServlet: " + illAccEx.toString(),illAccEx);
		} catch (InstantiationException instEx) {
			throw new ServletException("InstantiationException in UploadServlet: " + instEx.toString(),instEx);
		}
	}
	
	/**
	 * Obtiene el contenido de un fichero codificado en Base64.
	 * @param file Fichero cuyo contenido se desea codificar.
	 * @return El contenido del fichero cifrado en Base64.
	 * @throws IOException Si se produce algn error de acceso al fichero.
	 */
	public static String getContentInBase64(File file) throws IOException{
		byte[] fileContent;
		FileInputStream fileInputStream;
		
		fileInputStream = new FileInputStream(file);
		fileContent = new byte[(int)file.length()];
			
		fileInputStream.read(fileContent);
		fileInputStream.close();
			
		BASE64Encoder base64Encoder = new BASE64Encoder();
		String base64Data = base64Encoder.encodeBuffer(fileContent);
		
		return base64Data;
	}
	
	/**
	 * Genera un fichero a partir de su contenido cifrado en Base64.
	 * @param base64Data Contenido del fichero en Base64.
	 * @param file Fichero que va a generarse con el contenido descifrado.
	 * @param maxFileSize Tamao mximo en bytes permitido para el contenido del fichero.
	 * @throws IOException Si se produce algn error de acceso al fichero.
	 */
	public static void putContentFromBase64(String base64Data, File file, int maxFileSize) throws IOException{
		BASE64Decoder base64Decoder = new BASE64Decoder();
		byte[] fileContent;
		FileOutputStream fileOutputStream;
		
		fileContent = base64Decoder.decodeBuffer(base64Data);
		if(fileContent.length > maxFileSize) {
			throw new IOException("El tamao del fichero supera el lmite de "+maxFileSize+" bytes");
		}
		fileOutputStream = new FileOutputStream(file);
		fileOutputStream.write(fileContent);
		fileOutputStream.close();
	}
	

}
