/**
 * 
 * @author FMARTINI
 *
 */
package p12f.exe.pasarelapagos.helpers;

import java.io.ByteArrayOutputStream;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import n38a.exe.N38APISesion;

import org.w3c.dom.Document;

import com.ejie.r01f.log.R01FLog;
import com.ejie.r01f.xml.utils.XMLUtils;

public class SessionTokenHelper {
	
	private String _appId; // "P12G"
	private String _sessionToken;

	/**
	 * getter _appId
	 * @return
	 */
	public String getAppId() {
		return _appId;
	}

	/**
	 * getter _sessionToken
	 * @return
	 */
	public String getSessionToken() {
		return _sessionToken;
	}

	/**
	 * setter _appId
	 * @param appId
	 */
	public void setAppId(String appId) {
		_appId = appId;
	}

	/**
	 * setter _sessionToken
	 * @param strToken
	 */
	public void setSessionToken(String strToken) {
		_sessionToken = strToken;
	}

	/**
	 * Obtiene un token de sesin y lo asigna a este SignHelper
	 * @param appId id de aplicacin (p.ej. P12G)
	 * @param simulateXLNets true si el token es simulado (p.ej. para pruebas en local)
	 * @return
	 */
	public boolean setSessionToken(String appId, boolean simulateXLNets) {
		
		boolean resulTokenOk = false;

		_appId = appId;

		if (!simulateXLNets) { // Obtener el token XLNets de XLNets
			R01FLog.to("p12f.session").info("[P12F] SessionTokenHelper() Obteniendo token de sesin... (XLNETS)");

			_sessionToken = "";

			try {
				N38APISesion p12APISesion = new N38APISesion();

				Document p12SessionDoc = p12APISesion.n38APISesionCrearApp(appId); // Abrimos Sesin de Aplicacin obteniendo un xml de sesin

				if (p12SessionDoc != null) {
					R01FLog.to("p12f.session").info("[P12F] SessionTokenHelper() Sesin de aplicacin abierta para " + appId + "...");
					//R01FLog.to("p12f.session").info("[P12F] " + XMLUtils.write(p12SessionDoc,null));

					// Sesin abierta, convertimos token a String...
					TransformerFactory transformerFactory = TransformerFactory.newInstance ();
					Transformer transformer;

					transformer = transformerFactory.newTransformer ();
					transformer.setOutputProperty(OutputKeys.METHOD,  "xml");
					transformer.setOutputProperty(OutputKeys.ENCODING,"ISO-8859-1");
					transformer.setOutputProperty(OutputKeys.VERSION, "1.0");

					DOMSource source = new DOMSource(p12SessionDoc);

					ByteArrayOutputStream stream = new ByteArrayOutputStream();
					StreamResult streamResult = new StreamResult (stream); 

					transformer.transform (source, streamResult);

					_sessionToken = stream.toString();

					resulTokenOk = true;
				}
				else {
					R01FLog.to("p12f.session").info("[P12F] SessionTokenHelper() Sesin de aplicacin NO abierta para " + appId + "...");
				}

			} catch (TransformerConfigurationException e) {
				e.printStackTrace();
			} catch (TransformerException e) {
				e.printStackTrace();
			} catch (Exception ex) {
				ex.printStackTrace(System.out);
			}
		}
		else { // apa... para probar, sacamos el token de aqu como P12 (login de aplicacin):
			   //         https://xlnets.servicios.jakina.ejiedes.net/xlnets/serviciosLoginApp.htm
			R01FLog.to("p12f.session").info("[P12F] SessionTokenHelper() Obteniendo token de sesin... (SIMULADO)");

			_sessionToken =
					//"<n38><elementos tipo=\"n38APISesionCrearToken\"><elemento subtipo=\"N38Sesion\"><parametro id=\"n38fhum\"><valor>1354612153989</valor></parametro><parametro id=\"cn\"><valor>P121291035045446</valor></parametro><parametro id=\"n38uidsesion\"><valor>1291040952330</valor></parametro><parametro id=\"n38login\"><valor>P12</valor></parametro><parametro id=\"n38perfiles\"><valor>Publico</valor><valor>P12-PF-0002</valor><valor>P12-PF-0001</valor></parametro><parametro id=\"n38organizacion\"><valor>400</valor></parametro><parametro id=\"iphostnumber\"><valor>10.190.8.101</valor></parametro><parametro id=\"n38reintentossesion\"><valor>1</valor></parametro><parametro id=\"n38sexpiracion\"><valor>988999999999</valor></parametro><parametro id=\"n38tiposesion\"><valor>C/S</valor></parametro><parametro id=\"n38puestouid\"><valor>P12</valor></parametro><parametro id=\"n38fechacreacion\"><valor>1291035045446</valor></parametro><parametro id=\"n38personauid\"><valor>P12</valor></parametro><parametro id=\"n38fechalogoff\"><valor/></parametro><parametro id=\"n38valorhash\"><valor>MZrok7Ph8nTBBeWN7TldZpdGRc8=</valor></parametro><parametro id=\"n38fechainicio\"><valor>1291035045446</valor></parametro><parametro id=\"n38idioma\"><valor>1</valor></parametro><parametro id=\"dni\"><valor>P12</valor></parametro><parametro id=\"n38comportamientoip\"><valor>ipv</valor></parametro><parametro id=\"n38idcontexto\"><valor>intranet</valor><valor>internet</valor></parametro><parametro id=\"n38loginasociado\"><valor>https://xlnets.servicios.jakina.ejiedes.net:443/n38a/N38LoginInicioServlet</valor></parametro><parametro id=\"n38listacede\"><valor>x_cddes01:48880:1000</valor><valor>x_cddes02:48880:1000</valor></parametro><parametro id=\"n38comportamientofirma\"><valor>False</valor></parametro><parametro id=\"n38exclusivo\"><valor>False</valor></parametro><parametro id=\"n38dominiouid\"><valor>D0_Servidores11</valor></parametro><parametro id=\"n38urlsredireccion\"><valor>http://xlnets.servicios.jakina.ejiedes.net/n38nWar/N38NodoEstrellaServlet</valor></parametro><parametro id=\"n38listags\"><valor>x_ssdes01:47770:1000</valor><valor>x_ssdes02:47770:1000</valor></parametro><parametro id=\"n38rangoip\"><valor>10.200.56.11</valor><valor>10.170.9.109</valor><valor>10.170.8.4</valor><valor>10.170.8.5</valor><valor>10.170.9.4</valor><valor>10.170.9.5</valor><valor>10.170.9.137</valor><valor>10.170.9.79</valor><valor>10.200.56.57</valor><valor>10.170.17.114</valor><valor>10.200.8.216</valor><valor>10.200.8.217</valor><valor>10.200.8.218</valor><valor>10.190.8.14</valor><valor>10.170.17.105</valor><valor>192.168.56.231</valor><valor>192.168.56.233</valor><valor>192.168.56.242</valor><valor>192.168.56.241</valor><valor>10.190.20.25</valor><valor>10.190.24.15</valor><valor>192.168.56.234</valor><valor>10.190.8.101</valor><valor>10.190.20.20</valor><valor>192.168.56.21</valor><valor>192.168.56.13</valor><valor>192.168.56.17</valor><valor>192.168.56.11</valor><valor>192.168.56.228</valor><valor>10.170.8.216</valor><valor>192.168.56.58</valor><valor>10.190.20.14</valor><valor>10.170.17.201</valor><valor>555.</valor><valor>10.190.20.43</valor><valor>10.190.20.47</valor><valor>10.190.20.32</valor><valor>10.190.20.40</valor><valor>10.190.20.11</valor><valor>10.190.20.56</valor><valor>10.190.20.38</valor><valor>10.190.8.69</valor><valor>10.190.8.70</valor><valor>10.170.17.200</valor><valor>10.190.20.50</valor><valor>10.190.40.15</valor><valor>10.190.40.16</valor><valor>10.190.40.18</valor><valor>10.190.40.19</valor><valor>10.190.20.69</valor><valor>10.190.20.70</valor><valor>10.170.9.21</valor><valor>10.190.20.67</valor><valor>10.170.16.172</valor><valor>10.190.20.100</valor><valor>10.190.20.102</valor><valor>10.190.20.63</valor><valor>10.190.20.64</valor><valor>10.170.17.203</valor></parametro><parametro id=\"n38dominioscookies\"><valor>.jakina.ejgvdns</valor></parametro><parametro id=\"n38listamsservidores\"><valor>hola Aitzi</valor></parametro><parametro id=\"n38servidoraudit\"><valor>x_auddes01:4560</valor></parametro><parametro id=\"n38subjectcert\"><valor>NO</valor></parametro><parametro id=\"n38certificadopoliticas\"><valor>NO</valor></parametro><parametro id=\"n38oleserver\"><valor>false</valor></parametro><parametro id=\"n38cadescripciondominio\"><valor>ras internete</valor></parametro><parametro id=\"n38eudescripciondominio\"><valor>rak Interneta</valor></parametro><parametro id=\"n38crearsesionapp\"><valor>true</valor></parametro><parametro id=\"n38urlportal\"><valor>https://xlnets.servicios.jakina.ejiedes.net/n38a/N38servletPortal</valor></parametro><parametro id=\"n38urldispatcher\"><valor>http://xlnets.servicios.jakina.ejiedes.net/n38nWar/N38DispatcherEstrellaServlet</valor></parametro><parametro id=\"n38urlsgsd\"><valor>https://xlnets.servicios.jakina.ejiedes.net/n38nCDominioWar/N38GestorCDominioServlet</valor></parametro><parametro id=\"n38dominiocomuncookie\"><valor>.jakina.ejgvdns</valor></parametro><parametro id=\"n38sistemaxlnetsuid\"><valor>desarrollo-produccion</valor></parametro><parametro id=\"n38direccionproxy\"><valor>http://www.servicios.comunes.ejiedes.net/proxy</valor></parametro><parametro id=\"n38direcciongrc\"><valor>http://xlnets.servicios.jakina.ejiedes.net/n38rWar/N38rFachadaGRCServlet</valor></parametro><parametro id=\"n38grupoorganicouid\"><valor/></parametro><parametro id=\"n38orgunituid\"><valor/></parametro><parametro id=\"n38urlfinal\"><valor>http://www.euskadi.net</valor></parametro><parametro id=\"n38uidsesionglobal\"><valor>desarrollo-produccion-1291040952330</valor></parametro><parametro id=\"n38cadenaconfianza\"><valor>desarrollo-produccion</valor></parametro><parametro id=\"n38politicauid\"><valor>P2</valor></parametro></elemento></elementos></n38>";
					"<n38><elementos tipo=\"n38APISesionCrearToken\"><elemento subtipo=\"N38Sesion\"><parametro id=\"cn\"><valor>P12G1363796446086</valor></parametro><parametro id=\"n38uidsesion\"><valor>1363821468201</valor></parametro><parametro id=\"n38login\"><valor>P12G</valor></parametro><parametro id=\"n38perfiles\"><valor>Publico</valor></parametro><parametro id=\"n38organizacion\"><valor>400</valor></parametro><parametro id=\"iphostnumber\"><valor>10.190.20.197</valor></parametro><parametro id=\"n38reintentossesion\"><valor>1</valor></parametro><parametro id=\"n38fhum\"><valor>1363796446086</valor></parametro><parametro id=\"n38sexpiracion\"><valor>988999999999</valor></parametro><parametro id=\"n38tiposesion\"><valor>C/S</valor></parametro><parametro id=\"n38puestouid\"><valor>P12G</valor></parametro><parametro id=\"n38fechacreacion\"><valor>1363796446086</valor></parametro><parametro id=\"n38personauid\"><valor>P12G</valor></parametro><parametro id=\"n38fechalogoff\"><valor/></parametro><parametro id=\"n38valorhash\"><valor>fq7StAav46+ojVQmPNhE+2KvT1Y=</valor></parametro><parametro id=\"n38fechainicio\"><valor>1363796446086</valor></parametro><parametro id=\"n38idioma\"><valor>3</valor></parametro><parametro id=\"dni\"><valor>P12G</valor></parametro><parametro id=\"n38comportamientoip\"><valor>ipv</valor></parametro><parametro id=\"n38idcontexto\"><valor>intranet</valor><valor>internet</valor></parametro><parametro id=\"n38loginasociado\"><valor>https://xlnets.servicios.jakina.ejiedes.net:443/n38a/N38LoginInicioServlet</valor></parametro><parametro id=\"n38listacede\"><valor>x_cddes01:48880:1000</valor><valor>x_cddes02:48880:1000</valor></parametro><parametro id=\"n38comportamientofirma\"><valor>False</valor></parametro><parametro id=\"n38exclusivo\"><valor>False</valor></parametro><parametro id=\"n38dominiouid\"><valor>D0_Servidores11</valor></parametro><parametro id=\"n38urlsredireccion\"><valor>http://xlnets.servicios.jakina.ejiedes.net/n38nWar/N38NodoEstrellaServlet</valor></parametro><parametro id=\"n38listags\"><valor>x_ssdes01:47770:1000</valor><valor>x_ssdes02:47770:1000</valor></parametro><parametro id=\"n38rangoip\"><valor>10.200.56.11</valor><valor>10.170.9.109</valor><valor>10.170.8.4</valor><valor>10.170.8.5</valor><valor>10.170.9.4</valor><valor>10.170.9.5</valor><valor>10.170.9.137</valor><valor>10.170.9.79</valor><valor>10.200.56.57</valor><valor>10.170.17.114</valor><valor>10.200.8.216</valor><valor>10.200.8.217</valor><valor>10.200.8.218</valor><valor>10.190.8.14</valor><valor>10.170.17.105</valor><valor>192.168.56.231</valor><valor>192.168.56.233</valor><valor>192.168.56.242</valor><valor>192.168.56.241</valor><valor>10.190.20.25</valor><valor>10.190.24.15</valor><valor>192.168.56.234</valor><valor>10.190.8.101</valor><valor>10.190.20.20</valor><valor>192.168.56.21</valor><valor>192.168.56.13</valor><valor>192.168.56.17</valor><valor>192.168.56.11</valor><valor>192.168.56.228</valor><valor>10.170.8.216</valor><valor>192.168.56.58</valor><valor>10.190.20.14</valor><valor>10.170.17.201</valor><valor>555.</valor><valor>10.190.20.43</valor><valor>10.190.20.47</valor><valor>10.190.20.32</valor><valor>10.190.20.40</valor><valor>10.190.20.11</valor><valor>10.190.20.56</valor><valor>10.190.20.38</valor><valor>10.190.8.69</valor><valor>10.190.8.70</valor><valor>10.170.17.200</valor><valor>10.190.20.50</valor><valor>10.190.40.15</valor><valor>10.190.40.16</valor><valor>10.190.40.18</valor><valor>10.190.40.19</valor><valor>10.190.20.69</valor><valor>10.190.20.70</valor><valor>10.170.9.21</valor><valor>10.190.20.67</valor><valor>10.170.16.172</valor><valor>10.190.20.100</valor><valor>10.190.20.102</valor><valor>10.190.20.63</valor><valor>10.190.20.64</valor><valor>10.170.17.203</valor><valor>10.170.17.207</valor><valor>10.190.20.101</valor><valor>10.190.28.18</valor><valor>10.190.20.71</valor><valor>10.190.20.138</valor><valor>10.190.20.139</valor><valor>10.190.20.86</valor><valor>10.170.18.235</valor><valor>10.190.16.12</valor><valor>10.170.18.230</valor><valor>10.190.28.28</valor><valor>10.190.20.175</valor><valor>10.190.20.177</valor><valor>10.190.20.179</valor><valor>10.190.20.181</valor><valor>10.190.20.183</valor><valor>10.190.20.185</valor><valor>10.190.20.191</valor><valor>10.190.20.193</valor><valor>10.190.40.46</valor><valor>10.190.40.48</valor><valor>10.190.40.38</valor><valor>10.190.40.40</valor><valor>10.190.20.197</valor><valor>10.190.20.195</valor><valor>10.190.40.42</valor><valor>10.190.40.44</valor><valor>10.190.16.31</valor><valor>10.190.20.189</valor><valor>10.190.20.190</valor><valor>10.190.20.215</valor><valor>10.190.20.217</valor><valor>10.190.20.219</valor><valor>10.190.20.221</valor><valor>10.190.20.199</valor><valor>10.190.20.201</valor><valor>10.190.20.203</valor><valor>10.190.20.205</valor><valor>10.190.28.11</valor><valor>10.190.28.24</valor></parametro><parametro id=\"n38dominioscookies\"><valor>.jakina.ejgvdns</valor></parametro><parametro id=\"n38listamsservidores\"><valor>hola Aitzi</valor></parametro><parametro id=\"n38servidoraudit\"><valor>x_auddes01:4560</valor></parametro><parametro id=\"n38subjectcert\"><valor>NO</valor></parametro><parametro id=\"n38certificadopoliticas\"><valor>NO</valor></parametro><parametro id=\"n38oleserver\"><valor>false</valor></parametro><parametro id=\"n38cadescripciondominio\"><valor>ras internete</valor></parametro><parametro id=\"n38eudescripciondominio\"><valor>rak Interneta</valor></parametro><parametro id=\"n38crearsesionapp\"><valor>true</valor></parametro><parametro id=\"n38urlportal\"><valor>https://xlnets.servicios.jakina.ejiedes.net/n38a/N38servletPortal</valor></parametro><parametro id=\"n38urldispatcher\"><valor>http://xlnets.servicios.jakina.ejiedes.net/n38nWar/N38DispatcherEstrellaServlet</valor></parametro><parametro id=\"n38urlsgsd\"><valor>https://xlnets.servicios.jakina.ejiedes.net/n38nCDominioWar/N38GestorCDominioServlet</valor></parametro><parametro id=\"n38dominiocomuncookie\"><valor>.jakina.ejgvdns</valor></parametro><parametro id=\"n38sistemaxlnetsuid\"><valor>desarrollo-produccion</valor></parametro><parametro id=\"n38direccionproxy\"><valor>http://www.servicios.comunes.ejiedes.net/proxy</valor></parametro><parametro id=\"n38direcciongrc\"><valor>http://xlnets.servicios.jakina.ejiedes.net/n38rWar/N38rFachadaGRCServlet</valor></parametro><parametro id=\"n38grupoorganicouid\"><valor>400</valor></parametro><parametro id=\"n38orgunituid\"><valor>10500</valor></parametro><parametro id=\"n38urlfinal\"><valor>http://jakina</valor></parametro><parametro id=\"n38uidsesionglobal\"><valor>desarrollo-produccion-1363821468201</valor></parametro><parametro id=\"n38cadenaconfianza\"><valor>desarrollo-produccion</valor></parametro><parametro id=\"n38politicauid\"><valor>P2</valor></parametro></elemento></elementos></n38>";

			resulTokenOk = true;
		}

		R01FLog.to("p12f.session").info("[P12F] SessionTokenHelper() ##### TOKEN DE SESION (substring): " + _sessionToken.substring(0, Math.min(60,_sessionToken.length())).replace('\n', ' ') + "...");

		return resulTokenOk;
	}

}
