package p12f.exe.pasarelapagos.objects;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.bouncycastle.util.encoders.Hex;

import p12f.exe.pasarelapagos.security.crypto.TripleDESEBCDICCryptoHelper;

import com.ejie.r01f.util.StringUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

public class CreditCardData implements Serializable{

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public String timeStamp;
	public EncData encData; 
	public Map aditionalDataList;
	
	
	
	public  CreditCardData () {
		encData = new EncData();
	}
	
	
	/**
	 * Convierte un Objeto Concepto a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}

	
	/**
	 * Convierte un XML en un objeto DatosPago.
	 * @return Objeto DatosPago construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static CreditCardData getObject(String xml) throws XOMarshallerException {    
	    return (CreditCardData)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
	
	
	/**
	 * Metodo que realiza el XOR de dos arrays de bytes.
	 * @param clave1
	 * @param clave2
	 * @return Array de bytes con el resultado de la operacin.
	 */
	public static byte[] doXOR(String clave1, String clave2) throws Exception{
		
		if(clave1.length()!=clave2.length()){
			throw new Exception("Las dos cadenas deben tener la misma longitud.");
		}
		
		byte[] keyBytes1 = Hex.decode(clave1);
		byte[] keyBytes2 = Hex.decode(clave2);		
	
		byte[] keyBytesXOR = new byte[keyBytes1.length];
		
		for(int i=0;i<keyBytesXOR.length;i++){
			int intValue = new Byte(keyBytes1[i]).intValue()^new Byte(keyBytes2[i]).intValue();
			keyBytesXOR[i]=new Integer(intValue).byteValue();
		}
		
		return keyBytesXOR;
	}

	public static void main(String[] argv) throws Exception {
		
		String creditCardNumber="5540160500035358"; 
		//String creditCardUser="OLABARRIA OZAETA/PEPE"; 
		String creditCardYear="08";
		String creditCardMonth="07"; 
		
		creditCardNumber = StringUtils.rPadWithChar(creditCardNumber,' ',20);		
		
		String data = creditCardNumber + "20" + creditCardMonth + creditCardYear;
			
		data = StringUtils.rPadWithChar(data,' ',32);
	
	    
	    System.out.println(data);
	    
	    
	    String clave1="5D5850420" + "0080700" +  "0080700" + "0080700"  +  "0080700" + "F000406" + "FFFF";
	    String clave2="010009000" + "F000406" +  "F000406" + "F000406"  +  "7080700" + "F000406" + "FFFF";
	
	    System.out.println("Clave 1=" + clave1);
	    System.out.println("Clave 2=" + clave2);
	    
	    

	    System.out.println("Clave 1 Length=" + clave1.length());
	    System.out.println("Clave 1 Length=" + clave1.getBytes().length);
	    
	    System.out.println("Clave 2=" + clave2);
	    
	    
	       
	    
	    String claveFinal  = new String(Hex.encode(doXOR(clave1, clave2))).toUpperCase();

	    System.out.println("Clave Final="+claveFinal);
	    System.out.println("Clave Final=" + claveFinal.length());
	    System.out.println("Clave Final=" + claveFinal.getBytes().length);
	    
	    TripleDESEBCDICCryptoHelper encrypter = new TripleDESEBCDICCryptoHelper();
	    
	    System.out.println(data);
	    
	    
	    
	    String encrypted = encrypter.encrypt(claveFinal,data);
	    String encryptedHEX = ((TripleDESEBCDICCryptoHelper) encrypter).encryptHEX(claveFinal,data);
	    
	    
	    System.out.println("Encripted="+encrypted);    
	    System.out.println("EncriptedHex="+encryptedHEX);    
	    
	    String decrypted = encrypter.decrypt(claveFinal,encrypted);
	    System.out.println(decrypted);
	    
	    System.out.println(encrypted.length());   
	    
	    
	    CreditCardData  creditCardData = new CreditCardData();
	    creditCardData.encData.value = encrypted;
	    creditCardData.encData.encType =  "base64";
	    creditCardData.timeStamp = "" + System.currentTimeMillis();
	    	    
	    AditionalData aditionaData  = new AditionalData();
	    aditionaData.name = "datoAdicional 1";
	    aditionaData.value = "valor x";
	   
	    creditCardData.aditionalDataList = new HashMap();
	    creditCardData.aditionalDataList.put(aditionaData.name , aditionaData);
	    System.out.println(creditCardData.toXML());
	    
	    creditCardData = new CreditCardData();
	    creditCardData.encData.value = encryptedHEX;
	    creditCardData.encData.encType =  "hex";
	    creditCardData.timeStamp = "" + System.currentTimeMillis();
	    	    
	    aditionaData  = new AditionalData();
	    aditionaData.name = "datoAdicional 1";
	    aditionaData.value = "valor x";
	   
	    creditCardData.aditionalDataList = new HashMap();
	    creditCardData.aditionalDataList.put(aditionaData.name , aditionaData);
	    System.out.println(creditCardData.toXML());
	    
	    
	    
	    

	    
	    
	    
	    
	    
	    
	}
	
	

	
	
}
