/*
 * Created on 18-may-2005
 * @author co01347e
 */
package p12f.exe.pasarelapagos.objects;

import java.io.Serializable;
import java.util.Map;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Estructura que contiene el estado de un pago.
 */
public class Estado implements Serializable, Initializable {

	public static final String REGISTRADO = "00";
	public static final String EMITIDA_LIQUIDACION = "01";
	public static final String ENVIADO_ENTIDAD = "02";
	public static final String ANULADO = "03";
	public static final String PAGADO = "04";
	public static final String ERROR_PAGO = "05";
	public static final String NO_DATA = "06";
	public static final String PETICION_DEVOLUCION = "07";
	public static final String INGRESO_DEVUELTO = "08"; // puede que sea parcialmente devuelto
	public static final String PETICION_EJECUCION = "09";
	public static final String INGRESO_EJECUTADO = "10"; // puede que sea parcialmente ejecutado
	
	/**
	 * Cdigo que indica el estado del pago.
	 * El cdigo del estado puede ser uno de los siguientes:
	 * 		pagado  	El pago ha sido realizado correctamente.
	 * 		noPagado  	El pago no ha sido realizado por cualquier
	 * 					causa (razones tcnicas, saldo insuficiente,
	 * 					rfaga no vlida, etc.).
	 * 		noData		La Entidad Financiera no tiene datos sobre
	 * 					el pago.
	 */
	public String codigo;
	/**
	 * Este dato forma parte del NRC devuelto por la 
	 * Entidad Financiera.
	 * 	codigo=pagado	Indica la fecha en la que se realiz
	 * 					el pago en el formato ddmmaa	.
	 * 	codigo=noPagado	Este campo no se devuelve.
	 * 	codigo=noData	Este campo no se devuelve.
	 */
	public String fechaPago;
	/**
	 * Este dato forma parte del NRC devuelto por la
	 * Entidad Financiera.
	 * 	codigo=pagado	Indica la hora en la que se realiz
	 * 					el pago en el formato hhmmss.
	 * 	codigo=noPagado	Este campo no se devuelve.
	 * 	codigo=noData	Este campo no se devuelve.
	 */
	public String horaPago;
	/**
	 * Razn del error que ha imposibilitado el pago.
	 * razonError=noSaldo		No hay saldo suficiente
	 * razonError=noPosible		El emisor/sufijo no esta dado de alta en el HOST
	 * razonError=generalError	Error tecnico general
	 */
	public String razonError;
	/**
	 * Importe del pago efectuado en la Entidad Financiera.
	 */
	public String importe;
	/**
	 * Cdigo de la Entidad Financiera en la que se ha realizado el pago.
	 */
	public String entidad;
	/**
	 * Cdigo de la oficina en la que se ha realizado el pago.
	 */
	public String oficina;
	/**
	 * Nmero de operacin en la Entidad Financiera.
	 */
	public String numeroOperacion;
	/**
	 * Pago ID
	 */
	public String paymentID;
	/**
	 * NRC completo del pago.
	 */
	public String nrc;
	/**
	 * Mensajes en diferentes idiomas acerca de la consulta.
	 */
	public Map mensajes;
	/**
	 * Datos de estado de la vuelta de SIPCA
	 */
	public Map backendDataMap;
	
	public Estado(){
		ObjectUtils.initialize(this);
	}
	
	/**
	 * Convierte un Objeto Estado a XML.
	 * @return
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto estado.
	 * @return Objeto Estado construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static Estado getObject(String xml) throws XOMarshallerException {    
		return (Estado)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
	
}
