/**
 * Created on Oct 26, 2010
 * @author co00414a
 * (c) 2010 EJIE: Eusko Jaurlaritzako Informatika Elkartea 
 */
package p12f.exe.pasarelapagos.objects;

import java.io.Serializable;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * 
 * Estructura que contiene los datos de solicitud de un procesamiento batch.
 *
 */
public class FileManagerRequest implements Serializable, Initializable {

	public static final String DELETE = "DELETE";
	public static final String CREATE = "CREATE";
	
	/**
	 * Tipo de operacin a realizar
	 */
	public String operacion;
	
	/**
	 * Identificador de la administracin
	 */
	public String adminOid;
	
	/**
	 * Identificador del token 
	 */
	public String idToken;
	
	/**
	 * Nombre del fichero
	 */
	public String fileName;
	
	/**
	 * Contenido del fichero
	 */
	public String content;
	
	/**
	 * Constructor sin argumentos.
	 */
	public FileManagerRequest(){
	    ObjectUtils.initialize(this);
	}
	
	/**
	 * Convierte un Objeto BatchProcessRequest a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto BatchProcessRequest.
	 * @return Objeto BatchProcessRequest construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static FileManagerRequest getObject(String xml) throws XOMarshallerException {    
	    return (FileManagerRequest)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
	
	public static void main(String[] argv) throws XOMarshallerException {
		FileManagerRequest fmr = new FileManagerRequest();
		fmr.operacion = FileManagerRequest.DELETE;
		fmr.adminOid = "04833001";
		fmr.idToken = "Prueba";
		fmr.fileName = "PruebaFichero.xml";
		fmr.content = "Hola Mundo";
		System.out.println(fmr.toXML());
	}
}
