package p12f.exe.pasarelapagos.objects;

import java.io.IOException;
import java.io.Serializable;
import java.util.Map;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.util.encoders.BASE64Decoder;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Estructura que contiene los datos sobre un evento de la Pasarela.
 */

public class GatewayEvent implements Serializable, Initializable {
    
	public static final String PREINICIALIZAR = "00";
	public static final String INICIALIZAR = "01";
	public static final String COMENZAR = "02";
    public static final String CANCELAR = "03";
    public static final String PAGAR_ON_LINE = "04";
    public static final String PAGAR_OFF_LINE = "05";
    public static final String VALIDACION_OK = "06";
    public static final String VALIDACION_NOK = "07";
    public static final String PAGO_ON_LINE_OK = "08";
    public static final String PAGO_ON_LINE_NOK = "09";
	
	/**
	 * Tipo de evento lanzado (ver tabla anterior)
	 */
	public String tipo;
	/**
	 * Momento en el que se ha lanzado el evento en la Pasarela de Pagos
	 */
	public long timeStamp;
	/**
	 * Mensajes asociados al evento
	 */
	public Map mensajes;
	/**
	 * Identificador del objeto pago al que se refiere el evento 
	 */
	public String id;
	/**
	 * Datos del pago que ha dado lugar al evento;
	 */
	public DatosPago datosPago;

	
	/**
	 * Expediente (si lo hubiera)
	 * 
	 */
	public Expediente expediente;
	
	/**
	 * Estado del pago que ha dado lugar al evento.	 * 
	 * 
	 */
	
	public Estado estado;
	
	/**
	 * Constructor sin argumentos.
	 */
	public GatewayEvent(){
	    ObjectUtils.initialize(this);
	}
		
	/**
	 * Nombre imprimible del tipo de evento
	 * @return
	 */
	public String getTipoName() {
		if (tipo.equals("00")) return "PREINICIALIZAR";
		if (tipo.equals("01")) return "INICIALIZAR";
		if (tipo.equals("02")) return "COMENZAR";
		if (tipo.equals("03")) return "CANCELAR";
		if (tipo.equals("04")) return "PAGAR_ON_LINE";
		if (tipo.equals("05")) return "PAGAR_OFF_LINE";
		if (tipo.equals("06")) return "VALIDACION_OK";
		if (tipo.equals("07")) return "VALIDACION_NOK";
		if (tipo.equals("08")) return "PAGO_ON_LINE_OK";
		if (tipo.equals("09")) return "PAGO_ON_LINE_NOK";
		return tipo;
	}

	/**
	 * Convierte un Objeto GatewayEvent a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto GatewayEvent.
	 * @return Objeto GatewayEvent construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static GatewayEvent getObject(String xml) throws XOMarshallerException {    
	    return (GatewayEvent)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}

	/**
	 * Convierte un XML codificado 64 en un objeto GatewayEvent.
	 * @return Objeto GatewayEvent construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static GatewayEvent getObjectFromBase64(String xml64) throws XOMarshallerException {

		BASE64Decoder base64Decoder = new BASE64Decoder();
		String xmlGatewayEvent = null;
		try {
			xmlGatewayEvent = new String(base64Decoder.decodeBuffer(xml64.replaceFirst("<!.CDATA.", "").replaceFirst("]]>", "")));
		} catch (IOException e) {
			e.printStackTrace();
		}

	    return (GatewayEvent)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xmlGatewayEvent);
	}
}


