package p12f.exe.pasarelapagos.objects;

import java.io.Serializable;
import java.util.Map;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Estructura que contiene los datos bancarios a certificar.
 */

public class HolderCertData implements Serializable, Initializable{
	
	/*****************************************************************
	 * Constantes ****************************************************
	 *****************************************************************/
	
	// Tipos de Holder  (Cuenta o Credit Card Data)
	public static final String TYPE_ACCOUNTNUMBER = "accountNumber";
	public static final String TYPE_CREDITCARDATA = "creditCardData";
	
	// Formatos de Cuenta (Iban o Cuenta Corriente)
	public static final String ACCOUNT_NUMBER_FORMAT_IBAN = "iban";
	public static final String ACCOUNT_NUMBER_FORMAT_CC = "cc";
	
	// Longitudes de Cuenta (Iban o Cuenta Corriente)
	public static final int ACCOUNT_NUMBER_LENGTH_IBAN = 24;
	public static final int ACCOUNT_NUMBER_LENGTH_CC = 20;
	
	// Formato Final de Encriptacion (None, Base64, Hex)

	public static final  String  ENC_TYPE_HEX = "hex";
	public static final  String  ENC_TYPE_BASE64 = "base64";
	public static final  String  ENC_TYPE_NONE= "none";
	
	/*****************************************************************
	 * Miembros ******************************************************
	 *****************************************************************/
	
	// Atributos
	public String type;
	public String accountNumberFormat;
	public String encType;

	// Elementos
	public String citizenId;
	public String itemToCertNumber;	
	public String requestID;
	public String requestAdminID;
	public Map aditionalDataList;
	
	/**
	 * Constructor vaco
	 */
	public HolderCertData(){
	    ObjectUtils.initialize(this);
	}
	
	
	/**
	 * Convierte un objeto HolderCertData a XML.
	 * @return String con el XML que representa al HolderCertData
	 * @throws XOMarshallerException
	 */
    public String toXML() throws XOMarshallerException {   
         return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
    }
	
    /**
	 * Convierte un XML en un objeto HolderCertData.
	 * @return Objeto HolderCertData construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static HolderCertData getObject(String xml) throws XOMarshallerException {    
	    return (HolderCertData)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
	
	
	// Prueba
	public static void main(String[] argv) throws XOMarshallerException {
		
		HolderCertData  holderCertData = new HolderCertData();
		holderCertData.type = HolderCertData.TYPE_ACCOUNTNUMBER;
		holderCertData.accountNumberFormat = HolderCertData.ACCOUNT_NUMBER_FORMAT_IBAN;
		holderCertData.citizenId = "78802525T";
		holderCertData.itemToCertNumber = "209444444444444444444444";
		holderCertData.encType = HolderCertData.ENC_TYPE_NONE;
		
		AditionalData aditionaData  = new AditionalData();
		aditionaData.name = "datoAdicional 1";
		aditionaData.value = "valor x";
		 
		holderCertData.aditionalDataList.put(aditionaData.name, aditionaData);
		
		System.out.println(holderCertData.toXML());
	}
	
}
