package p12f.exe.pasarelapagos.objects;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Clase que devuelve el resultado de una CERTIFICACIN HOLDER..
 */
public class HolderCertResponse implements Serializable {
	
	// Tipos de Respuesta 
	/**
	 *  La cuenta existe y adems est asociada al tercero como titular/cotitular
	 */
	public static final int CERTCODE_VALID = 1;
	/**
	 *  La cuenta existe y adems est asociada al tercero como autorizado
	 */
	public static final int CERTCODE_AUTHORIZED = 2;
	/**
	 *  El nmero de cuenta no existe o no est asociado al tercero
	 */
	public static final int CERTCODE_INVALID = -1;
	/**
	 *  El dni es titular/cotitular de la cuenta, pero dicho nmero de cuenta ha cambiado y no est actualizado
	 */
	public static final int CERTCODE_VALID_ACCOUNT_MODIFIED = -2;
	/**
	 *  ERROR DEVUELTO POR LA EF -> este error puede equivaler a CERTCODE_INVALID segn la EF
	 *  TODO Habra que homogeneizar las respuestas de la EF
	 */
	public static final int CERTCODE_ERROR = -999;
	/**
	 *  ERROR AL ENVIAR A LA EF
	 */
	public static final int CERTCODE_ERROR_SENDING = -998;
	/**
	 *  SIN RESPUESTA - REENVIOS SUPERADOS/EF NO ADHERIDA
	 */
	public static final int CERTCODE_FINAL_ERROR = -1000;
	/**
	 *  SIN RESPUESTA
	 */
	public static final int CERTCODE_NO_RESPONSE = 0;
	
	public static final Set resultsNoFinal;
	static {
		resultsNoFinal = new HashSet();
		resultsNoFinal.add(new Integer(HolderCertResponse.CERTCODE_NO_RESPONSE));
		resultsNoFinal.add(new Integer(HolderCertResponse.CERTCODE_ERROR));
		resultsNoFinal.add(new Integer(HolderCertResponse.CERTCODE_ERROR_SENDING));
	}
	
	// Elementos
	public int certCode = HolderCertResponse.CERTCODE_NO_RESPONSE;
	public String timeStamp = "0";
	public String finantialOrgCode ;
	public String nrc;
	public Map aditionalHolderData = null;
	public Map accountConstraintList = null;
	
	/**
	 * Constructor vaco
	 */
	public HolderCertResponse(){ }
	
	/**
	 * Convierte un objeto HolderCertResponse a XML.
	 * @return String con el XML que representa al HolderCertResponse
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto HolderCertResponse.
	 * @return Objeto HolderCertResponse construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static HolderCertResponse getObject(String xml) throws XOMarshallerException {
		return (HolderCertResponse)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
	
}
