/*
 * Created on 18-may-2005
 * @author co01347e
 */
package p12f.exe.pasarelapagos.objects;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import p12f.exe.pasarelapagos.exceptions.PaymentException;
import p12f.exe.pasarelapagos.paymentrequest.ConceptoPeticion;
import p12f.exe.pasarelapagos.paymentrequest.PeticionPago;
import p12f.exe.pasarelapagos.utils.LimitDateHelper;

import com.ejie.r01f.log.R01FLog;
import com.ejie.r01f.reflection.ReflectionException;
import com.ejie.r01f.reflection.ReflectionUtils;
import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Estructura que contiene una instancia de pago.
 */
public class Pago implements Serializable, Initializable {
	
	/**
	 * Identificador del pago.
	 */
	public String id;
	/**
	 * Datos sobre el pago.
	 */
	public DatosPago datosPago;
	/**
	 * Descripciones del pago en diferentes idiomas.
	 */
	public Map descripcion;
	/**
	 * Desglose de los conceptos del pago.
	 */
	public List conceptos;
	/**
	 * Datos del emisor del pago.
	 */
	public Emisor emisor;
	/**
	 * Datos del expediente al que se refiere el pago.
	 */
	public Expediente expediente;
	/**
	 * Imagenes del pago.
	 */
	public Map imagenes;
	/**
	 * Mensajes del pago.
	 */
	public Map mensajes;
	/**
	 * Informacin referente a la domiciliacin del pago.
	 */
	public Domiciliacion domiciliacion;
	/**
	 * Informacin referente al TPV Virtual.
	 */
	public TPVVirtual tpvVirtual;
	/**
	 * Datos Adicionales
	 */
	public List datosAdicionales;
	
	/**
	 * Constructor sin parmetros.
	 */
	public Pago(){
	    ObjectUtils.initialize(this);
	}
	
	/**
	 * Constructor que obtiene un objeto Pago a partir de un objeto Peticion de Pago.
	 * @param peticionPago Objeto PeticionPago a partir del cual se quiere obtener el objeto Pago. 
	 * @throws PaymentException 
	 */
	public Pago (PeticionPago peticionPago) throws PaymentException {		
		
		Field[] fieldsThis = this.getClass().getDeclaredFields();
		Field thisField = null;
		Field otherField = null;
		try {
			for (int i = 0 ; i< fieldsThis.length ; i++){
				thisField = fieldsThis[i];			
				otherField = peticionPago.getClass().getField(thisField.getName());
                if (otherField!=null){                    
                    Object objValue = otherField.get(peticionPago);
                    otherField.set(this,objValue);
                }               
			}
            this._setConceptos(peticionPago.conceptos);
		} catch (SecurityException e) {
			e.printStackTrace();
			throw new PaymentException("Pago (PeticionPago peticionPago)::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
			throw new PaymentException("Pago (PeticionPago peticionPago).NoSuchFieldException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new PaymentException("Pago (PeticionPago peticionPago).IllegalAccessException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		}
 
	}

	/**
	 * Convierte un Objeto Pago a XML.
	 * @return
	 * @throws XOMarshallerException
	 */
    public String toXML() throws XOMarshallerException {   
         return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
    }
	
	/**
	 * Convierte un XML en un objeto Pago.
	 * @return Objeto Pago construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static Pago getObject(String xml) throws XOMarshallerException {    
	    return (Pago)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
    
   /**
    * Metodo que compara este objeto Pago con otro pasado como parmetro.
    * @param other Objeto Pago con el cual se va a comparar.
    * @return Resultado de la comparacin.
    */ 
    public boolean equals(Pago other) {
        if (!(other instanceof Pago)) return false;
        
        try {
            // Si los objetos son de la misma clase, compar los miembros
            if ( this.getClass().equals(other.getClass()) ) {
                Field fields[] = ReflectionUtils.getAllFieldsArray(this.getClass());  // Lista de miembros
                Field f = null;                                 // Miembro actual
                Object thisFValue = null;   // Valor del miembro actual en esta clase
                Object otherFValue = null;  // Valor del miembro actual en la otra clase
               
                    for (int i=0; i < fields.length; i++) {
                        f = fields[i];
                        thisFValue =    ReflectionUtils.getMemberValue(this,f.getName(),false);                      
                        otherFValue =    ReflectionUtils.getMemberValue(other,f.getName(),false);     
                        if ( thisFValue != null && otherFValue != null){ 
                            if ( thisFValue.getClass().getName().equalsIgnoreCase("java.lang.String")) {
                                R01FLog.to("p12f.pago").finest("This="+((String)thisFValue).length());
                                R01FLog.to("p12f.pago").finest("The other="+((String)otherFValue).length());
                            }
                         }            
                        // Comparar los valores dependiendo del tipo de objeto
                        // si se trata de otro AbstractPagoObj, hacer una llamada recursiva
                        if ( thisFValue != null && otherFValue != null) {
                            // Llamar a equals.
                            // NOTA: Si se trata a su vez de una instancia de AbstractPagoObject
                            //       esto resultara en una llamada recursiva.
                            if ( !thisFValue.equals(otherFValue) ) {
                    
                                R01FLog.to("p12f.pago").finest("Los objetos difieren en el (" + thisFValue.getClass().getName() + ")" + f.getName() );               
                                return false;
                            }
                            
                        } else if (thisFValue == null && otherFValue == null) {
                            // Seguir adelante, de momento son iguales
                        } else {
                            R01FLog.to("p12f.pago").finest("No son iguales");                    
                            return false;   // No son iguales
                        }
                    }
                    return true;    // Los objetos son iguales
        }
        } catch (ReflectionException reflex){
            
            R01FLog.to("p12f.pago").finest("Excepcion :" + reflex.getDetailedMessage());    
        }
            
        R01FLog.to("p12f.pago").finest("Los objetos no son de la misma clase");    
        return false;  // Son objetos de distinta clase
    }
    
    /****************************************************************************************
     * 
     *                          METODOS AUXILIARES PRIVADOS
     *             
     ***************************************************************************************/    
	
	/**
	 * Metodo que inserta los Conceptos de Pago a partir de los Conceptos de Peticion.
	 * Slo se utiliza para el constructor Pago(Peticion de Pago)
	 * @param  conceptosPeticionPago Lista con los conceptos de la peticin de pago.
	 * @throws PaymentException
	 **/
	private void _setConceptos (List conceptosPeticionPago) throws PaymentException {
		
		Concepto conceptoPago = null;
		ConceptoPeticion conceptoPeticionPago = null;
		if (conceptosPeticionPago != null){ 
			//Inicializar las lista de conceptos  
			this.conceptos = new ArrayList();
			for (Iterator it = conceptosPeticionPago.iterator();it.hasNext();){
				conceptoPeticionPago  = (ConceptoPeticion)it.next();
				conceptoPago = new Concepto(conceptoPeticionPago);
				this.conceptos.add(conceptoPago);
			}	
		}
	}
	
    /**
     * Metodo que permite obtener el importe de un pago, que coincide con el importe del periodo activo.
     * @return
     */
    public long getImporte(){
        Map periodosPago = this.datosPago.periodosPago;
        //long importe = 0;      
        for(Iterator it=periodosPago.keySet().iterator(); it.hasNext();){
            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(it.next());
            if (periodoPago.activo){
                return periodoPago.importe;
            }
        }

        // Si hay periodo normal y esta dentro de el
        if(periodosPago.get(PeriodoPago.PERIODO_NORMAL)!=null) {
            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_NORMAL);
            if(this.datosPago.formato.equals("521") || LimitDateHelper.comprobarFecha(periodoPago.fechaFin))
                return periodoPago.importe;
        } 
        // Si hay periodo voluntario y esta dentro de el
        if(periodosPago.get(PeriodoPago.PERIODO_VOLUNTARIO)!=null) {
            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_VOLUNTARIO);
            if(LimitDateHelper.comprobarFecha(periodoPago.fechaFin))
                return periodoPago.importe;
        }
        // Si hay periodo con recargo
        if(periodosPago.get(PeriodoPago.PERIODO_CON_RECARGO)!=null) {
            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_CON_RECARGO);
            return periodoPago.importe;
        }   
        
        return 0;
        
        /*if(importe.length()==2)
            importe = "0" + importe;
        return importe.substring(0,importe.length()-2) + "," + importe.substring(importe.length()-2, importe.length());*/
    }
    
    public void activarPeriodoPago(String idPeriodo) {
        // Se marcan como inactivos todos los periodos del pago
        for(Iterator it=this.datosPago.periodosPago.keySet().iterator();it.hasNext();) {
            PeriodoPago periodoPago = (PeriodoPago)this.datosPago.periodosPago.get(it.next());
            periodoPago.activo = false;
        }        
        
        // Se marca como activo el periodo indicado
        PeriodoPago periodoPago = (PeriodoPago)this.datosPago.periodosPago.get(idPeriodo);
        periodoPago.activo = true;
    }
	
}
