/*
 * Created on 18-may-2005
 * @author co01347e
 */
package p12f.exe.pasarelapagos.objects;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Estructura que contiene los datos de los pagos a realizar on-line.
 */
public class PaymentData implements Serializable, Initializable {

	/**
	 * Mltiples instancias de pagos a realizar.
	 */
	public Map pagos;
	/**
	 * Datos adicionales para utilizaciones futuras.
	 */
	public Map datosAdicionales;
	
	public PaymentData(){
	    ObjectUtils.initialize(this);
	}
	
	/**
	 * Convierte un Objeto PaymentData a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto PaymentData.
	 * @return Objeto PaymentData construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static PaymentData getObject(String xml) throws XOMarshallerException {    
	    return (PaymentData)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
	
	/**
	 * Metodo que permite obtener el importe total de los pagos, que coincide con 
	 * la suma de los importes de los periodos activos de cada una de ellos.
	 * @return
	 */
	public long getImporteTotal(){
	    
	    long total = 0;
	    
	    for(Iterator it=this.pagos.keySet().iterator(); it.hasNext();){
	        Pago pago = (Pago)this.pagos.get(it.next());
	        Map periodosPago = pago.datosPago.periodosPago;
	        for(Iterator iter=periodosPago.keySet().iterator(); iter.hasNext();){
	            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(iter.next());
	            if(periodoPago.activo){
	                total += periodoPago.importe;
	                break;
	            }
	        }
	    }
	    
	    /*String importe = new Long(total).toString();
	    if(importe.length()==2)
	        importe = "0" + importe;
        return importe.substring(0,importe.length()-2) + "," + importe.substring(importe.length()-2, importe.length());*/
	    
	    return total;
	}
	
	/**
	 * Metodo que permite obtener el importe total de los de pagos, que coincide con 
	 * la suma de los importes de los periodos activos de cada uno de ellos.
	 * @return
	 */
	public String showImporteTotal(){
	    
	    long total = 0;
	    
	    for(Iterator it=this.pagos.keySet().iterator(); it.hasNext();){
	        Pago pago = (Pago)this.pagos.get(it.next());
	        Map periodosPago = pago.datosPago.periodosPago;
	        for(Iterator iter=periodosPago.keySet().iterator(); iter.hasNext();){
	            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(iter.next());
	            if(periodoPago.activo){
	                total += periodoPago.importe;
	                break;
	            }
	        }
	    }
	    
	    String importe = new Long(total).toString();
	    while(importe.length()<3)
	        importe = "0" + importe;
        return importe.substring(0,importe.length()-2) + "," + importe.substring(importe.length()-2, importe.length());  
	}
	
	public static void main(String[] args) throws XOMarshallerException {
		
		PaymentData p = PaymentData.getObject("<paymentData><pagos><pago id='9050704833001507234234623651801011200000001110'><domiciliacion><permitir>false</permitir></domiciliacion><imagenes><imagen id='logoEmisor'><url><![CDATA[http://www.ej-gv.net/mipago/images/p72EJGVbilingueSmall.gif]]></url></imagen><imagen id='logoEmisor.mobile'><url><![CDATA[https://www6.euskadi.net/appcont/newPaymentGateway/images/logoemisores/logos_GV/logosGV_mobile/p72ejgvbilinguesmall.png]]></url></imagen></imagenes><datosPago><cpr>9050794</cpr><referencia>2342346236518</referencia><formato>507</formato><emisor>04833001-507</emisor><periodosPago><periodoPago id='periodoNormal'><fechaFin>010112</fechaFin><validarFechaFin>true</validarFechaFin><identificacion>010112</identificacion><importe>111</importe><activo>true</activo></periodoPago></periodosPago><validar>1</validar><tipo>507</tipo><codigo>9050704833001507234234623651801011200000001110</codigo></datosPago><descripcion><es>Cuadernillo 57 estndar</es><eu>Cuadernillo 57 estndar</eu></descripcion><emisor><cif>04833001</cif><nombre><es>Gobierno Vasco</es><eu>Eusko Jaurlaritza</eu></nombre><territorio>lava</territorio><municipio>Vitoria</municipio><codigoPostal>30000</codigoPostal><pais>Espaa</pais><codigo>04833001</codigo><calle>Avda. de Espana</calle></emisor><conceptos><concepto><numeroLinea>1</numeroLinea><baseImponible>0</baseImponible><descripcion><es>Cuadernillo 57 estndar</es><eu>Cuadernillo 57 estndar</eu></descripcion><unidades>0</unidades><tieneIVARepercutido>false</tieneIVARepercutido><IVARepercutido>false</IVARepercutido><tipoIVA>0</tipoIVA><importe>111</importe><importeIVA>0</importeIVA></concepto></conceptos></pago></pagos></paymentData>");
		System.out.println(p.toXML());
	}
}
