/*
 * Created on 18-may-2005
 * @author co01347e
 */
package p12f.exe.pasarelapagos.objects;

import java.io.Serializable;
import java.util.Map;

import p12f.exe.pasarelapagos.objects.validation.ValidationConstants;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.NumberUtils;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Estructura que contiene los datos de un periodo de pago.
 */
public class PeriodoPago implements Serializable, Initializable {
    
    public static final String PERIODO_NORMAL = "periodoNormal";
    public static final String PERIODO_VOLUNTARIO = "periodoVoluntario";
    public static final String PERIODO_CON_RECARGO = "periodoConRecargo";

    /**
	 * Identificacion del periodo de pago.
	 */
    public String id;
	/**
	 * Identificacion del pago en este periodo de pago.
	 */
	public String identificacion;
	/**
	 * Importe del pago en este periodo de pago.
	 */
	public long importe;
	/**
	 * Descripciones del periodo de pago en diferentes idiomas.
	 */
	public Map descripcion;
	/**
	 * Fecha de inicio del periodo de pago.
	 */
	public String fechaInicio;
	/**
	 * Fecha de fin del periodo de pago.
	 */
	public String fechaFin;
	/**
	 * Indica si hay que validar la fecha de fin del periodo de pago.
	 */
	public boolean validarFechaFin = true;
	/**
	 * Indica si es el periodo activo.
	 */
	public boolean activo = false;
	
	public PeriodoPago(){
	    ObjectUtils.initialize(this);
	}
	
	/**
	 * Convierte un Objeto PeriodoPago a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto PeriodoPago.
	 * @return Objeto PeriodoPago construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static PeriodoPago getObject(String xml) throws XOMarshallerException {    
	    return (PeriodoPago)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
    
    /**
     * Mtodo que realiza la validacin de los datos de este objeto.
     * @return
     */
    public String validate(String formato) {
        
        /** Se comprueba que la identificacion:
         *      - no es NULL.
         *      - que es correcto en funcion de formato del pago. 
         */
        if (identificacion==null) {
            return ValidationConstants.IDENTIFICACION_NULL;            
        }
        if (!NumberUtils.isNumber(identificacion)) {
            return ValidationConstants.IDENTIFICACION;            
        } 
        if(formato.equals("507")) {
            if (identificacion.length()!=6) {
                return ValidationConstants.IDENTIFICACION_LONGITUD;            
            } 
        } else if(formato.equals("502") || formato.equals("508")) {
            if (identificacion.length()!=7) {
                return ValidationConstants.IDENTIFICACION_LONGITUD;            
            } 
        } else if(formato.equals("521") || formato.equals("522")) {
            if (identificacion.length()!=10) {
                return ValidationConstants.IDENTIFICACION_LONGITUD;            
            } 
        }
        
        return null;
    }
}
