/*
 * Created on 18-may-2005
 * @author co01347e
 */
package p12f.exe.pasarelapagos.objects;

import java.io.Serializable;
import java.util.Map;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Estructura que permite intercambiar datos de contexto relativos a la llamada a funcin.
 */
public class ProtocolData implements Serializable, Initializable  {

	/**
	 * Token de seguridad para autenticar el origen del mensaje.
	 */
	public String token;
	/**
	 * Url a la que hay que enviar la respuesta a la peticin.
	 * Caso habitual:	En el caso de que este valor no llegue,
	 * 					la respuesta se devolver como response de
	 * 					la request original.
	 * Otros casos: 	Se puede indicar un valor en este
	 * 					parmetro, lo cual har que la respuesta a
	 * 					la llamada a funcin (operationResponse)
	 * 					se enve a la Url que se indica.
	 */
	public String responseURL;
	/**
	 * Identificador de la sesin en el servidor que inicia la llamada.
	 */
	public String sourceSessionId;
	/**
	 * Identificador de la sesin en el servidor que recibe la llamada.
	 */
	public String destinationSessionId;
	/**
	 * TimeStamp del envo del mansaje.
	 */
	public long timeStamp = System.currentTimeMillis();
	/**
	 * Nmero de operacin para el servicio que inicia la llamada.
	 */
	public String sourceOperationNumber;
	/**
	 * Diferentes urls utilizadas para el procesado de la peticin.
	 */
	public Map urls; 
	/**
	 * Diferentes datos adicionales utilizados para el procesado de la peticin.
	 */
	public Map datosAdicionales=null;
	
	public ProtocolData(){
	    ObjectUtils.initialize(this);
	}
	
	/**
	 * Convierte un Objeto ProtocolData a XML.
	 * @return
	 * @throws XOMarshallerException
	 */
    public String toXML() throws XOMarshallerException {   
    	return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
    }
    
    /**
	 * Convierte un XML en un objeto ProtocolData.
	 * @return
	 * @throws XOMarshallerException
	 */
	public static ProtocolData getObject(String xml) throws XOMarshallerException {    
	    return (ProtocolData)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
	
}
