/*
 * Created on 18-may-2005
 * @author co01347e
 */
package p12f.exe.pasarelapagos.objects;

import java.io.Serializable;
import java.util.List;

import p12f.exe.pasarelapagos.utils.PaymentComparator;

import com.ejie.r01f.log.R01FLog;
import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Estructura que contiene los datos del tercero al que hay que imputar el pago.
 */
public class Tercero implements Serializable, Initializable {

	public static final String TIPO_CIF = "1";
	public static final String TIPO_NIF = "2";
	public static final String TIPO_EXTRANJERO = "3";
	
	/**
	 * DNI / NIF
	 */
	public String dniNif;
	/**
	 * Razn Social del tercero.
	 */
	public String razonSocial;
	/**
	 * Domicilio.
	 */
	public String calle;
	/**
	 * Municipio.
	 */
	public String municipio;
	/**
	 * Territorio.
	 */
	public String territorio;
	/**
	 * Pas.
	 */
	public String pais;
	/**
	 * Cdigo Postal.
	 */
	public String codigoPostal;
	
	/**
	 * Tipo de tercero. Obligatorio si el derecho reconocido devenga a la emision de la liquidacion.
	 * 		1: CIF
	 * 		2: NIF
	 * 		3: Extranjero
	 */
	public String tipoTercero;
	/**
	 * Primer Apellido.
	 */
	public String primerApellido;
	/**
	 * Segundo Apellido.
	 */
	public String segundoApellido;
	
	
	
	/**
	 * Datos Adicionales
	 */
	public List datosAdicionales;
	
	public Tercero(){
		ObjectUtils.initialize(this);
	}
	
	/**
	 * Convierte un Objeto Tercero a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto Tercero.
	 * @return Objeto Tercero construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static Tercero getObject(String xml) throws XOMarshallerException {
		return (Tercero)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
	
	/**
	 * 
	 * @param other
	 * @return
	 */
	public boolean updateTercero(Tercero other) {
		boolean actualizado = false;
		List differences = PaymentComparator.compare(this, other);
		
		if(differences!=null && differences.size()>0) {
			actualizado = true;
			R01FLog.to("p12f.updatePeticionPago").fine("Se ha actualizado la peticion de pago......");
		}
		return actualizado;
	}
	
}
