/*
 * Created on 06-jun-2005
 * @author co01347e
 */
package p12f.exe.pasarelapagos.objects.config;


import java.util.Map;

import p12f.exe.pasarelapagos.objects.ObjectInheritance;

import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;
import p12f.exe.pasarelapagos.objects.ORGManager;
import p12f.exe.pasarelapagos.objects.AccesMethod;
import p12f.exe.pasarelapagos.paymentrequest.Aplicacion;
import p12f.exe.pasarelapagos.paymentrequest.Backend;
import p12f.exe.pasarelapagos.paymentrequest.Liquidacion;

/**
 * Estructura que define los datos de configuracin de una administracion
 */

public class AdminConfig extends BaseConfigObject{
	// Identificadores de los grupos:
	public static final String GOBIERNO_VASCO = "01";
	public static final String DIPU_BIZKAIA = "02";
    public static final String DIPU_GIPUZKOA = "03";
    public static final String DIPU_ARABA = "04";	
	public static final String LOCAL_BIZKAIA = "05";
    public static final String LOCAL_GIPUZKOA = "06";
	public static final String LOCAL_ARABA = "07";
	
    /**
     * OID de la Administracin.
     */
    public String oid;       	
    /**
     * Flag que indica si la entidad financiera esta activa.
     * Por defecto, la entidad financiera esta activa.
     */
    public boolean activa = true;
    /**
     * CIF de la Administracin.
     */
    public String cif;
    /**
     * Grupo de la Administracin.
     */
    public String grupo;
    /**
     * Descripcion de la Administracin.
     */
    public Map descripcion;
    /**
     * Calle de la direccin de la Administracin
     */
    public String calle; 
    /**
     * Municipio de la direccin de la Administracin
     */
    public String municipio;
    /**
     * Territorio de la direccin de la Administracin
     */
    public String territorio;
    /**
     * Pas de la direccin de la Administracin
     */
    public String pais;
    /**
     * Codigo Postal de la direccin de la Administracin
     */
    public String codigoPostal; 
    /**
     * Imgenes de la Administracin
     */
    public Map imagenes;
  
    /**
     * Modos de pago que admiten los pagos de esta entidad.
     */
    public Map paymentModes;
    /**
     * Mapa que almacena los codigos de emisor para los pagos de distintos cuadernos de una entidad.
     */
    public Map codEmisor;    
    
    /**
     * Informacion sobre la herencia
     */
    public String inherits;
    
    /**
     * Informacion sobre el manager de la organizacion.
     */
    
    public ORGManager orgManager;
    
    
    /**
     * Informacion sobre la forma de acceso a los usaurios de este este objeto.
     */
    
    public AccesMethod accesMethod;
    
    /**
     * Entidades financieras admitidas por la Administracin.
     */
    public Map finantialOrgs;
    
    /**
     * Datos relativos a la aplicacion.
     */
    public Aplicacion aplicacion;
    
    /**
     * Informacion sobre la liquidacion en PDF.
     */
    public Liquidacion liquidacion;
    
    /**
     * Datos relativos al sistema BackEnd.
     */
    public Backend backend;
    
    /**
     * Datos relativos al ConstraintList.
     */
    public ConstraintList constraintList=null;
    
    /***********************************************************************************/
    /*  Metodos Pblicos Auxiliares para manejar el Objeto.
   /************************************************************************************/
    
    /**
     * Aade al sufijo la herencia de un tipo de objeto. 
     */
    public void enableObjetInheritance(String objectTypeInheritance) throws Exception{
    	
    	if ( objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.APPLICATION_INHERITANCE)
    		|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.BACKENDATA_INHERITANCE)
    			|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.LIQUIDATION_INHERITANCE)
    				|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.FINANTIALORG_INHERITANCW) ){
    
	    	ObjectInheritance oi = new ObjectInheritance(this.inherits);
			oi.append(objectTypeInheritance);
			this.inherits = oi.toString();
			
    	} else  {
    		
    		  String message  = "\n[ Para activa la herencia debe pasar un objeto valido : ";
    		         message +=   "\n ObjectInheritance.APPLICATION_INHERITANCE ,";
    		         message +=   "\n ObjectInheritance.BACKENDATA_INHERITANCE,   ";
    		         message +=   "\n ObjectInheritance.LIQUIDATION_INHERITANCE";
    		         message +=   "\n ObjectInheritance.FINANTIALORG_INHERITANCW)]";
    		         
    		
    		  throw new Exception(message);
    		 
    	}
    }
    
    /**
     * Borra al sufijo la herencia de un tipo de objeto. 
     */
    public void disableObjetInheritance(String objectTypeInheritance) throws Exception{
    	
    	if ( objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.APPLICATION_INHERITANCE)
        		|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.BACKENDATA_INHERITANCE)
        			|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.LIQUIDATION_INHERITANCE)
        				|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.FINANTIALORG_INHERITANCW) ){
        
    	    	ObjectInheritance oi = new ObjectInheritance(this.inherits);
    			oi.delete(objectTypeInheritance);
    			this.inherits = oi.toString();
    			
        	} else  {
        		
        		  String message  = "\n[ Para desactivar la herencia debe pasar un objeto valido : ";
        		         message +=   "\n ObjectInheritance.APPLICATION_INHERITANCE ,";
        		         message +=   "\n ObjectInheritance.BACKENDATA_INHERITANCE,   ";
        		         message +=   "\n ObjectInheritance.LIQUIDATION_INHERITANCE";
        		         message +=   "\n ObjectInheritance.FINANTIALORG_INHERITANCW)]";
        		         
        		
        		  throw new Exception(message);
        		 
        	}
    	
    	
    	
    }
    /**
     * Comprueba si un sufijo tiene herencia de un tipo de objeto: aplicacion, liquidacion...
     */
    public boolean inheritsThisObject(String objectTypeInheritance) throws Exception  {
    	
     	
    	if ( objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.APPLICATION_INHERITANCE)
        		|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.BACKENDATA_INHERITANCE)
        			|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.LIQUIDATION_INHERITANCE)
        				|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.FINANTIALORG_INHERITANCW) ){
        
    	    	
    		     return  ! (inherits.indexOf(objectTypeInheritance) <  0 );
    			
        	} else  {
        		
        		  String message  = "\n[ Para activa la herencia debe pasar un objeto valido : ";
        		         message +=   "\n ObjectInheritance.APPLICATION_INHERITANCE ,";
        		         message +=   "\n ObjectInheritance.BACKENDATA_INHERITANCE,   ";
        		         message +=   "\n ObjectInheritance.LIQUIDATION_INHERITANCE";
        		         message +=   "\n ObjectInheritance.FINANTIALORG_INHERITANCW)]";
        		         
        		
        		  throw new Exception(message);
        		 
        	}
    	
    	
    }
    
    
 
    public AdminConfig(){
        ObjectUtils.initialize(this);
    }
    
    /**
     * Convierte de un String xml a un Objeto 
     * @param xml : XML de objeto.
     * @return Objeto convertido.
     * @throws XOMarshallerException
     */
    public static AdminConfig getObject( String xml) throws XOMarshallerException {
        return (AdminConfig)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"),xml);   
    }

    /**
     * Convierte un Objeto AdminConfig a XML.
     * @return
     * @throws XOMarshallerException
     */
    public String toXML() throws XOMarshallerException {   
         return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
    }
    
    
    public static void main(String[] argv) throws Exception {
    	
    	/* Ejemplo ==============================================:
    	     1. Aadir un Responsable a la Administracin.
    	     2. Establecer herencia a nivel de liquidacin
    	     3. Metodo de Autenticacin XLNES 
    	*==========================================================*/
    	
    	
    	AdminConfig admin = new AdminConfig();
    	
        ORGManager manager = new ORGManager();
		
		manager.dni = "78882020T";
		manager.nombre = "Francesco";
		manager.primerApellido = "Morosini";
		manager.segundoApellido = "Peloponnesiaco";
		manager.cargo = "gonfaloniere";
		manager.municipio = "Venezia";
		manager.pais = "Italia";
		manager.calle = "Arte Di Calimala 9";
		manager.codigoPostal = "48410";
		manager.municipio = "Venecia";
		manager.mail  = "francesco.morosini@calimala.it";
		manager.telefono = "616178968";
		
		admin.enableObjetInheritance(ObjectInheritance.LIQUIDATION_INHERITANCE );
		admin.orgManager  = manager;
		admin.accesMethod.setMethod(AccesMethod.XLNETS);
		admin.accesMethod.setMethod(AccesMethod.USERPASSWORD);
		
		System.out.println(admin.toXML());
    }
    
}
