package p12f.exe.pasarelapagos.objects.config;

import java.util.Map;

import p12f.exe.pasarelapagos.objects.AccesMethod;
import p12f.exe.pasarelapagos.objects.ORGManager;
import p12f.exe.pasarelapagos.paymentrequest.Aplicacion;
import p12f.exe.pasarelapagos.paymentrequest.Backend;
import p12f.exe.pasarelapagos.paymentrequest.Liquidacion;

import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Estructura que define los datos de configuracin de un grupo de Entidades Emisoras
 */

public class GroupConfig extends BaseConfigObject {	
    
	/**
     * OID del grupo de Entidades Emisoras.
     */
    public String oid;   
    /**
     * Flag que indica si la entidad financiera esta activa.
     * Por defecto, la entidad financiera esta activa.
     */
    public boolean activa = true;
    /**
     * Orden del grupo en el listado.
     */
    public int orden;
    /**
     * Descripcion del grupo de Entidades Emisoras.
     */
    public Map descripcion;
    
    /**
     * Informacion sobre el manager de la organizacion.
     */
    
    public ORGManager orgManager;
    
    /**
     * Informacion sobre la forma de acceso a los usaurios de este este objeto.
     */
    
    public AccesMethod accesMethod;
    
    
    /**
     * Entidades financieras admitidas por la Administracin.
     */
    public Map finantialOrgs;
    
    /**
     * Datos relativos a la aplicacion.
     */
    public Aplicacion aplicacion;
    
    /**
     * Informacion sobre la liquidacion en PDF.
     */
    public Liquidacion liquidacion;
    
    /**
     * Datos relativos al sistema BackEnd.
     */
    public Backend backend;
    
    
    public GroupConfig(){
        ObjectUtils.initialize(this);
    }
    
    /**
     * Convierte de un String xml a un Objeto 
     * @param xml : XML de objeto.
     * @return Objeto convertido.
     * @throws XOMarshallerException
     */
    public static GroupConfig getObject( String xml) throws XOMarshallerException {
        return (GroupConfig)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"),xml);   
    }

    /**
     * Convierte un Objeto GroupConfig a XML.
     * @return
     * @throws XOMarshallerException
     */
    public String toXML() throws XOMarshallerException {   
         return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
    }
    
    public static void main (String[]  argv) throws XOMarshallerException {
    	/* Ejemplo ==============================================	 
	     1. Establecer una plantilla de liquidacion
	     2. Metodo de Autenticacin Usaurio Password
	    /**********************************************/
    	
    	GroupConfig config = new GroupConfig();
    	config.liquidacion.urlPlantilla = "/datos/p12i/file/xml/recibo_pdf.xsl";
    	config.liquidacion.paymentGatewayVersion = "2";
    	config.accesMethod.setMethod(AccesMethod.USERPASSWORD);
    	
    	System.out.println(config.toXML());
    	
    }
}
