package p12f.exe.pasarelapagos.objects.config;

import java.io.Serializable;

import com.ejie.r01f.util.StringUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

public class HolderCertEventConfig implements Serializable {
	
	public static final String ALL_EVENTS_ENABLE  = "11111111111";
	public static final String ALL_EVENTS_DISABLE = "00000000000";
	
	public static final String SIMPLE_EVENTS			= "10010010000";
	public static final String MULTIPLE_EVENTS			= "01101101111";
	public static final String MULTIPLE_NO_BATCH_EVENTS = "01001001010";
	public static final String MULTIPLE_BATCH_EVENTS	= "00100100101";
	
	public static final String DEFAULT_CONF_EVENTS = "100";
	
	private static final char ENABLE_BIT  = '1';
	private static final char DISABLE_BIT = '0';
	
	/**
	 * BitMap de eventos.
	 * - Evento de inicializacin simple
	 * - Evento de inicializacin mltiple
	 * - Evento de inicializacin mltiple batch
	 * - Evento de envio a entidad financiera simple
	 * - Evento de envio a entidad financiera mltiple
	 * - Evento de envio a entidad financiera mltiple batch
	 * - Evento de recibo de resultado simple
	 * - Evento de recibo de resultado multiple
	 * - Evento de recibo de resultado mltiple batch
	 * - Evento de recibo de resultado parcial multiple
	 * - Evento de recibo de resultado parcial mltiple batch
	 */
	private String eventBitMap = null;
	
	/**
	 * BitMap de configuracion de eventos.
	 * - Enviar los eventos mltiples en zip
	 * - Enviar los datos adicionales en los eventos mltiples
	 * - Enviar los datos adicionales en los eventos mltiples batch
	 */
	private String configBitMap = DEFAULT_CONF_EVENTS;
	/**
	 * Se mantiene esta variable para tener retrocompatibilidad
	 */
	private Boolean sendEventsZipped = null;
		
	/**
	 * Constructor, con todos los eventos activados por defecto.
	 */
	public HolderCertEventConfig(){
		
	}
	
	/**
	 * Constructor a partir de un mapa de eventos.
	 * Los eventos que no vengan se inicializan a 0.
	 * Los eventos que no estn a 0/1 se ponen a 0.
	 * @param eventBitMapParam
	 */
	public HolderCertEventConfig(String eventBitMapParam){
		this.eventBitMap = _getBitMapFromString(eventBitMapParam, HolderCertEventConfig.ALL_EVENTS_ENABLE);
	}
	
	
	/**
	 * Constructor a partir de un mapa de eventos y un indicador de si se quieren
	 * recibir los resultados zipeados o no.
	 * Los eventos que no vengan se inicializan a 0.
	 * Los eventos que no estn a 0/1 se ponen a 0.
	 * @param eventBitMapParam
	 * @param sendResultsZipped
	 */
	public HolderCertEventConfig(String eventBitMapParam, boolean sendEventsZipped){
		this.eventBitMap = _getBitMapFromString(eventBitMapParam, HolderCertEventConfig.ALL_EVENTS_ENABLE);
		this.configBitMap = DEFAULT_CONF_EVENTS;
		this.setSendEventsZipped(sendEventsZipped);
	}
	
	/**
	 * Constructor a partir de un mapa de eventos y un mapa de configuracin.
	 * Los eventos que no vengan se inicializan a 0.
	 * Los eventos que no estn a 0/1 se ponen a 0.
	 * @param eventBitMapParam
	 * @param sendResultsZipped
	 */
	public HolderCertEventConfig(String eventBitMapParam, String eventConfBitMap){
		this.eventBitMap = _getBitMapFromString(eventBitMapParam, HolderCertEventConfig.ALL_EVENTS_ENABLE);
		this.configBitMap = _getBitMapFromString(eventConfBitMap, HolderCertEventConfig.DEFAULT_CONF_EVENTS);
	}
	
	public boolean isSendEventsZipped(){
		if (configBitMap.startsWith(new Character(HolderCertEventConfig.ENABLE_BIT).toString())) {
			return true;
		}
		return false;
	}
	public void setSendEventsZipped(boolean sendEventsZipped){
		if (sendEventsZipped) {
			enableSendMultipleEventsZipped();
		} else {
			disableSendMultipleEventsZipped();
		}
	}
	
	/**
	 * Devuelve el BitMap de eventos.
	 * @return String
	 */
	public String getEventBitMap() {
		return this.eventBitMap;
	}
	
	public void setVoidEventBitMap(){
		this.eventBitMap = null;
	}
	
	/**
	 * Asigna el BitMap de eventos.
	 */
	public void setEventBitMap(String eventBitMapParam) {
		this.eventBitMap = _getBitMapFromString(eventBitMapParam, HolderCertEventConfig.ALL_EVENTS_ENABLE);
	}
	
	/**
	 * Mtodo para activar todos los eventos.
	 */
	public void enableAllEvents(){
		this.eventBitMap = HolderCertEventConfig.ALL_EVENTS_ENABLE;
	}
	
	/**
	 * Mtodo para desactivar todos los eventos.
	 */
	public void disableAllEvents(){
		this.eventBitMap = HolderCertEventConfig.ALL_EVENTS_DISABLE;
	}
	
	
	/* EVENTOS DE INICIALIZACIN */
	
	/**
	 * Activar evento que se lanza al realizar la inicializacin de una
	 * peticin de validacin de cuenta simple.
	 */
	public void enableOnInitializeSimpleEvent(){
		_checkBitMap(true);
		eventBitMap = HolderCertEventConfig.ENABLE_BIT+eventBitMap.substring(1,eventBitMap.length());
	}
	/**
	 * Desactivar evento que se lanza al realizar la inicializacin de una
	 * peticin de validacin de cuenta simple.
	 */
	public void disableOnInitializeSimpleEvent(){
		_checkBitMap(false);
		eventBitMap = HolderCertEventConfig.DISABLE_BIT+eventBitMap.substring(1,eventBitMap.length());
	}
	
	/**
	 * Activar evento que se lanza al realizar la inicializacin de una
	 * peticin de validacin de cuenta mltiple.
	 */
	public void enableOnInitializeMultipleEvent(){
		_checkBitMap(true);
		eventBitMap = eventBitMap.substring(0,1)+HolderCertEventConfig.ENABLE_BIT+eventBitMap.substring(2,eventBitMap.length());
	}
	/**
	 * Desactivar evento que se lanza al realizar la inicializacin de una
	 * peticin de validacin de cuenta mltiple.
	 */
	public void disableOnInitializeMultipleEvent(){
		_checkBitMap(false);
		eventBitMap = eventBitMap.substring(0,1)+HolderCertEventConfig.DISABLE_BIT+eventBitMap.substring(2,eventBitMap.length());
	}
	
	/**
	 * Activar evento que se lanza al realizar la inicializacin de una
	 * peticin de validacin de cuenta mltiple batch.
	 */
	public void enableOnInitializeMultipleBatchEvent(){
		_checkBitMap(true);
		eventBitMap = eventBitMap.substring(0,2)+HolderCertEventConfig.ENABLE_BIT+eventBitMap.substring(3,eventBitMap.length());
	}
	/**
	 * Desactivar evento que se lanza al realizar la inicializacin de una
	 * peticin de validacin de cuenta mltiple batch.
	 */
	public void disableOnInitializeMultipleBatchEvent(){
		_checkBitMap(false);
		eventBitMap = eventBitMap.substring(0,2)+HolderCertEventConfig.DISABLE_BIT+eventBitMap.substring(3,eventBitMap.length());
	}
	
	
	/* EVENTOS DE ENVO A ENTIDAD */
	
	/**
	 * Activar evento que se lanza al enviar una peticin de validacin de 
	 * cuenta simple a la entidad financiera correspondiente.
	 */
	public void enableOnSendSimpleEvent(){
		_checkBitMap(true);
		eventBitMap = eventBitMap.substring(0,3)+HolderCertEventConfig.ENABLE_BIT+eventBitMap.substring(4,eventBitMap.length());
	}
	/**
	 * Desactivar evento que se lanza al enviar una peticin de validacin de 
	 * cuenta simple a la entidad financiera correspondiente.
	 */
	public void disableOnSendSimpleEvent(){
		_checkBitMap(false);
		eventBitMap = eventBitMap.substring(0,3)+HolderCertEventConfig.DISABLE_BIT+eventBitMap.substring(4,eventBitMap.length());
	}
	
	/**
	 * Activar evento que se lanza al enviar una peticin de validacin de 
	 * cuenta mltiple a la entidad financiera correspondiente.
	 */
	public void enableOnSendMultipleEvent(){
		_checkBitMap(true);
		eventBitMap = eventBitMap.substring(0,4)+HolderCertEventConfig.ENABLE_BIT+eventBitMap.substring(5,eventBitMap.length());
	}
	/**
	 * Desactivar evento que se lanza al enviar una peticin de validacin de 
	 * cuenta mltiple a la entidad financiera correspondiente.
	 */
	public void disableOnSendMultipleEvent(){
		_checkBitMap(false);
		eventBitMap = eventBitMap.substring(0,4)+HolderCertEventConfig.DISABLE_BIT+eventBitMap.substring(5,eventBitMap.length());
	}
	
	/**
	 * Activar evento que se lanza al enviar una peticin de validacin de 
	 * cuenta mltiple batch a la entidad financiera correspondiente.
	 */
	public void enableOnSendMultipleBatchEvent(){
		_checkBitMap(true);
		eventBitMap = eventBitMap.substring(0,5)+HolderCertEventConfig.ENABLE_BIT+eventBitMap.substring(6,eventBitMap.length());
	}
	/**
	 * Desactivar evento que se lanza al enviar una peticin de validacin de 
	 * cuenta mltiple batch a la entidad financiera correspondiente.
	 */
	public void disableOnSendMultipleBatchEvent(){
		_checkBitMap(false);
		eventBitMap = eventBitMap.substring(0,5)+HolderCertEventConfig.DISABLE_BIT+eventBitMap.substring(6,eventBitMap.length());
	}
	
	
	/* EVENTOS DE RECIBO DE RESULTADOS */
	
	/**
	 * Activar evento que se lanza al recibir un resultado de una peticin de 
	 * validacin de cuenta simple.
	 */
	public void enableOnSetResultSimpleEvent(){
		_checkBitMap(true);
		eventBitMap = eventBitMap.substring(0,6)+HolderCertEventConfig.ENABLE_BIT+eventBitMap.substring(7,eventBitMap.length());
	}
	/**
	 * Desactivar evento que se lanza al recibir un resultado de una peticin de 
	 * validacin de cuenta simple.
	 */
	public void disableOnSetResultSimpleEvent(){
		_checkBitMap(false);
		eventBitMap = eventBitMap.substring(0,6)+HolderCertEventConfig.DISABLE_BIT+eventBitMap.substring(7,eventBitMap.length());
	}
	
	/**
	 * Activar evento que se lanza al recibir un resultado de una peticin de 
	 * validacin de cuenta mltiple.
	 */
	public void enableOnSetResultMultipleEvent(){
		_checkBitMap(true);
		eventBitMap = eventBitMap.substring(0,7)+HolderCertEventConfig.ENABLE_BIT+eventBitMap.substring(8,eventBitMap.length());
	}
	/**
	 * Desactivar evento que se lanza al recibir un resultado de una peticin de 
	 * validacin de cuenta mltiple.
	 */
	public void disableOnSetResultMultipleEvent(){
		_checkBitMap(false);
		eventBitMap = eventBitMap.substring(0,7)+HolderCertEventConfig.DISABLE_BIT+eventBitMap.substring(8,eventBitMap.length());
	}
	
	/**
	 * Activar evento que se lanza al recibir un resultado de una peticin de 
	 * validacin de cuenta mltiple batch.
	 */
	public void enableOnSetResultMultipleBatchEvent(){
		_checkBitMap(true);
		eventBitMap = eventBitMap.substring(0,8)+HolderCertEventConfig.ENABLE_BIT+eventBitMap.substring(9,eventBitMap.length());
	}
	/**
	 * Desactivar evento que se lanza al recibir un resultado de una peticin de 
	 * validacin de cuenta mltiple batch.
	 */
	public void disableOnSetResultMultipleBatchEvent(){
		_checkBitMap(false);
		eventBitMap = eventBitMap.substring(0,8)+HolderCertEventConfig.DISABLE_BIT+eventBitMap.substring(9,eventBitMap.length());
	}
	
	
	/* EVENTOS DE RECIBO DE RESULTADOS PARCIALES */
	
	/**
	 * Activar evento que se lanza al recibir un resultado parcial de una 
	 * peticin de validacin de cuenta mltiple.
	 */
	public void enableOnSetPartialResultMultipleEvent(){
		_checkBitMap(true);
		eventBitMap = eventBitMap.substring(0,9)+HolderCertEventConfig.ENABLE_BIT+eventBitMap.substring(10,eventBitMap.length());
	}
	/**
	 * Desactivar evento que se lanza al recibir un resultado parcial de una 
	 * peticin de validacin de cuenta mltiple.
	 */
	public void disableOnSetPartialResultMultipleEvent(){
		_checkBitMap(false);
		eventBitMap = eventBitMap.substring(0,9)+HolderCertEventConfig.DISABLE_BIT+eventBitMap.substring(10,eventBitMap.length());
	}
	
	/**
	 * Activar evento que se lanza al recibir un resultado parcial de una 
	 * peticin de validacin de cuenta mltiple batch.
	 */
	public void enableOnSetPartialResultMultipleBatchEvent(){
		_checkBitMap(true);
		eventBitMap = eventBitMap.substring(0,10)+HolderCertEventConfig.ENABLE_BIT;
	}
	/**
	 * Desactivar evento que se lanza al recibir un resultado parcial de una
	 * peticin de  validacin de cuenta mltiple batch.
	 */
	public void disableOnSetPartialResultMultipleBatchEvent(){
		_checkBitMap(false);
		eventBitMap = eventBitMap.substring(0,10)+HolderCertEventConfig.DISABLE_BIT;
	}
	
	
	/* MTODOS PARA ACTIVAS LOS DATOS ADICIONALES EN LOS EVENTOS */
	/**
	 * Devuelve el BitMap de eventos.
	 * @return String
	 */
	public String getConfigBitMap() {
		return this.configBitMap;
	}
	
	/**
	 * Activar el envio de eventos mltiples en ZIP.
	 */
	public void enableSendMultipleEventsZipped(){
		configBitMap = HolderCertEventConfig.ENABLE_BIT+configBitMap.substring(1,configBitMap.length());
	}
	/**
	 * Desactivar el envio de eventos mltiples en ZIP.
	 */
	public void disableSendMultipleEventsZipped(){
		configBitMap = HolderCertEventConfig.DISABLE_BIT+configBitMap.substring(1,configBitMap.length());
	}
	/**
	 * Activar el envo de datos adicionales en eventos simples de validaciones
	 * mltiples.
	 */
	public void enableSendAditonalDataInMultipleEvents(){
		configBitMap = configBitMap.substring(0,1)+HolderCertEventConfig.ENABLE_BIT+configBitMap.substring(2,configBitMap.length());
	}
	/**
	 * Desactivar el envo de datos adicionales en eventos simples de validaciones
	 * mltiples.
	 */
	public void disableSendAditonalDataInMultipleEvents(){
		configBitMap = configBitMap.substring(0,1)+HolderCertEventConfig.DISABLE_BIT+configBitMap.substring(2,configBitMap.length());
	}
	/**
	 * Activar el envo de datos adicionales en eventos simples de validaciones
	 * mltiples.
	 */
	public void enableSendAditonalDataInMultipleBatchEvents(){
		configBitMap = configBitMap.substring(0,2)+HolderCertEventConfig.ENABLE_BIT;
	}
	/**
	 * Desactivar el envo de datos adicionales en eventos simples de validaciones
	 * mltiples.
	 */
	public void disableSendAditonalDataInMultipleBatchEvents(){
		configBitMap = configBitMap.substring(0,2)+HolderCertEventConfig.DISABLE_BIT;
	}
	
	/* MTODOS CON BITMAP */
	
	/**
	 * Mtodo que desactiva todos los eventos que no sean simples.
	 */
	public void simpleBitMapAND(){
		eventBitMap = _makeANDOp(eventBitMap, SIMPLE_EVENTS);
	}
	
	/**
	 * Mtodo que desactiva todos los eventos que no sean mltiples.
	 */
	public void multipleBitMapAND(){
		eventBitMap = _makeANDOp(eventBitMap, MULTIPLE_EVENTS);
	}
	
	/**
	 * Mtodo que desactiva todos los eventos que no sean mltiples no batch.
	 */
	public void multipleNoBatchBitMapAND(){
		eventBitMap = _makeANDOp(eventBitMap, MULTIPLE_NO_BATCH_EVENTS);
	}
	
	/**
	 * Mtodo que desactiva todos los eventos que no sean mltiples batch.
	 */
	public void multipleBatchBitMapAND(){
		eventBitMap = _makeANDOp(eventBitMap, MULTIPLE_BATCH_EVENTS);
	}
	
	/* MTODOS MARSHALL */
	
	/**
	 * Convierte un Objeto HolderCertEventConfig a XML.
	 * @return String
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte de un xml a un objeto HolderCertEventConfig
	 * @param xml XML del objeto.
	 * @return HolderCertEventConfig
	 * @throws XOMarshallerException
	 */
	public static HolderCertEventConfig getObject( String xml) throws XOMarshallerException {
		HolderCertEventConfig holderCertEventConfig = (HolderCertEventConfig) XOManager.getObject(XMLProperties.get("p12f","objectMapPath"),xml);
		if (holderCertEventConfig.sendEventsZipped!=null){
			if (holderCertEventConfig.sendEventsZipped.booleanValue()) {
				holderCertEventConfig.enableSendMultipleEventsZipped();
			} else {
				holderCertEventConfig.disableSendMultipleEventsZipped();
			}
		}
		return holderCertEventConfig;
	}
	
	
	/* ********************************************************************** */
	/* ************************* MTODOS PRIVADOS *************************** */
	/* ********************************************************************** */
	
	/**
	 * Mtodo que, a partir de un String, devuelve un bitMap bien formado.
	 * @param strBitMap
	 * @return String
	 */
	private String _getBitMapFromString(String strBitMap, String bitMapPatron) {
		String bitMapResult = StringUtils.rPadWithChar(strBitMap.substring(0, Math.min(strBitMap.length(), bitMapPatron.length())),
				'0', bitMapPatron.length());
		for (int i=0;i<bitMapPatron.length();i++){
			char c = bitMapResult.charAt(i);
			if (c!='0' && c!='1'){
				bitMapResult = bitMapResult.substring(0,i)+HolderCertEventConfig.DISABLE_BIT+bitMapResult.substring(i+1,bitMapResult.length());
			}
		}
		return bitMapResult;
	}
	
	private void _checkBitMap(boolean enabling) {
		if (this.eventBitMap==null){
			// Si no tiene bit map de eventos
			if (enabling){ // Si est activando un evento, deshabilitamos el resto (se quedar solo activo el que est activando)
				this.disableAllEvents();
			}else{ // Si est desactivando, activamos el resto
				this.enableAllEvents();
			}
		}
	}
	
	private String _makeANDOp(String bitMap, String key) {
		StringBuffer result = new StringBuffer("");
		for(int i = 0; i < bitMap.length(); i++) {
			result.append((char)(bitMap.charAt(i) & key.charAt(i % key.length())));
		}
		return result.toString();
	}
	
}
