/*
 * Created on 06-jun-2005
 * @author co01347e
 */
package p12f.exe.pasarelapagos.objects.config;


import java.util.HashMap;
import java.util.List;
import java.util.Map;

import p12f.exe.pasarelapagos.objects.Domiciliacion;
import p12f.exe.pasarelapagos.objects.DueDate;
import p12f.exe.pasarelapagos.objects.ObjectInheritance;
import p12f.exe.pasarelapagos.paymentrequest.Aplicacion;
import p12f.exe.pasarelapagos.paymentrequest.Backend;
import p12f.exe.pasarelapagos.paymentrequest.Liquidacion;

import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;


/**
 * Estructura que define los datos de confuracin de un tipo de pago
 */

public class PaymentTypeConfig extends BaseConfigObject {
    
	/**
	 * 
	 */

	// Constantes con los tipos de validacion posibles.
	public final static int NO_VALIDAR = 0;
	public final static int VALIDACION_NORMAL = 1;
	public final static int VALIDACION_TRAFICO = 2;
    
    // Constantes con los nombres de los atributos del tipo de pago
    public final static String EMISOR = "emisor";
    public final static String TRIBUTO = "tipo";
    public final static String FORMATO = "formato";
	
	/**
	 * OID del tipo de pago.
	 */
	public String oid;
	/**
     * Flag que indica si el tipo de pago esta activo.
     * Por defecto, el tipo de pago esta activo.
     */
    public boolean activa = true;
    /**
     * Oid del emisor del tipo de pago.
     */
    public String emisor;  
    /**
     * Tributo del tipo de pago.
     */
    public String tributo;  
    /**
     * Formato del cuadernillo del tipo de pago.
     */
    public String formato;
    /**
     * Indica la forma de validar los dgitos de control:
     */
    public int validar;
    /**
     * Descripcion del tipo de pago.
     */
    public Map descripcion;
    /**
     * Conceptos del tipo de pago.
     */
    public List conceptos;
    /**
     * Imagenes para el recibo.
     */
    public Map imagenes;             
    /**
     * Mensajes auxiliares.
     */
    public Map mensajes;
    /**
     * Datos relativos al sistema BackEnd.
     */
    public Backend backend;
    /**
     * Datos relativos a la aplicacion.
     */
    public Aplicacion aplicacion;
    
    public DueDate dueDate;
    /**
     * Datos sobre la domiciliacion del pago.
     */
    public Domiciliacion domiciliacion;
    /**
     * Entidades financieras que admiten este tipo de pago.
     */
    public Map finantialOrgs;
    /**
     * Modos de pago que admite este tipo de pago.
     */
    public Map paymentModes;
    /**
     * Codigos de comercio del emisor del pago en los distintos proveedores de TPV Virtual.
     */
    public Map codigosComercio;
    /**
     * Informacion sobre la liquidacion en PDF.
     */
    public Liquidacion liquidacion;
    
    /**
     * Informacion sobre la herencia
     */
    public String inherits;
    
    
    /**
     *  Debe mostrarse en la aplicacion generadora de Liquidacionesl.
     */
    public Boolean showOnLiquidationGenerator = null;
    
    
	/**
	 * Datos Adicionales
	 */
	public List datosAdicionales;
	
	
	/**
     * Datos relativos al ConstraintList.
     */
    public ConstraintList constraintList=null;
    
    
    
    /***********************************************************************************/
    /*  Metodos Pblicos Auxiliares para manejar el Objeto.
   /************************************************************************************/
    
    /**
     * Aade al sufijo la herencia de un tipo de objeto. 
     */
    public void enableObjetInheritance(String objectTypeInheritance) throws Exception{
    	
    	if ( objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.APPLICATION_INHERITANCE)
    		|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.BACKENDATA_INHERITANCE)
    			|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.LIQUIDATION_INHERITANCE)
    				|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.FINANTIALORG_INHERITANCW) ){
    
	    	ObjectInheritance oi = new ObjectInheritance(this.inherits);
			oi.append(objectTypeInheritance);
			this.inherits = oi.toString();
			
    	} else  {
    		
    		  String message  = "\n[ Para activa la herencia debe pasar un objeto valido : ";
    		         message +=   "\n ObjectInheritance.APPLICATION_INHERITANCE ,";
    		         message +=   "\n ObjectInheritance.BACKENDATA_INHERITANCE,   ";
    		         message +=   "\n ObjectInheritance.LIQUIDATION_INHERITANCE";
    		         message +=   "\n ObjectInheritance.FINANTIALORG_INHERITANCW)]";
    		         
    		
    		  throw new Exception(message);
    		 
    	}
    }
    
    /**
     * Borra al sufijo la herencia de un tipo de objeto. 
     */
    public void disableObjetInheritance(String objectTypeInheritance) throws Exception{
    	
    	if ( objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.APPLICATION_INHERITANCE)
        		|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.BACKENDATA_INHERITANCE)
        			|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.LIQUIDATION_INHERITANCE)
        				|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.FINANTIALORG_INHERITANCW) ){
        
    	    	ObjectInheritance oi = new ObjectInheritance(this.inherits);
    			oi.delete(objectTypeInheritance);
    			this.inherits = oi.toString();
    			
        	} else  {
        		
        		  String message  = "\n[ Para desactivar la herencia debe pasar un objeto valido : ";
        		         message +=   "\n ObjectInheritance.APPLICATION_INHERITANCE ,";
        		         message +=   "\n ObjectInheritance.BACKENDATA_INHERITANCE,   ";
        		         message +=   "\n ObjectInheritance.LIQUIDATION_INHERITANCE";
        		         message +=   "\n ObjectInheritance.FINANTIALORG_INHERITANCW)]";
        		         
        		
        		  throw new Exception(message);
        		 
        	}
    	
    	
    	
    }
    /**
     * Comprueba si un sufijo tiene herencia de un tipo de objeto: aplicacion, liquidacion...
     */
    public boolean inheritsThisObject(String objectTypeInheritance) throws Exception  {
    	
     	
    	if ( objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.APPLICATION_INHERITANCE)
        		|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.BACKENDATA_INHERITANCE)
        			|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.LIQUIDATION_INHERITANCE)
        				|| objectTypeInheritance.equalsIgnoreCase(ObjectInheritance.FINANTIALORG_INHERITANCW) ){
        
    	    	
    		     return  ! (inherits.indexOf(objectTypeInheritance) <  0 );
    			
        	} else  {
        		
        		  String message  = "\n[ Para activa la herencia debe pasar un objeto valido : ";
        		         message +=   "\n ObjectInheritance.APPLICATION_INHERITANCE ,";
        		         message +=   "\n ObjectInheritance.BACKENDATA_INHERITANCE,   ";
        		         message +=   "\n ObjectInheritance.LIQUIDATION_INHERITANCE";
        		         message +=   "\n ObjectInheritance.FINANTIALORG_INHERITANCW)]";
        		         
        		
        		  throw new Exception(message);
        		 
        	}
    	
    	
    }
    
    
    /**
     * Metodo que devuelve un mapa con los atributos principales de un tipo de pago:
     * emisor, tributo y formato.
     * @return
     */
    public Map getPaymentTypeAttributes() {
        Map atributos = new HashMap();
        
        if(oid!=null) {
            if(oid.length()==14){
                // Se trata de un tipo de pago del cuaderno 57
                String campo_emisor = oid.substring(0,8);
                String campo_tributo = oid.substring(8,11);
                String campo_formato = oid.substring(11,14);
                
                atributos.put(EMISOR, campo_emisor);
                atributos.put(TRIBUTO, campo_tributo);
                atributos.put(FORMATO, campo_formato);
            } else if(oid.length()==12){
                // Se trata de un tipo de pago del cuaderno 60
                String campo_emisor = oid.substring(0,6);
                String campo_tributo = oid.substring(6,9);
                String campo_formato = oid.substring(9,12);
                
                atributos.put(EMISOR, campo_emisor);
                atributos.put(TRIBUTO, campo_tributo);
                atributos.put(FORMATO, campo_formato);
            }
        }
        
        return atributos;
    }
    
    public PaymentTypeConfig(){
        ObjectUtils.initialize(this);
    }
    
    /**
     * Convierte un Objeto PaymentTypeConfig a XML.
     * @return
     * @throws XOMarshallerException
     */
    public String toXML() throws XOMarshallerException {   
         return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
    }
 
    /**
     * Convierte de un String xml a un Objeto 
     * @param xml: XML de objeto.
     * @return Objeto convertido.
     * @throws XOMarshallerException
     */
    public static PaymentTypeConfig getObject( String xml) throws XOMarshallerException {
        return (PaymentTypeConfig) XOManager.getObject(XMLProperties.get("p12f","objectMapPath"),xml);   
    }
    
}
