package p12f.exe.pasarelapagos.objects.data;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Clase que representa un fichero/directorio.
 * @author dbenitof
 */
public class DataFile implements Serializable {
	
	/**
	 * Ruta del fichero/directorio.
	 */
	private String filePath;
	/**
	 * Ruta del directorio padre.
	 */
	private String parentPath;
	/**
	 * Nombre del fichero/directorio.
	 */
	private String fileName;
	/**
	 * Tamao del fichero en bytes
	 */
	private long fileSize;
	/**
	 * Indicador de si se tienen o no permisos para leer el fichero/directorio.
	 */
	private boolean canRead;
	/**
	 * Indicador de si se tienen o no permisos para modificar el fichero/directorio.
	 */
	private boolean canWrite;
	/**
	 * Indicador de si es o no un fichero.
	 */
	private boolean isFile;
	/**
	 * Indicador de si el fichero/directorio es oculto.
	 */
	private boolean isHidden;
	/**
	 * ltima modificacin del fichero/directorio.
	 */
	private long lastModified;
	/**
	 * Ficheros dentro del directorio.
	 */
	private List directoryFiles;
	
	/**
	 * Constructor vaco.
	 */
	public DataFile(){
		directoryFiles = new ArrayList();
	}
	
	
	/**
	 * @return the filePath
	 */
	public String getFilePath() {
		return filePath;
	}
	/**
	 * @param filePath the filePath to set
	 */
	public void setFilePath(String filePath) {
		this.filePath = filePath;
	}
	/**
	 * @return the parentPath
	 */
	public String getParentPath() {
		return parentPath;
	}

	/**
	 * @param parentPath the parentPath to set
	 */
	public void setParentPath(String parentPath) {
		this.parentPath = parentPath;
	}

	/**
	 * @return the fileName
	 */
	public String getFileName() {
		return fileName;
	}
	/**
	 * @param fileName the fileName to set
	 */
	public void setFileName(String fileName) {
		this.fileName = fileName;
	}
	/**
	 * @return the canRead
	 */
	public boolean isCanRead() {
		return canRead;
	}
	/**
	 * @param canRead the canRead to set
	 */
	public void setCanRead(boolean canRead) {
		this.canRead = canRead;
	}
	/**
	 * @return the canWrite
	 */
	public boolean isCanWrite() {
		return canWrite;
	}
	/**
	 * @param canWrite the canWrite to set
	 */
	public void setCanWrite(boolean canWrite) {
		this.canWrite = canWrite;
	}
	/**
	 * @return the isFile
	 */
	public boolean isFile() {
		return isFile;
	}
	/**
	 * @param isFile the isFile to set
	 */
	public void setFile(boolean isFile) {
		this.isFile = isFile;
	}
	/**
	 * @return the isHidden
	 */
	public boolean isHidden() {
		return isHidden;
	}
	/**
	 * @param isHidden the isHidden to set
	 */
	public void setHidden(boolean isHidden) {
		this.isHidden = isHidden;
	}
	/**
	 * @return the lastModified
	 */
	public long getLastModified() {
		return lastModified;
	}
	/**
	 * @param lastModified the lastModified to set
	 */
	public void setLastModified(long lastModified) {
		this.lastModified = lastModified;
	}
	
	/**
	 * @return the directoryFiles
	 */
	public List getDirectoryFiles() {
		return directoryFiles;
	}
	/**
	 * @param directoryFiles the directoryFiles to set
	 */
	public void setDirectoryFiles(List directoryFiles) {
		this.directoryFiles = directoryFiles;
	}

	/**
	 * @return the fileSize
	 */
	public long getFileSize() {
		return fileSize;
	}
	/**
	 * @param fileSize the fileSize to set
	 */
	public void setFileSize(long fileSize) {
		this.fileSize = fileSize;
	}
	
	/**
	 * Convierte un Objeto DataFile a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto DataFile.
	 * @return Objeto DataFile construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static DataFile getObject(String xml) throws XOMarshallerException {
		return (DataFile)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
	
}
