package p12f.exe.pasarelapagos.objects.sms;

/**
 * 
 * MENSAJE: Si un mensaje es enviado a varios nmeros de telfono nos
 *          devuelve una lista con el resultado de cada uno de ellos.
 * - NUM: Nmero de telfono al que se ha enviado el mensaje.
 * - RESULTADO: OK o ERROR. Si el mensaje se ha entregado bien se devuelve
 *              un OK. Si ha sucedido algn error se devuelve ERROR.
 * - IDENTIFICADOR: Si el resultado ha sido OK, se devuelve el identificador
 *                  nico del mensaje, id interno del mensaje en la plataforma
 *                  (para la Q68) y en identificador externo, no nico, en el
 *                  caso de las aplicaciones no Q68. En este ltimo caso
 *                  Identificador + Num de telfono ser la manera de
 *                  identificar cada sms enviado.
 * - CODIGO_ERROR: Cdigo asignado al error que se ha producido.
 * - MENSAJE_ERROR: Descripcin del error que se ha generado.
 * 
 */
import java.io.Serializable;

import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

public class SMSMessageReceiverResult implements Serializable {

	private static final long serialVersionUID = 2888027333434071015L;
	
	public String receiverNumber; // TELEFONO (600123456)
	public String result;         // RESULTADO (OK/ERROR)
	public String messageId;      // IDENTIFICADOR (qwerty...)
	public String errorCode;      // CODIGO_ERROR (301)
	public String errorMessage;   // MENSAJE_ERROR ("El mensaje ha expirado")

	public SMSMessageReceiverResult(){
	    ObjectUtils.initialize(this);
	}
	
	/**
	 * Convierte un Objeto Url a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto Url.
	 * @return Objeto Url construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static SMSMessageReceiverResult getObject(String xml) throws XOMarshallerException {    
	    return (SMSMessageReceiverResult)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
}
