package p12f.exe.pasarelapagos.objects.sms.latinia;

import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * 
 * @author FMARTINI
 *
 * Ref. documentacin Latinia: - - - - - - - - - - - - - - - - - - - - - - -
 * 
 * Los parmetros de conexin necesarios para autenticacin en LATINIA son:
 * 
 * Pblicos (propios de la plataforma):
 *    - Empresa (Login enterprise): INNOVUS
 *    - Usuario Latinia: innovus.superusuario
 *    - Password Latinia: MARKSTAT
 * 
 * Privados (propios de cada aplicacin)
 *    - Aplicacin (refProduct): <cdigo aplicacin>
 *    - Contrato (idcontract): <cdigo de contrato asociado a la aplicacin>
 *    - Contrasea: <por defecto cdigo de aplicacin en desarrollo>
 * 
 * Con esto puede formarse un xml vlido para envo SMS a travs de los WebServices de W91D.
 * 
 * Ejemplo para una aplicacin Z99 con contrato 2000
 * 
 * <authenticationLatinia>
 *     <loginEnterprise>INNOVUS</loginEnterprise>
 *     <userLatinia>innovus.superusuario</userLatinia>
 *     <passwordLatinia>MARKSTAT</passwordLatinia>
 *     <refProduct>Z99</refProduct>
 *     <idContract>2000</idContract>
 *     <password>Z99</password>
 * </authenticationLatinia>
 *
 * (La opcin ms recomendable para la autenticacin es el uso de XLNETS. De
 * esta forma la aplicacin se abstrae de los cdigos y contraseas de
 * autenticacin en LATINIA que sern obtenidos por el conector W91D a partir
 * del token de sesin enviado en la peticin al WebService)
 * 
 */
public class AuthenticationLatinia {

	public String loginEnterprise; // INNOVUS
	public String userLatinia;     // innovus.superusuario
	public String passwordLatinia; // MARKSTAT
	public String refProduct;      // Z99 (P12G)
	public String idContract;      // 2000 (el id que nos asignen...)
	public String password;        // Z99 (P12G)

	/**
	 * Cargar configuracin de properties... 
	 */
	public AuthenticationLatinia() {
//		loginEnterprise = "INNOVUS";
//		userLatinia     = "innovus.superusuario";
//		passwordLatinia = "MARKSTAT";
//		refProduct      = "P12G";  // P12G
//		idContract      = "2000";  // el id que nos asignen...
//		password        = "P12G";  // P12G

		loginEnterprise = XMLProperties.get("p12g","sms/latinia/authentication/loginEnterprise");
		userLatinia     = XMLProperties.get("p12g","sms/latinia/authentication/userLatinia");
		passwordLatinia = XMLProperties.get("p12g","sms/latinia/authentication/passwordLatinia");
		refProduct      = XMLProperties.get("p12g","sms/latinia/authentication/refProduct");  // P12G
		idContract      = XMLProperties.get("p12g","sms/latinia/authentication/idContract");  // el id que nos asignen...
		password        = XMLProperties.get("p12g","sms/latinia/authentication/password");  // P12G
	}

	/**
	 * Convierte un Objeto Url a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto Url.
	 * @return Objeto Url construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static AuthenticationLatinia getObject(String xml) throws XOMarshallerException {    
	    return (AuthenticationLatinia)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		AuthenticationLatinia latiniaAuth = new AuthenticationLatinia();
//		latiniaAuth.loginEnterprise = "INNOVUS";
//		latiniaAuth.userLatinia     = "innovus.superusuario";
//		latiniaAuth.passwordLatinia = "MARKSTAT";
//		latiniaAuth.refProduct      = "Z99";  // P12G
//		latiniaAuth.idContract      = "2000"; // el id que nos asignen... = 2043
//		latiniaAuth.password        = "Z99";  // P12G
		try {
			System.out.println("XML: " + latiniaAuth.toXML());

			String strXML = "<authenticationLatinia><password>Z99</password><idContract>2000</idContract><loginEnterprise>INNOVUS</loginEnterprise><passwordLatinia>MARKSTAT</passwordLatinia><refProduct>Z99</refProduct><userLatinia>innovus.superusuario</userLatinia></authenticationLatinia>";
			AuthenticationLatinia latiniaAuth1 = AuthenticationLatinia.getObject(strXML);
			System.out.println("XML(1): " + latiniaAuth1.toXML());
		} catch (XOMarshallerException e) {
			e.printStackTrace();
		}
	}

}
