package p12f.exe.pasarelapagos.objects.sms.latinia;

import java.io.Serializable;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * 
 * @author FMARTINI
 *
 * Ejemplo (de documentacin Latinia):
 * 
 * [ MENSAJE_INFO ==> SMSMessage ]
 * 
 * <?xml version="1.0" encoding="UTF-8"?>
 * <PETICION>
 * 	<LATINIA>
 * 		<MENSAJES>
 * 			<MENSAJE_INFO ACUSE="S">
 * 				<TS_DIFERIDO>620000000</TS_DIFERIDO> <!-- TimeStamp (long)-->
 * 				<TEXTO><![CDATA[prueba mensaje viernes 2]]></TEXTO>
 * 				<GSM_DEST>659000001,666000001</GSM_DEST>
 * 			</MENSAJE_INFO>
 * 			<MENSAJE_INFO ACUSE="N">
 * 				<TS_EXPIRE>10</TS_EXPIRE> <!-- Minutos de vida del mensaje-->
 * 				<TEXTO><![CDATA[Hola mundo!!!]]></TEXTO>
 * 				<GSM_DEST>659000001</GSM_DEST>
 * 			</MENSAJE_INFO>
 * 		</MENSAJES>
 * 	</LATINIA>
 * </PETICION>
 *
 * MENSAJES: Lista de MENSAJE_INFO (mensajes) que se desean enviar.
 * + MENSAJE_INFO: informacin del mensaje que se quiere enviar.
 * 	- ACUSE: S o N.
 * 		 S: Se pide notificacin de entrega al proveedor.
 * 		 N: No se pide notificacin de entrega al  proveedor.
 * 	- TS_DIFERIDO: Timestamp en milisegundos (long) con la fecha concreta en la
 *                 que se quiere entregar el mensaje.
 * 	- TS_EXPIRE: Minutos de vida del mensaje (int). Fecha a partir de la cual
 *               el mensaje ya no se va a enviar.
 * 	- TEXTO: Seccin CDATA con el texto del mensaje que se quiere
 *           enviar al usuario.
 * 	- GSM_DEST: Lista de los nmeros de telfono a los que se quiere enviar el
 *              mensaje. Como mximo se pueden enviar 10 y tienen que ir
 *              separados por comas.
 */

public class LatiniaMessage implements Serializable, Initializable {

	private static final long serialVersionUID = 3707683691743824854L;

	public String acuse;          // ACUSE (S/N)
	public String timeStamp;      // TS_DIFERIDO
	public String expireTime;     // TS_EXPIRE
	public String messageContent; // TEXTO
	public String receiverNumber; // GSM_DEST ("600123456,600987654") mx. 150 nmeros por mensaje

	/**
	 * Convierte un Objeto Url a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto Url.
	 * @return Objeto Url construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static LatiniaMessage getObject(String xml) throws XOMarshallerException {    
	    return (LatiniaMessage)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}

	/*
	public static void main(String[] args) {
		SMSMessage sms = new SMSMessage();
		sms.acuse          = "N";
		sms.receiverNumber = "600123456";
		sms.messageContent = "Hola mundo...";
		sms.expireTime     = "30";
		try {
			System.out.println("XML: " + sms.toXML());

			String strXML = "<MENSAJE_INFO ACUSE=\"S\"><TS_DIFERIDO>620000000</TS_DIFERIDO><TEXTO><![CDATA[Hola mundo SMS]]></TEXTO><GSM_DEST>659000001,666000001</GSM_DEST></MENSAJE_INFO>";
			SMSMessage sms1 = SMSMessage.getObject(strXML);
			System.out.println("XML(1): " + sms1.toXML());
		} catch (XOMarshallerException e) {
			e.printStackTrace();
		}
	}
	*/

}
