package p12f.exe.pasarelapagos.objects.sms.latinia;

/**
 * 
 * @author FMARTINI
 *
 * Resultado de uno de los mensajes para una peticin SMS (LatiniaRequest)
 * 
 * (un sms se ha podido enviar a n nmeros de telfono)
 *
 * Ejemplo (de documentacin Latinia):
 * 	<MENSAJE NUM="1">
 * 		<TELEFONO NUM="659000001">
 * 			<RESULTADO>OK</RESULTADO>
 * 			<IDENTIFICADOR>UGsiZ7E1naZX/Uey32A1hFUq</IDENTIFICADOR>
 * 		</TELEFONO>
 * 		<TELEFONO NUM="666000001">
 * 			<RESULTADO>OK</RESULTADO>
 * 			<IDENTIFICADOR>UGsiZ7E2efSshUey32A1mU7o</IDENTIFICADOR>
 * 		</TELEFONO>
 * 		<TELEFONO NUM="600123456">
 * 			<RESULTADO>ERROR</RESULTADO>
 * 			<CODIGO_ERROR>301</CODIGO_ERROR>
 * 			<MENSAJE_ERROR>El mensaje ha expirado</MENSAJE_ERROR>
 * 		</TELEFONO>
 * 	</MENSAJE>
 * 
 * MENSAJE: Si un mensaje es enviado a varios nmeros de telfono nos
 *          devuelve una lista con el resultado de cada uno de ellos.
 * NUM: nmero del mensaje que se ha enviado.
 * - TELEFONO: Mensaje que se ha enviado. 
 * - NUM: Nmero de telfono al que se ha enviado el mensaje.
 * - RESULTADO: OK o ERROR. Si el mensaje se ha entregado bien se devuelve
 *              un OK. Si ha sucedido algn error se devuelve ERROR.
 * - IDENTIFICADOR: Si el resultado ha sido OK, se devuelve el identificador
 *                  nico del mensaje, id interno del mensaje en la plataforma
 *                  (para la Q68) y en identificador externo, no nico, en el
 *                  caso de las aplicaciones no Q68. En este ltimo caso
 *                  Identificador + Num de telfono ser la manera de
 *                  identificar cada sms enviado.
 * - CODIGO_ERROR: Cdigo asignado al error que se ha producido.
 * - MENSAJE_ERROR: Descripcin del error que se ha generado.
 * 
 */
import java.io.Serializable;
import java.util.List;

import p12f.exe.pasarelapagos.objects.ERPI.ERPIEvent;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

public class LatiniaMessageAllReceiversResult implements Serializable, Initializable {

	public String messageResultId;
	public List smsMessageReceiverResultList;

	public LatiniaMessageAllReceiversResult(){
	    ObjectUtils.initialize(this);
	}
	
	/**
	 * Aade el resultado de un mensaje SMS para un nmero de telfono a la
	 * lista de resultados del SMS (para cada nmero de telfono al que va
	 * dirigido un SMS habr un resultado.
	 * @param receiverResult
	 * @return
	 */
	public boolean addMessageReceiverResult(LatiniaMessageReceiverResult receiverResult) {
		this.smsMessageReceiverResultList.add(receiverResult);
		return true;
	}

	/**
	 * Convierte un Objeto Url a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXMLW91D() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this).replaceAll("</?RESULTADO_SMS_TELEFONOS>", "");
	}
	
	/**
	 * Convierte un XML en un objeto Url.
	 * @return Objeto Url construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static LatiniaMessageAllReceiversResult getObjectW91D(String xml) throws XOMarshallerException {    
		String xmlSMSResul = xml.matches(".*</?RESULTADO_SMS_TELEFONOS>.*") ? xml : xml.replaceAll("<MENSAJE NUM='([0-9]{1,4})'>(.*)</MENSAJE>",
				                                                                                   "<MENSAJE NUM='$1'><RESULTADO_SMS_TELEFONOS>$2</RESULTADO_SMS_TELEFONOS></MENSAJE>");
		//System.out.println("TRAZA(SMSMessageResult.getObject): "+ xmlSMSResul);
	    return (LatiniaMessageAllReceiversResult)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xmlSMSResul);
	}

	/**
	 * Convierte un Objeto Url a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		//return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
		return this.toXMLW91D();
	}
	
	/**
	 * Convierte un XML en un objeto Url.
	 * @return Objeto Url construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static LatiniaMessageAllReceiversResult getObject(String xml) throws XOMarshallerException {    
	    //return (SMSMessageResult)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	    return LatiniaMessageAllReceiversResult.getObjectW91D(xml);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		LatiniaMessageAllReceiversResult resultadoSMS = new LatiniaMessageAllReceiversResult();

		// hemos mandado un sms a 2 telfonos...
		try {
			resultadoSMS.messageResultId = "1";

			// el primer telfono receptor lo ha recibido bien...
			LatiniaMessageReceiverResult resultadoSMS0 = new LatiniaMessageReceiverResult();
			resultadoSMS0.receiverNumber = "600987654";
			resultadoSMS0.result         = "OK";
			resultadoSMS0.messageId      = "qwertyuiop";

			System.out.println("XML resultado SMS OK: " + resultadoSMS0.toXML());

			resultadoSMS.addMessageReceiverResult(resultadoSMS0);

			// el segundo telfono receptor ha fallado...
			LatiniaMessageReceiverResult resultadoSMS1 = new LatiniaMessageReceiverResult();
			resultadoSMS1.receiverNumber = "600123456";
			resultadoSMS1.result         = "ERROR";
			resultadoSMS1.errorCode      = "999";
			resultadoSMS1.errorMessage   = "Error de prueba";

			System.out.println("XML resultado SMS ERROR: " + resultadoSMS1.toXML());

			resultadoSMS.addMessageReceiverResult(resultadoSMS1);

			System.out.println("XML resultado del SMS a todos los telfonos:\n" + resultadoSMS.toXML());

			resultadoSMS = LatiniaMessageAllReceiversResult.getObject("<MENSAJE NUM='1'><TELEFONO NUM='600555555'><IDENTIFICADOR><![CDATA[qwertyuiop]]></IDENTIFICADOR><RESULTADO>OK</RESULTADO></TELEFONO><TELEFONO NUM='600123456'><RESULTADO>ERROR</RESULTADO><CODIGO_ERROR>999</CODIGO_ERROR><MENSAJE_ERROR><![CDATA[Error de prueba]]></MENSAJE_ERROR></TELEFONO></MENSAJE>");
			System.out.println("XML test getObject:\n" + resultadoSMS.toXML());

		} catch (XOMarshallerException e) {
			e.printStackTrace();
		}
	}

}
