package p12f.exe.pasarelapagos.objects.sms.latinia;

/**
 * 
 * @author FMARTINI
 *
 * Resultado de uno de los mensajes para un nico nmero de telfono en
 * una peticin SMS (SMSRequest)
 * 
 * (un sms se ha podido enviar a n nmeros de telfono)
 *
 * Ejemplo OK (de documentacin Latinia):
 * 		<TELEFONO NUM="659000001">
 * 			<RESULTADO>OK</RESULTADO>
 * 			<IDENTIFICADOR>UGsiZ7E1naZX/Uey32A1hFUq</IDENTIFICADOR>
 * 		</TELEFONO>
 * 
 * Ejemplo ERROR (de documentacin Latinia):
 * 		<TELEFONO NUM="600123456">
 * 			<RESULTADO>ERROR</RESULTADO>
 * 			<CODIGO_ERROR>301</CODIGO_ERROR>
 * 			<MENSAJE_ERROR>El mensaje ha expirado</MENSAJE_ERROR>
 * 		</TELEFONO>
 * 
 * MENSAJE: Si un mensaje es enviado a varios nmeros de telfono nos
 *          devuelve una lista con el resultado de cada uno de ellos.
 * NUM: nmero del mensaje que se ha enviado.
 * - TELEFONO: Mensaje que se ha enviado. 
 * - NUM: Nmero de telfono al que se ha enviado el mensaje.
 * - RESULTADO: OK o ERROR. Si el mensaje se ha entregado bien se devuelve
 *              un OK. Si ha sucedido algn error se devuelve ERROR.
 * - IDENTIFICADOR: Si el resultado ha sido OK, se devuelve el identificador
 *                  nico del mensaje, id interno del mensaje en la plataforma
 *                  (para la Q68) y en identificador externo, no nico, en el
 *                  caso de las aplicaciones no Q68. En este ltimo caso
 *                  Identificador + Num de telfono ser la manera de
 *                  identificar cada sms enviado.
 * - CODIGO_ERROR: Cdigo asignado al error que se ha producido.
 * - MENSAJE_ERROR: Descripcin del error que se ha generado.
 * 
 */
import java.io.Serializable;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

public class LatiniaMessageReceiverResult implements Serializable, Initializable {

	public String receiverNumber; // TELEFONO (600123456)
	public String result;         // RESULTADO (OK/ERROR)
	public String messageId;      // IDENTIFICADOR (qwerty...)
	public String errorCode;      // CODIGO_ERROR (301)
	public String errorMessage;   // MENSAJE_ERROR ("El mensaje ha expirado")

	/**
	 * Convierte un Objeto Url a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto Url.
	 * @return Objeto Url construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static LatiniaMessageReceiverResult getObject(String xml) throws XOMarshallerException {    
	    return (LatiniaMessageReceiverResult)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		LatiniaMessageReceiverResult resultadoSMS = new LatiniaMessageReceiverResult();
		resultadoSMS.receiverNumber = "600987654";
		resultadoSMS.result         = "OK";
		resultadoSMS.messageId      = "qwertyuiop";

		LatiniaMessageReceiverResult resultadoSMS1 = new LatiniaMessageReceiverResult();
		resultadoSMS1.receiverNumber = "600123456";
		resultadoSMS1.result         = "ERROR";
		resultadoSMS1.errorCode      = "999";
		resultadoSMS1.errorMessage   = "Error de prueba";

		try {
			System.out.println("XML resultado SMS OK: " + resultadoSMS.toXML());
			System.out.println("XML resultado SMS ERROR: " + resultadoSMS1.toXML());
		} catch (XOMarshallerException e) {
			e.printStackTrace();
		}
	}
}
