package p12f.exe.pasarelapagos.objects.sms.latinia;

/**
 * 
 * @author FMARTINI
 *
 * Ejemplo (de documentacin Latinia):
 * <?xml version="1.0" encoding="UTF-8"?>
 * <PETICION>
 *	<LATINIA>
 *		<MENSAJES>
 *			<MENSAJE_INFO ACUSE="S">
 *				<TS_DIFERIDO>620000000</TS_DIFERIDO> <!-- TimeStamp (long)-->
 *				<TEXTO><![CDATA[prueba mensaje viernes 2]]></TEXTO>
 *				<GSM_DEST>659000001,666000001</GSM_DEST>
 *			</MENSAJE_INFO>
 *			<MENSAJE_INFO ACUSE="N">
 *				<TS_EXPIRE>10</TS_EXPIRE> <!-- Minutos de vida del mensaje-->
 *				<TEXTO><![CDATA[Hola mundo!!!]]></TEXTO>
 *				<GSM_DEST>659000001</GSM_DEST>
 *			</MENSAJE_INFO>
 *		</MENSAJES>
 *	</LATINIA>
 * </PETICION>
 *
 * MENSAJES: Lista de MENSAJE_INFO (mensajes) que se desean enviar.
 * MENSAJE_INFO: informacin del mensaje que se quiere enviar.
 * ACUSE: S o N.
 *        S => Se pide notificacin de entrega al proveedor.
 *        N => No se pide notificacin de entrega al  proveedor.
 * TS_DIFERIDO: Timestamp en milisegundos (long) con la fecha concreta en la
 *              que se quiere entregar el mensaje.
 * TS_EXPIRE: Minutos de vida del mensaje (int). Fecha a partir de la cual el
 *            mensaje ya no se va a enviar.
 * TEXTO: Seccin CDATA con el texto del mensaje que se quiere enviar al usuario.
 * GSM_DEST: Lista de los nmeros de telfono a los que se quiere enviar el
 *           mensaje. Como mximo se pueden enviar 10 y tienen que ir separados
 *           por comas.
 */
import java.io.Serializable;
import java.util.List;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

public class LatiniaRequest implements Serializable, Initializable {

	public LatiniaMessageList smsRequestMessageList;
	public List smsMessageResultList;

	public LatiniaRequest(){
	    ObjectUtils.initialize(this);
	}

	/**
	 * Aade un mensaje SMS a la lista de mensajes de esta peticin
	 * @param sms
	 * @return
	 */
	public boolean addMessageToRequestMessageList(LatiniaMessage sms) {
		this.smsRequestMessageList.addMessage(sms);
		return true;
	}

	/**
	 * Aade el resultado de un mensaje SMS
	 * @param smsResult
	 * @return
	 */
	public boolean addResultToMessageResultList(LatiniaMessageAllReceiversResult smsResult) {
		this.smsMessageResultList.add(smsResult);
		return true;
	}

	/**
	 * Convierte un Objeto Url a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto Url.
	 * @return Objeto Url construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static LatiniaRequest getObject(String xml) throws XOMarshallerException {    
	    return (LatiniaRequest)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		LatiniaRequest peticion0 = new LatiniaRequest(); // test peticin de envo...
		LatiniaRequest peticion1 = new LatiniaRequest(); // test resultado de envo...


		LatiniaMessage sms = new LatiniaMessage();
		sms.acuse          = "N";
		sms.receiverNumber = "600123456";
		sms.messageContent = "Hola mundo...";
		sms.expireTime     = "30";

		peticion0.addMessageToRequestMessageList(sms);

		try {
			String strXML = "<MENSAJE_INFO ACUSE=\"S\"><TS_DIFERIDO>620000000</TS_DIFERIDO><TEXTO><![CDATA[Hola mundo SMS]]></TEXTO><GSM_DEST>659000001,666000001</GSM_DEST></MENSAJE_INFO>";
			LatiniaMessage sms1 = LatiniaMessage.getObject(strXML);

			peticion0.addMessageToRequestMessageList(sms1);

			//System.out.println("XML(0): " + sms.toXML());
			//System.out.println("XML(1): " + sms1.toXML());
			//System.out.println("XML lista mensajes: " + peticion0.smsRequestMessageList.toXML());
			System.out.println("XML peticin SMS: " + peticion0.toXML());

		} catch (XOMarshallerException e) {
			e.printStackTrace();
		}

	}

}
