package p12f.exe.pasarelapagos.objects.sms.latinia;

import java.io.Serializable;
import java.util.Iterator;
import java.util.List;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * 
 * @author FMARTINI
 *
 * Resultado de todos los mensajes para todas las peticiones SMS para todos
 * sus destinatarios.
 * 
 * Un LatiniaResult puede contener n LatiniaMessageAllReceiverResult
 *
 * Ejemplo (de documentacin Latinia):
 * <?xml version="1.0" encoding="UTF-8"?>
 * <PETICION>
 * 	<MENSAJE NUM="1">
 * 		<TELEFONO NUM="659000001">
 * 			<RESULTADO>OK</RESULTADO>
 * 			<IDENTIFICADOR>UGsiZ7E1naZX/Uey32A1hFUq</IDENTIFICADOR>
 * 		</TELEFONO>
 * 		<TELEFONO NUM="666000001">
 * 			<RESULTADO>OK</RESULTADO>
 * 			<IDENTIFICADOR>UGsiZ7E2efSshUey32A1mU7o</IDENTIFICADOR>
 * 		</TELEFONO>
 * 	</MENSAJE>
 * 	<MENSAJE NUM="2">
 * 		<TELEFONO NUM="659000001">
 * 			<RESULTADO>ERROR</RESULTADO>
 * 			<CODIGO_ERROR>301</CODIGO_ERROR>
 * 			<MENSAJE_ERROR>El mensaje ha expirado</MENSAJE_ERROR>
 * 		</TELEFONO>
 * 	</MENSAJE>
 * </PETICION>
 * 
 * 
 * MENSAJE: Si un mensaje es enviado a varios nmeros de telfono nos
 *          devuelve una lista con el resultado de cada uno de ellos.
 * NUM: nmero del mensaje que se ha enviado.
 * - TELEFONO: Mensaje que se ha enviado. 
 * - NUM: Nmero de telfono al que se ha enviado el mensaje.
 * - RESULTADO: OK o ERROR. Si el mensaje se ha entregado bien se devuelve
 *              un OK. Si ha sucedido algn error se devuelve ERROR.
 * - IDENTIFICADOR: Si el resultado ha sido OK, se devuelve el identificador
 *                  nico del mensaje, id interno del mensaje en la plataforma
 *                  (para la Q68) y en identificador externo, no nico, en el
 *                  caso de las aplicaciones no Q68. En este ltimo caso
 *                  Identificador + Num de telfono ser la manera de
 *                  identificar cada sms enviado.
 * - CODIGO_ERROR: Cdigo asignado al error que se ha producido.
 * - MENSAJE_ERROR: Descripcin del error que se ha generado.
 * 
 */
public class LatiniaResult implements Serializable, Initializable {

	public List smsMessageResultList;	

	public LatiniaResult(){
	    ObjectUtils.initialize(this);
	}

	/**
	 * Aade el resultado de un mensaje SMS
	 * @param smsResult
	 * @return
	 */
	public boolean addMessageResult(LatiniaMessageAllReceiversResult smsResult) {
		this.smsMessageResultList.add(smsResult);
		return true;
	}

	/**
	 * Convierte un Objeto Url a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXMLW91D() throws XOMarshallerException {
		String xml = XOManager.getXML(XMLProperties.get("p12f","objectMapPath"), this);
		
		xml = xml.replaceAll("</?RESULTADO_SMS_TELEFONOS>", "");
		xml = xml.replaceAll("</?RESULTADO_MENSAJES>", "");
		xml = xml.replaceAll("RESULTADO_PETICION>", "PETICION>");

		return xml;
	}

	/**
	 * Convierte un Objeto Url a XML.
	 * @return String que contiene el XML.
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {   
		//return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
		return this.toXMLW91D();
	}

	/**
	 * @param xml
	 * @return
	 * @throws XOMarshallerException
	 */
	public static LatiniaResult getObjectW91D(String xml) throws XOMarshallerException {    
		System.out.println("TRAZA(LatiniaResult.getObjectW91D): "+ xml);

		String xmlSMSResul = xml.matches(".*</?RESULTADO_SMS_TELEFONOS>.*") ? xml : xml.replaceAll("<MENSAJE NUM='([0-9]{1,4})'>(.*)</MENSAJE>",
				                                                                                   "<MENSAJE NUM='$1'><RESULTADO_SMS_TELEFONOS>$2</RESULTADO_SMS_TELEFONOS></MENSAJE>");
		//System.out.println("TRAZA(SMSMessageResult.getObject): "+ xmlSMSResul);
	    return (LatiniaResult)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xmlSMSResul);
	}
	
	/**
	 * Convierte un XML en un objeto Url.
	 * @return Objeto Url construido a partir del XML.
	 * @throws XOMarshallerException
	 */
	public static LatiniaResult getObject(String xml) throws XOMarshallerException {    
		System.out.println("TRAZA(LatiniaResult.getObject): "+ xml);
		
		String xmlW91D = xml.replaceAll("PETICION>(.*)</PETICION", "RESULTADO_PETICION><RESULTADO_MENSAJES>$1</RESULTADO_MENSAJES></RESULTADO_PETICION");

//		xmlW91D = xmlW91D.replaceAll("<MENSAJE NUM='([0-9]{1,4})'>(^MENSAJE+)</MENSAJE>",
//                                     "<MENSAJE NUM='$1'><RESULTADO_SMS_TELEFONOS>$2</RESULTADO_SMS_TELEFONOS></MENSAJE>");
		xmlW91D = xmlW91D.replaceAll("<MENSAJE NUM='([0-9]{1,4})'>(.*)</MENSAJE>",
				                     "<MENSAJE NUM='$1'><RESULTADO_SMS_TELEFONOS>$2</RESULTADO_SMS_TELEFONOS></MENSAJE>");
		
		System.out.println("TRAZA(LatiniaResult.getObject): "+ xmlW91D);
		
	    //return (LatiniaResult)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	    return getObjectW91D(xmlW91D);
	}

	/**
	 * Extraer campos de un resultado Latinia para un SMS simple (solo un mensaje para un nico destinatario)
	 * 
	 * @param fieldName
	 * @return
	 */
	public String getDataFromSimpleResult(String fieldName) {
		String fieldValue = "";
				
		Iterator ir = this.smsMessageResultList.iterator();
		
		LatiniaMessageAllReceiversResult smsResult = (LatiniaMessageAllReceiversResult)ir.next();

		Iterator it = smsResult.smsMessageReceiverResultList.iterator();

		LatiniaMessageReceiverResult receiverResult = (LatiniaMessageReceiverResult)it.next();

		if (fieldName.equalsIgnoreCase("TELEFONO"))
			return receiverResult.receiverNumber.replaceFirst(".*<TELEFONO NUM=\"(.*)\">.*</TELEFONO>.*", "$1");

		if (fieldName.equalsIgnoreCase("RESULTADO"))
			return receiverResult.result.replaceFirst(".*<RESULTADO>(.*)</RESULTADO>.*", "$1");

		if (receiverResult.result.replaceFirst(".*<RESULTADO>(.*)</RESULTADO>.*", "$1").equalsIgnoreCase("OK")) {
			if (fieldName.equalsIgnoreCase("IDENTIFICADOR")) {
				return receiverResult.messageId.replaceFirst(".*<IDENTIFICADOR>(.*)</IDENTIFICADOR>.*", "$1");
			}
		}
		else { // ERROR
			if (fieldName.equalsIgnoreCase("CODIGO_ERROR"))
				return receiverResult.errorCode.replaceFirst(".*<CODIGO_ERROR>(.*)</CODIGO_ERROR>.*", "$1");
			if (fieldName.equalsIgnoreCase("MENSAJE_ERROR"))
				return receiverResult.errorMessage.replaceFirst(".*<MENSAJE_ERROR>(.*)</MENSAJE_ERROR>.*", "$1");
		}
		return "";
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		LatiniaResult resul = new LatiniaResult();
		
		try {
			// ---------------------------------------------------------------
			// Resultado de SMS para telefonos 600987000, 601123111 y 602987222
			LatiniaMessageAllReceiversResult resultadoSMS0 = new LatiniaMessageAllReceiversResult();
			resultadoSMS0.messageResultId = "1";

			// el primer telfono receptor lo ha recibido bien...
			LatiniaMessageReceiverResult resultadoSMS0Tel0 = new LatiniaMessageReceiverResult();
			resultadoSMS0Tel0.receiverNumber = "600987000";
			resultadoSMS0Tel0.result         = "OK";
			resultadoSMS0Tel0.messageId      = "qwertyuiop";
			//System.out.println("XML resultado SMS OK: " + resultadoSMS0.toXML());

			resultadoSMS0.addMessageReceiverResult(resultadoSMS0Tel0);

			// el segundo telfono receptor ha fallado...
			LatiniaMessageReceiverResult resultadoSMS0Tel1 = new LatiniaMessageReceiverResult();
			resultadoSMS0Tel1.receiverNumber = "601123111";
			resultadoSMS0Tel1.result         = "ERROR";
			resultadoSMS0Tel1.errorCode      = "999";
			resultadoSMS0Tel1.errorMessage   = "Error de prueba";
			//System.out.println("XML resultado SMS ERROR: " + resultadoSMS1.toXML());

			resultadoSMS0.addMessageReceiverResult(resultadoSMS0Tel1);

			// el tercero telfono receptor lo ha recibido bien...
			LatiniaMessageReceiverResult resultadoSMS0Tel2 = new LatiniaMessageReceiverResult();
			resultadoSMS0Tel2.receiverNumber = "602987222";
			resultadoSMS0Tel2.result         = "OK";
			resultadoSMS0Tel2.messageId      = "zxcvbnm";
			//System.out.println("XML resultado SMS OK: " + resultadoSMS0.toXML());

			resultadoSMS0.addMessageReceiverResult(resultadoSMS0Tel2);

			//System.out.println("XML resultado del SMS a todos los telfonos:\n" + resultadoSMS0.toXML());

			//resultadoSMS = SMSMessageResult.getObject("<MENSAJE NUM='1'><TELEFONO NUM='600555555'><IDENTIFICADOR><![CDATA[qwertyuiop]]></IDENTIFICADOR><RESULTADO>OK</RESULTADO></TELEFONO><TELEFONO NUM='600123456'><RESULTADO>ERROR</RESULTADO><CODIGO_ERROR>999</CODIGO_ERROR><MENSAJE_ERROR><![CDATA[Error de prueba]]></MENSAJE_ERROR></TELEFONO></MENSAJE>");
			//System.out.println("XML test getObject:\n" + resultadoSMS.toXML());
			resul.addMessageResult(resultadoSMS0);

			// ---------------------------------------------------------------
			// Resultado de SMS para telefonos 677987654 ------------------
			LatiniaMessageAllReceiversResult resultadoSMS1 = new LatiniaMessageAllReceiversResult();
			resultadoSMS1.messageResultId = "2";

			// el primer telfono receptor lo ha recibido bien...
			LatiniaMessageReceiverResult resultadoSMS1Tel0 = new LatiniaMessageReceiverResult();
			resultadoSMS1Tel0.receiverNumber = "677987654";
			resultadoSMS1Tel0.result         = "OK";
			resultadoSMS1Tel0.messageId      = "asdfghjkl";
			//System.out.println("XML resultado SMS OK: " + resultadoSMS0.toXML());

			resultadoSMS1.addMessageReceiverResult(resultadoSMS1Tel0);

			resul.addMessageResult(resultadoSMS1);

			// ---------------------------------------------------------------
			String xmlResul = resul.toXML();
			System.out.println("XML resultado de todos los SMS:\n" + xmlResul);

			// test getObject... - - - - - - - - - - - - - - - - - - - - - - -
			// test n sms a k telfonos...
			//xmlResul = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><PETICION><MENSAJE NUM=\"1\"><TELEFONO NUM=\"777123456\"><RESULTADO>OK</RESULTADO><IDENTIFICADOR>UIOY2nF0xYnehkey32ARgJxA</IDENTIFICADOR></TELEFONO></MENSAJE></PETICION>";

			// test 1 sms a n telfonos...
			//xmlResul = "<PETICION><MENSAJE NUM='1'><TELEFONO NUM='600987000'><IDENTIFICADOR><![CDATA[qwertyuiop]]></IDENTIFICADOR><RESULTADO>OK</RESULTADO></TELEFONO><TELEFONO NUM='601123111'><RESULTADO>ERROR</RESULTADO><CODIGO_ERROR>999</CODIGO_ERROR><MENSAJE_ERROR><![CDATA[Error de prueba]]></MENSAJE_ERROR></TELEFONO><TELEFONO NUM='602987222'><IDENTIFICADOR><![CDATA[zxcvbnm]]></IDENTIFICADOR><RESULTADO>OK</RESULTADO></TELEFONO></MENSAJE></PETICION>";

			LatiniaResult r = LatiniaResult.getObject(xmlResul);
			System.out.println("XML sobre getObject...:\n" + r.toXML());

		} catch (XOMarshallerException e) {
			e.printStackTrace();
		}
	}

}
