package p12f.exe.pasarelapagos.paymentrequest;

import java.io.Serializable;
import java.util.Map;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

import p12f.exe.pasarelapagos.objects.Concepto;

/**
 *  Objeto Concepto de Peticion de Pago usado para SIPCA.
 *  Hereda los atributos del Concepto de Pago y aade
 *  los atributos para el backend (SIPCA)
 *  Para  SIPCA dichos atributos son :
 * 
 * 
	 numeroLinea;     // Numero de la linea en la que aparece el concepto    
	 codigoIngreso;            // SIPCA
	 aplicPresupuestaria;      // SIPCA
	 numInmovilizado;          // SIPCA
	 baseImponible;            // Base imponible sobre la que se repercute el iva
	 tipoIVA;                   // Tipo de iva que se aplica - porcentaje
	 ivaRepercutido = false;  /* 1- IVA SI Deducible-Repercutido: El IVA va como un concepto individual (en caso de un alquiler, etc...)
										A SIPCA se le pasan la baseImponible, tipoIVA, territorioIVA y conceptoIVA.
								 2- IVA NO Deducible-Repercutido: El IVA va incluido en el importe del concepto. 
										 - SI hay base imponible y tipoIVA (precios pblicos, como venta de libros, etc):
										 				el IVA se desglosa en la liquidacin.
										 - NO hay base imponible ni tipoIVA (tasas): no hay IVA.
									
     importeIVA;               // importe del iva, se enva si el IVA es NO Deducible-Repercutido y se quiere mostrar el IVA desglosado.
	 conceptoIVA;              // Concepto del iva que se enva a SIPCA
	 territorioIVA;            // Territorio IVA

 *  Adicionalmente se considera si el concepto es o no presupuestario. Caso de
 *  no serlo no se validara aplicPresupuestaria.
 * 
 * 
 */
public class ConceptoResolucion extends Concepto implements Serializable, Initializable{	
	
	public Map backendDataMap ; 			 // Map de objetos BackEnd (Sipca, etc);

	public Boolean noPresupuestario = null; // Flag que indica si el concepto es NO presupuestario (true/false)

	public ConceptoResolucion(){
	    ObjectUtils.initialize(this);
	}
	
	/**
     * Convierte de un String xml a un Objeto 
     * @param xml : XML de objeto.
     * @return Objeto convertido.
     * @throws XOMarshallerException
     */
    public static Concepto getObject( String xml) throws XOMarshallerException {
        return (ConceptoResolucion)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"),xml);   
    }

    /**
     * Convierte un Objeto ConceptoPeticion a XML.
     * @return
     * @throws XOMarshallerException
     */
    public String toXML() throws XOMarshallerException {   
         return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
    }
   
}
