/*
 * Created on 18-may-2005
 * @author co01347e
 */
package p12f.exe.pasarelapagos.paymentrequest;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import p12f.exe.pasarelapagos.objects.PeriodoPago;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Estructura que encapsula un lote de peticiones de pago.
 */
public class PaymentRequestData implements Serializable, Initializable {

	
	public Map peticionesPago;    // Multiples instancias de pagos a realizar-
	public Map datosAdicionales; //  Datos adicionales.
	
	public PaymentRequestData(){
	    ObjectUtils.initialize(this);
	}
	
	/**
     * Convierte un Objeto Aplicacion a XML.
     * @return
     * @throws XOMarshallerException
     */
	public String toXML() throws XOMarshallerException {   
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	/**
	 * Convierte un XML en un objeto PaymentRequestData.
	 * @return
	 * @throws XOMarshallerException
	 */
	public static PaymentRequestData getObject(String xml) throws XOMarshallerException {    
	    return (PaymentRequestData)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
	
	/**
	 * Metodo que permite obtener el importe total de las peticiones de pago, que coincide con 
	 * la suma de los importes de los periodos activos de cada una de ellas.
	 * @return
	 */
	public long getImporteTotal(){
	    
	    long total = 0;
	    
	    for(Iterator it=this.peticionesPago.keySet().iterator(); it.hasNext();){
	        PeticionPago peticionPago = (PeticionPago)this.peticionesPago.get(it.next());
	        //Map periodosPago = peticionPago.datosPago.periodosPago;
            total+= peticionPago.getImporte();
	    }
	    
	    /*String importe = new Long(total).toString();
	    while(importe.length()<3)
	        importe = "0" + importe;
        return importe.substring(0,importe.length()-2) + "," + importe.substring(importe.length()-2, importe.length());*/
	    
	    return total;   
	}
    
    /**
     * Metodo que permite obtener el importe total de las peticiones de pago, que coincide con 
     * la suma de los importes de los periodos normales o voluntarios de cada una de ellas.
     * @return
     */
    public long getImporteNormalTotal(){
        
        long total = 0;
        
        for(Iterator it=this.peticionesPago.keySet().iterator(); it.hasNext();){
            PeticionPago peticionPago = (PeticionPago)this.peticionesPago.get(it.next());
            Map periodosPago = peticionPago.datosPago.periodosPago;
            
            if(periodosPago.get(PeriodoPago.PERIODO_NORMAL)!=null) {
                PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_NORMAL);
                total += periodoPago.importe;
            } else if(periodosPago.get(PeriodoPago.PERIODO_VOLUNTARIO)!=null) {
                PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_VOLUNTARIO);
                total += periodoPago.importe;
            }       
        }
        
        /*String importe = new Long(total).toString();
        while(importe.length()<3)
            importe = "0" + importe;
        return importe.substring(0,importe.length()-2) + "," + importe.substring(importe.length()-2, importe.length());*/
        
        return total;   
    }
	
	/**
	 * Metodo que permite obtener el importe total de las peticiones de pago, que coincide con 
	 * la suma de los importes de los periodos activos de cada una de ellas.
	 * @return
	 */
	public String showImporteTotal(){
	    
	    long total = 0;
	    
	    for(Iterator it=this.peticionesPago.keySet().iterator(); it.hasNext();){
	        PeticionPago peticionPago = (PeticionPago)this.peticionesPago.get(it.next());
            total += peticionPago.getImporte();
            
            
	        /*Map periodosPago = peticionPago.datosPago.periodosPago;
	        for(Iterator iter=periodosPago.keySet().iterator(); iter.hasNext();){
	            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(iter.next());
	            if(periodoPago.activo){
	                total += periodoPago.importe;
	                break;
	            }
	        }*/
	    }
	    
	    String importe = new Long(total).toString();
	    while(importe.length()<3)
	        importe = "0" + importe;
        return importe.substring(0,importe.length()-2) + "," + importe.substring(importe.length()-2, importe.length());  
	}
	
	/**
	 * Metodo que comprueba los periodos de pago de los pagos del lote, descartando aquellos pagos 
	 * cuya fecha limite haya sido rebasada.
	 */
	public void comprobarPeriodos(){
	    Map peticionesPagoTemp = new HashMap();
	    // Se recorren los pagos del lote
	    for(Iterator it=peticionesPago.keySet().iterator(); it.hasNext(); ){
	        PeticionPago peticionPago = (PeticionPago)peticionesPago.get(it.next());
	        try {
	            peticionPago.comprobarPeriodos();
                peticionesPagoTemp.put(peticionPago.id, peticionPago);
            } catch (Exception e) {
                // Si salta la excepcion al actualizar el periodo, este pago se retira del lote.
            }
	    }
	    // Se devuelven lo pagos del lote cuya fecha limite no se ha superado.
	    peticionesPago = peticionesPagoTemp;
	}
}
