/*
 * Created on 08-jun-2005
 * @author co00414a
 */
package p12f.exe.pasarelapagos.paymentrequest;

import java.io.Serializable;
import java.util.Map;

import p12f.exe.pasarelapagos.exceptions.ConfigLoadException;
import p12f.exe.pasarelapagos.exceptions.LimitDateException;
import p12f.exe.pasarelapagos.objects.Pago;
import p12f.exe.pasarelapagos.objects.PeriodoPago;
import p12f.exe.pasarelapagos.objects.validation.ValidationConstants;
import p12f.exe.pasarelapagos.utils.LimitDateHelper;
import p12f.exe.pasarelapagos.utils.exceptions.PaymentDataHelpperException;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Objeto que representa un  objeto de Peticion de Pago
 */
public class PeticionPago extends Pago implements Serializable, Initializable{

	public static final String DEVENGO_PAGO = "0";
	public static final String DEVENGO_EMISION = "1";
	
	
	public Backend backend;			// Objeto BackEnd (Sipca, etc)
	public Aplicacion aplicacion; 	// Datos de la aplicacin que realiza la Peticion de Pago.
	public Liquidacion liquidacion;
	public Map finantialOrgs;
	public Map paymentModes;
    public Validacion validacion;
    public String devengo;
    public Boolean exento = null;
    
    public PeticionPago(){
        ObjectUtils.initialize(this);
        
        // Objetos que se deben inicializar a NULL
        validacion = null;
    }

    /**
     * Convierte de un String xmlPeticionPago a un Objeto 
     * @param xmlPeticionPago : XML de objeto.
     * @return Objeto convertido.
     * @throws XOMarshallerException
     */
    public static Pago getObject( String xmlPeticionPago ) throws XOMarshallerException {
    	return (PeticionPago) XOManager.getObject(XMLProperties.get("p12f","objectMapPath"),xmlPeticionPago);	 
    }

    /**
     * Convierte un Objeto PeticionPago a XML.
     * @return
     * @throws XOMarshallerException
     */
    public String toXML() throws XOMarshallerException {   
         return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
    }
    
    /**
     * Metodo que permite obtener el importe de un pago, que coincide con el importe del periodo activo.
     * @return
     */
    /*public long getImporte(){
        Map periodosPago = this.datosPago.periodosPago;
        long importe = 0;      
        for(Iterator it=periodosPago.keySet().iterator(); it.hasNext();){
            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(it.next());
            if (periodoPago.activo){
                return periodoPago.importe;
            }
        }

        // Si hay periodo normal y esta dentro de el
        if(periodosPago.get(PeriodoPago.PERIODO_NORMAL)!=null) {
            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_NORMAL);
            if(this.datosPago.formato.equals("521") || LimitDateHelper.comprobarFecha(periodoPago.fechaFin))
                return periodoPago.importe;
        } 
        // Si hay periodo voluntario y esta dentro de el
        if(periodosPago.get(PeriodoPago.PERIODO_VOLUNTARIO)!=null) {
            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_VOLUNTARIO);
            if(LimitDateHelper.comprobarFecha(periodoPago.fechaFin))
                return periodoPago.importe;
        }
        // Si hay periodo con recargo
        if(periodosPago.get(PeriodoPago.PERIODO_CON_RECARGO)!=null) {
            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_CON_RECARGO);
            return periodoPago.importe;
        }   
        
        return 0;
    }*/
    
    /**
     * Metodo que permite obtener el importe de un pago, que coincide con el importe del periodo normal o voluntario.
     * @return
     */
    public long getImporteNormal(){
        Map periodosPago = this.datosPago.periodosPago;
        long importe = 0;
        if(periodosPago.get(PeriodoPago.PERIODO_NORMAL)!=null) {
            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_NORMAL);
            importe = periodoPago.importe;
        } else if(periodosPago.get(PeriodoPago.PERIODO_VOLUNTARIO)!=null) {
            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_VOLUNTARIO);
            importe = periodoPago.importe;
        }                
        return importe;
    }
    
    /**
     * Metodo que permite obtener el importe de un pago, que coincide con el importe del periodo con recargo.
     * @return
     */
    public long getImporteConRecargo(){
        Map periodosPago = this.datosPago.periodosPago;
        long importe = 0;
        if(periodosPago.get(PeriodoPago.PERIODO_CON_RECARGO)!=null) {
            PeriodoPago periodoPago = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_CON_RECARGO);
            importe = periodoPago.importe;
        }        
     
        return importe;
    }

    /**
     * Mtodo que comprueba el estado de los periodos de pago de una peticion.
     * @param periodosPago Mapa de periodos de pago de la peticion.
     * @throws ConfigLoadException
     */
    public void comprobarPeriodos() throws LimitDateException, PaymentDataHelpperException{
        Map periodosPago = this.datosPago.periodosPago;
        // Se comprueba el periodo de pago normal
        PeriodoPago periodoNormal = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_NORMAL);
        if(periodoNormal!=null){
            // Si existe el periodo normal este sera el unico periodo de pago.
            // Se obtiene la fecha limite de este periodo.
            String fechaLimite = periodoNormal.fechaFin;
            if(fechaLimite==null){
                // Si la fecha limite es nula se interpreta que no hay fecha limite.
                periodoNormal.activo = true;
            }else{
                // Si la fecha limite no es nula se comprueba su validez.
                if(LimitDateHelper.comprobarFecha(fechaLimite)){
                    periodoNormal.activo = true;
                }else{
                    // La fecha limite se ha superado, este pago ya no se puede realizar.
                    periodoNormal.activo = false;
                    throw new LimitDateException();
                }
            }            
        }else{
            // Si no existe periodo normal, existiran periodos voluntario y con recargo.
            PeriodoPago periodoVoluntario = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_VOLUNTARIO);
            PeriodoPago periodoConRecargo = (PeriodoPago)periodosPago.get(PeriodoPago.PERIODO_CON_RECARGO);
            if(periodoVoluntario!=null && periodoConRecargo!=null){
                // Se obtiene la fecha limite del periodo voluntario.
                String fechaLimiteVol = periodoVoluntario.fechaFin;
                if(LimitDateHelper.comprobarFecha(fechaLimiteVol)){
                    periodoVoluntario.activo = true;
                    periodoConRecargo.activo = false;
                }else{
                    periodoVoluntario.activo = false;
                    // Se obtiene la fecha limite del periodo con recargo.
                    String fechaLimiteRec = periodoConRecargo.fechaFin;
                    if(LimitDateHelper.comprobarFecha(fechaLimiteRec)){
                        periodoConRecargo.activo = true;
                    }else{
                        // La fecha limite se ha superado, este pago ya no se puede realizar.
                        periodoConRecargo.activo = false;
                        throw new LimitDateException();
                    }
                }
            }else{
                // Si no contiene estos dos periodos se lanza una excepcion
                throw new PaymentDataHelpperException();
            }
        }
    }
    
    /**
	 * Metodo que permite las entidades financieras que permiten un  pago.
	 * @param idioma Idioma en el que se quiere presentar la informacin.
	 * @return
	 */
	/*
    public String getEntidadesFinancieras(String idioma){
	    int iteracion = 0;
	    Map finantialOrgs = null;
	    String entidadesPago = null;
        
        PeticionPago peticionPago = this;
        if(iteracion++==0){
            finantialOrgs = peticionPago.finantialOrgs;
        }else{
            Map finantialOrgsTemp = new HashMap();
	        for(Iterator iter=peticionPago.finantialOrgs.keySet().iterator();iter.hasNext(); ){
	            FinantialOrg entidad = (FinantialOrg)peticionPago.finantialOrgs.get(iter.next());
	            if(finantialOrgs.get(entidad.oid)!=null){
	                finantialOrgsTemp.put(entidad.oid, entidad);
	            }
	        }
	        finantialOrgs = finantialOrgsTemp;
        }
	   	    
	    try{
	        for(Iterator iter=finantialOrgs.keySet().iterator();iter.hasNext(); ){
	            FinantialOrg entidad = (FinantialOrg)finantialOrgs.get(iter.next());
	            // Se obtiene la configuracion de la entidad financiera
	            ConfigAPI configAPI = new ConfigAPI();
		        FinantialOrgConfig finantialOrgConfig = configAPI.loadFinantialOrgConfig(entidad.oid);
		        String nombre = (String)finantialOrgConfig.descripcion.get(idioma);
		        if(entidadesPago==null)
		            entidadesPago = nombre;
		        else
		            entidadesPago+=" - " + nombre;
	        }
		    
	    }catch(ConfigLoadException ex){
	        ex.printStackTrace();	
        }
	    
	    // Si no hay entidades de pago coincidentes entre todos los pagos.
	    if(entidadesPago==null) entidadesPago = "---------";
	    
	    return entidadesPago;
	}
    */
    /**
     * Mtodo que realiza la validacin de los datos de este objeto.
     * @return
     */
    public String validate() {
        
        if (datosPago==null) {
            return ValidationConstants.DATOSPAGO_NULL;
        } 
        
        String error_validacion = null;
        error_validacion = datosPago.validate();
        if(error_validacion!=null) {
            return error_validacion;
        }
            
        // Se comprueba si hay informacion de SIPCA
        if(backend!=null && backend.backendDataMap!=null && backend.backendDataMap.size()>0) {
            BackendData informarSipca = (BackendData)backend.backendDataMap.get(BackendDataConstants.INFORMAR_SIPCA);
            if(informarSipca!=null && "true".equals(informarSipca.value)) {
                // Se comprueban los conceptos
                if(conceptos!=null && conceptos.size()>0) {
                    // Se recorren los conceptos y se comprueban los datos
                    for(int i=0; i<conceptos.size(); i++) {
                        ConceptoPeticion concepto = (ConceptoPeticion)conceptos.get(i);
                        
                        // Comprobaciones generales de los conceptos
                        if(concepto.numeroLinea!=i+1) {
                            return ValidationConstants.NUMERO_LINEA;
                        }
                        if(concepto.descripcion==null || concepto.descripcion.size()==0) {
                            return ValidationConstants.CONCEPTO_DESCRIPCION;
                        }
                        if(concepto.importe==0) {
                            return ValidationConstants.CONCEPTO_IMPORTE;
                        }
                        if(concepto.backendDataMap==null || concepto.backendDataMap.size()==0) {
                            return ValidationConstants.CONCEPTO_BACKEND_NULL;
                        }
                        BackendData codigoIngreso = (BackendData)concepto.backendDataMap.get(BackendDataConstants.CODIGO_INGRESO);
                        if (codigoIngreso==null || codigoIngreso.value==null) {
                            return ValidationConstants.CODIGO_INGRESO;
                        }                        
                        
                        if(concepto.IVARepercutido) {
                            // Se trata de un concepto de IVA repercutido asociado a otro concepto 
                            if(concepto.baseImponible==0) {
                                return ValidationConstants.BASE_IMPONIBLE;
                            }
                            if(concepto.tipoIVA==0) {
                                return ValidationConstants.TIPO_IVA;
                            }
                            if(concepto.importeIVA==0) {
                                return ValidationConstants.IMPORTE_IVA;
                            }                            
                            
                            BackendData conceptoIVA = (BackendData)concepto.backendDataMap.get(BackendDataConstants.CONCEPTO_IVA);
                            if (conceptoIVA==null || conceptoIVA.value==null) {
                                return ValidationConstants.CONCEPTO_IVA;
                            }
                            BackendData territorioIVA = (BackendData)concepto.backendDataMap.get(BackendDataConstants.TERRITORIO_IVA);
                            if (territorioIVA==null || territorioIVA.value==null) {
                                return ValidationConstants.TERRITORIO_IVA;
                            }
                        } else {
                            
                        	if (concepto.noPresupuestario!=null && !concepto.noPresupuestario.booleanValue()) { // si el concepto es presupuestario validamos aplicacin presupuestaria
                        		BackendData aplicacionPresupuestaria = (BackendData)concepto.backendDataMap.get(BackendDataConstants.APLICACION_PRESUPUESTARIA);
                        		if (aplicacionPresupuestaria==null || aplicacionPresupuestaria.value==null) {
                        			return ValidationConstants.APLICACION_PRESUPUESTARIA;
                        		}
                        	} // si el concepto es NO presupuestario no llegar cdigo de aplicacin presupuestaria
                            
                            if(concepto.tieneIVARepercutido) {
                                // Se trata de un pago con IVA repercutido 
                                // Comprobar que tiene un concepto de IVA repercutido asociado
                                if(conceptos.size()>i+1) {
                                    ConceptoPeticion conceptoIVARepercutido = (ConceptoPeticion)conceptos.get(i+1);
                                    if(conceptoIVARepercutido==null) {
                                        // No existen mas conceptos
                                        return ValidationConstants.CONCEPTO_IVA_REPERCUTIDO;
                                    }
                                    if(!conceptoIVARepercutido.IVARepercutido) {
                                        // El siguiente concepto no es de IVA repercutido
                                        return ValidationConstants.CONCEPTO_IVA_REPERCUTIDO;
                                    }
                                } else {
                                    return ValidationConstants.CONCEPTO_IVA_REPERCUTIDO;
                                }                            
                            } else {
                                
                                // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                                // Se retira esta validacion por incompatibilidad con las aplicaciones antiguas
                                // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                                
                                /*if(concepto.baseImponible>0 ||
                                        concepto.tipoIVA>0 ||
                                        concepto.importeIVA>0) {
                                    // Se trata de un pago con IVA no repercutido
                                    if(concepto.baseImponible==0) {
                                        return ValidationConstants.BASE_IMPONIBLE;
                                    }
                                    if(concepto.tipoIVA==0) {
                                        return ValidationConstants.TIPO_IVA;
                                    }
                                    if(concepto.importeIVA==0) {
                                        return ValidationConstants.IMPORTE_IVA;
                                    }                                        
                                }else {
                                    // Se trata de un pago sin IVA
                                    // No hay comprobaciones adicionales....
                                }*/
                            }
                        }
                    }
                } else {
                	
                	 // Cambio 07/07/08.
                	 // Si Backend informarSipca es true y  los conpeptos son nulos, se
                	//  cogen de la configuracin del sufijo.
                	  return null;
                	//return ValidationConstants.CONCEPTOS_NULL;
                }
            }
        }
        
        return null;
    }
}
