package p12f.exe.pasarelapagos.paymentrequest;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

import p12f.exe.pasarelapagos.exceptions.PaymentRequestException;
import p12f.exe.pasarelapagos.objects.DatosPago;
import p12f.exe.pasarelapagos.objects.DatosResolucion;
import p12f.exe.pasarelapagos.objects.Expediente;
import p12f.exe.pasarelapagos.objects.PeriodoPago;
import p12f.exe.pasarelapagos.utils.CSBConstants;
import p12f.exe.pasarelapagos.utils.CSBHelpper;
import p12f.exe.pasarelapagos.utils.exceptions.CSBFormatException;

import com.ejie.r01f.util.DateUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

public class ResolutionRequestData implements Serializable {

	public final static String MOTIVO_DEVOLUCION_ESTIMACION_RECURSO = "1";
	public final static String MOTIVO_DEVOLUCION_COBRO_DUPLICADO = "2";
	public final static String MOTIVO_DEVOLUCION_COBRO_ERRONEO = "3";
	public final static String MOTIVO_DEVOLUCION_OTRAS_CAUSAS = "10";
	
	public String pagoID;
	
	public DatosPago datosPago;
	
	public DatosResolucion datosResolucion;

	public Backend backend; // Datos de backend asociados a la devolucin
	
	public List conceptos; // Lista de Concpetos Resolucion
	
	public Aplicacion aplicacion;
	
	public Expediente expediente;
	
	public Validacion validacion = null;
	
	/**
	 * Convierte de un String xml a un Objeto ResolutionRequestData.
	 * @param xml : XML de objeto.
	 * @return Objeto convertido.
	 * @throws XOMarshallerException
	 */
	public static ResolutionRequestData getObject( String xml) throws XOMarshallerException {
		return (ResolutionRequestData) XOManager.getObject(XMLProperties.get("p12f","objectMapPath"),xml);
	}
	
	/**
	 * Convierte un Objeto ResolutionRequestData a XML.
	 * @return
	 * @throws XOMarshallerException
	 */
	public String toXML() throws XOMarshallerException {
		return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
	}
	
	
	/**
	 * Mtodo que realiza la validacin de los datos de este objeto.
	 */
	public void validate() {
		String errorMsg = null;
		if(this.validacion == null){
			try {			
				// Validacin de sufijo-referencia-identificador
				Map paymentDataMap =  CSBHelpper.decode507(this.pagoID);
				String sufijo = (String) paymentDataMap.get(CSBConstants.CSB507.SUFIJO);
				String referencia =  (String) paymentDataMap.get(CSBConstants.CSB507.REFERENCIA);
				String identificacion  = (String) paymentDataMap.get(CSBConstants.CSB507.IDENTIFICACION);
				if (referencia.length()!=13) {
					throw new PaymentRequestException ("La referencia debe ser de longitud 13 y ese de longitud "+referencia.length());
				}
				
				if (this.backend == null) {
					throw new PaymentRequestException ("No hay datos de backend");
				}
				// ejercicio contable
				if (this.backend.get(BackendDataConstants.EJERCICIO_CONTABLE)==null) {
					throw new PaymentRequestException ("No hay datos de backend del EJERCICIO CONTABLE");
				}
				// validacin de que la fecha limite de pago est bien formada
				PeriodoPago periodoPago = (PeriodoPago)this.datosPago.periodosPago.values().iterator().next();
				//La identificacion es la fecha lmite de pago solo si el sufijo es mayor de 500 
				if ( Integer.parseInt(sufijo) > 500) {
					DateUtils.getDateFromFormatedString(identificacion,"ddMMyy");
				} else if (periodoPago.fechaFin != null) {
					DateUtils.getDateFromFormatedString(periodoPago.fechaFin,"dd/MM/yyyy");
				}
				// validacin de que el importe est bien formado
				Long.toString(periodoPago.importe);
				
				// motivo devolucin
				if(datosResolucion == null || datosResolucion.motivo == null || "".equals(datosResolucion.motivo)){
					throw new PaymentRequestException ("No hay MOTIVO DEVOLUCION");
				}
				/*if (this.backend.get(BackendDataConstants.MOTIVO_DEVOLUCION)==null) {
					throw new PaymentRequestException ("No hay datos de backend del MOTIVO DEVOLUCION");
				}*/
				
				// TODO realizar validaciones del tercero
				
				// TODO comprobar si es necesario realizar ms validaciones
				
				
				
			} catch (PaymentRequestException e) {
				errorMsg = e.getRawMessage();
			} catch (CSBFormatException e) {
				errorMsg = e.getRawMessage();
			} catch (Exception e) {
				e.printStackTrace();
				try {
					errorMsg = " Se ha producido algn tipo de excepcin no recogida en la validacin: "+ e.getMessage() +
							   " de la siguiente peticin de Resolucin:\n" + this.toXML();
				} catch (XOMarshallerException e1) {			
					e1.printStackTrace();
				}
			}
			Validacion validacion = new Validacion();
			if (errorMsg == null) {
				validacion.valido = true;
			} else {
				validacion.valido = false;
				validacion.mensaje = errorMsg;
			}
			this.validacion = validacion;
		}
	}
	
}