package p12f.exe.pasarelapagos.security;


import java.util.List;

import javax.servlet.http.*;



import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Clase a utilizar cuando NO est disponible el N38API,
 * por ejemplo cuando se desarrolle en local.     
 */   
public class SecurityHelperDefault implements ISecurityInfo {
    private HttpServletRequest request;

    public  SecurityHelperDefault() {}	
    
    public String[] getPerfiles () {        
        List listaPerfiles = XMLProperties.getPropertyList("p12i","security/test/usr/perfil");
        String[] perfiles = new String[listaPerfiles.size()];
        for(int i=0; i<listaPerfiles.size(); i++){
            perfiles[i] = (String)listaPerfiles.get(i);
        }
        return perfiles;
    } 

    public boolean hasPermission(String uidFuncion) {
        
        // Perfil Asociado a la funcion a la que se quiere saber si se tiene acceso.
        String perfil = XMLProperties.getProperty("p12i","security/test/functions/function[@id='"+uidFuncion+"']/perfil");
        if (perfil == null )
            return false;
        String[] perfilesUsuario = this.getPerfiles();
        for(int i=0; i<perfilesUsuario.length; i++){         
            String perfilUsuario = (String)perfilesUsuario[i];
            if(perfil.equals(perfilUsuario))
                return true;
        }
        return false;
    }	
    
    public boolean init( HttpServletRequest request, HttpServletResponse response){
        this.request = request;
        return true;
    }	
    
    /**
     * Crea un objecto de seguridad para una funcion dada.
     * Si devuelve Null es que no se tiene permiso
     * para esa Funcion o recurso de LDAP :
     * 
     * <security>
		<provider>p12f.exe.pasarelapagos.security.SecurityHelperDefault</provider>
		<functions>
			<pagoVentanilla>
				<uid>P12I-FN-0001</uid>
			</pagoVentanilla>
		</functions>
		<test>
			<usr>
				<uid>P12I-1</uid>
				<perfil>P12I-PF-0001</perfil>
				<perfil>P12I-PF-0003</perfil>
			</usr>
			<function>
				<uid>P12I-FN-0001</uid>
				<perfil>P12I-PF-0001</perfil>
			</function>
		</test>
	</security>
	 *
     * 
     */  			
    public SecurityObject getSecurityObject(String uidFuncion) throws SecurityHelperException {    	
        SecurityObject objetoSeguridad = null;
        if (this.hasPermission(uidFuncion)) {	         
            objetoSeguridad = new SecurityObject(XMLProperties.getProperty("p12i","security/test/usr/uid"));
            objetoSeguridad.setPerfiles(this.getPerfiles());
        }	
        return objetoSeguridad;   	
    }
    
    /**
     * Redirige a la Pagina de Login de XLNETS.
     */
    public void redirectToLogin(String finalUri)throws SecurityHelperException {
        throw new SecurityHelperException("NOT IMPLEMENTED EXCEPTION : En SecurityHelperDefault NUNCA DEBE REDIRIGIR A LOGIN");
    }
    
    
}	
