package p12f.exe.pasarelapagos.security;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import n38c.exe.N38API;
import n38i.exe.N38Excepcion;
import n38i.exe.N38ParameterException;

import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

import com.ejie.r01f.log.R01FLog;

/**
 * Clase auxiliar para tratar con LDAP
 */
public class SecurityHelperXLNets implements ISecurityInfo {
	
    private HttpServletRequest request;
    private HttpServletResponse response;
    private N38API n38API;
    
    public  SecurityHelperXLNets()  {
    }	
    
    /* Asociar la request de la sesin N38 */ 
    public boolean init(HttpServletRequest request,HttpServletResponse response) throws SecurityHelperException{
        try {
            R01FLog.to("p12f.security").info("\n\n>>>>>>>>>>>> INICIALIZANDO OBJETO SEGURIDAD XLNETS : p12f.exe.pasarelapagos.security.SecurityHelperXLNets VZ-4X"); 
            this.request=request;
            this.response=response;
            n38API =  new N38API (request);
            R01FLog.to("p12f.security").info("\n\n\n>>>>SESSION:" + XLNetsPrintWritter.print(n38API.n38ItemSesion())); 
            return this.isValidSession();         
        } catch( Exception e ) {   
            R01FLog.to("p12f.security").info(">>Excepcion capturada en Initialize de SecurityHelper:" + e);
            return false;
        }	
    }

    /* Obtener los perfiles asociados al Usuario que realiza Login.*/
    public String[] getPerfiles () throws SecurityHelperException {
        try {
            String[] perfiles = this.n38API.n38ItemSesion("n38perfiles");
            return perfiles; 
        } catch (Throwable e) {
            throw new SecurityHelperException("SecurityHelpper:getPerfiles");
        }	   
    } 
    
    /* Obtener la Persona asociada al Usuario que realiza Login.*/
    public String getPersonaNombre () throws SecurityHelperException {
        NodeList lista;
        try {
            String uidPersona = this.getPersonaUID();
            Document personaDocument = this.n38API.n38ItemObtenerPersonas("n38PuestoUID="+uidPersona);
            R01FLog.to("p12f.security").info(XLNetsPrintWritter.print(personaDocument)); 
            String filtro = "//elemento[@subtipo=\"n38persona\"]/parametro[@id=\"cn\"]/valor/text()";
            //El cn contiene el nombre y apellidos de la persona.
            String nombre = XPathAPI.selectNodeList(personaDocument,filtro).item(0).getNodeValue();
            return nombre;
        } catch (Exception e) {
            throw new SecurityHelperException("SecurityHelpper:getPersonaNombre");
        }	   
    } 
   
    /* Obtener el uid del usuario que realiza Login.*/
    public String getPersonaUID () throws SecurityHelperException {
        try {
            return n38API.n38ItemSesion("n38puestouid")[0];
        } catch (Exception e) {
            throw new SecurityHelperException("SecurityHelpper:getPersonaUID");
        }	   
    } 
        /* Obtener el uid del usuario que realiza Login.*/
    public String getPersonaDNI () throws SecurityHelperException {
        try {
            return n38API.n38ItemSesion("dni")[0];
        } catch (Exception e) {
            throw new SecurityHelperException("SecurityHelpper:getPersonaDNI");
        }	   
    } 
   
    /* Comprobar si el usuario tiene permiso para la funcin dada */ 
    public boolean hasPermission(String uidFuncion) throws SecurityHelperException {
        NodeList lista;
        try {
            R01FLog.to("p12f.security").info("SecurityHelpper :: Obtener el item de seguridad para la funcion: " + uidFuncion);
            Document itemSeguridad =  n38API.n38ItemSeguridad(uidFuncion);
            R01FLog.to("p12f.security").info(XLNetsPrintWritter.print(itemSeguridad)); 
            R01FLog.to("p12f.security").info(" SecurityHelpper ::  Obtener la lista de permisos.");
            lista = XPathAPI.selectNodeList(itemSeguridad,"//elementos");
        } catch (Exception e) {
            R01FLog.to("p12f.security").info("SecurityHelpper ::  Se ha producido una excepcion al tratar de comprobar los permisos");
            throw new SecurityHelperException("SecurityHelpperXLNets:hasPermision:" +e);	  
        }	  
        R01FLog.to("p12f.security").info("Longitud de la lista de permisos: " + lista.getLength());
        return ( lista.getLength() != 0 );
    }	
      
    /**
     * Se crea un objecto de seguridad para una funcinn dada.
     * Si devuelve Null es que no se tiene permiso para esa Funcin o recurso de LDAP.
     */
    public SecurityObject getSecurityObject(String uidFuncion)throws SecurityHelperException {
        SecurityObject objetoSeguridad = null;            
        try {	     
            
            //System.out.println("\n\n\n>>>>SESSION:" + XLNetsPrintWritter.print(n38API.n38ItemSesion()));
            R01FLog.to("p12f.security").info("\n\n\n>>>>SecurityHelpper ::SESSION:" + XLNetsPrintWritter.print(n38API.n38ItemSesion())); 
            
            //System.out.println(">SESSION:" + XLNetsPrintWritter.print(n38API.n38ItemSesion()));
            if (this.hasPermission(uidFuncion)) {	
                R01FLog.to("p12f.security").info(">>>>>>>>>>>> SecurityHelpper :: Obteniendo objeto de seguridad para " + n38API.n38ItemSesion("n38puestouid")[0] );
                objetoSeguridad = new SecurityObject(n38API.n38ItemSesion("n38puestouid")[0]);
                objetoSeguridad.setPerfiles(this.getPerfiles());
                objetoSeguridad.setPersona(this.getPersonaNombre());
                objetoSeguridad.dni=this.getPersonaDNI();                
            } else  
                R01FLog.to("p12f.security").info(" SecurityHelpper :: El usuario no tiene permiso para esta funcin.");
        } catch (Exception e) {
            R01FLog.to("p12f.security").info( " SecurityHelpper :: Se ha producido una excepcion al tratar de obtener el SecurityObject");
            throw new SecurityException(" SecurityHelpper ::  SecurityHelpperException:getSecurityObject:" + e);
        }	
        return objetoSeguridad; 
    }	
    
    /**
     * Redirige a Login de XLNETS.
     * @param finalURI : Url final a donde redirecciona XLNETS.
     * @param response
     * @throws IOException
     */
    public void redirectToLogin(String finalURI) {    
      
        R01FLog.to("p12f.security").info(">>>>>>>>>>>> Redirigiendo a :" + finalURI);
        try {
            response.sendRedirect(finalURI);
        } catch (IOException e) {
            throw new SecurityException("SecurityHelpperException:redirectToLogin:" + e);
        }
    }
    
    
    /*
     * Es valida la Sesion.
     */
    public boolean isValidSession(){       
        String uidSession = null;
        try {
            uidSession = n38API.n38ItemSesion("n38uidsesion")[0];
            return ( uidSession != null) &&  ( !uidSession.equalsIgnoreCase(""));
        } catch (N38ParameterException e) {
            throw new SecurityException("SecurityHelpperException:isValidSession:" + e);
        } catch (N38Excepcion e) {
            throw new SecurityException("SecurityHelpperException:isValidSession:" + e);
        }
        
    }
    
    
    
}	
