package p12f.exe.pasarelapagos.security.app.manager.custom;

import java.util.Date;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import org.apache.commons.codec.binary.Hex;

import p12f.exe.pasarelapagos.objects.security.SecurityContext;
import p12f.exe.pasarelapagos.security.app.custom.DefaultXMLToken;
import p12f.exe.pasarelapagos.security.app.manager.SecurityManagerInterface;

import com.ejie.r01f.xml.marshalling.XOMarshallerException;

import com.ejie.r01f.xmlproperties.XMLProperties;

/****
 * Clase que implementa una validacin simple de los contextos enviados desde Aplicaciones Clientes contra Aplicaciones Servers.
 * 
 * 1.La clase cliente envia un Contexto con un token generado (HMAC) a travs de una trama de datos coontra 
 *   una clave asignada por un paquete servidor (pe:z99g) a un paquete cliente (p.e: z99i)
 * 2.La clase cliente envia en la llamdas a los servicios del servidor el contexto generado.
 * 3.La clase servidora regenera el HMAC a travs de los datos-trama enviados desde el cliente.
 * 4. Si el HMAC enviado desde Cliente coincide con el HMAC generado en el Servidor (adems de comprobarse que no ha caducado),
 *    se considera contexto vlido con permiso.
 *   Definicin de un APP Client en el Server: 
 *   A travs de del HashMAC enviado desde una app cliente se calcula en el servidor. Si coinciden tiene permiso.
 *   <defaultSecurityManagerDefinition>
	 *    <app id =''>
	 *        <appClientKey></appClientKey>
	 *        <expirationMillis></expirationMillis>
	 *        
	 *    </app>
 *   </defaultSecurityManagerDefinition>

 */
public class DefaultSecurityManager  implements SecurityManagerInterface {
	
	
	String _appServerCode;
	
	public DefaultSecurityManager(String appCode) {
		
		this._appServerCode = appCode;
	}
	
	/**
	 * Determina si se tiene permiso a partir de una deternminado contextoo.
	 * @param ctx
	 * @throws T08FSecurityException
	 */
	public boolean  hasPermission(SecurityContext ctx) throws SecurityException {
		
		try {
		
			String appClientID = ctx.getAppClientID();
			String service = ctx.getServiceName();
			String appClientKey =  XMLProperties.get(_appServerCode,"defaultSecurityManagerDefinition/services/service[@id='"+ service + "']/app[@id='"+ appClientID + "']/appClientKey");
			String expirationMillis =  XMLProperties.get(_appServerCode,"defaultSecurityManagerDefinition/services/service[@id='"+ service + "']/app[@id='"+ appClientID + "']/expirationMillis");
			
	
			
			if (appClientKey == null ){
			   
				String message  = "\n\n";
				message += "[ATENCIN ] No est definido en la parte servidora el defaultSecurityManagerDefinition:" + "\n";;
				message +=       "<defaultSecurityManagerDefinition>" + "\n";
				message +=       "<services>" + "\n";
				message +=       "<services id='"+ service + "'>" + "\n";

				message +=       "      <app id ='"+  appClientID + "'>"  + "\n";
				message +=       "      	<appClientKey>????????????</appClientKey>"  + "\n";
				message +=       "      	<expirationMillis>5000</expirationMillis>"  + "\n";
				message +=       "      </app>"  + "\n";
				message +=       "</defaultSecurityManagerDefinition>" + "\n";
				
				throw new SecurityException(message);
			}
			
			
			long  howMillisLongWasThisCtx = _howMillisLongWasThisCTX(new Date(Long.parseLong(ctx.getTimestamp())));
			
			if (expirationMillis != null )
				if (howMillisLongWasThisCtx > Long.parseLong(expirationMillis)){
					throw new SecurityException("SecurityContext No Vlido : Caducado :" + ( howMillisLongWasThisCtx / 1000 ) +" segundos " );			
				}
				
			//En el Servidor se vuelve a cacular el token en base a unos datos de entrada  y la clave asignada a la aplicacin cliente. 
			DefaultXMLToken  serverGeneratedToken =  _generateTokenData (appClientID,appClientKey,ctx.getAppClientSessionID(),ctx.getTimestamp());			
			DefaultXMLToken  clientToken =  DefaultXMLToken.getObjectFromBase64(ctx.getTokenData().getContent());
			
			if(serverGeneratedToken.getValue().equalsIgnoreCase(clientToken.getValue())) return true;
			
	
		} catch (Exception ex) {	
			throw new SecurityException(ex.getMessage());			
		}catch (Throwable ex) {		
			throw new SecurityException(ex.getMessage());
		}
		
		return false;
	}

	
	
	/**
	 * Genera un Objeto Token a travs de una trama de datos
	 * @param appClientID
	 * @param appClientKey
	 * @param appClientSessionID
	 * @param timestamp
	 * @return
	 * @throws Exception
	 */
	public static DefaultXMLToken _generateTokenData(String appClientID,  String appClientKey,String appClientSessionID, String timestamp  ) throws Exception {
		
			String token_data = appClientID + appClientSessionID + timestamp;			 		
			String hmac = _HMAC_MD5_encode(appClientKey, token_data);		
			DefaultXMLToken defaultToken = new DefaultXMLToken();
			defaultToken.setValue(hmac);
            return defaultToken;
	}
	
	/**
	 * Obtiene el HMAC de una trama de datos bandonos en un clave privada, la trama misma y el algoritmo MD5.
	 * @param appClientID
	 * @param appClientKey
	 * @param appClientSessionID
	 * @param timestamp
	 * @return
	 * @throws Exception
	 */
    public static String _HMAC_MD5_encode(String key, String message) throws Exception {
        SecretKeySpec keySpec = new SecretKeySpec( key.getBytes(), "HmacMD5");

        Mac mac = Mac.getInstance("HmacMD5");
        mac.init(keySpec);
        byte[] rawHmac = mac.doFinal(message.getBytes()); 
        return new String( Hex.encodeHex(rawHmac));
    }

	
	
    /**
	  * Devuelve cuantos milisegundos han pasado desde que el token se gener.
	  */
	 public static long _howMillisLongWasThisCTX (Date  eventDate){
		
		 long today = System.currentTimeMillis();
		 return (today -  eventDate.getTime()) ;
	 }
	
    
	
	/*************************************************
	 * Test
	 * @throws MarshallerException 
	 ********************************************************/
	public static void main(String[] argv) throws XOMarshallerException  {
		
		String  strCTX = "<securityContext type='custom' appClientSessionID='z99icxb89386498177b3cd5712a5d5854d5bde' appClientID='z99i' serviceName='configService'><tokenData><content><![CDATA[PGRlZmF1bHRUb2tlbj48dmFsdWU+NWVmZmNhOTFkMDQ3MTE5NWQ0YzZjZTVlODE2NDYwNjY8L3ZhbHVlPjwvZGVmYXVsdFRva2VuPg==]]></content></tokenData><timestamp>1355216872819</timestamp></securityContext>";
			
		SecurityContext ctx  = SecurityContext.getObject(strCTX);
		DefaultSecurityManager mng = new DefaultSecurityManager("t08g");
		System.out.println(mng.hasPermission(ctx));

	
		
	}
	
	
	
}
