package p12f.exe.pasarelapagos.security.crypto;
import java.io.UnsupportedEncodingException;
import java.security.NoSuchProviderException;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.KeySpec;

import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;


/**
 * Clase de ayuda para la encriptacin/desencriptacin simtrica.
 * @author ie00191m
 *
 */
public class CryptoHelper {
    
    
    public static final String encoding = "UTF-8";
    
    
    private Cipher ecipher; // encriptador
    private Cipher dcipher; // desencriptador.

    // Saltos.
    byte[] salt = {
        (byte)0xA9, (byte)0x9B, (byte)0xC8, (byte)0x32,
        (byte)0x56, (byte)0x35, (byte)0xE3, (byte)0x03
    };

    // Iterations.
    int iterationCount = 19;    
    
    
    /**
     * Devuelve una instancia de CryptoHelper (escriptador/desencriptador) para una pwd concreta.
     * @return
     */
    public static CryptoHelper getInstance(String pass){        
        return new CryptoHelper(pass);
     }
    
    
    /**
     * COnstructor Privado.
     * @param passPhrase
     * @throws CryptoHelperException
     */
    private CryptoHelper(String passPhrase) {
       
        try {
            
            // Se crea la clave : PBE, Password Based Encription.
            
            KeySpec keySpec = new PBEKeySpec(passPhrase.toCharArray(), salt, iterationCount);
            
            // PBEWith<digest>And<encryption> or PBEWith<prf>And<encryption>:
            
            SecretKey key = SecretKeyFactory.getInstance("PbeWithMD5AndDES").generateSecret(keySpec);            
            //SecretKey key = SecretKeyFactory.getInstance("PBEWithHmacSHA1AndDESede").generateSecret(keySpec);
           
            ecipher = Cipher.getInstance(key.getAlgorithm(), "SunJCE");
            dcipher = Cipher.getInstance(key.getAlgorithm(), "SunJCE");

            // Prepare the parameter to the ciphers
            AlgorithmParameterSpec paramSpec = new PBEParameterSpec(salt, iterationCount);

            // Instancias de Cipher, una para encriptar, otra para dessencriptar.
            ecipher.init(Cipher.ENCRYPT_MODE, key, paramSpec);
            dcipher.init(Cipher.DECRYPT_MODE, key, paramSpec);
            
        } catch (java.security.InvalidAlgorithmParameterException e) {
            e.printStackTrace();
        } catch (java.security.spec.InvalidKeySpecException e) {
            e.printStackTrace();
        } catch (javax.crypto.NoSuchPaddingException e) {
            e.printStackTrace();
        } catch (java.security.NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (java.security.InvalidKeyException e) {
            e.printStackTrace();
        } catch (NoSuchProviderException e){
        	e.printStackTrace();
        }
    }

    /**
     * Encripta unca cadena.
     * @param str
     * @return
     * @throws CryptoHelperException 
     */
    public String encrypt(String str) throws CryptoHelperException {
        try {
            // Encodear segn utf-8
            byte[] utf8 = str.getBytes("UTF-8");
            // Encrypt
            byte[] enc = ecipher.doFinal(utf8);
            // Se devuelve en Base64.
            return new sun.misc.BASE64Encoder().encode(enc);
        } catch (javax.crypto.BadPaddingException e) {
            throw new CryptoHelperException(e);
        } catch (IllegalBlockSizeException e) {
            throw new CryptoHelperException(e);
        } catch (UnsupportedEncodingException e) {
            throw new CryptoHelperException(e);
        } 
    }

    /**
     * Desencripta una cadena.
     * @param str
     * @return
     * @throws CryptoHelperException 
     */
    public String decrypt(String str) throws CryptoHelperException {
        try {
          
            byte[] dec = new sun.misc.BASE64Decoder().decodeBuffer(str);
            byte[] utf8 = dcipher.doFinal(dec);
            return new String(utf8, "UTF-8");
        } catch (javax.crypto.BadPaddingException e) {
            throw new CryptoHelperException(e);
        } catch (IllegalBlockSizeException e) {
            throw new CryptoHelperException(e);
        } catch (UnsupportedEncodingException e) {
            throw new CryptoHelperException(e);
        } catch (java.io.IOException e) {
            throw new CryptoHelperException(e);
        }
        
    }

    
    /**
     * Guarda una instancia de CryptoHelper.
     * @author ie00191m
     */
    public static final class CryptoHelperHolder {        
        static final CryptoHelper  cryptoHelperInstance = new CryptoHelper("balalallala");        
        
    }
    
    /**
     * Excepcion.
     * @author ie00191m
     */
    public class CryptoHelperException  extends Exception {
        
        public CryptoHelperException(){        
            super();
        }
        
       public CryptoHelperException(Exception kieX){        
            super(kieX);
        }
        
    }


}







