package p12f.exe.pasarelapagos.security.crypto;

import java.io.UnsupportedEncodingException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import org.bouncycastle.util.encoders.Hex;

import com.ejie.r01f.util.StringUtils;

/**
 * Clase de ayuda para el cifrado/descifrado simtrico TRIPL DES EBCDIC
 * @author ie00191m
 */
public class TripleDESEBCDICCryptoHelper {
	
	public static final String CIPHER_3DES_CBC_PKCS5Padding = "DESede/CBC/PKCS5Padding";
	public static final String CIPHER_3DES_CBC_NOPADDING = "DESede/CBC/NoPadding";
	public static final String CIPHER_3DES_CBC_SpacePADDING = "DESede/CBC/SpacePadding"; // no existe, es para indicar que se haga padding con espacios y luego se use el NoPadding
	
	public static final String CIPHER_3DES_ECB_PKCS5Padding = "DESede/ECB/PKCS5Padding";
	public static final String CIPHER_3DES_ECB_NOPADDING  = "DESede/ECB/NoPadding";
	
	public static final String ENC_TYPE_HEX = "hex";
	public static final String ENC_TYPE_BASE64 = "base64";
	public static final String ENC_TYPE_NONE= "none";
	
	public static final String encoding = "CP500";
	
	private static final String ALGORITHM_3DES = "DESede";
	
	private static final byte[] ivBytes ={0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,0x00};
	
	/**
	 * Constructor TripleDESEBCDICCryptoHelper
	 */
	public TripleDESEBCDICCryptoHelper() {
		
	}
	
	/**
	 * Cifra la cadena dada str y la devuelve en el encType indicado, en base a unas claves y un modo de cifrado.
	 * @param clave1
	 * @param clave2
	 * @param encType Tipo de cifrado (base64, hexadecimal, ninguno)
	 * @param modeCipherPadding
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public String encrypt(String clave1, String clave2, String encType, String modeCipherPadding, String str) throws CryptoHelperException {
		if (ENC_TYPE_NONE.equals(encType)){
			return str;
		}
		try {
			String passPhrase = new String(Hex.encode(doXOR(clave1, clave2))).toUpperCase();
			if (ENC_TYPE_BASE64.equals(encType)){
				return encrypt(passPhrase, modeCipherPadding, str);
			} else if (ENC_TYPE_HEX.equals(encType)){
				return encryptHEX(passPhrase, modeCipherPadding, str);
			}
		} catch (Exception e) {
			throw new CryptoHelperException(e);
		}
		return null;
	}
	
	/**
	 * Descifra la cadena dada str en el encType indicado, en base a unas claves y un modo de cifrado.
	 * @param clave1
	 * @param clave2
	 * @param encType
	 * @param modeCipherPadding
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public String decrypt(String clave1, String clave2, String encType, String modeCipherPadding, String str) throws CryptoHelperException {
		if (ENC_TYPE_NONE.equals(encType)){
			return str;
		}
		try {
			String passPhrase = new String(Hex.encode(doXOR(clave1, clave2))).toUpperCase();
			if (ENC_TYPE_BASE64.equals(encType)){
				return decrypt(passPhrase, modeCipherPadding, str);
			} else if (ENC_TYPE_HEX.equals(encType)){
				return decryptHEX(passPhrase, modeCipherPadding, str);
			}
		} catch (Exception e) {
			throw new CryptoHelperException(e);
		}
		return null;
	}
	
	/**
	 * Cifra la cadena dada str y la devuelve en Base64, en base a un password y un modo de cifrado.
	 * @param passPhrase
	 * @param modeCipherPadding CIPHER_3DES_CBC_NOPADDING
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public String encrypt(String passPhrase, String str) throws CryptoHelperException {
		
		return  encrypt(passPhrase,CIPHER_3DES_CBC_NOPADDING,str);
	}
	
	/**
	 * Cifra la cadena dada str y la devuelve en Base64, en base a un password y un modo de cifrado.
	 * @param passPhrase
	 * @param modeCipherPadding
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public String encrypt(String passPhrase, String modeCipherPadding, String str) throws CryptoHelperException {
		try {
			String strToCipher = str;
			byte [] seed_key = Hex.decode(passPhrase);
			SecretKeySpec keySpec = new SecretKeySpec(seed_key,ALGORITHM_3DES);
			if (CIPHER_3DES_CBC_SpacePADDING.equals(modeCipherPadding)) {
				if (str.length()%8!=0){
					int modulo = str.length()%8;
					strToCipher=StringUtils.rPadWithChar(str,' ',str.length()+8-modulo);
				}
				modeCipherPadding = CIPHER_3DES_CBC_NOPADDING;
			}
			Cipher ecipher = Cipher.getInstance(modeCipherPadding);
			IvParameterSpec iv = new IvParameterSpec(ivBytes);
			ecipher.init(Cipher.ENCRYPT_MODE, keySpec, iv);
			
			// Encodear segn EBCDIC
			byte[] toEBCDIC = strToCipher.getBytes(TripleDESEBCDICCryptoHelper.encoding);
			// Encrypt
			byte[] enc = ecipher.doFinal(toEBCDIC);
			return new sun.misc.BASE64Encoder().encode(enc);
			
		} catch (javax.crypto.BadPaddingException e) {
			throw new CryptoHelperException(e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptoHelperException(e);
		} catch (UnsupportedEncodingException e) {
			throw new CryptoHelperException(e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptoHelperException(e);
		} catch (NoSuchPaddingException e) {
			throw new CryptoHelperException(e);
		} catch (InvalidKeyException e) {
			throw new CryptoHelperException(e);
		} catch (InvalidAlgorithmParameterException e) {
			throw new CryptoHelperException(e);
		}
	}
	
	/**
	 * Cifra la cadena str en TripleDES a partir del password passPhrase y el modo de cifrado : CIPHER_3DES_CBC_NOPADDING
	 * @param passPhrase
	 * @param modeCipherPadding
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public String encryptHEX (String passPhrase, String str) throws CryptoHelperException {
		return encryptHEX (passPhrase,CIPHER_3DES_CBC_NOPADDING, str); 
	}
	
	/**
	 * Cifra la cadena str en TripleDES a partir del password passPhrase y el modo de cifrado.
	 * @param passPhrase
	 * @param modeCipherPadding
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public String encryptHEX (String passPhrase,String modeCipherPadding, String str) throws CryptoHelperException{   			
		try {
			String strToCipher = str;
			byte [] seed_key =  Hex.decode(passPhrase);
			SecretKeySpec keySpec = new SecretKeySpec(seed_key,ALGORITHM_3DES);
			
			if (CIPHER_3DES_CBC_SpacePADDING.equals(modeCipherPadding)) {
				if (str.length()%8!=0){
					int modulo = str.length()%8;
					strToCipher=StringUtils.rPadWithChar(str,' ',str.length()+8-modulo);
				}
				modeCipherPadding = CIPHER_3DES_CBC_NOPADDING;
			}
			Cipher ecipher = Cipher.getInstance(modeCipherPadding);
			IvParameterSpec iv = new IvParameterSpec(ivBytes);
			ecipher.init(Cipher.ENCRYPT_MODE, keySpec, iv);

			byte[] toEBCDIC = strToCipher.getBytes(TripleDESEBCDICCryptoHelper.encoding);
			byte[] enc = ecipher.doFinal(toEBCDIC);
			return new String(Hex.encode(enc)).toUpperCase();
		} catch (Exception ex ){
			ex.printStackTrace();
			throw new CryptoHelperException (ex);
		}
	}
	
	/**
	 * Descifra una cadena en hexadecimal en base a un passWord y un modo de cifrado por defecto : CIPHER_3DES_CBC_NOPADDING
	 * @param passPhrase
	 * @param modeCipherPadding CIPHER_3DES_CBC_NOPADDING
	 * @param encryptedChain
	 * @return
	 * @throws CryptoHelperException
	 */
	public String decryptHEX(String passPhrase, String encryptedChain) throws CryptoHelperException {
		return decryptHEX (passPhrase,CIPHER_3DES_CBC_NOPADDING,encryptedChain);
	}
	
	/**
	 * Descifra una cadena en hexadecimal en base a un passWord y un modo de cifrado.
	 * @param passPhrase
	 * @param modeCipherPadding
	 * @param encryptedChain
	 * @return
	 * @throws CryptoHelperException
	 */
	public String decryptHEX(String passPhrase,String modeCipherPadding, String encryptedChain) throws CryptoHelperException {
		try {
			
			byte [] seed_key =  Hex.decode(passPhrase);
			SecretKeySpec keySpec = new SecretKeySpec(seed_key,ALGORITHM_3DES);
			
			boolean removeLastWithespaces = false;
			if (CIPHER_3DES_CBC_SpacePADDING.equals(modeCipherPadding)) {
				modeCipherPadding = CIPHER_3DES_CBC_NOPADDING;
				removeLastWithespaces = true;
			}
			Cipher dcipher =  Cipher.getInstance(modeCipherPadding);

			IvParameterSpec iv = new IvParameterSpec(ivBytes);
			dcipher.init(Cipher.DECRYPT_MODE, keySpec,iv);
			
			byte[] encData = Hex.decode(encryptedChain);
			String decodedStr = new String( dcipher.doFinal(encData),TripleDESEBCDICCryptoHelper.encoding);
			
			if (removeLastWithespaces) {
				return decodedStr.replaceAll("\\s+$", "");
			}
			return decodedStr;

		} catch (javax.crypto.BadPaddingException e) {
			throw new CryptoHelperException(e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptoHelperException(e);
		} catch (UnsupportedEncodingException e) {
			throw new CryptoHelperException(e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptoHelperException(e);
		} catch (NoSuchPaddingException e) {
			throw new CryptoHelperException(e);
		} catch (InvalidKeyException e) {
			throw new CryptoHelperException(e);
		} catch (InvalidAlgorithmParameterException e) {
			throw new CryptoHelperException(e);
		}
	}
	
	/**
	 * Descifra una cadena en Base 64 CIPHER_3DES_CBC_NOPADDING
	 * @param passPhrase
	 * @param modeCipherPadding: Por defecto : CIPHER_3DES_CBC_NOPADDING
	 * @param encryptedChain
	 * @return
	 * @throws CryptoHelperException
	 */
	public String decrypt(String passPhrase, String encryptedChain ) throws CryptoHelperException {
		return decrypt(passPhrase, CIPHER_3DES_CBC_NOPADDING, encryptedChain );
	}
	
	/**
	 * Descifra una cadena en Base 64.
	 * @param passPhrase
	 * @param modeCipherPadding
	 * @param encryptedChain
	 * @return
	 * @throws CryptoHelperException
	 */
	public String decrypt(String passPhrase, String modeCipherPadding, String encryptedChain ) throws CryptoHelperException {
		try {
			
			byte [] seed_key =  Hex.decode(passPhrase);
			SecretKeySpec keySpec = new SecretKeySpec(seed_key,ALGORITHM_3DES);
			
			boolean removeLastWithespaces = false;
			if (CIPHER_3DES_CBC_SpacePADDING.equals(modeCipherPadding)) {
				modeCipherPadding = CIPHER_3DES_CBC_NOPADDING;
				removeLastWithespaces = true;
			}
			Cipher dcipher = Cipher.getInstance(modeCipherPadding);

			IvParameterSpec iv = new IvParameterSpec(ivBytes);
			dcipher.init(Cipher.DECRYPT_MODE, keySpec, iv);
			
			byte[] dec = new sun.misc.BASE64Decoder().decodeBuffer(encryptedChain); 
			String decodedStr = new String( dcipher.doFinal(dec),TripleDESEBCDICCryptoHelper.encoding);
			
			if (removeLastWithespaces) {
				return decodedStr.replaceAll("\\s+$", "");
			}
			return decodedStr;
			
		} catch (javax.crypto.BadPaddingException e) {
			throw new CryptoHelperException(e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptoHelperException(e);
		} catch (UnsupportedEncodingException e) {
			throw new CryptoHelperException(e);
		} catch (java.io.IOException e) {
			throw new CryptoHelperException(e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptoHelperException(e);
		} catch (NoSuchPaddingException e) {
			throw new CryptoHelperException(e);
		} catch (InvalidKeyException e) {
			throw new CryptoHelperException(e);
		} catch (InvalidAlgorithmParameterException e) {
			throw new CryptoHelperException(e);
		}
	}
	
	/**
	 * Metodo que realiza el XOR de dos arrays de bytes.
	 * @param clave1
	 * @param clave2
	 * @return Array de bytes con el resultado de la operacin.
	 */
	public static byte[] doXOR(String clave1, String clave2) throws Exception{
		
		if(clave1.length()!=clave2.length()){
			throw new Exception("Las dos cadenas deben tener la misma longitud.");
		}
		
		byte[] keyBytes1 = Hex.decode(clave1);
		byte[] keyBytes2 = Hex.decode(clave2);
		
		byte[] keyBytesXOR = new byte[keyBytes1.length];
		
		for(int i=0;i<keyBytesXOR.length;i++){
			int intValue = new Byte(keyBytes1[i]).intValue()^new Byte(keyBytes2[i]).intValue();
			keyBytesXOR[i]=new Integer(intValue).byteValue();
		}
		
		return keyBytesXOR;
	}
	
}
