/*
 * Created on 13-jun-2005
 * @author co00414a
 */
package p12f.exe.pasarelapagos.transformer;

import com.ejie.r01f.xml.marshalling.XOMarshallerException;

import p12f.exe.pasarelapagos.helpers.PresentationDataHelper;
import p12f.exe.pasarelapagos.objects.PresentationData;
import p12f.exe.pasarelapagos.objects.ProtocolData;
import p12f.exe.pasarelapagos.paymentrequest.PaymentGatewayData;
import p12f.exe.pasarelapagos.paymentrequest.PaymentRequestData;
import p12f.exe.pasarelapagos.paymentrequest.PeticionPago;
import p12f.exe.pasarelapagos.paymentrequest.PresentationRequestData;
import p12f.exe.pasarelapagos.transformer.ObjectTransformerAPI;
import p12f.exe.pasarelapagos.transformer.ObjectTransformerException;
import p12f.exe.pasarelapagos.transformer.ObjectTransformerHelpper;

/**
 * 
 *
 */
public class ObjectTransformer implements ObjectTransformerAPI {
	
	
	
	/**************************************************************************************
	 * Tansforma de Objetos de Peticion de Pago antiguos a la nueva coleccion de datos.
	 * @param oldPeticionPago Peticion de Pago de la Pasarela Antigua
	 * @return PaymentGatewayData de los objetos de la Pasrela de Pagos : (PaymentRequestData + ProtocolData + PresentationData)
	 * @throws ObjectTransformerException
	 */
	public PaymentGatewayData transformToNewPaymentData(p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago) throws ObjectTransformerException {
		
		PaymentGatewayData paymentGatewayData = new PaymentGatewayData();
		paymentGatewayData.paymentRequestData = ObjectTransformerHelpper.getNewPaymentRequestData(oldPeticionPago);
		paymentGatewayData.protocolData = ObjectTransformerHelpper.getNewProtocolData(oldPeticionPago);
		paymentGatewayData.presentationData = ObjectTransformerHelpper.getNewPresentationRequestData(oldPeticionPago);
		return paymentGatewayData;
		
	}
	
	/**
	 * Mtodo que obtiene un objeto PeticionPago antiguo a partir de
	 * los PaymentGateway Data : PaymentData + ProtocolData + PresentationData
	 * @param  Map : PaymentGateway Data : PaymentData + ProtocolData + PresentationData
	 * @return PeticionPago : PeticionPago Pasarela Antigua
	 **/
	public p12f.exe.pasarelapagos.v1.objects.PeticionPago transformToOldPaymentData(PaymentGatewayData paymentGatewayData) throws ObjectTransformerException {
		PeticionPago newPeticionPago = (PeticionPago)paymentGatewayData.paymentRequestData.peticionesPago.values().iterator().next();
		p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago = ObjectTransformerHelpper.getOldPeticionPago(newPeticionPago,paymentGatewayData.protocolData,paymentGatewayData.presentationData);
		return oldPeticionPago;
	}
	
	/**************************************************************************************
	 * Override del mtodo anterior.
	 * @param paymentRequestDataXML
	 * @param presentationRequestDataXML
	 * @param protocolDataXML
	 * @return p12f.exe.pasarelapagos.v1.objects.PeticionPago (Peticion de Pago Antigua)
	 * @throws ObjectTransformerException
	 */
	public p12f.exe.pasarelapagos.v1.objects.PeticionPago transformToOldPaymentData(String paymentRequestDataXML, 
	        String presentationRequestDataXML, 
	        String protocolDataXML) throws ObjectTransformerException {
		
		p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago  = null;		
		try {
		    
			PresentationRequestData prd = PresentationRequestData.getObject(presentationRequestDataXML);
			PresentationData presentationData = PresentationDataHelper.composePresentationData(prd);
			PaymentGatewayData paymentGatewayData2 = new PaymentGatewayData();
			paymentGatewayData2.paymentRequestData = PaymentRequestData.getObject(paymentRequestDataXML);
			paymentGatewayData2.protocolData = ProtocolData.getObject(protocolDataXML);
			paymentGatewayData2.presentationData = prd;	
			oldPeticionPago = this.transformToOldPaymentData(paymentGatewayData2);
		 
		} catch (XOMarshallerException e) {
			 throw new ObjectTransformerException(e);
		}
		return oldPeticionPago;
	}

	/**
	 * Transforma de oid antiguo a nuevo.
	 */
	public String getNewOIDFormat(String oldOid) throws ObjectTransformerException {
		return ObjectTransformerHelpper.getNewOIDFormat(oldOid);
	}
	
	/**
	 * Transforma de oid nuevo a antiguo..
	 */
	public String getOldOIDFormat(String oldOid) throws ObjectTransformerException {
		return ObjectTransformerHelpper.getOldOIDFormat(oldOid);
	}
	

	
}
