/*
 * Created on 10-jun-2005
 *
 */
package p12f.exe.pasarelapagos.transformer;

import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import p12f.exe.pasarelapagos.objects.Concepto;
import p12f.exe.pasarelapagos.objects.DatosPago;
import p12f.exe.pasarelapagos.objects.Estado;
import p12f.exe.pasarelapagos.objects.Expediente;
import p12f.exe.pasarelapagos.objects.Pago;
import p12f.exe.pasarelapagos.objects.PaymentStateData;
import p12f.exe.pasarelapagos.objects.PeriodoPago;
import p12f.exe.pasarelapagos.objects.ProtocolData;
import p12f.exe.pasarelapagos.objects.config.AdminConfig;
import p12f.exe.pasarelapagos.objects.config.PaymentTypeConfig;
import p12f.exe.pasarelapagos.paymentrequest.Aplicacion;
import p12f.exe.pasarelapagos.paymentrequest.Backend;
import p12f.exe.pasarelapagos.paymentrequest.BackendData;
import p12f.exe.pasarelapagos.paymentrequest.ConceptoPeticion;
import p12f.exe.pasarelapagos.paymentrequest.FinantialOrg;
import p12f.exe.pasarelapagos.paymentrequest.Liquidacion;
import p12f.exe.pasarelapagos.paymentrequest.PaymentMode;
import p12f.exe.pasarelapagos.paymentrequest.PaymentRequestData;
import p12f.exe.pasarelapagos.paymentrequest.PeticionPago;
import p12f.exe.pasarelapagos.paymentrequest.PresentationRequestData;
import p12f.exe.pasarelapagos.utils.CSBConstants;
import p12f.exe.pasarelapagos.v1.objects.ConceptoPago;
import p12f.exe.pasarelapagos.v1.objects.ConceptoPeticionPago;
import p12f.exe.pasarelapagos.v1.objects.DatoAdicional;
import p12f.exe.pasarelapagos.v1.objects.DatosOperacion;
import p12f.exe.pasarelapagos.v1.objects.Descripcion;
import p12f.exe.pasarelapagos.v1.objects.EntidadFinanciera;
import p12f.exe.pasarelapagos.v1.objects.EstadoPago;
import p12f.exe.pasarelapagos.v1.objects.Imagen;
import p12f.exe.pasarelapagos.v1.objects.InetAddr;
import p12f.exe.pasarelapagos.v1.objects.MedioPago;
import p12f.exe.pasarelapagos.v1.objects.Mensaje;
import p12f.exe.pasarelapagos.v1.objects.Presentacion;
import p12f.exe.pasarelapagos.v1.objects.Protocolo;
import p12f.exe.pasarelapagos.v1.objects.ResultadoPago;
import p12f.exe.pasarelapagos.v1.objects.config.ConfigEntidad;
import p12f.exe.pasarelapagos.v1.objects.config.ConfigSufijo;

import com.ejie.r01f.util.MoneyUtils;
import com.ejie.r01f.util.StringUtils;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * @author co00414
 *  Clase de ayuda para la conversin de objetos. 
 *  Los mtodos que proporciona esta clase se utilizarn por parte de la
 *  clase que implemente el ObjectConverterAPI (p.e. ObjectConverter).
 * 			
 */
public class ObjectTransformerHelpper {
	
	/* Constantes Idiomas */
	public static final String IDIOMA_ES = "es";
	public static final String IDIOMA_EU = "eu";
	
	/* Constantes ObjectConverter */
	public static final String CPR_507 = "90507";
	public static final String OLD_CPR = "9050794";
	public static final String OLD_CSB = "507";
	public static final String BACKEND_SIPCA_ID = "SIPCA";
	public static final String OLD_PAYMENT_DEFAULT_INSTANCE = "1";
	
	/* Periodos de Pago */
	public static final String PERIODOS_PAGO_ID = "ID";
	
	/* Periodos de Pago */
	public static final String PROTOCOLO_URL_VUELTA = "urlVuelta";
	public static final String PROTOCOLO_URL_VALIDACION = "urlValidacion";
	public static final String PROTOCOLO_URL_RESULTADO = "urlResultado";	
	
	/* Formateador de Fechas */
	static SimpleDateFormat dateFormatter = new SimpleDateFormat("ddMMyy");
	static SimpleDateFormat dateFormatter2 = new SimpleDateFormat("dd/MM/yyyy [HH:mm:ss]");

	/* Identificadores de los datos adicionales del campo Aplicacion */
	public static final String URL_VUELTA = "urlVuelta";
	public static final String IDIOMA = "idioma";

	/*
	 *  METODOS Pblicos. Los mtodos que proporciona esta clase se utilzarn
	 *  por parte de la case que implmente ObjectConverterAPI.
	 */
	
	/**
	 * Mtodo que obtiene un objeto PaymentRequestData, conjunto de un unico
	 * objeto Peticion de Pago a partir de un objeto de Peticion de Pago ANTIGUO.
	 * un objeto PeticionPago de la pasarela antigua. 
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return PaymentRequestData : PaymentRequestData Pasarela Nueva
	 */	
	public static PaymentRequestData getNewPaymentRequestData(p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago) throws ObjectTransformerException {
		PaymentRequestData paymentRequestData = new PaymentRequestData();
		paymentRequestData.peticionesPago = new HashMap();
		PeticionPago newPeticionPago = getNewPeticionPago(oldPeticionPago);
		paymentRequestData.peticionesPago.put(newPeticionPago.id,newPeticionPago);
		return paymentRequestData;
	}
	
	/**
	 * Mtodo que obtiene un objeto PeticionPago nuevo a partir de
	 * un objeto PeticionPago de la pasarela antigua. 
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return PeticionPago : PeticionPago Pasarela Nueva
	 */
	public static PeticionPago getNewPeticionPago (p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago) throws ObjectTransformerException {
	
		PeticionPago  newPeticionPago = new PeticionPago();
		newPeticionPago.id = getNewOIDFormat(oldPeticionPago.protocolo.pagoID);
		newPeticionPago.datosPago = _getNewDatosPago(oldPeticionPago);
		newPeticionPago.conceptos = _getNewConceptosPeticionPago(oldPeticionPago.conceptosPeticionPago);
		newPeticionPago.backend = _getBackendDataMapOfPeticionPago(oldPeticionPago);
		// Cambio. Las liquidaciones pasan a ser parte del objeto Presentation.newPeticionPago.liquidacion = _getNewLiquidacionPeticionPago(oldPeticionPago.liquidacion);
		newPeticionPago.emisor = _getNewEmisor(oldPeticionPago.emisor);
		newPeticionPago.expediente = _getNewExpediente(oldPeticionPago);
		newPeticionPago.aplicacion = _getNewAplicacion(oldPeticionPago.aplicacion);
		// Dentro del objeto Aplicacion se han recogido datos auxiliares heredados de la V1 que se corresponden con
		// la aplicacion que emite el pago.
		newPeticionPago.aplicacion.datosAdicionales = _setDatosAdicionales(oldPeticionPago);
		newPeticionPago.descripcion = _getNewDescripcion(oldPeticionPago);
		// Copia de imagenes.
		newPeticionPago.imagenes = _getNewImages(oldPeticionPago.imagenes);
		// Copia de mensajes
		newPeticionPago.mensajes = _getNewMessageMap(oldPeticionPago.mensajes);
		// Copia de Financial Orgs.
		newPeticionPago.finantialOrgs = _getNewFinantialOrgs(oldPeticionPago.presentacion.entidadesFinancieras);
		// Copia de modos de Pago.
		newPeticionPago.paymentModes = _getNewPaymentModes(oldPeticionPago.presentacion.mediosPago);
		// Datos adicionales
		newPeticionPago.datosAdicionales = _getNewDatosAdicionales(oldPeticionPago.datosAdicionales);
		
		return newPeticionPago;
	}

	/**
	 * Mtodo que obtiene un mapa de imgenes nuevas a partir de otro de imagenes viejas
	 * @param oldImages Mapa de imgenes viejas
	 * @return
	 */
	private static Map _getNewImages(Map oldImages) {
		Map newImages = new HashMap();
		
		// Copia de imagenes.
		if ( oldImages != null ){
			for (Iterator it = oldImages.values().iterator();it.hasNext(); ){
				Imagen imagenOld = (Imagen) it.next();
				p12f.exe.pasarelapagos.objects.Imagen newImagen = new p12f.exe.pasarelapagos.objects.Imagen();
				newImagen = new p12f.exe.pasarelapagos.objects.Imagen ();
				newImagen.id = imagenOld.id;
				//Imp.Cambiar en la defincion de la clase a un byte[]
				if (imagenOld.bin != null) {
					newImagen.bin = imagenOld.bin.toString();
				}
				newImagen.url = imagenOld.url;
				newImages.put(newImagen.id,newImagen);
			}
		}
		
		return newImages;
	}
	
	/**
	 * Mtodo que obtiene un objeto Pago nuevo a partir de
	 * un objeto Pago de la pasarela antigua. 
	 * @param  Pago : Pago Pararela Antigua.
	 * @return Pago : Pago Pasarela Nueva
	 */
	public static Pago getNewPago (p12f.exe.pasarelapagos.v1.objects.Pago oldPago) throws ObjectTransformerException {
	
		Pago newPago = new Pago();
		newPago.id = getNewOIDFormat(oldPago.protocolo.pagoID);
		newPago.datosPago = _getNewDatosPago(oldPago);
		newPago.conceptos = _getNewConceptosPago(oldPago.conceptosPago);
		//newPago.backend = _getBackendDataMapOfPeticionPago(oldPeticionPago);
		// Cambio. Las liquidaciones pasan a ser parte del objeto Presentation.newPeticionPago.liquidacion = _getNewLiquidacionPeticionPago(oldPeticionPago.liquidacion);
		newPago.emisor = _getNewEmisor(oldPago.emisor);
		//newPago.expediente = _getNewExpediente(oldPago);
		//newPago.aplicacion = _getNewAplicacion(oldPeticionPago.aplicacion);
		newPago.descripcion = _getNewDescripcion(oldPago);
		
		return newPago;
	}
	
	/**
	 * Mtodo que obtiene un objeto PeticionPago a partir de
	 * un objeto PeticionPago de la pasarela antigua. 
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return PeticionPago : PeticionPago Pasarela Nueva
	 */
	public static p12f.exe.pasarelapagos.v1.objects.PeticionPago getOldPeticionPago (PeticionPago newPeticionPago, ProtocolData newProtocolData, PresentationRequestData newPresentationData ) throws ObjectTransformerException {
		
		if ( newPeticionPago == null ) { throw new ObjectTransformerException("Objeto PeticionPago nulo. Repase los datos"); }
		else if ( newProtocolData == null ) { throw new ObjectTransformerException("Objeto ProtocolData nulo. Repase los datos"); }
		else if ( newPresentationData == null ) { throw new ObjectTransformerException("Objeto PresentationRequestData nulo. Repase los datos"); }
		
		p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago = new p12f.exe.pasarelapagos.v1.objects.PeticionPago();
		
		oldPeticionPago.referencia = newPeticionPago.datosPago.referencia;
		PeriodoPago newPeriodoPago = (PeriodoPago)newPeticionPago.datosPago.periodosPago.values().iterator().next();
		oldPeticionPago.identificacion = newPeriodoPago.identificacion;
		
		// Comprobar si la fecha lmite de pago no es null (puede serlo si sufijo < 500)
		if (newPeriodoPago.fechaFin != null) {
			oldPeticionPago.fechaLimitePago = newPeriodoPago.fechaFin.substring(0,2) + "/" + newPeriodoPago.fechaFin.substring(2,4) + "/20" + newPeriodoPago.fechaFin.substring(4,6);
		}
		
		if(newPeticionPago.aplicacion!=null) {
			oldPeticionPago.aplicacion = _getOldAplicacion(newPeticionPago.aplicacion);
		}
		if(newPeticionPago.conceptos!=null) {
			oldPeticionPago.conceptosPeticionPago = _getOldConceptosPeticionPago(newPeticionPago.conceptos);
		}
		if(newPeticionPago.emisor!=null){
			String codigo = newPeticionPago.datosPago.emisor;
			oldPeticionPago.emisor = _getOldEmisor(newPeticionPago.emisor, codigo);
		}
		
		if ( newPeticionPago.descripcion != null ){
			oldPeticionPago.descripcion = new Descripcion();
			if(newPeticionPago.descripcion.get(IDIOMA_ES)!=null) { oldPeticionPago.descripcion.castellano = (String)newPeticionPago.descripcion.get(IDIOMA_ES); }
			if(newPeticionPago.descripcion.get(IDIOMA_EU)!=null) { oldPeticionPago.descripcion.euskera = (String)newPeticionPago.descripcion.get(IDIOMA_EU); }
		}
		
		//Presentacion
		oldPeticionPago.presentacion = _getOldPresentationRequestData(newPresentationData);
	
		//Protocol
		oldPeticionPago.protocolo  = _getOldProtocolData(newPeticionPago,newProtocolData);
		
		//Insertar en oldPeticiondePago los BackendData de newPeticionPago.
		oldPeticionPago = _setBackendDataOfPeticionPago(newPeticionPago,oldPeticionPago);
		
		// Tercero
		if(newPeticionPago.expediente!=null) {
			oldPeticionPago.tercero =_getOldTercero(newPeticionPago.expediente.tercero);
		}
		
		//Liquidacion 
		if (newPeticionPago.liquidacion!=null) {
			oldPeticionPago.liquidacion = _getOldLiquidacionPeticionPago(newPeticionPago.liquidacion);
		}
		
		// Mensajes 
		//oldPeticionPago.mensajes = _getOldMessageMap(newPresentationData.mensajes);
		oldPeticionPago.mensajes = _getOldMessageMap(newPeticionPago.mensajes);
		
		// Importe
		oldPeticionPago.importe = getOldImporte(newPeticionPago);
		
		// Imagenes
		oldPeticionPago.imagenes = _getOldImageMap(newPeticionPago.imagenes);
		
		// Datos adicionales del pago
		oldPeticionPago.datosAdicionales = _getOldDatosAdicionales(newPeticionPago.datosAdicionales);
		
		return oldPeticionPago;
	}
	
	/**
	 * Devuelve el oid en el Formato Nuevo para OID de la Pasarela V.1
	 * @param oldOidFormat
	 * @return
	 */
	public static String getNewOIDFormat( String pagoID ){
		
		String emisorCSB = pagoID.substring(0,8);
		String sufijoCSB = pagoID.substring(8,11);
		String referencia = pagoID.substring(11,24);
		String identificacion = pagoID.substring(24,30);
		String centeuros = pagoID.substring(30,pagoID.length());
		
		while ( centeuros.length() < 10 ) { centeuros = "0" + centeuros; }
		
		String csbCode = CSBConstants.CSB507.ID_APP.defaultValue + CSBConstants.CSB507.csbCode;
		csbCode += emisorCSB 
				+ sufijoCSB
				+ referencia
				+ identificacion
				+ centeuros
				+ CSBConstants.CSB507.DIGITO_PARIDAD.defaultValue;
		
		return csbCode;
	}
	
	/**
	 * Devuelve el oid en el Formato Viejo para OID de la Pasarela V.2
	 * @param oldOidFormat
	 * @return
	 */
	public static String getOldOIDFormat( String pagoID ){
		
		String emisorCSB = pagoID.substring(5,13);
		String sufijoCSB = pagoID.substring(13,16);
		String referencia = pagoID.substring(16,29);
		String identificacion = pagoID.substring(29,35);
		String centeuros = pagoID.substring(35,pagoID.length()-1);
		
		long centeurosLong = Long.parseLong(centeuros);
		
		String csbCode = emisorCSB + sufijoCSB + referencia + identificacion + centeurosLong ;
		
		return csbCode;
	}
	
	/**
	 * Crea un sufijo/tibuto en el formato nuevo.
	 * @param sufijoV1
	 * @return
	 * @throws ObjectTransformerException 
	 */
	public static PaymentTypeConfig getNewPaymentTypeConfig(ConfigSufijo sufijoV1) throws ObjectTransformerException{
		
		PaymentTypeConfig newPaymentTypeConfig = new PaymentTypeConfig();
		
		// Componer nuevo oid
		newPaymentTypeConfig.oid = sufijoV1.emisor.cif + sufijoV1.oid + CSBConstants.CSB507.csbCode;
		newPaymentTypeConfig.activa = true;
		newPaymentTypeConfig.emisor = sufijoV1.emisor.codigo.substring(0, 8);
		newPaymentTypeConfig.tributo = sufijoV1.emisor.codigo.substring(9, 12);
		newPaymentTypeConfig.formato = "507";
		newPaymentTypeConfig.validar = 1;
		
		// Datos de la descripcion
		newPaymentTypeConfig.descripcion.put("es", sufijoV1.descripcion.castellano);
		newPaymentTypeConfig.descripcion.put("eu", sufijoV1.descripcion.euskera);
		
		// Conceptos del tipo de Pago. 
		newPaymentTypeConfig.conceptos = _getNewConceptosPeticionPago(sufijoV1.conceptosPeticionPago);
		// Imagenes del tipo de Pago. 
		newPaymentTypeConfig.imagenes =  _getNewImages(sufijoV1.imagenes);
		// Se copia la imagen con id "1" a "logoEmisor"
		p12f.exe.pasarelapagos.objects.Imagen imagen = (p12f.exe.pasarelapagos.objects.Imagen)newPaymentTypeConfig.imagenes.get("1");
		if(imagen!=null) {
			p12f.exe.pasarelapagos.objects.Imagen copiaImagen = new  p12f.exe.pasarelapagos.objects.Imagen();
			copiaImagen.alt = imagen.alt;
			copiaImagen.bin = imagen.bin;
			copiaImagen.url = imagen.url;
			copiaImagen.id = "logoEmisor";
			
			newPaymentTypeConfig.imagenes.put(copiaImagen.id, copiaImagen);
		}
		
		// Mensajes del tipo de Pago. 
		newPaymentTypeConfig.mensajes =  _getNewMessageMap(sufijoV1.mensajes);
		// Datos de Backcend (Por reflection los coge del config, igual que para la peticion de Pago.)
		newPaymentTypeConfig.backend = _getBackendDataMapOfSufijo(sufijoV1);
		// Datos de la aplicacion
		newPaymentTypeConfig.aplicacion = _getNewAplicacion(sufijoV1.aplicacion);	
		// Entidades Financieras del tipo de Pago. 
		newPaymentTypeConfig.finantialOrgs = _getNewFinantialOrgs(sufijoV1.presentacion.entidadesFinancieras);
		// Modos de pago del tipo de Pago.		
		newPaymentTypeConfig.paymentModes = _getNewPaymentModes(sufijoV1.presentacion.mediosPago);
		// Plantilla de liquidacion del tipo de Pago.
		if(sufijoV1.liquidacion!=null) {
			newPaymentTypeConfig.liquidacion = _getNewLiquidacionPeticionPago(sufijoV1.liquidacion);
		}
		
		return newPaymentTypeConfig;
	}
	
	/**
	 * Crea una entidad emisora en el formato nuevo.
	 * @param entidadV1
	 * @param sufijoV1
	 * @return
	 * @throws ObjectTransformerException 
	 */
	public static AdminConfig getNewAdminConfig(ConfigEntidad emisorV1, ConfigSufijo sufijoV1){
		
		AdminConfig newAdminConfig = new AdminConfig();

		newAdminConfig.oid = emisorV1.oid;
		newAdminConfig.cif = emisorV1.cif;

		// Datos de la descripcion
		newAdminConfig.descripcion.put("es", emisorV1.descripcion.castellano);
		newAdminConfig.descripcion.put("eu", emisorV1.descripcion.euskera);
		
		newAdminConfig.calle = sufijoV1.emisor.calle;
		newAdminConfig.municipio = sufijoV1.emisor.municipio;
		newAdminConfig.territorio = sufijoV1.emisor.territorio;
		newAdminConfig.codigoPostal = sufijoV1.emisor.codigoPostal;
		
		return newAdminConfig;
	}
	
	/************************************************************************************************
	 * 										METODOS PRIVADOS
	 ***********************************************************************************************/
	
	
	/**
	 * Mtodo que obtiene un objeto DatosPago a partir de
	 * un objeto PeticionPago de la pasarela antigua. 
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return DatosPago : DatosPago Pasarela Nueva
	 */
	private static DatosPago _getNewDatosPago (p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago){
	
		DatosPago newDatosPago = new DatosPago();
		
		String entidadEmisora = oldPeticionPago.emisor.codigo.substring(0, 8); //emisor.cif;
		String sufijo = oldPeticionPago.emisor.codigo.substring(oldPeticionPago.emisor.codigo.length()-3 ,oldPeticionPago.emisor.codigo.length());
		String referencia = oldPeticionPago.referencia;
		String identificacion = oldPeticionPago.identificacion;	
		
		// IMPORTANTE: Se debe acordar el formato del importe. Actualmente en la nueva version 
		// lo tenemos como String. En la Pasarela Version 1 como double.
		String importe = Long.toString(MoneyUtils.getCenteuros(oldPeticionPago.importe));
		StringUtils.rPadWithChar(importe, '0', 10);
		
		newDatosPago.formato = OLD_CSB;
		newDatosPago.validar = DatosPago.VALIDACION_NORMAL;
		newDatosPago.cpr = OLD_CPR;
		newDatosPago.codigo = CPR_507 + entidadEmisora + sufijo + referencia + identificacion + importe + '0';
		newDatosPago.emisor = entidadEmisora + "-" + sufijo;
		newDatosPago.tipo = sufijo;
		newDatosPago.referencia = referencia;
		newDatosPago.periodosPago = _getNewPeriodosPago(oldPeticionPago);
		
		return newDatosPago;
	}
	
	/**
	 * Mtodo que obtiene un objeto DatosPago a partir de
	 * un objeto Pago de la pasarela antigua. 
	 * @param  Pago : Pago Pararela Antigua.
	 * @return DatosPago : DatosPago Pasarela Nueva
	 */
	private static DatosPago _getNewDatosPago (p12f.exe.pasarelapagos.v1.objects.Pago oldPago){
	
		DatosPago newDatosPago = new DatosPago();

		String entidadEmisora = oldPago.emisor.cif;
		String sufijo = oldPago.emisor.codigo.substring(oldPago.emisor.codigo.length()-3 ,oldPago.emisor.codigo.length());
		String referencia = oldPago.referencia;
		String identificacion = oldPago.identificacion;	
		// IMPORTANTE: Se debe acordar el formato del importe. Actualmente en la nueva version 
		// lo tenemos como String. En la Pasarela Version 1 como double.
		String importe = oldPago.importe;
		while(importe.length()<10) importe = "0" + importe;
		newDatosPago.formato = OLD_CSB;
		newDatosPago.validar = DatosPago.VALIDACION_NORMAL;
		newDatosPago.cpr = OLD_CPR;
		newDatosPago.codigo = CPR_507 + entidadEmisora + sufijo + referencia + identificacion + importe + "0";
		newDatosPago.emisor = entidadEmisora + "-" + sufijo;
		newDatosPago.tipo = sufijo;
		newDatosPago.referencia = referencia;
		newDatosPago.periodosPago = _getNewPeriodosPago(oldPago);
		
		return newDatosPago;
	}
		
	/**
	 * Mtodo que obtiene un Map de Periodos de Pago, con un nico 
	 * periodo de pago un objeto PeticionPago de la pasarela antigua. 
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return Map : Mapa con un periodo de pago.
	 */
	private static Map _getNewPeriodosPago (p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago){
	
		Map periodosPagoMap = new java.util.HashMap();	
		
		PeriodoPago periodoPago = new PeriodoPago();
		periodoPago.id = PeriodoPago.PERIODO_NORMAL;
		periodoPago.identificacion = oldPeticionPago.identificacion;
		
		// El importe en los antiguos pagos viene en Euros. 
		// Se debe pasar a centeuros.
		periodoPago.importe = getNewImporte(oldPeticionPago);
		
		//Date fechaAyer = new Date(new Date().getTime() - (24*60*60*1000));
		//periodoPago.fechaInicio = dateFormatter.format(fechaAyer);
		String fechaLimite = oldPeticionPago.fechaLimitePago;
		if(fechaLimite!=null) {
			String dia = fechaLimite.substring(0,2);
			String mes = fechaLimite.substring(3,5);
			String anio = fechaLimite.substring(8,10);
			periodoPago.fechaFin = dia + mes + anio;
		}
		periodoPago.validarFechaFin = true;
		periodoPago.activo = true;
		periodosPagoMap.put(periodoPago.id,periodoPago);
		return periodosPagoMap;
	}
	
	/**
	 * Mtodo que obtiene un Map de Periodos de Pago, con un nico 
	 * periodo de pago un objeto Pago de la pasarela antigua. 
	 * @param  Pago : Pago Pararela Antigua.
	 * @return Map : Mapa con un periodo de pago.
	 */
	private static Map _getNewPeriodosPago (p12f.exe.pasarelapagos.v1.objects.Pago oldPago){
	
		Map periodosPagoMap = new java.util.HashMap();	
		
		PeriodoPago periodoPago = new PeriodoPago();
		periodoPago.id = PeriodoPago.PERIODO_NORMAL;
		periodoPago.identificacion = oldPago.identificacion;
		
		// El importe en los antiguos pagos viene en Euros. 
		// Se debe pasar a centeuros.
		periodoPago.importe = (new Double(oldPago.importe)).longValue();
		
		//Date fechaAyer = new Date(new Date().getTime() - (24*60*60*1000));
		//periodoPago.fechaInicio = dateFormatter.format(fechaAyer);
		String fechaLimite = oldPago.fechaLimitePago;
		
		String dia = fechaLimite.substring(0,2);
		String mes = fechaLimite.substring(3,5);
		String anio = fechaLimite.substring(8,10);
		periodoPago.fechaFin = dia + mes + anio;
		periodoPago.validarFechaFin = true;
		periodoPago.activo = true;
		periodosPagoMap.put(periodoPago.id,periodoPago);
		return periodosPagoMap;
	}
	
	private static long _getCenteuros(double cantidad){
		StringBuffer centeuros = new StringBuffer(new Double(cantidad).toString());
		
		// Se comprueba si se trata de un decimal
		if(centeuros.indexOf(".")!=-1){
			// Si el numero viene con mas de 2 decimales, se eliminan
			if(centeuros.length()>(centeuros.indexOf(".")+3)) {
				centeuros.delete(centeuros.indexOf(".")+3,centeuros.length());
			} else {
				// Si el numero no viene con 2 decimales, se aaden
				while(centeuros.length()<(centeuros.indexOf(".")+3)) {
					centeuros.append("0");
				}
			}
			// Se elimina el decimal
			centeuros.deleteCharAt(centeuros.indexOf("."));
		} else {
			// Se trata de un entero,
			// Se aaden 2 cifras decimales
			centeuros.append("00");
		}
		
		return Long.parseLong(centeuros.toString());
	}
	
	/**
	 * Mtodo que obtiene un Map de Periodos de Pago, con un nico 
	 * periodo de pago un objeto PeticionPago de la pasarela antigua. 
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return Map : Mapa con los Conceptos de Peticion de Pago.
	 */
	private static List _getNewConceptosPeticionPago (List oldConceptosPeticionPago) throws ObjectTransformerException  {
		
		List newConceptosPeticionPago = new java.util.ArrayList();
		if(oldConceptosPeticionPago!=null){
			for(int i=0; i<oldConceptosPeticionPago.size(); i++) {
				// Se obtiene el pago actual
				ConceptoPeticionPago oldCPP = (ConceptoPeticionPago)oldConceptosPeticionPago.get(i);
				// Se convierte a pago nuevo	
				ConceptoPeticion newCPP = _getNewConceptoPeticionPago(oldCPP);
				
				// En el caso de que el pago tenga IVA hay que extrer los datos 
				// del IVA del siguiente concepto, si este existe.
				if(i+1<oldConceptosPeticionPago.size()) {
					ConceptoPeticionPago oldCPP2 = (ConceptoPeticionPago)oldConceptosPeticionPago.get(i+1);
					ConceptoPeticion newCPP2 = _getNewConceptoPeticionPago(oldCPP2);
					if(newCPP2.IVARepercutido) {
						// El siguiente concepto es de IVA repercutido, por tanto se
						// informa que el primer concepto tiene IVA repercutido
						newCPP.tieneIVARepercutido = true;
					}else if(newCPP2.tipoIVA!=0 && newCPP2.importeIVA!=0){
						// El siguiente concepto es de IVA repercutido, por tanto se
						// informa que el primer concepto tiene IVA repercutido
						newCPP.tipoIVA = newCPP2.tipoIVA;
						newCPP.tipoIVA = newCPP2.importeIVA;
					}
				}
				
				newConceptosPeticionPago.add(newCPP);
			}
		}
		return newConceptosPeticionPago;
	}
	
	/**
	 * Mtodo que obtiene un List de Conceptos de Pago, con un nico 
	 */
	private static List _getNewConceptosPago (List oldConceptosPago) throws ObjectTransformerException {
		List newConceptosPago = new java.util.ArrayList();
		if(oldConceptosPago!=null){
			for(int i=0; i<oldConceptosPago.size(); i++) {
				// Se obtiene el concepto actual
				ConceptoPago oldCPP = (ConceptoPago)oldConceptosPago.get(i);
				// Se convierte a concepto nuevo
				Concepto newCPP = _getNewConceptoPago(oldCPP);
				
				// En el caso de que el pago tenga IVA hay que extrer los datos 
				// del IVA del siguiente concepto, si este existe.
				if(i+1<oldConceptosPago.size()) {
					ConceptoPago oldCPP2 = (ConceptoPago)oldConceptosPago.get(i+1);
					Concepto newCPP2 = _getNewConceptoPago(oldCPP2);
					if(newCPP2.IVARepercutido) {
						// El siguiente concepto es de IVA repercutido, por tanto se
						// informa que el primer concepto tiene IVA repercutido
						newCPP.tieneIVARepercutido = true;
					}else if(newCPP2.tipoIVA!=0 && newCPP2.importeIVA!=0){
						// El siguiente concepto es de IVA repercutido, por tanto se
						// informa que el primer concepto tiene IVA repercutido
						newCPP.tipoIVA = newCPP2.tipoIVA;
						newCPP.tipoIVA = newCPP2.importeIVA;
					}
				}
				
				newConceptosPago.add(newCPP);
			}
		}
		return newConceptosPago;
	}
	
	/**
	 * Mtodo que obtiene un Concepto de Pago nuevo a partir de uno viejo 
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return Map : Mapa con los Conceptos de Peticion de Pago.
	 */
	private static ConceptoPeticion _getNewConceptoPeticionPago (ConceptoPeticionPago oldCPP) throws ObjectTransformerException {
		ConceptoPeticion newCPP = null;

		newCPP = new ConceptoPeticion();
		newCPP.numeroLinea = oldCPP.numeroLinea;
		newCPP.descripcion = new HashMap();
		newCPP.descripcion.put(IDIOMA_ES,oldCPP.castellano);
		newCPP.descripcion.put(IDIOMA_EU,oldCPP.euskera);
		newCPP.importe = _getCenteuros(oldCPP.importe);
		newCPP.baseImponible = _getCenteuros(oldCPP.baseImponible);
		newCPP.tipoIVA = _getCenteuros(oldCPP.tipoIVA);
		newCPP.IVARepercutido = oldCPP.ivaRepercutido;
		newCPP.importeIVA = _getCenteuros(oldCPP.importeIVA);
		newCPP.backendDataMap = _getBackendDataOfConceptoPeticion(oldCPP);
		// De momento el numero de unidades por concepto se deja sin valor para la conversin
		//newCPP.unidades = 0;
		return newCPP;
	}
	
	/**
	 * Mtodo que obtiene un Concepto de Pago nuevo a partir de uno viejo 
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return Map : Mapa con los Conceptos de Peticion de Pago.
	 */
	private static Concepto _getNewConceptoPago (ConceptoPago oldCPP) throws ObjectTransformerException {
		ConceptoPeticion newCPP = null;

		newCPP = new ConceptoPeticion();
		newCPP.descripcion = new HashMap();
		newCPP.descripcion.put(IDIOMA_ES,oldCPP.castellano);
		newCPP.descripcion.put(IDIOMA_EU,oldCPP.euskera);
		newCPP.importe = _getCenteuros(Long.parseLong(oldCPP.importe));
		// De momento el numero de unidades por concepto se deja sin valor para la conversin
		//newCPP.unidades = 0;

		return newCPP;
	}
	
	/**
	 * Mtodo que obtiene un List Datos Adicionales de la nueva pasarela. 
	 * @param  oldDatosAdicionales : Datos Adicionales de la Pasarela Antigua.
	 * @return List con los Datos Adicionales de la Pasarela Nuevao.
	 */
	private static List _getNewDatosAdicionales (List oldDatosAdicionales) {
		
		List newDatosAdicionales = new ArrayList();
		
		if ( oldDatosAdicionales != null ){
			for ( Iterator it = oldDatosAdicionales.iterator();it.hasNext(); ){
				DatoAdicional oldDatoAdicional = (DatoAdicional) it.next();
				p12f.exe.pasarelapagos.objects.DatoAdicional newDatoAdicional = new p12f.exe.pasarelapagos.objects.DatoAdicional();
				newDatoAdicional.id = oldDatoAdicional.id;
				newDatoAdicional.valor = oldDatoAdicional.valor;
				if(oldDatoAdicional.castellano!=null){
					newDatoAdicional.texto.put("es",oldDatoAdicional.castellano);
				}
				if(oldDatoAdicional.euskera!=null){
					newDatoAdicional.texto.put("eu",oldDatoAdicional.euskera);
				}
				newDatosAdicionales.add(newDatoAdicional);
			}
		}
		
		return newDatosAdicionales;	
	}
	
	/**
	 * Mtodo que obtiene un Map de Periodos de Pago, con un nico 
	 * periodo de pago un objeto PeticionPago de la pasarela antigua. 
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return Map : Mapa con los Conceptos de Peticion de Pago.
	 */
	private static List _getOldConceptosPeticionPago (List newConceptosPeticionPago) throws ObjectTransformerException  {
		
		List oldConceptosPeticionPago = new java.util.ArrayList();
		ConceptoPeticionPago oldCPP = null;
		ConceptoPeticion newCPP = null;
		for (Iterator it = newConceptosPeticionPago.iterator(); it.hasNext(); ) {
			
			newCPP = (ConceptoPeticion)it.next();
			oldCPP = new ConceptoPeticionPago();
			oldCPP.numeroLinea = newCPP.numeroLinea;
			oldCPP.importe = new Long(newCPP.importe).doubleValue()/100;
			oldCPP.castellano = (String)newCPP.descripcion.get(IDIOMA_ES);
			oldCPP.euskera = (String)newCPP.descripcion.get(IDIOMA_EU);
			oldCPP = _setBackendDataOfConceptoPeticion(newCPP,oldCPP);
			oldConceptosPeticionPago.add(oldCPP);
			
		}
		return oldConceptosPeticionPago;
	}
	
	/**
	 * Mtodo que obtiene un Map con los BackEndData para los Conceptos
	 * de Peticion de Pago (datos Sipca : ivaRepercutido, aplicPresupuestaria..etc.)
	 * de un objeto PeticionPago de la pasarela antigua. 
	 * @param  ConceptoPeticionPago : ConceptoPeticionPago Pararela Antigua.
	 * @return Map : Mapa con los  datos SIPCA de los Conceptos de Peticion de Pago.
	 */
	private static Map _getBackendDataOfConceptoPeticion (ConceptoPeticionPago oldCPP) throws ObjectTransformerException {
	
		Map backendDataMap = new HashMap();
		String fieldName = null;
		String fieldValue = null;
		BackendData bendData = null;
		Field field = null;
		
		/* En los antiguos objetos de Conceptos de Peticion de Pago los campos
		 * para sipca eran miembros de la clase. En esta nueva versin 
		 * los guardamos en un BackendMap, en los que los campos de SIPCA los
		 * cogemos de un properties. Para pasar de un
		 */
		List fields = XMLProperties.getPropertyList("p12f","sipca/conceptoPeticionPago/fields/field");
		if ( fields == null ) throw new ObjectTransformerException("No ha definido los campos Backend SIPCA para ConceptoPeticionPago :getPropertyList(\"p12f\",\"sipca/conceptoPeticionPago/fields/field\") ");
		
		try {
			for ( Iterator it = fields.iterator();it.hasNext(); ){
				fieldName = (String)it.next();
				field = oldCPP.getClass().getDeclaredField(fieldName);
				if ( field.get(oldCPP) != null ){
					fieldValue = field.get(oldCPP).toString();
					bendData = new BackendData (fieldName,fieldValue);
					backendDataMap.put(fieldName,bendData);
				}
			}
		} catch (SecurityException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackEndDataOfConceptoPeticion.SecurityException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackEndDataOfConceptoPeticion.NoSuchFieldException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackEndDataOfConceptoPeticion.IllegalAccessException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		}
		return backendDataMap;
	}
	
	/**
	 * Mtodo que inserta por reflection en un objeto antiguo Concepto de Peticion de Pago 
	 * los datos backendSipca  del Concepto de Peticion de Pago nuevo
	 * @param  ConceptoPeticion newCPP, ConceptoPeticionPago olcCPP
	 * @return ConceptoPeticionPago : Concepto peticion pago antiguo modificado con los datos de SIPCA.
	 */
	private static ConceptoPeticionPago _setBackendDataOfConceptoPeticion (ConceptoPeticion newCPP, ConceptoPeticionPago oldCPP) throws ObjectTransformerException {
	
		String fieldName = null;
		Field  fieldOldCPP = null;
		BackendData backendData = null;
		
		/* En los antiguos objetos de Concpetos de Peticion de Pago los campos
		 * para sipca eran miembros de la clase. En esta nueva versin 
		 * los guardamos en un BackendMap, en los que los campos de SIPCA los
		 * cogemos de un properties. Para pasar de un
		 */
		List fields = XMLProperties.getPropertyList("p12f","sipca/conceptoPeticionPago/fields/field");
		if ( fields == null ) { throw new ObjectTransformerException("No ha definido los campos Backend SIPCA para ConceptoPeticionPago :getPropertyList(\"p12f\",\"sipca/conceptoPeticionPago/fields/field\") "); }
		try {
			for ( Iterator it = fields.iterator();it.hasNext(); ){
				fieldName = (String)it.next();
				backendData = (BackendData) newCPP.backendDataMap.get(fieldName);
				fieldOldCPP = oldCPP.getClass().getDeclaredField(fieldName);
				if ( backendData != null ){
					if ( fieldOldCPP.getType().getClass().getName().equals("java.lang.String")){
						fieldOldCPP.set(oldCPP,backendData.value);
					} else if ( fieldOldCPP.getType().getClass().getName().equals("double")){
						fieldOldCPP.setDouble(oldCPP,Double.parseDouble(backendData.value));
					}  else if ( fieldOldCPP.getType().getClass().getName().equals("int")) {
						fieldOldCPP.setInt(oldCPP,Integer.parseInt(backendData.value));
					} else if ( fieldOldCPP.getType().getClass().getName().equals("boolean")){
						fieldOldCPP.setBoolean(oldCPP,Boolean.valueOf(backendData.value).booleanValue());
					}
				}
			}
		} catch (SecurityException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackEndDataOfConceptoPeticion.SecurityException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackEndDataOfConceptoPeticion.NoSuchFieldException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackEndDataOfConceptoPeticion.IllegalAccessException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		}
		return oldCPP;
	}
	
	/**
	 * Mtodo que obtiene un objeto Backend para la Peticion de Pago
	 * (datos Sipca : sipca)
	 * @param  PeticionPago : ConceptoPeticionPago Pararela Antigua.
	 * @return Backend : Backend con los  datos SIPCA de los Conceptos de Peticion de Pago.
	 */
	private static Backend _getBackendDataMapOfPeticionPago (p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago) throws ObjectTransformerException {
	
		/* Crear un objeto backend  */
		Backend backend = new Backend();
		backend.systemID = BACKEND_SIPCA_ID;
		backend.enabled = oldPeticionPago.informarSIPCA;
		
		/* Obtener los campos del BackEndSipca */
		String fieldName = null;
		String fieldValue = null;
		BackendData bendData = null;
		Field field = null;
		
		/* En los antiguos objetos de Concpetos de Peticion de Pago los campos
		 * para sipca eran miembros de la clase. En esta nueva versin 
		 * los guardamos en un BackendMap, en los que los campos de SIPCA los
		 * cogemos de un properties. Para pasar de un
		 */
		List fields = XMLProperties.getPropertyList("p12f","sipca/peticionPago/fields/field");
		if ( fields == null ) { throw new ObjectTransformerException("No ha definido los campos Backend SIPCA para ConceptoPeticionPago :getPropertyList(\"p12f\",\"sipca/PeticionPago/fields/field\") "); }
	
		try {
			for ( Iterator it = fields.iterator();it.hasNext(); ){
				fieldName = (String)it.next();
				field = oldPeticionPago.getClass().getDeclaredField(fieldName);
				if ( field.get(oldPeticionPago) != null ) {
					fieldValue = field.get(oldPeticionPago).toString();
					bendData = new BackendData (fieldName,fieldValue);
					backend.put(bendData);
				}
			}
		} catch (SecurityException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackendDataMapForNewPeticionPago.SecurityException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackendDataMapForNewPeticionPago.NoSuchFieldException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackendDataMapForNewPeticionPago.IllegalAccessException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		}
		return backend;
	}
	
	/**
	 * Mtodo que obtiene un objeto Backend para el Sufijo (Teoricamente son los mismos que para Peticion de Pago.)
	 * (datos Sipca : sipca)
	 * @param  ConfigSufijo : ConceptoPeticionPago Pararela Antigua.
	 * @return Backend : Backend con los  datos SIPCA de los Conceptos de ConfigSufijo
	 */
	private static Backend _getBackendDataMapOfSufijo (ConfigSufijo oldSufijo) throws ObjectTransformerException {
	
		/* Crear un objeto backend  */
		Backend backend = new Backend();
		backend.systemID = BACKEND_SIPCA_ID;
		backend.enabled = oldSufijo.informarSIPCA;
		
		/* Obtener los campos del BackEndSipca */
		String fieldName = null;
		String fieldValue = null;
		BackendData bendData = null;
		Field field = null;
		
		/* En los antiguos objetos de Concpetos de Peticion de Pago los campos
		 * para sipca eran miembros de la clase. En esta nueva versin 
		 * los guardamos en un BackendMap, en los que los campos de SIPCA los
		 * cogemos de un properties. Para pasar de un
		 */
		List fields = XMLProperties.getPropertyList("p12f","sipca/peticionPago/fields/field");
		if ( fields == null ) { throw new ObjectTransformerException("No ha definido los campos Backend SIPCA para ConceptoPeticionPago :getPropertyList(\"p12f\",\"sipca/PeticionPago/fields/field\") "); }
	
		try {
			for ( Iterator it = fields.iterator();it.hasNext(); ){
				fieldName = (String)it.next();
				field = oldSufijo.getClass().getDeclaredField(fieldName);
				if ( field.get(oldSufijo) != null ) {
					fieldValue = field.get(oldSufijo).toString();
					bendData = new BackendData (fieldName,fieldValue);
					backend.put(bendData);
				}
			}
		} catch (SecurityException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackendDataMapForNewPeticionPago.SecurityException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackendDataMapForNewPeticionPago.NoSuchFieldException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackendDataMapForNewPeticionPago.IllegalAccessException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		}
		return backend;
	}
	
	/**
	 * Mtodo que inserta por reflection en un objeto antiguo de Peticion de Pago 
	 * los datos backendSipca  de la Peticion de Pago nueva
	 * @param  Peticion newCPP, ConceptoPeticionPago olcCPP
	 * @return PeticionPago : Concepto peticion pago antiguo modificado con los datos de SIPCA.
	 */
	private static p12f.exe.pasarelapagos.v1.objects.PeticionPago _setBackendDataOfPeticionPago (PeticionPago newPP, p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPP) throws ObjectTransformerException {
		String fieldName = null;
		Field fieldOldCPP = null;
		BackendData backendData = null;

		/*
		 * En los antiguos objetos de Concpetos de Peticion de Pago los campos
		 * para sipca eran miembros de la clase. En esta nueva versin los
		 * guardamos en un BackendMap, en los que los campos de SIPCA los
		 * cogemos de un properties. Para pasar de un
		 */
		List fields = XMLProperties.getPropertyList("p12f", "sipca/peticionPago/fields/field");
		if (fields == null) {
			throw new ObjectTransformerException("No ha definido los campos Backend SIPCA para ConceptoPeticionPago :getPropertyList(\"p12f\",\"sipca/conceptoPeticionPago/fields/field\") ");
		}
		try {
			for (Iterator it = fields.iterator(); it.hasNext();) {
				fieldName = (String)it.next();
				if (newPP.backend != null) {
					backendData = newPP.backend.get(fieldName);
					fieldOldCPP = oldPP.getClass().getDeclaredField(fieldName);
					if (backendData != null) {
						if (fieldOldCPP.getType().getClass().getName().equals("java.lang.String") ||
								fieldOldCPP.getType().getName().equals("java.lang.String")) {
							fieldOldCPP.set(oldPP, backendData.value);
						} else if (fieldOldCPP.getType().getClass().getName().equals("double") ||
								fieldOldCPP.getType().getName().equals("double")) {
							fieldOldCPP.setDouble(oldPP, Double.parseDouble(backendData.value));
						} else if (fieldOldCPP.getType().getClass().getName().equals("int") ||
								fieldOldCPP.getType().getName().equals("int")) {
							fieldOldCPP.setInt(oldPP, Integer.parseInt(backendData.value));
						} else if (fieldOldCPP.getType().getClass().getName().equals("boolean") ||
								fieldOldCPP.getType().getName().equals("boolean")) {
							fieldOldCPP.setBoolean(oldPP, Boolean.valueOf(backendData.value).booleanValue());
						}
					}
				}
			}
		} catch (SecurityException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackEndDataOfConceptoPeticion.SecurityException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackEndDataOfConceptoPeticion.NoSuchFieldException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			throw new ObjectTransformerException("getBackEndDataOfConceptoPeticion.IllegalAccessException::No se pueden obtener el BackEndDataMap para los Conceptos de Peticion de Pago.Revisa la configuracion de los campos");
		}
		return oldPP;
	}
	
	/**
	 * Mtodo que obtiene un objeto Liquidacion a partir de un objeto
	 * Peticion de Pago Antiguo.
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return Map : Liquidacion
	 */
	private static Liquidacion _getNewLiquidacionPeticionPago (p12f.exe.pasarelapagos.v1.objects.Liquidacion oldLiquidacion){
		
		Liquidacion liqui = new Liquidacion();
		liqui.paymentGatewayVersion = "1";
		if(oldLiquidacion.plantillasLiquidacion!=null) {
			if(oldLiquidacion.plantillasLiquidacion.get("pdf")!=null) {
				String plantilla = ((InetAddr)oldLiquidacion.plantillasLiquidacion.get("pdf")).urlCastellano;
				String cadena1 = plantilla.substring(0, plantilla.indexOf("/aplic/p72/html"));
				// Establece la ruta correcta para las plantillas en softbase-desarrollo 
				//String cadena2 = "/serapp/softbase_ejie/datos/p12i/file";
				// Establece la ruta correcta para las plantillas en produccion 
				String cadena2 = "/datos/p12i/file";
				String cadena3 = plantilla.substring(plantilla.indexOf("/xml/"));
				
				liqui.urlPlantilla = cadena1 + cadena2 + cadena3;
			}
		}
		
		// Imagenes.
		liqui.imagenes = _getNewImages(oldLiquidacion.imagenes);
		
		return liqui;
	}
	
	private static p12f.exe.pasarelapagos.v1.objects.Liquidacion _getOldLiquidacionPeticionPago (Liquidacion newLiquidacion){
		
		p12f.exe.pasarelapagos.v1.objects.Liquidacion liqui = new p12f.exe.pasarelapagos.v1.objects.Liquidacion();
		
		// Imagenes.
		liqui.imagenes = _getOldImageMap(newLiquidacion.imagenes);
		
		return liqui;
	}
	
	/**
	 * Mtodo que obtiene un objeto Emisor a partir de un objeto
	 * Peticion de Pago Antiguo.
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return Emisor
	 */
	private static p12f.exe.pasarelapagos.objects.Emisor _getNewEmisor (p12f.exe.pasarelapagos.v1.objects.Emisor oldEmisor){
		
		p12f.exe.pasarelapagos.objects.Emisor emisor = new p12f.exe.pasarelapagos.objects.Emisor();
		emisor.calle = oldEmisor.calle;
		emisor.cif = oldEmisor.cif;
		emisor.codigoPostal = oldEmisor.codigoPostal;
		emisor.municipio = oldEmisor.municipio;
		emisor.nombre = new HashMap();
		emisor.nombre.put(IDIOMA_ES,oldEmisor.nombreCastellano);
		emisor.nombre.put(IDIOMA_EU,oldEmisor.nombreEuskera);
		emisor.territorio = oldEmisor.territorio;
		return emisor;
	}
	
	private static p12f.exe.pasarelapagos.v1.objects.Emisor _getOldEmisor (p12f.exe.pasarelapagos.objects.Emisor newEmisor, String codigo){
		StringTokenizer st = new StringTokenizer(codigo);
		String cif = st.nextToken("-");
		//String sufijo = st.nextToken("-");
		
		p12f.exe.pasarelapagos.v1.objects.Emisor emisor = new p12f.exe.pasarelapagos.v1.objects.Emisor();
		emisor.calle = newEmisor.calle;
		emisor.cif = cif;
		emisor.codigo = codigo;
		emisor.codigoPostal = newEmisor.codigoPostal;
		emisor.municipio = newEmisor.municipio;
		emisor.nombreCastellano = (String)newEmisor.nombre.get(IDIOMA_ES);
		emisor.nombreEuskera = (String)newEmisor.nombre.get(IDIOMA_EU);
		emisor.territorio = newEmisor.territorio;
		return emisor;
	}
	
	/**
	 * Mtodo que obtiene un objeto Expediente a partir de un objeto
	 * Peticion de Pago Antiguo.
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return Emisor
	 */
	private static Expediente _getNewExpediente (p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago){
	
		Expediente expediente = new Expediente();
		expediente.codigo = oldPeticionPago.expedienteAsociado; // Es correcta esta equivalencia?
		expediente.descripcion = new HashMap();
		if(oldPeticionPago.tercero!=null) {
			expediente.tercero = new p12f.exe.pasarelapagos.objects.Tercero();
			expediente.tercero.calle = oldPeticionPago.tercero.domicilio;
			expediente.tercero.codigoPostal = oldPeticionPago.tercero.codigoPostal;
			expediente.tercero.dniNif = oldPeticionPago.tercero.dniNif;
			expediente.tercero.municipio= oldPeticionPago.tercero.municipio;
			expediente.tercero.pais = oldPeticionPago.tercero.pais;
			expediente.tercero.territorio = oldPeticionPago.tercero.provincia;
			expediente.tercero.razonSocial = oldPeticionPago.tercero.razonSocial;
			// Datos Adicionales.
			expediente.tercero.datosAdicionales = _getNewDatosAdicionales(oldPeticionPago.tercero.datosAdicionales);
		}
		return expediente;
	}
	
	/**
	 * Mtodo que obtiene un objeto Expediente a partir de un objeto
	 * Peticion de Pago Antiguo.
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return Emisor
	 */
	private static p12f.exe.pasarelapagos.v1.objects.Tercero _getOldTercero (p12f.exe.pasarelapagos.objects.Tercero newTercero){
		
		p12f.exe.pasarelapagos.v1.objects.Tercero oldTercero = new p12f.exe.pasarelapagos.v1.objects.Tercero();
		oldTercero.codigoPostal = newTercero.codigoPostal;
		oldTercero.razonSocial = newTercero.razonSocial;
		oldTercero.dniNif = newTercero.dniNif;
		oldTercero.domicilio = newTercero.calle;
		oldTercero.localidad = newTercero.municipio;
		oldTercero.municipio = newTercero.municipio;
		oldTercero.pais = newTercero.pais;
		oldTercero.provincia = newTercero.territorio;
		// Datos Adicionales.
		oldTercero.datosAdicionales = _getOldDatosAdicionales(newTercero.datosAdicionales);
		return oldTercero;	
	}
	
	/**
	 * Mtodo que obtiene un objeto Aplicacion a partir de un objeto
	 * Peticion de Pago Antiguo.
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return Aplicacion
	 */
	private static Aplicacion _getNewAplicacion (p12f.exe.pasarelapagos.v1.objects.Aplicacion oldPeticionPagoAPP){
		Aplicacion aplicacion = new Aplicacion();
		if(oldPeticionPagoAPP!=null) {
			aplicacion.nombre = new HashMap();
			aplicacion.nombre.put(IDIOMA_ES, oldPeticionPagoAPP.castellano);
			aplicacion.nombre.put(IDIOMA_EU, oldPeticionPagoAPP.euskera);
			// Se pasa siempre el codigo de aplicacion a minusculas para evitar posibles errores
			aplicacion.codigo = oldPeticionPagoAPP.codigo.toLowerCase();
			if(aplicacion.codigo.equals("p72")) {
				aplicacion.codigo = "p12";
			}
			aplicacion.responsable = oldPeticionPagoAPP.responsable;
		}
		
		// Se configura el EventListener.
		// La pasarela V1 solo responde a los siguientes eventos:
		//	- Pago pagado en la entidad financiera correctamente.
		//	- Error al realizar el pago en la entidad financiera.
		// 	- Pago cancelado.
		//  - Liquidacion emitida.
		aplicacion.eventListenerConfig.eventListenerMethodBitMap = "0000000000";
		aplicacion.eventListenerConfig.eventListenerClass = "p12g.exe.pasarelapagos.events.DefaultEventListenerImpl";
		aplicacion.eventListenerConfig.properties = new HashMap();
		if(oldPeticionPagoAPP!=null && oldPeticionPagoAPP.urlVueltaResultadoPago!=null){
			aplicacion.eventListenerConfig.properties.put("urlClient", oldPeticionPagoAPP.urlVueltaResultadoPago);
		}
		aplicacion.eventListenerConfig.properties.put("paymentGatewayVersion", "1");
		return aplicacion;
	}
	
	/**
	 * Mtodo que obtiene un mapa de datos adicionales del objeto Aplicacion
	 * @param  PeticionPago : ConceptoPeticionPago Pararela Antigua.
	 * @return Map : Mapa con los datos adicionales.
	 */
	private static Map _setDatosAdicionales(p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago) throws ObjectTransformerException {
		Map datosAdicionales = new HashMap();
		
		// URL de vuelta
		if(oldPeticionPago.protocolo.urlVuelta!=null) {
			p12f.exe.pasarelapagos.objects.DatoAdicional datoAdicional = new p12f.exe.pasarelapagos.objects.DatoAdicional();
			datoAdicional.id = URL_VUELTA;
			datoAdicional.valor = oldPeticionPago.protocolo.urlVuelta;
			datosAdicionales.put(datoAdicional.id, datoAdicional);
		}
		// Idioma 
		if(oldPeticionPago.presentacion.idioma!=null) {
			p12f.exe.pasarelapagos.objects.DatoAdicional datoAdicional = new p12f.exe.pasarelapagos.objects.DatoAdicional();
			datoAdicional.id = IDIOMA;
			datoAdicional.valor = oldPeticionPago.presentacion.idioma;
			datosAdicionales.put(datoAdicional.id, datoAdicional);
		}
		
		return datosAdicionales;
	}
	
	/**
	 * Mtodo que obtiene un objeto Aplicacion antiguo a partir de un objeto
	 * APlicacion nuevo.
	 * @param  Aplicacion : Aplicacion Pararela Antigua.
	 * @return Aplicacion
	 */
	private static p12f.exe.pasarelapagos.v1.objects.Aplicacion _getOldAplicacion (Aplicacion newPeticionPagoAPP){
		p12f.exe.pasarelapagos.v1.objects.Aplicacion aplicacion = new p12f.exe.pasarelapagos.v1.objects.Aplicacion();
		if(newPeticionPagoAPP.nombre!=null){
			aplicacion.castellano = (String)newPeticionPagoAPP.nombre.get(IDIOMA_ES);
			aplicacion.euskera = (String)newPeticionPagoAPP.nombre.get(IDIOMA_EU);
		}
		if(newPeticionPagoAPP.codigo!=null) {
			// Se pasa siempre el codigo de aplicacion a minusculas para evitar posibles errores
			aplicacion.codigo = newPeticionPagoAPP.codigo.toLowerCase();
		}
		if(newPeticionPagoAPP.responsable!=null) {
			aplicacion.responsable = newPeticionPagoAPP.responsable;
		}
		if (( newPeticionPagoAPP.eventListenerConfig != null ) && (newPeticionPagoAPP.eventListenerConfig.properties != null)) {
			if(newPeticionPagoAPP.eventListenerConfig.properties.get("url")!=null) {
				aplicacion.urlVueltaResultadoPago = (String)newPeticionPagoAPP.eventListenerConfig.properties.get("urlClient");
			}
		}
		return aplicacion;
	}
	
	/**
	 * Mtodo que obtiene un objeto ProtocolData a partir de un objeto
	 * Peticion de Pago Antiguo.
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return ProtocolData
	 */
	public static ProtocolData getNewProtocolData (p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago){
		
		p12f.exe.pasarelapagos.v1.objects.Protocolo oldProtocolo = oldPeticionPago.protocolo;
		ProtocolData protocolData  = new ProtocolData();
		protocolData.token = oldProtocolo.token;
		protocolData.responseURL = null;
		protocolData.sourceSessionId = oldProtocolo.sessionID;
		protocolData.destinationSessionId = oldProtocolo.sessionID;
		try {	
			if(oldProtocolo.timeStamp!=null){
				Calendar fecha = Calendar.getInstance();
				StringTokenizer st = new StringTokenizer(oldProtocolo.timeStamp.trim(), "/[:");
				int numTokens = st.countTokens();
				if(numTokens>=3 && numTokens<=4){
					int dia = Integer.parseInt(st.nextToken());
					int mes = Integer.parseInt(st.nextToken());
					int anio = Integer.parseInt(st.nextToken());
					
					fecha.set(anio, mes, dia);
				}
				if(numTokens>=6){
					int dia = Integer.parseInt(st.nextToken().trim());
					int mes = Integer.parseInt(st.nextToken().trim());
					int anio = Integer.parseInt(st.nextToken().trim());
					int hora = Integer.parseInt(st.nextToken().trim());
					int minuto = Integer.parseInt(st.nextToken().trim());
					String segundos =  st.nextToken().trim();
					int segundo;
					if (segundos.indexOf(".") > 0 ) {
						StringTokenizer stSeconds = new StringTokenizer(segundos, ".");
						segundo = Integer.parseInt(stSeconds.nextToken().trim());
					} else {
						String secPa = segundos.substring(0,2);
						segundo = Integer.parseInt(secPa);
					}
					fecha.set(anio, mes-1, dia, hora, minuto, segundo);
				}
				protocolData.timeStamp = fecha.getTimeInMillis();
			}
		} catch(Exception ex) {
			ex.printStackTrace();
		}
		protocolData.sourceOperationNumber = oldProtocolo.numeroOperacion;
		protocolData.urls = new HashMap();
		// URL Vuelta
		p12f.exe.pasarelapagos.objects.Url urlVuelta = new p12f.exe.pasarelapagos.objects.Url();
		urlVuelta.url = oldProtocolo.urlVuelta;
		urlVuelta.id = PROTOCOLO_URL_VUELTA;		
		protocolData.urls.put(PROTOCOLO_URL_VUELTA, urlVuelta);
		// URL Resultado
		p12f.exe.pasarelapagos.objects.Url urlResultado = new p12f.exe.pasarelapagos.objects.Url();
		urlResultado.url = oldProtocolo.urlResultado;	
		urlResultado.id = PROTOCOLO_URL_RESULTADO;
		protocolData.urls.put(PROTOCOLO_URL_RESULTADO, urlResultado);
		// URL Validacion
		p12f.exe.pasarelapagos.objects.Url urlValidacion = new p12f.exe.pasarelapagos.objects.Url();
		urlValidacion.url = oldProtocolo.urlValidacion;
		urlValidacion.id = PROTOCOLO_URL_VALIDACION;
		protocolData.urls.put(PROTOCOLO_URL_VALIDACION, urlValidacion);
		
		return  protocolData;
	}
	
	/**
	 * Mtodo que obtiene un objeto ProtocolData antiguo  a partir de un objeto
	 * Protocol Nuevo
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return ProtocolData antiguo
	 */
	private static Protocolo _getOldProtocolData (PeticionPago newPeticionPago, ProtocolData newProtocolData){
		
		Protocolo protocolData  = new Protocolo();
		//PeticionPago.getPagoOID(newCodigoEmisor,newReferencia,newIdentificacion,newImporte)
		
		PeriodoPago newPeriodoPago = (PeriodoPago)newPeticionPago.datosPago.periodosPago.values().iterator().next();
		
		protocolData.pagoID = 	p12f.exe.pasarelapagos.v1.objects.PeticionPago.getPagoOID(newPeticionPago.datosPago.emisor,
				newPeticionPago.datosPago.referencia, 
				newPeriodoPago.identificacion,
				getOldImporte(newPeticionPago));
		
		protocolData.token = newProtocolData.token;
		protocolData.sessionID = newProtocolData.sourceSessionId;
		
		Date fecha = new Date();
		fecha.setTime(newProtocolData.timeStamp);
		protocolData.timeStamp = dateFormatter2.format(fecha);
		
		// Siempre se manda el mismo y lo necesita el BBVA
		protocolData.numeroOperacion = "0001";
		
		if(newProtocolData.urls.get(PROTOCOLO_URL_VUELTA)!=null){
			p12f.exe.pasarelapagos.objects.Url urlVuelta = (p12f.exe.pasarelapagos.objects.Url)newProtocolData.urls.get(PROTOCOLO_URL_VUELTA);
			protocolData.urlVuelta = urlVuelta.url;
		}
		protocolData.urlValidacion = XMLProperties.get("p12f","urls/urlValidacion");
		protocolData.urlResultado = XMLProperties.get("p12f","urls/urlResultado");
		
		return  protocolData;
	}
	
	/**
	 * Convierte de un map de antiguas entidades financieras a un map con las nuevas.
	 * @param oldEntidadesFinancieras
	 * @return
	 */
	private static Map _getNewFinantialOrgs(List oldEntidadesFinancieras){
		// Copia de Financial Orgs.
		HashMap finantialOrgs = new HashMap();
		if ( oldEntidadesFinancieras!= null )
			for (Iterator it = oldEntidadesFinancieras.iterator();it.hasNext();){
				EntidadFinanciera entidadFinancieraOld = (EntidadFinanciera) it.next();
				if(entidadFinancieraOld.id!=null) {
					FinantialOrg financialOrg = new FinantialOrg();
					financialOrg.oid = entidadFinancieraOld.id;
					financialOrg.text = entidadFinancieraOld.descripcion;
					financialOrg.def = entidadFinancieraOld.def;
					finantialOrgs.put(financialOrg.oid,financialOrg);
				}
			}
		return finantialOrgs;
	}
	
	/**
	 * Convierte de un map de antiguos modos de pago a un map con los nuevos.
	 * @param oldEntidadesFinancieras
	 * @return
	 */
	private static Map _getNewPaymentModes(List oldPaymentModes){
		Map paymentModes = new HashMap();
		if ( oldPaymentModes != null ) {
			for (Iterator it = oldPaymentModes.iterator();it.hasNext();){
				MedioPago oldMedioPago = (MedioPago)it.next();
				if(oldMedioPago.id!=null){
					PaymentMode newMedioPago = new PaymentMode();
					newMedioPago.oid = oldMedioPago.id;
					paymentModes.put(newMedioPago.oid,newMedioPago);
				}
			}
		}
		return paymentModes;
	}
	
	/**
	 * Mtodo que obtiene un objeto PresentationData antiguo a partir de un objeto
	 * PresentationData antig
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return PresentationData nueva
	 */
	
	public static PresentationRequestData getNewPresentationRequestData(p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPeticionPago){
		
		PresentationRequestData  newPresentationData = new PresentationRequestData();
		Presentacion oldPresentationData = oldPeticionPago.presentacion;
		newPresentationData.idioma = oldPresentationData.idioma;
		newPresentationData.imagenes = new HashMap();
	
		// Copia de imagenes.
		if ( oldPresentationData.imagenes != null ) {
			for (Iterator it = oldPresentationData.imagenes.values().iterator();it.hasNext(); ){
				Imagen imagenOld = (Imagen) it.next();
				p12f.exe.pasarelapagos.objects.Imagen newImagen = new p12f.exe.pasarelapagos.objects.Imagen();
				newImagen = new p12f.exe.pasarelapagos.objects.Imagen ();
				newImagen.id = imagenOld.id;
				//Imp.Cambiar en la defincion de la clase a un byte[]
				if (imagenOld.bin != null) {
					newImagen.bin = imagenOld.bin.toString();
				}
				newImagen.url = imagenOld.url;
				newPresentationData.imagenes.put(newImagen.id,newImagen);
			}
		}
		// Copia de mensajes.
		newPresentationData.mensajes = new HashMap();
		if ( oldPeticionPago.mensajes != null){
			for (Iterator it = oldPeticionPago.mensajes.values().iterator();it.hasNext(); ){
				Mensaje oldMensaje = (Mensaje) it.next();
				p12f.exe.pasarelapagos.objects.Mensaje newMensaje = new p12f.exe.pasarelapagos.objects.Mensaje();
				newMensaje.texto = new HashMap();
				newMensaje.texto.put(IDIOMA_ES,oldMensaje.castellano);
				newMensaje.texto.put(IDIOMA_EU,oldMensaje.euskera);
				newMensaje.id = oldMensaje.id;
				newPresentationData.mensajes.put(newMensaje.id,newMensaje);
			}
		}
		return newPresentationData;
	}
	
	/**
	 * Mtodo que obtiene un objeto Presentacion  antiguo  a partir de un objeto
	 * PresentationData Nuevo
	 * @param  PeticionPago : PeticionPago Pararela Antigua.
	 * @return PresentationData nueva
	 */
	private static Presentacion _getOldPresentationRequestData(PresentationRequestData newPresentationData){
		
		Presentacion oldPresentationData = new Presentacion();
		oldPresentationData.idioma = newPresentationData.idioma;	
		oldPresentationData.instance = OLD_PAYMENT_DEFAULT_INSTANCE;
		newPresentationData.idioma = oldPresentationData.idioma;
		oldPresentationData.imagenes = new HashMap();
	
		// Copia de imagenes.
		if ( newPresentationData.imagenes != null ){
			for (Iterator it = newPresentationData.imagenes.values().iterator();it.hasNext(); ){
				p12f.exe.pasarelapagos.objects.Imagen newImagen  = (p12f.exe.pasarelapagos.objects.Imagen ) it.next();
				Imagen oldImagen = new Imagen();
				
				oldImagen.id = newImagen.id;
				//Imp.Cambiar en la defincion de la clase a un byte[]
				if (newImagen.bin != null) {
					oldImagen.bin = newImagen.bin.getBytes();
				}
				oldImagen.url = newImagen.url;
				oldPresentationData.imagenes.put(oldImagen.id,oldImagen);
			}
		}
		// Copia de Financial Orgs.
		oldPresentationData.entidadesFinancieras = new ArrayList();
		if ( newPresentationData.finantialOrgs != null ) {
			for (Iterator it = newPresentationData.finantialOrgs.values().iterator();it.hasNext();){
				FinantialOrg financialOrg = (FinantialOrg)it.next();
				EntidadFinanciera entidadFinancieraOld = new EntidadFinanciera();
				entidadFinancieraOld.id = financialOrg.oid;
				entidadFinancieraOld.descripcion = financialOrg.text;
				entidadFinancieraOld.def = financialOrg.def;
				oldPresentationData.entidadesFinancieras.add(entidadFinancieraOld);
			}
		}
		// Copia de modos de Pago.
		oldPresentationData.mediosPago = new ArrayList();
		if (( newPresentationData.paymentModes != null ) && ( newPresentationData.paymentModes.size() > 0)) {
			for (Iterator it = newPresentationData.paymentModes.values().iterator();it.hasNext();){
				PaymentMode newMedioPago = (PaymentMode)it.next();
				MedioPago oldMedioPago = new MedioPago();
				oldMedioPago.id = newMedioPago.oid;
				oldPresentationData.mediosPago.add(oldMedioPago);
			}
		} else {
			// Como minimo pago ON-LINE
			MedioPago oldMedioPago = new MedioPago();
			oldMedioPago.id = MedioPago.BANCA_ONLINE;
			oldPresentationData.mediosPago.add(oldMedioPago);
		}
		return oldPresentationData;
	}
	
	/**
	 * Mtodo que obtiene un Map de los  antiguos objetos Mensaje de la Peticon de Pago y que
	 * en la nueva versin se colocan el objeto Presentation
	 * PresentationData Nuevo
	 * @param   Map con mensajes nuevo.
	 * @return  Map con mensajes antiguos.
	 */
	private static Map _getOldMessageMap (Map newMessageMap){
		
		Map oldMensajes = new HashMap();
		if ( newMessageMap != null){
			for (Iterator it = newMessageMap.values().iterator();it.hasNext(); ){
				p12f.exe.pasarelapagos.objects.Mensaje newMensaje = (p12f.exe.pasarelapagos.objects.Mensaje) it.next();
				Mensaje  oldMessage = new Mensaje();
				oldMessage.id = newMensaje.id;
				oldMessage.castellano = (String)newMensaje.texto.get(IDIOMA_ES);
				oldMessage.euskera = (String)newMensaje.texto.get(IDIOMA_EU);
				oldMensajes.put(oldMessage.id,oldMessage);
			}
		}
		return oldMensajes;
		
	}
	
	/**
	 * Mtodo que obtiene un Map de los  antiguos objetos Imagen de la Peticon de Pago
	 * PresentationData Nuevo
	 * @param   Map con imagenes nuevas.
	 * @return  Map con imagenes antiguas.
	 */
	private static Map _getOldImageMap (Map newImageMap){
		Map oldImagenes = new HashMap();
		if ( newImageMap != null){
			for (Iterator it = newImageMap.values().iterator();it.hasNext(); ){
				p12f.exe.pasarelapagos.objects.Imagen newImagen = (p12f.exe.pasarelapagos.objects.Imagen) it.next();
				Imagen  oldImagen = new Imagen();
				oldImagen.id = newImagen.id;
				oldImagen.url = newImagen.url;
				oldImagenes.put(oldImagen.id,oldImagen);
			}
		}
		return oldImagenes;	
	}
	
	/**
	 * Obtener el importe para la antigua Peticion de Pago.
	 * @param peticionPago
	 * @return
	 */
	private static double getOldImporte(PeticionPago peticionPago){
		PeriodoPago periodoPago = (PeriodoPago) peticionPago.datosPago.periodosPago.values().iterator().next();
		return (new Long(periodoPago.importe).doubleValue())/100;
	}

	/**
	 * Obtener el importe para la nueva Peticion de Pago.
	 * @param Pago
	 * @return
	 */
	private static long getNewImporte(p12f.exe.pasarelapagos.v1.objects.PeticionPago peticionPago) {
		double dblCantidad = peticionPago.importe;
		return MoneyUtils.getCenteuros(dblCantidad);
	}
	
	/**
	 * Obtener las descripciones para la nueva Peticion de Pago.
	 * @param Pago
	 * @return
	 */
	private static Map _getNewDescripcion(p12f.exe.pasarelapagos.v1.objects.Pago oldPago) {
		Map descripcion = new HashMap();
		if(oldPago.descripcion.castellano!=null) descripcion.put(IDIOMA_ES, oldPago.descripcion.castellano);
		if(oldPago.descripcion.euskera!=null) descripcion.put(IDIOMA_EU, oldPago.descripcion.euskera);
		return descripcion;
	}
	
	/**
	 * Obtener las descripciones para la nueva Peticion de Pago.
	 * @param peticionPago
	 * @return
	 */
	private static Map _getNewDescripcion(p12f.exe.pasarelapagos.v1.objects.PeticionPago oldPago) {
		Map descripcion = new HashMap();
		if(oldPago.descripcion.castellano!=null) descripcion.put(IDIOMA_ES, oldPago.descripcion.castellano);
		if(oldPago.descripcion.euskera!=null) descripcion.put(IDIOMA_EU, oldPago.descripcion.euskera);
		return descripcion;
	}
	
	/**
	 * Obtener el estado de un pago a partir de una estructura ResultadoPago.
	 * @param ResultadoPago
	 * @return
	 */
	public static Estado getNewEstado(ResultadoPago resultadoPago) {
		Estado estado = new Estado();
		try {
			if (resultadoPago.resultado) {
				estado.codigo = Estado.PAGADO;
			} else {
				estado.codigo = Estado.ERROR_PAGO;
			}
			String fecha = resultadoPago.datosOperacion.fecha;
			estado.fechaPago = _formateOldDate(fecha);
			String hora = resultadoPago.datosOperacion.hora;
			estado.horaPago = _formateOldHour(hora);
			
			estado.entidad = resultadoPago.datosOperacion.entidadFinanciera.id;
			
			String importe = resultadoPago.datosOperacion.importe;
			while(importe.length()<10) { importe = "0" + importe; }
			estado.importe = importe;
			estado.mensajes = _getNewMessageMap(resultadoPago.mensajes);
			
		} catch(Exception ex) {
			ex.printStackTrace();
		}
		return estado;
	}
	
	/**
	 * Mtodo que obtiene un Map de los antiguos objetos Mensaje de la Peticon de Pago y que
	 * en la nueva versin se colocan el objeto Presentation
	 * PresentationData Nuevo
	 * @param   Map con mensajes nuevo.
	 * @return  Map con mensajes antiguos.
	 */
	private static Map _getNewMessageMap (List oldMessageList){
		Map newMensajes = new HashMap();
		if ( oldMessageList != null){
			int i = 0;
			for (Iterator it = oldMessageList.iterator();it.hasNext(); ){
				Mensaje oldMessage = (Mensaje)it.next();
				p12f.exe.pasarelapagos.objects.Mensaje newMessage = new p12f.exe.pasarelapagos.objects.Mensaje();
				newMessage.id = new Integer(i).toString(); //oldMessage.id;
				newMessage.texto.put(IDIOMA_ES, oldMessage.castellano);
				newMessage.texto.put(IDIOMA_EU, oldMessage.euskera);
				
				newMensajes.put(newMessage.id,newMessage);
			}
		}
		return newMensajes;
	}
	
	/**
	 * Mtodo que obtiene un Map de mensajes nuevos a partir de un Map de los antiguos objetos Mensaje
	 * PresentationData Nuevo
	 * @param   Map con mensajes nuevo.
	 * @return  Map con mensajes antiguos.
	 */
	private static Map _getNewMessageMap (Map oldMessageList){
		Map newMensajes = new HashMap();
		if ( oldMessageList != null){
			for (Iterator it = oldMessageList.keySet().iterator();it.hasNext(); ){
				Mensaje oldMessage = (Mensaje)oldMessageList.get(it.next());
				p12f.exe.pasarelapagos.objects.Mensaje newMessage = new p12f.exe.pasarelapagos.objects.Mensaje();
				newMessage.id = oldMessage.id;
				newMessage.texto.put(IDIOMA_ES, oldMessage.castellano);
				newMessage.texto.put(IDIOMA_EU, oldMessage.euskera);
				
				newMensajes.put(newMessage.id,newMessage);
			}
		}
		return newMensajes;
	}
	
	/**
	 * Metodo que convierte un objeto PaymentSateData de la version 2 de la Pasarela de Pagos
	 * a un objeto ResultadoPago de la version 1.
	 * @param paymentStateData
	 * @return
	 */
	public static ResultadoPago getOldResultadoPago(PaymentStateData paymentStateData){
		ResultadoPago resultadoPago = new ResultadoPago();
		
		// Objeto Resultado que informa del resultado de la operacion
		resultadoPago.resultado = true;
		
		resultadoPago.datosOperacion = new DatosOperacion();
		// Objeto DatosOperacion que informa sobre los datos referentes al pago
		// Se comprueba si el pago ha sido pagado
		if(paymentStateData.estado.codigo.equals(Estado.PAGADO)){
			// Fecha en que se realizo el pago
			resultadoPago.datosOperacion.fecha = _formatearFecha(paymentStateData.estado.fechaPago);
			// Hora en que se realizo el pago
			resultadoPago.datosOperacion.hora = _formatearHora(paymentStateData.estado.horaPago);
			// Entidad financiera en la que se realizo el pago
			EntidadFinanciera entidadFinanciera = new EntidadFinanciera();
			entidadFinanciera.id = paymentStateData.estado.entidad;
			entidadFinanciera.descripcion = "Entidad " + paymentStateData.estado.entidad;
			resultadoPago.datosOperacion.entidadFinanciera = entidadFinanciera;
			// Numero de operacion en la entidad financiera
			resultadoPago.datosOperacion.numeroOperacion = paymentStateData.estado.numeroOperacion;
		}
		// Codigo emisor del pago
		resultadoPago.datosOperacion.codigoEmisor = paymentStateData.datosPago.emisor;
		// Referencia del pago
		resultadoPago.datosOperacion.referencia = paymentStateData.datosPago.referencia;
		// Identificacion e importe para el periodo de validez del pago
		PeriodoPago periodoNormal = (PeriodoPago)paymentStateData.datosPago.periodosPago.get(PeriodoPago.PERIODO_NORMAL);
		resultadoPago.datosOperacion.identificacion = periodoNormal.identificacion;
		resultadoPago.datosOperacion.importe = Long.toString(periodoNormal.importe);
		
		// Objeto EstadoPago que informa del estado del pago
		if(paymentStateData.estado.codigo.equals(Estado.PAGADO)){
			resultadoPago.estadoPago = new EstadoPago(EstadoPago.PAGO_ONLINE_OK);
		} else {
			resultadoPago.estadoPago = new EstadoPago(EstadoPago.PAGO_ONLINE_NOK);
		}
		
		// Objeto Mensajes que agrupa los mensajes del resultado del pago.
		Map mensajes = paymentStateData.estado.mensajes;
		if(mensajes!=null && mensajes.size()>0) {
			resultadoPago.mensajes = new ArrayList();
			for(Iterator it=mensajes.keySet().iterator(); it.hasNext();){
				p12f.exe.pasarelapagos.objects.Mensaje mensaje = (p12f.exe.pasarelapagos.objects.Mensaje)mensajes.get(it.next());
				Mensaje mensajeV1 = new Mensaje();
				mensajeV1.id = mensaje.id;
				mensajeV1.castellano = (String)mensaje.texto.get("es");
				mensajeV1.euskera = (String)mensaje.texto.get("eu");
			}
		}
		
		resultadoPago.protocolo = new Protocolo();
		resultadoPago.protocolo.pagoID = getOldOIDFormat(paymentStateData.id);
		
		return resultadoPago;
	}
	
	/**
	 * Metodo privado para convertir la fecha al formato que maneja la version 1
	 * @param fecha
	 * @return
	 */
	private static String _formatearFecha(String fecha){
		return (fecha.substring(0,2) + "/" + fecha.substring(2,4) + "/" + fecha.substring(4,6));
	}
	
	/**
	 * Metodo privado para convertir la hora al formato que maneja la version 1
	 * @param fecha
	 * @return
	 */
	private static String _formatearHora(String hora){
		return (hora.substring(0,2) + ":" + hora.substring(2,4) + ":" + hora.substring(4,6));
	}
	
	
	/**
	 * Mtodo que obtiene una Lista de los antiguos Datos Adicionales de la Peticoon de Pago
	 * PresentationData Nuevo
	 * @param   List con Datos Adicionales nuevos.
	 * @return  Map con Datos Adicionales antiguos.
	 */
	private static List _getOldDatosAdicionales (List newDatosAdicionales){
		List oldDatosAdicionales = new ArrayList();
		if ( newDatosAdicionales != null ){
			for (Iterator it = newDatosAdicionales.iterator(); it.hasNext() ; ){
				p12f.exe.pasarelapagos.objects.DatoAdicional newDatoAdicional = (p12f.exe.pasarelapagos.objects.DatoAdicional) it.next();
				DatoAdicional oldDatoAdicional = new DatoAdicional();
				oldDatoAdicional.id = newDatoAdicional.id;
				oldDatoAdicional.valor = newDatoAdicional.valor;
				oldDatoAdicional.castellano = (String)newDatoAdicional.texto.get("es");
				oldDatoAdicional.euskera = (String)newDatoAdicional.texto.get("eu");
				oldDatosAdicionales.add(oldDatoAdicional);
			}
		}
		return oldDatosAdicionales;
	}
	
	/**
	 * Transforma una hora en el formato viejo al formato nuevo
	 * @param currentHour
	 * @return
	 */
	private static String _formateOldHour(String currentHour) {
		try {
			StringTokenizer st = new StringTokenizer(currentHour,":");
			String  hora = st.nextToken();
			if (hora.length() == 1)  hora = "0" + hora;
			String  minutos = st.nextToken();  
			if (minutos.length() == 1)  minutos = "0" + minutos;
			String  segundos = st.nextToken();	 
			if (segundos.length()== 1) segundos = "0" + segundos;
			return hora + minutos + segundos;
		} catch (Exception ex) {
			ex.printStackTrace();
			return currentHour;
		}
	}
	
	/**
	 * Transforma una fecha en el formato nuevo al formato viejo
	 * @param currentDate
	 * @return
	 */
	private static String _formateOldDate(String currentDate) {
		try {
			StringTokenizer st = new StringTokenizer(currentDate,"/");
			String  dia = st.nextToken();
			if (dia.length() == 1)  dia = "0" + dia;
			String  mes = st.nextToken();  
			if (mes.length() == 1)  mes= "0" + mes;
			String  anyo = st.nextToken();
			if (anyo.length()== 1) {
				anyo = "0" + anyo;
			} else if (anyo.length()>2){
				anyo = anyo.substring(anyo.length()-2, anyo.length());
			}
			return dia + mes + anyo;
		} catch (Exception ex) {
			ex.printStackTrace();
			return currentDate;
		}
	}
	
	/*
	public static void main(String args[]) throws XOMarshallerException {

		String protocol = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?><protocolo><pagoID>0483300150901061007497310311064500</pagoID><timeStamp>24/10/2006 [17:09:43.45]</timeStamp><urlVuelta><![CDATA[http://www.hiru.com/ikastaroak/e-lea/izena_eman_bukatu?zentroa=4]]></urlVuelta><urlValidacion><![CDATA[https://www.ej-gv.net/p72/p72ppaServlet?module=if]]></urlValidacion><urlResultado><![CDATA[https://www.ej-gv.net/p72/p72ppaServlet?module=if]]></urlResultado></protocolo>";

		Protocolo protocolo  = Protocolo.getObject(protocol);
		p12f.exe.pasarelapagos.v1.objects.PeticionPago pp = new p12f.exe.pasarelapagos.v1.objects.PeticionPago();
		pp.protocolo  = protocolo;
		ProtocolData newProtocolData = getNewProtocolData(pp);
		System.out.println("NewProtocolData: " + newProtocolData.toXML());

		Protocolo oldProtocolo = _getOldProtocolData(null, newProtocolData);
		System.out.println("OldProtocolo: " + oldProtocolo.toXML());
	}
	*/
}