package p12f.exe.pasarelapagos.transformer;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import p12f.exe.pasarelapagos.paymentrequest.PaymentGatewayData;
import p12f.exe.pasarelapagos.paymentrequest.PeticionPago;
import p12f.exe.pasarelapagos.v1.objects.ResultadoOperacion;

import com.ejie.r01f.log.R01FLog;
import com.ejie.r01f.rpcdispatcher.RPCCall;
import com.ejie.r01f.rpcdispatcher.RPCClient;
import com.ejie.r01f.rpcdispatcher.RPCException;
import com.ejie.r01f.rpcdispatcher.RPCFunction;
import com.ejie.r01f.rpcdispatcher.RPCParameter;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 * Clase que implementa el BZD PaymentManagerAPI para la Pasarela Versin1, es decir cuando una aplicacin desee conectarse
 * contra la versin 1 de la Pasarela con el API de la versin 2
 * @author ie00191m
 *
 */

public class PaymentManagerAPIToV1 {

	
	/**
	 * Mtodo Server que crea las peticiones de pago a partir de un objeto PaymentRequestData
	 * @param  PaymentRequestData : Conjunto de Peticiones de Pago.
	 * @return PaymentRequestData : Conjunto de Peticiones de Pago inicializadas.
	 **/
	
	public PaymentGatewayData doExecInitializePayment( PaymentGatewayData paymentGatewayData) {
		R01FLog.to("p12f.transformer").finest(":::::::::::: CLIENT : PaymentManagerAPIToV1.createPayment ::::::::::::");
		ObjectTransformer transformer = new ObjectTransformer();	
		RPCCall rpcCall = null;
		try {
			
			// Transformar la peticion de pago.
			p12f.exe.pasarelapagos.v1.objects.PeticionPago peticionPago = transformer.transformToOldPaymentData(paymentGatewayData);
		
			// Llamada de Peticion de Pago.
			rpcCall = _createNewPeticionPagoRPCCALL(peticionPago); 
			//System.out.println(rpcCall.toXML());
			R01FLog.to("p12f.transformer").finest(rpcCall.toXML());	
			
			// Resultado de la operacion
			sendRPCCall(rpcCall);
			
			// Peticion de Pago Inicializada
		    PeticionPago peticionPagoInicializada = (PeticionPago)paymentGatewayData.paymentRequestData.peticionesPago.values().iterator().next();
		    peticionPagoInicializada.id = transformer.getNewOIDFormat(peticionPago.protocolo.pagoID);
		    paymentGatewayData.paymentRequestData.peticionesPago.put(peticionPagoInicializada.id,peticionPagoInicializada);
		
		    //System.out.println(paymentGatewayData.toXML());
		    
			return paymentGatewayData;
		} catch (ObjectTransformerException e) {		
			e.printStackTrace();
			return null;
		} catch (RPCException e) {
			e.printStackTrace();
			return null;
		} catch (XOMarshallerException e) {			
			e.printStackTrace();
			return null;
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	 
	}
	
	
	/**
	 * Crea una Peticion de Pago en la Pasarela Versin 1 desde el API de la Versin 2.
	 * @param peticionPago
	 * @return RPCCall
	 * @throws RPCException
	 * @throws XOMarshallerException
	 *  Ejemplo :	 *  
	 *  
		/**************************  Obtener Parmetros *******************************************
		  <param name="instanciaPasarela" type="String">1</param> 
		  <param name="emisor" type="String">04833001-904</param> 
		  <param name="referencia" type="String">1000000056708</param> 
		  <param name="identificacion" type="String">210105</param> 
		  <param name="importe" type="double">15.98</param> 
		  <param name="peticionPagoXML" type="XML">
		**************************************************************************************
	 */

	public RPCCall _createNewPeticionPagoRPCCALL (p12f.exe.pasarelapagos.v1.objects.PeticionPago peticionPago ) throws RPCException, XOMarshallerException{
		RPCCall  rpcCall = new RPCCall();
		rpcCall.setModule("ia");		
		RPCFunction function = new RPCFunction();
		function.setName("newPeticionPago");
		RPCParameter parameterInstancia = new RPCParameter("instanciaPasarela","String","1");
		RPCParameter parameterEmisor = new RPCParameter("emisor","String",	peticionPago.emisor.codigo);
		RPCParameter parameterReferencia = new RPCParameter("referencia","String",peticionPago.referencia);
		RPCParameter parameterIdentificacion = new RPCParameter("identificacion","String",peticionPago.identificacion);
		RPCParameter parameterImporte = new RPCParameter("importe","double",""+peticionPago.importe);
		RPCParameter parameterPeticionPagoXML = new RPCParameter("peticionPagoXML","XML",peticionPago.toXML());		
		
		function.putParameter(parameterInstancia);
		function.putParameter(parameterEmisor);
		function.putParameter(parameterReferencia);
		function.putParameter(parameterIdentificacion);
		function.putParameter(parameterImporte);
		function.putParameter(parameterPeticionPagoXML);
		
		rpcCall.addFunction(function);
		return rpcCall;
		
	}
	
	/**
	 * Envia una peticion RPC de Peticion de Pago a la Pasarela antigua.
	 * @param rpcCall
	 * @throws IOException 
	 */	
	public ResultadoOperacion sendRPCCall(RPCCall rpcCall ) throws IOException {		
		RPCClient rpcClient = null;
		String url = XMLProperties.get("p12f","integracionPasarelaV1/urlPasarela");	
		String proxyRequired = XMLProperties.getPropertyNode("p12f","integracionPasarelaV1/proxy/@required").getNodeValue() ;
	     if (proxyRequired.equalsIgnoreCase("true")){
	           String proxyHost = XMLProperties.getProperty("p12f","integracionPasarelaV1/proxy/host") ;
	           String proxyPort = XMLProperties.getProperty("p12f","integracionPasarelaV1/proxy/port") ;
	           rpcClient = new RPCClient(url,proxyHost,proxyPort);
	           if ( XMLProperties.getPropertyNode("p12f","integracionPasarelaV1/proxy/authorization") != null ){
	               String authorizationRequired = XMLProperties.getProperty("p12f","integracionPasarelaV1/proxy/authorization/@required") ;
	               if (authorizationRequired.equalsIgnoreCase("true")){
	                   String proxyUsr = XMLProperties.getProperty("p12f","integracionPasarelaV1/proxy/authorization/usr") ;
	                   String proxyPwd = XMLProperties.getProperty("p12f","integracionPasarelaV1/proxy/authorization/pwd") ;
	                   rpcClient.setProxyAuthorization(proxyUsr,proxyPwd);
	               }	           }
	       } else {
	    	   //::::::::::::  No se requiere Proxy  ::::::::::::::::::::::::::::::::
	    	   rpcClient = new RPCClient(url);
	       }		 
		  InputStream is =  rpcClient.sendRPCCall(rpcCall);
		  
		  /************************************* Tratamiento de Resultado de Vuelta *************************/
		  BufferedReader in = new BufferedReader(new InputStreamReader(is));
		  String returnString = new String();
		  String inputLine = new String();
		  
		  while ((inputLine = in.readLine()) != null){  	
			  returnString += inputLine;
		  } 
		  ResultadoOperacion resultadoOperacion = null;
		  try {
			 resultadoOperacion = ResultadoOperacion.getObject(returnString);		  
		  } catch (XOMarshallerException e) {
			  throw new IOException(e.getDetailedMessage());
		  }
		  return resultadoOperacion;
	}
	
	
	/**
	 * Tratar el retorno de la Pasarela Versin1.
	 * @param is
	 * @throws IOException 
	 */
	
	public void  _returnVale(InputStream is) throws IOException {		
		  
		
	
	    
	}
	
	
	
	
	

}
