package p12f.exe.pasarelapagos.utils;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.bouncycastle.util.encoders.Hex;

import p12f.exe.pasarelapagos.exceptions.ConfigLoadException;
import p12f.exe.pasarelapagos.objects.AccountData;
import p12f.exe.pasarelapagos.objects.HolderCertData;
import p12f.exe.pasarelapagos.objects.config.FinantialOrgConfig;
import p12f.exe.pasarelapagos.security.crypto.TripleDESEBCDICCryptoHelper;

import com.ejie.r01f.util.StringUtils;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;

/**
 * @author IE00191M
 * Utilidad de ayuda para la clase de validacin de cuentas.  Originalmente encriptaba en el NUCLEO, pero se ha pasado a P12J , ya que cuando
 * 
 * 
 * 
 *       IBAN
 *       ================
 *       
 *       El IBAN consta de un mximo de 34 caracteres alfanumricos. Los dos primeros son de carcter alfabtico e identifican el pas. Los dos siguientes son dgitos de control y son el elemento de validacin de la totalidad del IBAN. Los restantes son el nmero de cuenta, que en la mayora de los casos identifica adems la entidad y la oficina.

		En el caso espaol estn los cuatro primeros, formados por los elementos explicados en el prrafo anterior, y luego se sigue con los 20 caracteres numricos actuales del Cdigo Cuenta Cliente (es decir, consta de un total de 24 caracteres). Dichos cuatro nuevos caracteres corresponden a:
		
		Los dos primeros, al pas (ES para Espaa). 
		Los dos siguientes, a los dgitos de control IBAN. 
		Ejemplos: comparativa entre el cdigo cuenta cliente (CCC) y cdigo IBAN de una misma cuenta.
		
		Cdigo Cuenta Cliente (C.C.C.): 2077 0024 00 3102575766
		
		Cdigo IBAN: IBAN ES76 2077 0024 0031 0257 5766 (formato papel).
		
		Cdigo IBAN: ES7620770024003102575766 (formato electrnico).

 * 
 * 
 */
public class AccountDataUtil {
	
	public static String generateOID(String admin, String type , String reference) {
		
	    SimpleDateFormat sd = new SimpleDateFormat("ddMMyy");	
	    
		String identificacion = sd.format(new Date(System.currentTimeMillis()));
		String referenceWith00 = reference + "00";
		String dc = CSBHelpper.getDigitoControlCSB507(admin,type, referenceWith00,identificacion, "0"); 
		String oid =  admin + type + reference + dc + identificacion;
		String formattedString = StringUtils.lPadWithChar(oid, '0',19);
		
		return formattedString;
	}
	
	/**
	 * Obtiene el cdigo de entidad financera de los datos a validar.
	 * @param accountData
	 * @return
	 */
	public static String getFinantialOrgCode(AccountData nonEncryptedHolderCertData)throws Exception {
	
		
		String itemToCert = nonEncryptedHolderCertData.encData.value;

		
	   try {
			if (nonEncryptedHolderCertData.accountNumberFormat.equalsIgnoreCase(HolderCertData.ACCOUNT_NUMBER_FORMAT_CC)){				
				return  itemToCert.substring(0, 4);
			}
				
			else if (nonEncryptedHolderCertData.accountNumberFormat.equalsIgnoreCase(HolderCertData.ACCOUNT_NUMBER_FORMAT_IBAN)){
				return  itemToCert.substring(4, 8);
			}
			
		   } catch (Exception ex) {
			   throw new Exception("Error en el formato de cuenta :, pe. Cdigo IBAN: ES7620770024003102575766 (formato electrnico).\n \n  :"+ ex.getLocalizedMessage() + "@ itemToCert=" + itemToCert);
		   }
			
			
		
		return null;
				
	}

	/** Cifrar el dato a enviar a la entidad financiera
	 * @throws Exception */
	public static AccountData encrypt (AccountData accountData, FinantialOrgConfig  entidad) throws Exception{
		
		/**
		 * 	public String timeStamp;
			public EncData encData; 
			public Map aditionalDataList;
			public String accountNumberFormat
		 */
			
		AccountData encryptedAccountData = AccountData.getObject(accountData.toXML());

		try {
						
			String clave1 = (String)entidad.claves.get("accountPaymentKey1");
			String clave2 = (String)entidad.claves.get("accountPaymentKey2");
			
			if ((clave1 == null) ||  (clave2 == null) )
			    throw new Exception ("No se han definido las claves accountPaymentKey1 y  accountPaymentKey1 para la Entidad Financiera: " + entidad.oid );
						
			encryptedAccountData.encData.value = _encryptAccountData(clave1,clave2,accountData);				
			
		    return encryptedAccountData;
		
		} catch (XOMarshallerException e) {		
			throw new Exception(e);
		} catch (ConfigLoadException e) {
			throw new Exception(e);
		}
				
	}
	

	
	/** Descrifrar el dato a enviar a la administracin
	 * @throws Exception */
	public static HolderCertData decrypt (AccountData encryptedAccountData, FinantialOrgConfig  entidad) throws Exception{
			
		HolderCertData decryptedHolderCertData = new HolderCertData();

		try {
						
			String clave1 = (String)entidad.claves.get("accountPaymentKey1");
			String clave2 = (String)entidad.claves.get("accountPaymentKey1");
			
			if ((clave1 == null) ||  (clave2 == null) )
			    throw new Exception ("No se han definido las claves accountHolderKey1 y  accountHolderKey2 para la Entidad Financiera: " + entidad.oid );
		
			decryptedHolderCertData.itemToCertNumber = _decryptData(clave1,clave2,encryptedAccountData).trim();				
		    return decryptedHolderCertData;
		
		} catch (XOMarshallerException e) {		
			throw new Exception(e);
		} catch (ConfigLoadException e) {
			throw new Exception(e);
		}
				
	}
	

	
	/*******************************************************************************************/
    /*   Metodos Privados 																	   */
	/**
	 * @throws Exception *****************************************************************************************/
	/*
	private static String _encryptCreditCardData (String clave1, String clave2 ,HolderCertData holderCertData) throws Exception{
		String creditCardNumber  = holderCertData.itemToCertNumber;		
		String creditCardYear = ((AditionalData)holderCertData.aditionalDataList.get("creditCardYear")).value;		
		String creditCardMonth = ((AditionalData)holderCertData.aditionalDataList.get("creditCardMonth")).value;		
		
		creditCardNumber = StringUtils.rPadWithChar(creditCardNumber,' ',20); 				
		String data = creditCardNumber + "20" + creditCardYear + creditCardMonth;			
		// 2 Rellenar a 32 los datps finales a encriptar
		data = StringUtils.rPadWithChar(data,' ',32);
        // Datos variables.	
	
        String encType = holderCertData.encType;  
        
        String claveFinal  = new String(Hex.encode(TripleDESEBCDICCryptoHelper.doXOR(clave1, clave2))).toUpperCase();
        TripleDESEBCDICCryptoHelper encrypter = new TripleDESEBCDICCryptoHelper();	
        if ((encType!=null)&&(encType.equalsIgnoreCase("hex")))
        	return encrypter.encryptHEX(claveFinal,data);
        else return encrypter.encrypt(claveFinal,data); 
	}
	*/
	
	private static String _encryptAccountData (String clave1, String clave2 ,AccountData accountData) throws Exception{

		System.out.println("[P12F] AccountDataUtil._encryptAccountData ---- CIFRAR");
		System.out.println("======================================================");
		System.out.println("[P12F] >>>>> Semiclaves de entrada:");
		System.out.println("[P12F]      .Semiclave1: [" + clave1.substring(0,clave1.length()/2) + "...]");
		System.out.println("[P12F]      .Semiclave2: [" + clave2.substring(0,clave2.length()/2) + "...]");

		String encType = accountData.encData.encType;
		String data    = accountData.encData.value;

	    if  (accountData.accountNumberFormat.equalsIgnoreCase(AccountData.ACCOUNT_NUMBER_FORMAT_CC)){
	    	data = StringUtils.rPadWithChar(data,' ',32);
	    } else {
	    	data = StringUtils.rPadWithChar(data,' ',32);
	    }
		
	    System.out.println(">>>>>Trama final a encriptar (rellenar a 32 espacios por la derecha para lograr paridad x 8:"+ data  + "data.length()=" + data.length());
	 
        String claveFinal  = new String(Hex.encode(TripleDESEBCDICCryptoHelper.doXOR(clave1, clave2))).toUpperCase();
        System.out.println("[P12F] Clave final XOR: [" + claveFinal.substring(0,claveFinal.length()/2) + "...]");
        TripleDESEBCDICCryptoHelper encrypter = new TripleDESEBCDICCryptoHelper();	
        if ((encType!=null)&&(encType.equalsIgnoreCase("hex")))
        	return encrypter.encryptHEX(claveFinal,data);
        else return encrypter.encrypt(claveFinal,data); 
	}

	private static String _decryptData (String clave1, String clave2 ,AccountData accountData) throws Exception {
		
		
		String encType =  accountData.encData.encType;
		String data = 	  accountData.encData.value;
		
		String claveFinal  = new String(Hex.encode(TripleDESEBCDICCryptoHelper.doXOR(clave1, clave2))).toUpperCase();
		TripleDESEBCDICCryptoHelper encrypter = new TripleDESEBCDICCryptoHelper();
		
		if ((encType!=null)&&(encType.equalsIgnoreCase("hex")))
        	return encrypter.decryptHEX(claveFinal,data);
        else return encrypter.decrypt(claveFinal,data); 
	}
	
	
}

