package p12f.exe.pasarelapagos.utils;

import java.util.HashMap;

/*
 *  Clase que guarda la estructura de los los diferentes cuadernos.
 */
public class CSBConstants {

	
	/***************************************************************************************************
	 *     								  Implementacin trama  CSB507
	 ***************************************************************************************************/
	public static class CSB507 {
		
		public static Integer csbCode = new Integer(507);
        
		// Campos de Acceso Publico
		public static final CSBFIELD ID_APP =  new CSBFIELD("Identificativo APP",2,"90");
		public static final CSBFIELD FORMATO = new CSBFIELD("Tipo de Formato",3);
		public static final CSBFIELD EMISOR = new CSBFIELD("Emisor",8);		
		public static final CSBFIELD SUFIJO = new CSBFIELD("Sufijo",3);		
		public static final CSBFIELD REFERENCIA = new CSBFIELD("Referencia",13);
		public static final CSBFIELD IDENTIFICACION = new CSBFIELD("Identificacion",6);
		public static final CSBFIELD IMPORTE = new CSBFIELD("Importe",10);
		public static final CSBFIELD DIGITO_PARIDAD= new CSBFIELD("Digitoparidad",1,"0");
	    
		// Conjunto de tramas (una trama es un Array de CSBFields) que se accederan para descomponer un codigo determinado :
		//  p.e : Se puede descomponer un codigo completo (900000), o un campo concreto, p.e descomponer el emisor.
		
		public static HashMap tramas = new HashMap();
		
		// Estructura de la Tramas
		// Solo se incluye la trama completa. 
		//En este caso no tiene sentido descomponer por otras tramas (Emisor, Identificacion, Referencia..) como en el caso del CSB60
		
		private static final CSBFIELD[] tramaCompleta = {ID_APP, FORMATO,EMISOR,SUFIJO, REFERENCIA, IDENTIFICACION, IMPORTE,DIGITO_PARIDAD};
		
		// Cargar las diferentes tramas.
		static {
			tramas.put(CSB507.csbCode,tramaCompleta);		
		}
			
	}	
	
	
	/***************************************************************************************************
	 *     								  Implementacin trama  CSB502
	 ***************************************************************************************************/
	public static class CSB502 {
		
		public static Integer csbCode = new Integer(502);
		
		// Campos de CSB502
		public static final CSBFIELD ID_APP =  new CSBFIELD("Identificativo APP",2,"90");
		public static final CSBFIELD FORMATO = new CSBFIELD("Tipo de Formato",3);
		public static final CSBFIELD EMISOR = new CSBFIELD("Emisor",6);		
		public static final CSBFIELD REFERENCIA = new CSBFIELD("Referencia",12);
		public static final CSBFIELD IDENTIFICACION = new CSBFIELD("Identificacion",7);
		public static final CSBFIELD IMPORTE = new CSBFIELD("Importe",8);
		
		//Campos de CSB502.Emisor
		public static final CSBFIELD PROVINCIA = new CSBFIELD("Provincia",2);	
		public static final CSBFIELD MUNICIPIO = new CSBFIELD("Municipio",3);
		public static final CSBFIELD DIGITO_CONTROL_EMISOR = new CSBFIELD("Digito de Control",1);
		

		//Campos de CSB502.Referencia
		public static final CSBFIELD IDENTIFICACION_DOCUMENTO = new CSBFIELD("Identificacion DOC",10);	
		public static final CSBFIELD DIGITOS_CONTROL_REFERENCIA= new CSBFIELD("Digitos Control",2);
			
        
       //Campos de CSB502.Identificacion
		public static final CSBFIELD TRIBUTO = new CSBFIELD("Tributo",3);	
		public static final CSBFIELD EJERCICIO = new CSBFIELD("Ejercicio",2);
		public static final CSBFIELD REMESA = new CSBFIELD("Remesa",2);
		
		
		
		// Estructura de la Trama.		
		private static final CSBFIELD[] tramaCompleta = {ID_APP, FORMATO, EMISOR, REFERENCIA, IDENTIFICACION, IMPORTE};
 		private static final CSBFIELD[] tramaEmisor = {PROVINCIA, MUNICIPIO,DIGITO_CONTROL_EMISOR};
 		private static final CSBFIELD[] tramaReferencia = {IDENTIFICACION_DOCUMENTO,DIGITOS_CONTROL_REFERENCIA};
 		private static final CSBFIELD[] tramaIdentificacion = {TRIBUTO,EJERCICIO,REMESA};
 	 	
 		// Conjunto de tramas (Map) que se accederan para descomponer :
		//  p.e : Se puede descomponer un codigo completo (900000), o un campo concreto, p.e descomponer el emisor.
		
		public static HashMap tramas = new HashMap();
		
 		
 		static {
 			tramas.put(CSB502.csbCode,tramaCompleta);
 			tramas.put(EMISOR.id,tramaEmisor);
 			tramas.put(REFERENCIA.id,tramaReferencia);
 			tramas.put(IDENTIFICACION.id,tramaIdentificacion);
 		 }
		
	}	
		
	/***************************************************************************************************
	 *     								  Implementacin trama  CSB508
	 ***************************************************************************************************/
	public static class CSB508 {
		
		public static Integer csbCode = new Integer(508); 
		
		// Campos de CSB508		
		public static final CSBFIELD ID_APP =  new CSBFIELD("Identificativo APP",2,"90");
		public static final CSBFIELD FORMATO = new CSBFIELD("Tipo de Formato",3);
		public static final CSBFIELD ENTIDAD_TESORERA = new CSBFIELD("Entidad Tesorera",4);
		public static final CSBFIELD FECHA_LIMITE_PAGO = new CSBFIELD("Fecha Limite de Pago",6);
		public static final CSBFIELD EMISOR = new CSBFIELD("Emisor",6);
		public static final CSBFIELD REFERENCIA = new CSBFIELD("Referencia",12);
		public static final CSBFIELD IDENTIFICACION = new CSBFIELD("Identificacion",7);
		public static final CSBFIELD IMPORTE = new CSBFIELD("Importe",8);
		
		//Campos de CSB508.Emisor
		public static final CSBFIELD PROVINCIA = new CSBFIELD("Provincia",2);	
		public static final CSBFIELD MUNICIPIO = new CSBFIELD("Municipio",3);
		public static final CSBFIELD DIGITO_CONTROL_EMISOR = new CSBFIELD("Digito de Control",1);
		

		//Campos de CSB508.Referencia
		public static final CSBFIELD IDENTIFICACION_REFERENCIA= new CSBFIELD("Identificacion DOC",10);	
		public static final CSBFIELD DIGITOS_CONTROL_REFERENCIA= new CSBFIELD("Digitos Control",2);
		
		  
       //Campos de CSB508.Identificacion
		public static final CSBFIELD TRIBUTO = new CSBFIELD("Tributo",3);	
		public static final CSBFIELD EJERCICIO = new CSBFIELD("Ejercicio",2);
		public static final CSBFIELD REMESA = new CSBFIELD("Remesa",2);
		
	
 		// Conjunto de tramas (Map) que se accederan para descomponer :
		//  p.e : Se puede descomponer un codigo completo (900000), o un campo concreto, p.e descomponer el emisor.
		
		public static HashMap tramas = new HashMap();
		
		
		// Estructura de la Trama.		
		private static final CSBFIELD[] tramaCompleta = {ID_APP, FORMATO, ENTIDAD_TESORERA,FECHA_LIMITE_PAGO,EMISOR, REFERENCIA,IDENTIFICACION,IMPORTE};
		private static final CSBFIELD[] tramaEmisor = {PROVINCIA, MUNICIPIO,DIGITO_CONTROL_EMISOR};
		private static final CSBFIELD[] tramaReferencia = {IDENTIFICACION_REFERENCIA,DIGITOS_CONTROL_REFERENCIA};
		private static final CSBFIELD[] tramaIdentificacion = {TRIBUTO,EJERCICIO,REMESA};
 	 	
		
 		static {
 			tramas.put(CSB508.csbCode,tramaCompleta);
 			tramas.put(EMISOR.id,tramaEmisor);
 			tramas.put(REFERENCIA.id,tramaReferencia);
 			tramas.put(IDENTIFICACION.id,tramaIdentificacion);
 		}
		
	}
	
	
	/***************************************************************************************************
	 *     								  Implementacin trama  CSB521
	 ***************************************************************************************************/
	
	public static class CSB521 {
		
		public static Integer csbCode = new Integer(521); 
		
		
		public static final CSBFIELD ID_APP =  new CSBFIELD("Identificativo APP",2,"90");
		public static final CSBFIELD FORMATO = new CSBFIELD("Tipo de Formato",3);
		public static final CSBFIELD EMISOR = new CSBFIELD("Emisora",6);
		public static final CSBFIELD REFERENCIA = new CSBFIELD("Referencia",12);
		public static final CSBFIELD IDENTIFICACION = new CSBFIELD("Identificacion",10);
		public static final CSBFIELD IMPORTE = new CSBFIELD("Importe",8);
		public static final CSBFIELD DIGITO_PARIDAD= new CSBFIELD("DIGITO_PARIDAD",1,"0");
		
		
		//Campos de CSB521.Emisor
		public static final CSBFIELD PROVINCIA = new CSBFIELD("Provincia",2);	
		public static final CSBFIELD MUNICIPIO = new CSBFIELD("Municipio",3);
		public static final CSBFIELD DIGITO_CONTROL_EMISOR = new CSBFIELD("Digito de Control",1);
		
		

		//Campos de CSB521.Referencia
		public static final CSBFIELD IDENTIFICACION_REFERENCIA= new CSBFIELD("Identificacion DOC",10);	
		public static final CSBFIELD DIGITOS_CONTROL_REFERENCIA= new CSBFIELD("Digitos Control",2);
		
		//Campos de CSB521.Identificacion
		
		public static final CSBFIELD DISCRIMINANTE_PERIODO =  new CSBFIELD("Discriminante_Perido",1);
		public static final CSBFIELD TRIBUTO = new CSBFIELD("Tributo",3);
		public static final CSBFIELD EJERCICIO_DEVENGO = new CSBFIELD("Ejercicio Devengo",2);
		public static final CSBFIELD ANYO_FECHA_LIMITE = new CSBFIELD("Ao fecha limite",1);
		public static final CSBFIELD FECHA_JULIANA = new CSBFIELD("Fecha Juliana",3);
		
		
		// Conjunto de tramas (Map) que se accederan para descomponer :
		//  p.e : Se puede descomponer un codigo completo (900000), o un campo concreto, p.e descomponer el emisor.
		
		public static HashMap tramas = new HashMap();
		
		
		
		// Estructura de la Trama.		
		public static final CSBFIELD[]  tramaCompleta = {ID_APP, FORMATO, EMISOR,REFERENCIA,IDENTIFICACION,IMPORTE,DIGITO_PARIDAD};
		private static final CSBFIELD[] tramaEmisor = {PROVINCIA, MUNICIPIO,DIGITO_CONTROL_EMISOR};
		private static final CSBFIELD[] tramaReferencia = {IDENTIFICACION_REFERENCIA,DIGITOS_CONTROL_REFERENCIA};
		private static final CSBFIELD[] tramaIdentificacion = {DISCRIMINANTE_PERIODO,TRIBUTO,EJERCICIO_DEVENGO,ANYO_FECHA_LIMITE,FECHA_JULIANA};
	

 		static {
 			tramas.put(CSB521.csbCode,tramaCompleta);
 			tramas.put(EMISOR.id,tramaEmisor);
 			tramas.put(REFERENCIA.id,tramaReferencia);
 			tramas.put(IDENTIFICACION.id,tramaIdentificacion);
 		}
	
	}

	/***************************************************************************************************
	 *     								  Implementacin trama  CSB522
	 ***************************************************************************************************/
	
	public static class CSB522 {
		
		public static Integer csbCode = new Integer(522);  
		
		public static String ID_SIN_RECARGO = "periodoVoluntario";
		public static String ID_CON_RECARGO = "periodoConRecargo";
		
		
		public static final CSBFIELD ID_APP =  new CSBFIELD("Identificativo APP",2,"90");
		public static final CSBFIELD FORMATO = new CSBFIELD("Tipo de Formato",3);
		public static final CSBFIELD EMISOR = new CSBFIELD("Emisora",6);
		public static final CSBFIELD REFERENCIA = new CSBFIELD("Referencia",14);              
		public static final CSBFIELD IDENTIFICACION_SIN_RECARGO = new CSBFIELD("Identificacion",10);
        public static final CSBFIELD FECHA_SIN_RECARGO = new CSBFIELD("Fecha sin recargo",4);
		public static final CSBFIELD IMPORTE_VOLUNTARIA = new CSBFIELD("Importe",8);
		public static final CSBFIELD IDENTIFICACION_CON_RECARGO = new CSBFIELD("Identificacion",10);
        public static final CSBFIELD FECHA_CON_RECARGO = new CSBFIELD("Fecha con recargo",4);
		public static final CSBFIELD IMPORTE_RECARGO = new CSBFIELD("Importe",8);
		public static final CSBFIELD DIGITO_PARIDAD= new CSBFIELD("DIGITO_PARIDAD",1,"0");
		
		//Campos de CSB522.Emisor
		public static final CSBFIELD PROVINCIA = new CSBFIELD("Provincia",2);	
		public static final CSBFIELD MUNICIPIO = new CSBFIELD("Municipio",3);
		public static final CSBFIELD DIGITO_CONTROL_EMISOR = new CSBFIELD("Digito de Control",1);
		

		//Campos de CSB522.Referencia
		public static final CSBFIELD IDENTIFICACION_REFERENCIA= new CSBFIELD("Identificacion DOC",10);	
		public static final CSBFIELD DIGITOS_CONTROL_IMPORTES_SIN_RECARGO = new CSBFIELD("Digitos Control sin recargo",2);
		public static final CSBFIELD DIGITOS_CONTROL_IMPORTES_CON_RECARGO = new CSBFIELD("Digitos Control con recargo",2);

		//Campos de CSB521.Identificacion		
		public static final CSBFIELD DISCRIMINANTE_PERIODO =  new CSBFIELD("Discriminante_Perido",1);
		public static final CSBFIELD TRIBUTO = new CSBFIELD("Tributo",3);
		public static final CSBFIELD EJERCICIO_DEVENGO = new CSBFIELD("Ejercicio Devengo",2);
		public static final CSBFIELD ANYO_FECHA_LIMITE = new CSBFIELD("Ao fecha limite",1);
		public static final CSBFIELD FECHA_JULIANA = new CSBFIELD("Fecha Juliana",3);
		
	
		// Conjunto de tramas (Map) que se accederan para descomponer :
		//  p.e : Se puede descomponer un codigo completo (900000), o un campo concreto, p.e descomponer el emisor.
		
		public static HashMap tramas = new HashMap();			
		
		// Estructura de la Trama.		
		public static final  CSBFIELD[] tramaCompleta = {ID_APP, 
            FORMATO, 
            EMISOR , 
            REFERENCIA,
            TRIBUTO,
            EJERCICIO_DEVENGO,
            FECHA_SIN_RECARGO,
            IMPORTE_VOLUNTARIA,
            FECHA_CON_RECARGO,
            IMPORTE_RECARGO};
		private static final CSBFIELD[] tramaEmisor = {PROVINCIA, 
            MUNICIPIO,
            DIGITO_CONTROL_EMISOR};
		private static final CSBFIELD[] tramaReferencia = {IDENTIFICACION_REFERENCIA,
            DIGITOS_CONTROL_IMPORTES_SIN_RECARGO,
            DIGITOS_CONTROL_IMPORTES_CON_RECARGO};
		private static final CSBFIELD[] tramaIdentificacionSinRecargo = {DISCRIMINANTE_PERIODO,TRIBUTO,
            EJERCICIO_DEVENGO,
            ANYO_FECHA_LIMITE,
            FECHA_JULIANA};
		private static final CSBFIELD[] tramaIdentificacionConRecargo = {DISCRIMINANTE_PERIODO,
            TRIBUTO,EJERCICIO_DEVENGO,
            ANYO_FECHA_LIMITE,
            FECHA_JULIANA};
        private static final CSBFIELD[] fechaSinRecargo = {ANYO_FECHA_LIMITE,
            FECHA_JULIANA};
        private static final CSBFIELD[] fechaConRecargo = {ANYO_FECHA_LIMITE,
            FECHA_JULIANA};

		static {
 			tramas.put(CSB522.csbCode,tramaCompleta);
 			tramas.put(EMISOR.id,tramaEmisor);
 			tramas.put(REFERENCIA.id,tramaReferencia);
 			tramas.put(IDENTIFICACION_SIN_RECARGO.id,tramaIdentificacionSinRecargo);
 			tramas.put(IDENTIFICACION_CON_RECARGO.id,tramaIdentificacionConRecargo);
            tramas.put(FECHA_CON_RECARGO.id,fechaConRecargo);
            tramas.put(FECHA_CON_RECARGO.id,fechaConRecargo);
 		}
	
	
	
	}
	
	
}