package p12f.exe.pasarelapagos.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import p12f.exe.pasarelapagos.utils.exceptions.CSBFormatException;
/*
 *  Clase de ayuda para los cuadernos CSB
 */
public class CSBHelpper {
	
	/**
	 * Decodifica un codigo completo(String) a todos sus valores 502.
	 * @param code
	 * @return
	 * @throws CSBFormatException
	 */
		
	public static Map decode502(String code) throws CSBFormatException {	
		CSBFIELD[] tramaCompleta502 = (CSBFIELD[])CSBConstants.CSB502.tramas.get(CSBConstants.CSB502.csbCode); 
		return _decode(tramaCompleta502,code);
	}
	
	/**
	 * Decodifica un campo concreto de un CSB502. 
	 * @param code
	 * @return
	 * @throws CSBFormatException
	 */
	public static Map decode502(String code, CSBFIELD field) throws CSBFormatException {
		CSBFIELD[] tramaCompleta502 = (CSBFIELD[])CSBConstants.CSB502.tramas.get(field.id); 
		if ( tramaCompleta502 == null ) throw new CSBFormatException("El campo " + field.id  + " no se puede descomponer porque es un campo simple o porque no se ha defindo una trama para su decodificacin");
		return _decode(tramaCompleta502,code);
	}
		
	/**
	 * Decodifica un codigo a sus campos en CSB507.
	 * @param code
	 * @return
	 * @throws CSBFormatException
	 */
	public static Map decode507(String code) throws CSBFormatException {
	
		CSBFIELD[] tramaCompleta507 = (CSBFIELD[])CSBConstants.CSB507.tramas.get(CSBConstants.CSB507.csbCode); 
		return _decode(tramaCompleta507,code);
	}
	
	public static Map decode507(String code, CSBFIELD field) throws CSBFormatException {
		CSBFIELD[] tramaField507 = (CSBFIELD[])CSBConstants.CSB507.tramas.get(field.id); 
		if ( tramaField507 == null ) throw new CSBFormatException("El campo " + field.id  + " no se puede descomponer porque es un campo simple o porque no se ha defindo una trama para su decodificacin.El CSB507 actuamente no tiene tramas compuestas");
		return _decode(tramaField507,code);
	}
	
	/**
	 * Decodifica un codigo completo(String) a todos sus valores 508.
	 * @param code
	 * @return
	 * @throws CSBFormatException
	 */
		
	public static Map decode508(String code) throws CSBFormatException {		
		CSBFIELD[] tramaCompleta508 = (CSBFIELD[])CSBConstants.CSB508.tramas.get(CSBConstants.CSB508.csbCode); 
		return _decode(tramaCompleta508,code);
	}
	
	/**
	 * Decodifica un campo concreto de un CSB508. 
	 * @param code
	 * @return
	 * @throws CSBFormatException
	 */
	public static Map decode508(String code, CSBFIELD field) throws CSBFormatException {
		CSBFIELD[] tramaField508 = (CSBFIELD[])CSBConstants.CSB508.tramas.get(field.id); 
		if ( tramaField508 == null ) throw new CSBFormatException("El campo " + field.id  + " no se puede descomponer porque es un campo simple o porque no se ha defindo una trama para su decodificacin");
		return _decode(tramaField508,code);
	}
		
	/**
	 * Decodifica un codigo completo(String) a todos sus valores 521.
	 * @param code
	 * @return
	 * @throws CSBFormatException
	 */
		
	public static Map decode521(String code) throws CSBFormatException {		
		CSBFIELD[] tramaCompleta521 = (CSBFIELD[])CSBConstants.CSB521.tramas.get(CSBConstants.CSB521.csbCode); 
		return _decode(tramaCompleta521,code);
	}
	
	/**
	 * Decodifica un campo concreto de un CSB521. 
	 * @param code
	 * @return
	 * @throws CSBFormatException
	 */
	public static Map decode521(String code, CSBFIELD field) throws CSBFormatException {
		CSBFIELD[] tramaField521 = (CSBFIELD[])CSBConstants.CSB521.tramas.get(field.id); 
		if ( tramaField521 == null ) throw new CSBFormatException("El campo " + field.id  + " no se puede descomponer porque es un campo simple o porque no se ha defindo una trama para su decodificacin.");
		return _decode(tramaField521,code);
	}
		
	
	/**
	 * Decodifica un codigo completo(String) a todos sus valores 521.
	 * @param code
	 * @return
	 * @throws CSBFormatException
	 */
		
	public static Map decode522(String code) throws CSBFormatException {			
	    CSBFIELD[] tramaCompleta522 = (CSBFIELD[])CSBConstants.CSB522.tramas.get(CSBConstants.CSB522.csbCode); 
	    return _decode(tramaCompleta522,code);
	}
	
	/**
	 * Decodifica un campo concreto de un CSB521. 
	 * @param code
	 * @return
	 * @throws CSBFormatException
	 */
	public static Map decode522(String code, CSBFIELD field) throws CSBFormatException {
		CSBFIELD[] tramaField522 = (CSBFIELD[])CSBConstants.CSB522.tramas.get(field.id); 
		if ( tramaField522 == null ) throw new CSBFormatException("El campo " + field.id  + " no se puede descomponer porque es un campo simple o porque no se ha defindo una trama para su decodificacin");
		return _decode(tramaField522,code);
	}
	
	
	
/**
	 * Devuelve un String con un debug de la decodificacin.
	 * @param trama
	 * @param code
	 * @return String cadena descriptiva con el deug de la Decodificacin
	 * @throws CSBFormatException
	 */
	 public static String debug ( CSBFIELD[] tramaCompleta,String code) throws CSBFormatException {	
    	String  debugText = 			 "\nDEBUG CSB CODE.\n";
		debugText += "=======================================================\n";
		debugText += "Code : " + code + "\n";
		List listaRegExp =  _getRegExpList(  tramaCompleta );
		String code502Copy = new String (code);
	  	for (int i = 0 ; i < listaRegExp.size() ; i++ ){
			String regExp = ( String ) listaRegExp.get(i);
			Pattern p = null;
			if ( i <  listaRegExp.size() -1  )
				p = Pattern.compile(regExp + "([(\\d || \\w)]*)");
			else p = Pattern.compile(regExp);
			Matcher m = p.matcher(code502Copy);
			if (m.matches())
				debugText += "\n" + regExp + " " + tramaCompleta[i].id  + ":" + m.group(1);					
			else 
				debugText += "\n" + regExp + " " + tramaCompleta[i].id  + ": NOT MATCHES";			
			code502Copy = code502Copy.replaceFirst(regExp,"");			   
		}
	  	return debugText;
	 }
	 
     
     
    /**
     * Calcula el dgito de control de un CSB 57 (Consejo Superior Bancario)
     * El dgito de control del CSB se calcula de la siguiente forma:
     *  - Sumar emisor(8) + sufijo (3) + referencia(11) + identificacion(6) + importe en centeuros
     *  - Dividir por 97 el resultado de la suma
     *  - Obtener los dos primeros decimales de la divisin
     *  - El digito de control es (100 - los dos decimales)       
     */
    public static String getDigitoControlCSB507(String emisor,String sufijo,String referencia,String identificacion,String importeCenteuros) {
      try {
        long lngEmisor = Long.parseLong(emisor);
        long lngSufijo = Long.parseLong(sufijo);
        long lngReferencia = Long.parseLong(referencia.substring(0,11));    // Solo 11 digitos de la referencia
        long lngIdentificacion = Long.parseLong(identificacion);
        long lngImporte = Long.parseLong(importeCenteuros);

        // Obtener la suma
        long suma = lngEmisor + lngSufijo + lngReferencia + lngIdentificacion + lngImporte;
        // Obtener los decimales de la division
        long resto = suma % 97; 
        double decimales = resto / 97.0;    // OJO!!! No poner 97 sin mas ya que JAVA utilizaria integer
                                            //        y devolveria 0.0

        // Obtener los dos primeros digitos de la division
        String strDecimales = Double.toString(decimales);
        int comaIndx = strDecimales.indexOf('.');
        if (comaIndx > 0) {
            strDecimales = strDecimales.substring(comaIndx+1);
        } else {
            comaIndx = strDecimales.indexOf(',');
            if (comaIndx > 0) strDecimales = strDecimales.substring(comaIndx+1);
        }

        if (Double.parseDouble(strDecimales) == 0) {
            return "00";
        } else if (strDecimales.length() == 1) {
            strDecimales = strDecimales + "0";
        } else {
            strDecimales = strDecimales.substring(0,2);
        }
            
        int digitoControl = 100 - Integer.parseInt(strDecimales);
        if (digitoControl < 10) return  "0" + digitoControl;                
        return Integer.toString(digitoControl);
    } catch (Exception ex) {
        ex.printStackTrace();
        return "EX";    // Indica que ha habido una exception al generar el digito de control
    }
   }
      
 
     
 /********************************************************************************************
  * 						METODOS PRIVADOS
  ********************************************************************************************/


	/**
	 * Decodifica la cadena code a una trama especfica.
	 * @param trama
	 * @param code
	 * @return
	 * @throws CSBFormatException
	 */
	 private static Map _decode ( CSBFIELD[] trama,String code) throws CSBFormatException {
		 Map map = new HashMap();
		 String regExp = _getRegExp(trama);
		 Pattern p = Pattern.compile(regExp);
		 Matcher m = p.matcher(code);
		 if (! m.matches()){	
			 throw new CSBFormatException("La trama : "  + code + " no sigue el formato correcto : " + regExp + "\n" + debug(trama,code));
		 } 
		 String value = null;
		 CSBFIELD field = null;
		 for (int i=0; i< trama.length ;i++){
			field = trama[i];
			// Para obtener el campo descompuesto "i+1", pues el array empieza por 0, pero el match-group desde 1
			map.put(field,m.group(i+1));	
		 }
		 	 
		 return map;
	}
	 
	
	/**
	 * Devuelve la expresin regular para una trama determinada.
	 * @param trama
	 * @return
	 */
	private static String _getRegExp( CSBFIELD[] trama ){
		String regExp = new String();
		String grupo = null;
		for (int i = 0 ; i < trama.length ; i++ ){
			CSBFIELD campo = trama[i];
			grupo = "([0-9]{" + campo.length + "})";
			regExp += grupo;	
		}			
		return regExp;
	}
		
	/**
	 * Devuelve la expresin regular en forma de grupos
	 * @param trama
	 * @return
	 */
	private static List _getRegExpList( CSBFIELD[] trama ){
		List  regDescompuesto = new ArrayList();
		String grupo = null;
		for (int i = 0 ; i < trama.length ; i++ ){
			CSBFIELD campo = trama[i];
			//grupo = "([0-9]{" + campo.length + "})";
			grupo = "([0-9]{" + campo.length + "})";
			regDescompuesto.add(grupo);
		}			
		return regDescompuesto;
	}
	
	/*
	 *  Metodo para probar la clase;
	 */
	public static void main(String args[]) throws CSBFormatException {
		
	/*/	//([0-9]{2})([0-9]{3})([0-9]{6})([0-9]{12})([0-9]{7})([0-9]{8})
		String  id = "90";
		String formato = "503";
		String emisora = "048810";
		String referencia = "012345678912";
		String identificacion = "1234567";
		String importe = "1234568788888";
		String code502 = id + formato + emisora + referencia + identificacion + importe;*/
		
				
		String de = debug((CSBFIELD[])CSBConstants.CSB507.tramas.get(CSBConstants.CSB507.csbCode),"9050700100000100314000045015025061400000075940");
	
		System.out.println(de);
	}
	
	
	 
	}


