package p12f.exe.pasarelapagos.utils;

import java.util.Iterator;

import p12f.exe.pasarelapagos.objects.config.ConstraintItem;
import p12f.exe.pasarelapagos.objects.config.ConstraintList;

public class ConstraintUtils {
	
	/**
	 * Constructor privado para evitar que se instancie.
	 */
	private ConstraintUtils() { }
	
	/**
	 * Indica si en la constraintList existe un constraint con el identificador constraintId.
	 * @param constraintList
	 * @param constraintId
	 * @return boolean
	 */
	public static boolean containsConstraint(ConstraintList constraintList, String constraintId) {
		if (constraintList==null || constraintList.constraintList == null) {
			return false;
		}
		Iterator listadoConstraint = constraintList.constraintList.iterator();
		while (listadoConstraint.hasNext()) {
			ConstraintItem cItem=(ConstraintItem)listadoConstraint.next();
			if(cItem.id!=null && cItem.id.equals(constraintId)){
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Devuelve del constraintList la constraint identificada por el constraintId,
	 * null en caso de no encontrar ninguna constraint con dicho identificador.
	 * @param constraintList
	 * @param constraintId
	 * @return ConstraintItem
	 */
	public static ConstraintItem getConstraint(ConstraintList constraintList, String constraintId) {
		if (constraintList==null || constraintList.constraintList == null) {
			return null;
		}
		Iterator listadoConstraint = constraintList.constraintList.iterator();
		while (listadoConstraint.hasNext()) {
			ConstraintItem cItem=(ConstraintItem)listadoConstraint.next();
			if(cItem.id!=null && cItem.id.equals(constraintId)){
				return cItem;
			}
		}
		return null;
	}
	
}
