/*
 * Created on 15-dic-2005
 *
 * @author co01347e
 */
package p12f.exe.pasarelapagos.utils;

import java.util.StringTokenizer;

import p12f.exe.pasarelapagos.objects.DatosPago;
import p12f.exe.pasarelapagos.objects.PeriodoPago;

import com.ejie.r01f.log.R01FLog;

/**
 * 
 */
public class ControlCodeHelper {

	/**
	 * Mtodo que realiza la validacin de los dgitos de control de un pago en funcin de su formato.
	 * @param peticionPago Informacin del pago.
	 * @return
	 */
	public static boolean compruebaCC(DatosPago datosPago){
		
		int tipoValidacion = datosPago.validar;
		
		if(tipoValidacion==DatosPago.NO_VALIDAR){
			// La peticion de pago no se valida
			return true;
		} else if(tipoValidacion==DatosPago.VALIDACION_NORMAL){
			if("507".equals(datosPago.formato)){
				return validarDC57(datosPago);
			}
			if("502".equals(datosPago.formato)){
				return validarDC60(datosPago);
			}
			if("508".equals(datosPago.formato)){
				return validarDC60(datosPago);
			}
			if("521".equals(datosPago.formato)){
				return validarDC60(datosPago);
			}
			if("522".equals(datosPago.formato)){
				return validarDC60(datosPago);
			}
		} else if(tipoValidacion==DatosPago.VALIDACION_TRAFICO){
			if(datosPago.formato.equals("507")){
				return validarDC57NoEstandar(datosPago);
			}
		}
		
		return false;
	}
	
	/**
	 * Mtodo que comprueba los digitos de control para pagos del tipo 507.
	 */
	public static boolean validarDC57(DatosPago datosPago) {
		String referencia = datosPago.referencia;
		String strDigitoControlActual = referencia.substring(referencia.length()-2);
		String strDigitoControlCalculado = obtenerDCReferencia57(datosPago);
		
		if(strDigitoControlActual.equals(strDigitoControlCalculado)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Mtodo que obtiene los digitos de control para pagos del tipo 507.
	 * @param peticionPago
	 * @return
	 */
	public static String obtenerDCReferencia57(DatosPago datosPago) {
		
		// Sumar emisor + sufijo + referencia (11 caracteres) + identificacion + importe (centeuros)
		String emisor = datosPago.emisor;
		long numEntidad = Long.parseLong(emisor.substring(0,emisor.length()-4));
		long numSufijo = Long.parseLong(emisor.substring(emisor.length()-3));
		String referencia = datosPago.referencia;
		long numReferencia = Long.parseLong(referencia.substring(0,referencia.length()-2));
		PeriodoPago periodoNormal = (PeriodoPago)datosPago.periodosPago.get("periodoNormal");
		long numIdentificacion = Long.parseLong(periodoNormal.identificacion);
		long numImporte = periodoNormal.importe;
		long numSuma = numEntidad + numSufijo + numReferencia + numIdentificacion + numImporte;
		long resto = numSuma % 97;    
		
		// Dividir la suma por 97
		double numDivision = resto / 97.0;
		
		// Obtener los dos primeros digitos de la division
		String strDecimales = Double.toString(numDivision);
		// Comprobar que los decimales tengan tres digitos
		if(strDecimales.length()==3) { 
			strDecimales = strDecimales + "0";
		}
		int comaIndx = strDecimales.indexOf('.');
		if (comaIndx > 0) {
			strDecimales = strDecimales.substring(comaIndx+1, comaIndx+3);
		} else {
			comaIndx = strDecimales.indexOf(',');
			if (comaIndx > 0) strDecimales = strDecimales.substring(comaIndx+1, comaIndx+3);
		}
		
		//Caso en el que la division de exacto.En este caso devolvemos 00.
		if (strDecimales.equalsIgnoreCase("00") || strDecimales.equalsIgnoreCase("0")){
			return "00";
		}
		// Los digitos de control son: 100 - los dos primeros decimales de la division anterior
		int digitoControl = 100 - Integer.parseInt(strDecimales);
		String strDigitoControl = new Integer(digitoControl).toString();
		if (new Integer(digitoControl).toString().length() == 1) {
			strDigitoControl = "0" + digitoControl;
		}
		return strDigitoControl;
	}
	
	/**
	 * Mtodo que comprueba los digitos de control para pagos del cuadernillo 60
	 */
	public static boolean validarDC60(DatosPago datosPago) {
		String emisor = datosPago.emisor;
		String strDCEmisorActual = emisor.substring(emisor.length()-1);
		String strDCEmisorCalculado = obtenerDCEmisor60(emisor);
		
		String referencia = datosPago.referencia;
		String strDCReferenciaActual;
		String strDCReferenciaCalculado;
		String formato = datosPago.formato;
		
		if("522".equals(formato)){
			strDCReferenciaActual = referencia.substring(referencia.length()-4);
			strDCReferenciaCalculado = obtenerDCReferencia60_recargo(datosPago);
		}else{
			strDCReferenciaActual = referencia.substring(referencia.length()-2);
			strDCReferenciaCalculado = obtenerDCReferencia60(datosPago);
		}
		
		if(!strDCEmisorActual.equals(strDCEmisorCalculado)){
			return false;
		}
		if(strDCReferenciaActual.equals(strDCReferenciaCalculado)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Metodo que calcula el digito de control del emisor del cuadernillo 60
	 */
	public static String obtenerDCEmisor60(String emisor) {
		
		// Separar los digitos de emisor.
		int dig1 = Integer.parseInt(emisor.substring(0,1));
		int dig2 = Integer.parseInt(emisor.substring(1,2));
		int dig3 = Integer.parseInt(emisor.substring(2,3));
		int dig4 = Integer.parseInt(emisor.substring(3,4));
		int dig5 = Integer.parseInt(emisor.substring(4,5));
		
		// Multiplicar los digitos por su peso
		dig1 *= 6;
		dig2 *= 5;
		dig3 *= 4;
		dig4 *= 3;
		dig5 *= 2;
		
		// Sumar los digitos
		int total = dig1 + dig2 + dig3 + dig4 + dig5;
		
		// Obtener el digito de control
		int digitoControl = total%11;
		if(digitoControl==10) digitoControl=0;
		
		// Devolver resultados
		return (new Integer(digitoControl).toString());
	}
	
	/**
	 * Metodo que calcula el digito de control de la referencia del cuadernillo 60
	 * para los formatos con un unico periodo de pago (502, 508, 521).
	 */
	public static String obtenerDCReferencia60(DatosPago datosPago) {
		
		// Obtener los campos
		long numEntidad = Long.parseLong(datosPago.emisor);
		String referencia = datosPago.referencia;
		long numReferencia = Long.parseLong(referencia.substring(0,10));
		PeriodoPago periodoNormal = (PeriodoPago)datosPago.periodosPago.get("periodoNormal");
		String identificacion = periodoNormal.identificacion;
		long numIdentificacion = Long.parseLong(identificacion);
		long numImporte = periodoNormal.importe;
		long campo1 = numEntidad;
		long campo2 = numReferencia;
		long campo3 = numIdentificacion + numImporte;
		
		// Multiplicar los campos por sus pesos y dividir por 97
		campo1 *= 76;
		campo2 *= 9;
		campo3 = (campo3-1) * 55;
		
		long numSuma = campo1 + campo2 + campo3;
		long resto = numSuma % 97;
		
		// Dividir la suma por 97
		double numDivision = resto / 97.0;
		R01FLog.to("p12f.controlCode").info("Num Division:"+numDivision);
		
		// Obtener los dos primeros digitos de la division
		String strDecimales = Double.toString(numDivision);
		int comaIndx = strDecimales.indexOf('.');
		if (comaIndx > 0) {
			StringTokenizer st = new StringTokenizer(strDecimales,".");
			st.nextToken(); // entero
			String decimal = st.nextToken();
			if (decimal.length()==1) {
				strDecimales = decimal + "0";
			} else {
				strDecimales = decimal.substring(0,2);
			}
		} else {
			comaIndx = strDecimales.indexOf(',');
			if (comaIndx > 0) {
				StringTokenizer st = new StringTokenizer(strDecimales,",");
				st.nextToken(); // entero
				String decimal = st.nextToken();
				if (decimal.length()==1) {
					strDecimales = decimal + "0";
				} else {
					strDecimales = decimal.substring(0,2);
				}
			}
		}
		
		// Calcular el digito de control
		int digitoControl = 99 - Integer.parseInt(strDecimales);
		String strDigitoControl = new Integer(digitoControl).toString();
		if(strDigitoControl.length()==1) {
			strDigitoControl = "0" + strDigitoControl;
		}
		if(strDigitoControl.equals("00")) {
			strDigitoControl="99";
		}
		return strDigitoControl;
	}
	
	/**
	 * Metodo que calcula el digito de control de la referencia del cuadernillo 60
	 * para el formatos con dos periodos de pago (522).
	 */
	public static String obtenerDCReferencia60_recargo(DatosPago datosPago) {
		
		// Obtener los campos
		long numEntidad = Long.parseLong(datosPago.emisor);
		String referencia = datosPago.referencia;
		long numReferencia = Long.parseLong(referencia.substring(0,10));
		PeriodoPago periodoVoluntario = (PeriodoPago)datosPago.periodosPago.get("periodoVoluntario");
		String identificacionVol = periodoVoluntario.identificacion;
		long numIdentificacionVol = Long.parseLong(identificacionVol);
		long numImporteVol = periodoVoluntario.importe;
		PeriodoPago periodoConRecargo = (PeriodoPago)datosPago.periodosPago.get("periodoConRecargo");
		String identificacionRec = periodoConRecargo.identificacion;
		long numIdentificacionRec = Long.parseLong(identificacionRec);
		long numImporteRec = periodoConRecargo.importe;
		// PARCHE 20-08-2013: DGITOS DE CONTROL CUADERNO 522 SIN RECARGO
		// Para que coincidan con los calculados con recargo han de calcularse con el identificador a todo 0's.
		// Para detectar que no hay recargo se comprueba que el importe y la fecha fin del periodo recargo viene a 0.
		if (numImporteRec == 0 && identificacionRec.endsWith("0000")) {
			numIdentificacionRec = 0;
			R01FLog.to("p12f.controlCode").info("SIN RECARGO > IDENTIFICADOR A 0s");
		}
		// FIN PARCHE
		long campo1 = numEntidad;
		long campo2 = numReferencia;
		long campo3Vol = numIdentificacionVol + numImporteVol;
		long campo3Rec = numIdentificacionRec + numImporteRec;
		
		// Multiplicar los campos por sus pesos y dividir por 97
		campo1 *= 76;
		campo2 *= 9;
		campo3Vol = (campo3Vol-1) * 55;
		campo3Rec = (campo3Rec-1) * 55;
		
		long numSumaVol = campo1 + campo2 + campo3Vol;
		long numSumaRec = campo1 + campo2 + campo3Rec;
		long restoVol = numSumaVol % 97;
		long restoRec = numSumaRec % 97;
		
		// Dividir la suma por 97
		double numDivisionVol = restoVol / 97.0;
		double numDivisionRec = restoRec / 97.0;
		
		// Obtener los dos primeros digitos de la division
		String strDecimalesVol = Double.toString(numDivisionVol);
		String strDecimalesRec = Double.toString(numDivisionRec);
		
		//-------------- Calculo de decimales de voluntaria.---------------------------
		
		int comaIndx = strDecimalesVol.indexOf('.');
		if (comaIndx > 0) {
			StringTokenizer st = new StringTokenizer(strDecimalesVol,".");
			st.nextToken(); // entero
			String decimal = st.nextToken();
			if (decimal.length()==1) {
				strDecimalesVol = decimal + "0";
			} else {
				strDecimalesVol = decimal.substring(0,2);
			}
		} else {
			comaIndx = strDecimalesVol.indexOf(',');
			if (comaIndx > 0) {
				StringTokenizer st = new StringTokenizer(strDecimalesVol,",");
				st.nextToken(); // entero
				String decimal = st.nextToken();
				if (decimal.length()==1) {
					strDecimalesVol = decimal + "0";
				} else {
					strDecimalesVol = decimal.substring(0,2);
				}
			}
		}
		
		comaIndx = strDecimalesRec.indexOf('.');
		R01FLog.to("p12f.controlCode").info("Clculo comoIndex:"+comaIndx);
		R01FLog.to("p12f.controlCode").info("strDecimalesRec:"+strDecimalesRec);
		
		if (comaIndx > 0) {
			StringTokenizer st = new StringTokenizer(strDecimalesRec,".");
			st.nextToken(); // entero
			String decimal = st.nextToken();
			if (decimal.length()==1) {
				strDecimalesRec = decimal + "0";
			} else {
				strDecimalesRec = decimal.substring(0,2);
			}
		} else {
			comaIndx = strDecimalesRec.indexOf(',');
			if (comaIndx > 0) {
				StringTokenizer st = new StringTokenizer(strDecimalesRec,",");
				st.nextToken(); // entero
				String decimal = st.nextToken();
				if (decimal.length()==1) {
					strDecimalesRec = decimal + "0";
				} else {
					strDecimalesRec = decimal.substring(0,2);
				}
			}
		}
		
		// Calcular el digito de control voluntario
		int digitoControlVol = 99 - Integer.parseInt(strDecimalesVol);
		String strDigitoControlVol = new Integer(digitoControlVol).toString();
		if(strDigitoControlVol.length()==1){
			strDigitoControlVol = "0" + strDigitoControlVol;
		}
		if(strDigitoControlVol.equals("00")) {
			strDigitoControlVol="99";
		}
		// Calcular el digito de control con recargo
		int digitoControlRec = 99 - Integer.parseInt(strDecimalesRec);
		String strDigitoControlRec = new Integer(digitoControlRec).toString();
		if(strDigitoControlRec.length()==1){
			strDigitoControlRec = "0" + strDigitoControlRec;
		}
		if(strDigitoControlRec.equals("00")) {
			strDigitoControlRec="99";
		}
		return strDigitoControlVol + strDigitoControlRec;
	}
	
	/**
	 * Mtodo que comprueba los digitos de control para pagos del tipo 507 no estndar.
	 */
	public static boolean validarDC57NoEstandar(DatosPago datosPago) {
		String referencia = datosPago.referencia;
		String strDigitoControlActual = referencia.substring(referencia.length()-2);
		String strDigitoControlCalculado = obtenerDCReferencia57NoEstandar(datosPago);
		
		if(strDigitoControlActual.equals(strDigitoControlCalculado)) {
			return true;
		}
		return false;
	}
	
	/**
	 * Mtodo que obtiene los digitos de control para pagos del tipo 507 no estndar.
	 * @param peticionPago
	 * @return
	 */
	public static String obtenerDCReferencia57NoEstandar(DatosPago datosPago) {
		
		// Sumar emisor + sufijo + referencia (11 caracteres) + identificacion + importe (centeuros)
		String emisor = datosPago.emisor;
		long numEntidad = Long.parseLong(emisor.substring(0,emisor.length()-4));
		long numSufijo = Long.parseLong(emisor.substring(emisor.length()-3));
		String referencia = datosPago.referencia;
		long numReferencia = Long.parseLong(referencia.substring(0,referencia.length()-2));
		// Para el calculo de los digitos de control no se tienen en cuenta los campos de identificacion e importe 
		//PeriodoPago periodoNormal = (PeriodoPago)datosPago.periodosPago.get("periodoNormal");
		//long numIdentificacion = Long.parseLong(periodoNormal.identificacion);
		//long numImporte = periodoNormal.importe;
		long numSuma = numEntidad + numSufijo + numReferencia; // + numIdentificacion + numImporte;  
		long resto = numSuma % 97;
		
		// Dividir la suma por 97
		double numDivision = resto / 97.0;
		
		if (numDivision == 0) {
			return "00";
		}
		
		// Obtener los dos primeros digitos de la division
		String strDecimales = Double.toString(numDivision);
		int comaIndx = strDecimales.indexOf('.');
		if (comaIndx > 0) {
			strDecimales = strDecimales.substring(comaIndx+1, comaIndx+3);
		} else {
			comaIndx = strDecimales.indexOf(',');
			if (comaIndx > 0) {
				strDecimales = strDecimales.substring(comaIndx+1, comaIndx+3);
			}
		}
		//Caso en el que la division de exacto.En este caso devolvemos 00.
		if (strDecimales.equalsIgnoreCase("00") || strDecimales.equalsIgnoreCase("0")) {
			return "00";
		}
		// Los digitos de control son: 100 - los dos primeros decimales de la division anterior
		int digitoControl = 100 - Integer.parseInt(strDecimales);
		String strDigitoControl = new Integer(digitoControl).toString();
		if (new Integer(digitoControl).toString().length() == 1) {
			strDigitoControl = "0" + digitoControl;
		}
		return strDigitoControl;
	}
	
}
