/*
 * Created on 15-dic-2005
 *
 * @author co01347e
 */
package p12f.exe.pasarelapagos.utils;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import p12f.exe.pasarelapagos.objects.DatosPago;
import p12f.exe.pasarelapagos.objects.PeriodoPago;
import p12f.exe.pasarelapagos.paymentrequest.PeticionPago;

/**
 * Clase de ayuda que contiene mtodos que permiten la comprobacin de la fecha lmite de los pagos
 */
public class LimitDateHelper {

    public static boolean compruebaFecha(PeticionPago peticionPago) {
        
        int tipoValidacion = peticionPago.datosPago.validar;
        
        if(tipoValidacion==DatosPago.NO_VALIDAR){
            // La fecha limite del pago no se valida
            return true;
        } else if(tipoValidacion==DatosPago.VALIDACION_NORMAL || tipoValidacion==DatosPago.VALIDACION_TRAFICO){         
            // La fecha limite del pago se valida 
            return realizarComprobacionFecha(peticionPago);   
        } 
        
        return false;
    }
    
    
    
    /**
     * Mtodo que realiza la comprobacin de la fecha lmite e un pago en funcin de su formato.
     * @param peticionPago Informacin del pago.
     * @return
     */
    public static boolean realizarComprobacionFecha(PeticionPago peticionPago){   
        
        // Se comprueba que la fecha limite del pago no se haya superado.
        if(peticionPago.datosPago.formato.equals("507")){
            String sufijo = peticionPago.datosPago.emisor.substring(peticionPago.datosPago.emisor.length()-3);
            int numeroSufijo = Integer.parseInt(sufijo);
            // Para el formato 507 solo se realiza la comprobacin de la fecha limite de pago
            // si el sufijo es menor que 500.
            if(numeroSufijo<500){
                return true;
            }
            
            // Los pagos en el formato 507 tienen un unico periodo de pago
            PeriodoPago periodoNormal = (PeriodoPago)peticionPago.datosPago.periodosPago.get(PeriodoPago.PERIODO_NORMAL);           
            if(periodoNormal==null){
                return false;
            }
            // Se comprueba si se debe validar la fecha de fin del periodo 
            if(!periodoNormal.validarFechaFin) {
                return true;
            }    
            return comprobarFecha(periodoNormal.fechaFin);    
        }
        
        if(peticionPago.datosPago.formato.equals("502")){
            // Para el formato 502 no se realiza la comprobacin de la fecha limite de pago
            return true;
        }
        if(peticionPago.datosPago.formato.equals("508")){
            // Los pagos en el formato 508 tienen un unico periodo de pago
            PeriodoPago periodoNormal = (PeriodoPago)peticionPago.datosPago.periodosPago.get(PeriodoPago.PERIODO_NORMAL);
            if(periodoNormal==null){
                return false;
            }
            // Se comprueba si se debe validar la fecha de fin del periodo 
            if(!periodoNormal.validarFechaFin) {
                return true;
            }    
            return comprobarFecha(periodoNormal.fechaFin);
        }
        if(peticionPago.datosPago.formato.equals("521")){
            // Los pagos en el formato 521 tienen un unico periodo de pago
            /*PeriodoPago periodoNormal = (PeriodoPago)peticionPago.datosPago.periodosPago.get(PeriodoPago.PERIODO_NORMAL);
            if(periodoNormal==null){
                return false;
            }
            // Se comprueba si se debe validar la fecha de fin del periodo 
            if(!periodoNormal.validarFechaFin) {
                return true;
            }    
            return comprobarFecha(periodoNormal.fechaFin);*/
            
            /*
             * Para la modalidad 2 del cuaderno 60 se debe permitir continuar el proceso de pago
             * aunque la fecha limite de pago se haya superado.
             */
            return true;
        }
        if(peticionPago.datosPago.formato.equals("522")){
            // Los pagos en el formato 522 tienen dos periodo de pago
            /*PeriodoPago periodoVoluntario = (PeriodoPago)peticionPago.datosPago.periodosPago.get(PeriodoPago.PERIODO_VOLUNTARIO);
            if(periodoVoluntario==null){
                return false;
            }
            // Se comprueba si se debe validar la fecha de fin del periodo 
            if(!periodoVoluntario.validarFechaFin) {
                return true;
            }    
            if(comprobarFecha(periodoVoluntario.fechaFin)){
                return true;
            }
            PeriodoPago periodoConRecargo = (PeriodoPago)peticionPago.datosPago.periodosPago.get(PeriodoPago.PERIODO_CON_RECARGO);
            if(periodoConRecargo==null){
                return false;
            }
            // Se comprueba si se debe validar la fecha de fin del periodo 
            if(!periodoConRecargo.validarFechaFin) {
                return true;
            }    
            return comprobarFecha(periodoConRecargo.fechaFin);*/
            
            /*
             * Para la modalidad 2 del cuaderno 60 se debe permitir continuar el proceso de pago
             * aunque la fecha limite de pago se haya superado.
             */
            return true;
        }

        return false;
    }
    
    /**
     * Metodo que combrueba una fecha respecto a la actual
     * @param fecha fecha en formato DDMMAA
     * @return
     */
    public static boolean comprobarFecha(String fecha){
        
        // Si la fecha es null es que no hay fecha limite de pago
        if(fecha==null) return true;
        
        if(!comprobarFechaCorrecta(fecha)){
        	return false;
        }
        
        Calendar diaActual = Calendar.getInstance();
        String anioActual = (new Integer(diaActual.get(Calendar.YEAR))).toString();
        // Obtener por separado los dos primeros digitos del ao
        int sigloActual = Integer.parseInt(anioActual.substring(0,2));
        Date fechaActual = new Date();
       
        int dia = Integer.parseInt(fecha.substring(0,2));
        int mes = Integer.parseInt(fecha.substring(2,4));
        int anio = Integer.parseInt(sigloActual + fecha.substring(4,6));        
        
        Calendar diaPago = Calendar.getInstance();
        diaPago.set(Calendar.DAY_OF_MONTH,dia);
        diaPago.set(Calendar.MONTH,mes-1);
        diaPago.set(Calendar.YEAR,anio);      
        Date fechaPago = diaPago.getTime();  
        
        if(fechaActual.compareTo(fechaPago)<=0){
            return true;
        }
        
        return false;
    }
    
    /**
     * Metodo que combrueba si una fecha existe
     * @param fecha fecha en formato ddMMyy
     * @return
     */
    public static boolean comprobarFechaCorrecta(String fecha){
	    SimpleDateFormat dateFormat = new SimpleDateFormat("ddMMyy");
	    if (fecha.trim().length() != dateFormat.toPattern().length())
	    	return false;
	    dateFormat.setLenient(false);
	    try {
	    	dateFormat.parse(fecha.trim());
	    }catch (Exception pe) {
	    	return false;
	    }
	    return true;
    }
    
    
    
   /**
    * Metodo que me devuelve una fecha en formato DD/MM/AAAA
    * @param anio Ao actual.
    * @param fechaJuliana Fecha en formato Juliano.
    * @return
    */
    public static String formatearFecha(String anio, String fechaJuliana){
        // Se calcula el ao de finalizacin
        SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy");
        String anioActual = dateFormatter.format(new Date());
        int intDecenioActual = Integer.parseInt(anioActual.substring(2,3)); // Digito del decenio de la fecha
        int intAnioActual = Integer.parseInt(anioActual.substring(3,4)); // Ultimo digito del ao de la fecha actual
        int intAnio = Integer.parseInt(anio); // Ultimo digito del ao de la fecha de finalizacion del pago
        
        String anioCompleto = "20" + intDecenioActual + anio;
        
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.YEAR,Integer.parseInt(anioCompleto));
        calendar.set(Calendar.DAY_OF_YEAR,Integer.parseInt(fechaJuliana));
        Date fecha = calendar.getTime();
        dateFormatter = new SimpleDateFormat("ddMMyy");
        return dateFormatter.format(fecha);
    }

    /**
     * Metodo que devuelve la fecha lmite de pago a partir de un pagoID
     * @param pagoID
     * @return
     */
    public static Date obtenerFechaLimiteDePagoID(String pagoID) {
    	if (pagoID==null) return null;
    	
    	Date fecha = new Date();
    	
    	String strFormato = pagoID.substring(2,5); // 507: 90|507|04833001...
    	String strSufijo;
    	String strDevengo;
    	String strFecha; // 4  6 dgitos (4:juliana)

        String strYYYY = null;

        Calendar hoy = Calendar.getInstance();
        String yyyyHoy = (new Integer(hoy.get(Calendar.YEAR))).toString();
        int sigloHoy = Integer.parseInt(yyyyHoy.substring(0,2));

        Calendar calLimite = Calendar.getInstance();

        if (strFormato.equals("507")) { // c57 - identificacin 311255
    		strSufijo = pagoID.substring(13,16); // 800: 9050704833001|800|9999...

    		if (Integer.parseInt(strSufijo)<500) return null; // no hay fecha lmite

    		strFecha = pagoID.substring(29,35); //System.out.println("TRAZA: "+strFecha+" - "+pagoID);
    		
            calLimite.set(Calendar.DAY_OF_MONTH,Integer.parseInt(strFecha.substring(0,2)));
            calLimite.set(Calendar.MONTH,Integer.parseInt(strFecha.substring(2,4))-1);
            calLimite.set(Calendar.YEAR, Integer.parseInt(sigloHoy + strFecha.substring(4,6)));      
            fecha = calLimite.getTime();  
    	}
    	else { // c60 - fecha juliana 0000
            //String strDecenioActual = yyyyHoy.substring(2,3); // Digito del decenio de la fecha

            // 2019 -- 2020 ==> 190030 ==> devengo 19, juliana 0|030
            int nDevengo; // ejercicio de devengo (2 dgitos)
            int yJuliana; // dgito ao de la fecha juliana
            int nDecenio; // nmero dcada de la fecha lmite de pago (a calcular...)

            if (strFormato.equals("521")) {
                strSufijo  = pagoID.substring(24,27); // 047: 905214860009999...|047|...
        		strDevengo = pagoID.substring(27,29);
        		strFecha   = pagoID.substring(29,33); //System.out.println("TRAZA: "+strFecha+" - "+pagoID);
        		if (strDevengo.equalsIgnoreCase("00") && strFecha.equalsIgnoreCase("0000")) return null;

        		nDevengo = Integer.parseInt(strDevengo);
        		yJuliana = Integer.parseInt(pagoID.substring(29,30));
        		
    			//if (nDevengo%10>yJuliana) siguiente dcada... // 2019 vs 2020 => 19|0|030 => 19%10>0 => 9>0
        		nDecenio = Integer.parseInt(strDevengo.substring(0,1))+(nDevengo%10>yJuliana ? 1 : 0);

        		strYYYY = String.valueOf(sigloHoy) + String.valueOf(nDecenio) + strFecha.substring(0,1);

        		//calLimite.set(Calendar.YEAR,Integer.parseInt(sigloHoy + strDecenioActual + strFecha.substring(0,1)));
                calLimite.set(Calendar.YEAR,Integer.parseInt(strYYYY));
                calLimite.set(Calendar.DAY_OF_YEAR,Integer.parseInt(strFecha.substring(1,4))); // revisar...
                fecha = calLimite.getTime();
            }
            else {
                if (strFormato.equals("522")) {
            		strSufijo = pagoID.substring(25,28); // 051: 905224860009999...|051|...
                	
            		strDevengo = pagoID.substring(28,30);
            		//strFecha = pagoID.substring(30,34); // voluntaria
            		strFecha = pagoID.substring(42,46); // recargo (tomamos esta fecha como lmite)
            		//System.out.println("TRAZA: "+strFecha+" - "+pagoID);

            		nDevengo = Integer.parseInt(strDevengo);

            		// Si se ha introducido manualmente y se est en periodo voluntario, la fecha de
            		// recargo estar cumplimentada a ceros; tomaramos como fecha lmite la de periodo
            		// voluntario... (si no, 0000 juliana es 31/12/2009
            		if (strFecha.equalsIgnoreCase("0000")) {
                		strFecha = pagoID.substring(30,34);
            		}
            		if (strDevengo.equalsIgnoreCase("00") && strFecha.equalsIgnoreCase("0000")) return null;
            		yJuliana = Integer.parseInt(strFecha.substring(0,1));

        			//if (nDevengo%10>yJuliana) siguiente dcada... // 2019 vs 2020 => 19|0|030 => 19%10>0 => 9>0
            		nDecenio = Integer.parseInt(strDevengo.substring(0,1))+(nDevengo%10>yJuliana ? 1 : 0);

            		strYYYY = String.valueOf(sigloHoy) + String.valueOf(nDecenio) + strFecha.substring(0,1);

                    //calLimite.set(Calendar.YEAR,Integer.parseInt(sigloHoy + strDecenioActual + strFecha.substring(0,1)));
                    calLimite.set(Calendar.YEAR,Integer.parseInt(strYYYY));
                    calLimite.set(Calendar.DAY_OF_YEAR,Integer.parseInt(strFecha.substring(1,4))); // revisar...
                    fecha = calLimite.getTime();
                }
                else { // 508
                    if (strFormato.equals("508")) {
                    	// no se usa...
                    	// C60: longitud 6, posicin 10-15
                    	return null;
                    }
                    else { // 502
                        // Para el formato 502 no se realiza la comprobacin de la fecha limite de pago
                    	return null;
                    }
                }
            }
    	}
        //System.out.println("TRAZA: yyyy: " + yyyyHoy + ", " + sigloHoy + ", " + strYYYY + ", pagoID: " + pagoID + ", fecha limite: " + fecha.toString());
    	return fecha;
    }
    
   /* public static void main (String[] argv){
    	
    	//System.out.println(formatearFecha("0","0001"));

    	// Recibo C60 -----
        System.out.println("FECHA RECIBO ----------------------------------------------------------------");
    	String strDevengo = "11";
    	String strFecha   = "1032"; // 032 == 1 de febrero
    	String strY       = strFecha.substring(0,1);
    	String strDDD     = strFecha.substring(1,4);
        System.out.println("\tFecha recibo: devengo " + strDevengo + ", fecha juliana " + strY + "|" + strDDD);

    	//String strYDevengo = strDevengo.substring(1,2);
        //System.out.println("Ao fecha juliana: " + strY + " vs. dgito ao devengo: " + );

    	// HOY ------------
        System.out.println("FECHA HOY -------------------------------------------------------------------");
        Calendar hoy = Calendar.getInstance();
        String yyyyHoy = (new Integer(hoy.get(Calendar.YEAR))).toString(); // ao hoy: YYYY --> Y0|Y1|Y2|Y3
        //String decadaHoy = yyyyHoy.substring(2,3);

        System.out.println("\tYYYY(hoy): " + yyyyHoy);

        // Componer fecha limite
        System.out.println("FECHA LIMITE ----------------------------------------------------------------");

        System.out.println("-----------------------------------------------------------------------------\n\n");

        //int sigloHoy = Integer.parseInt(yyyyHoy.substring(0,2));
    	
    	String sigloHoy = "20"; // 2011 -- dgitos 20
    	String strDecenioActual = "1"; // yyyyHoy.substring(2,3); // Digito del decenio de la fecha
    	String strYYYY = sigloHoy + strDecenioActual + strFecha.substring(0,1);
        int y = Integer.parseInt(strYYYY);
        int doy = Integer.parseInt(strFecha.substring(1,4));

    	System.out.println("y: " + sigloHoy + "|" + strDecenioActual + "|" + strFecha.substring(0,1));
    	System.out.println("Calendar.YEAR: " + y);
    	System.out.println("Calendar.DAY_OF_YEAR: " + doy);

    	// C57
        System.out.println("TEST C57 ------------------------------------");
        System.out.println("TEST C57 - 507 ------------------------------");
    	System.out.println("090909: "+obtenerFechaLimiteDePagoID("9050704833001800912912912918909090900000000010"));
    	System.out.println("310311: "+obtenerFechaLimiteDePagoID("9050704833001800000779912077931031100000000070"));
    	System.out.println("180810: "+obtenerFechaLimiteDePagoID("9050704833001501001006029955618081000000072000"));
    	System.out.println("020411: "+obtenerFechaLimiteDePagoID("9050704833001800000771657325002041100000000070"));
    	System.out.println("130611: "+obtenerFechaLimiteDePagoID("9050704833001800000771657325013061100000000070"));

    	// C60
        System.out.println("TEST C60 ------------------------------------");
    	// C60 - 521 corto
        System.out.println("TEST C60 - 521 ------------------------------");
    	System.out.println("11.1.161: "+obtenerFechaLimiteDePagoID("905214860000019213084731047111161000000120")); // 10-jun-2011
    	System.out.println("11.1.126: "+obtenerFechaLimiteDePagoID("905214860000020213084501047111126000000130")); // 6-may-2011
    	System.out.println("11.2.161: "+obtenerFechaLimiteDePagoID("905214860000019213084731047112161000000120")); // 9-jun-2012
    	System.out.println("11.2.126: "+obtenerFechaLimiteDePagoID("905214860000020213084501047112126000000130")); // 5-may-2012 (2012 es bisiesto)
    	System.out.println("19.0.161: "+obtenerFechaLimiteDePagoID("905214860000019213084731047190161000000120")); // 10-jun-2020
    	System.out.println("19.0.126: "+obtenerFechaLimiteDePagoID("905214860000020213084501047190126000000130")); // 5-may-2020
    	System.out.println("19.1.126: "+obtenerFechaLimiteDePagoID("905214860000020213084501047191126000000130")); // 6-may-2021
    	System.out.println("20.0.126: "+obtenerFechaLimiteDePagoID("905214860000020213084501047200126000000130")); // 5-may-2020
    	System.out.println("20.9.126: "+obtenerFechaLimiteDePagoID("905214860000020213084501047209126000000130")); // 6-may-2029
    	System.out.println("Ayto.Vit: "+obtenerFechaLimiteDePagoID("905210105956110000364830648000000000004300")); // null

    	// C60 - 522 largo
        System.out.println("TEST C60 - 522 ------------------------------");
    	System.out.println("11.1.223: "+obtenerFechaLimiteDePagoID("905224860000022960604183905111115800000015122300000025")); // 11-ago-2011 recargo
    	System.out.println("11.1.223: "+obtenerFechaLimiteDePagoID("905224860000022960604183905111115800000000000000000000")); // 7-jun-2011 voluntaria
    	System.out.println("11.2.223: "+obtenerFechaLimiteDePagoID("905224860000022960604183905111215800000015222300000025")); // 11-ago-2012 recargo
    	System.out.println("11.2.223: "+obtenerFechaLimiteDePagoID("905224860000022960604183905111215800000000000000000000")); // 7-jun-2012 voluntaria
    	System.out.println("19.0.223: "+obtenerFechaLimiteDePagoID("905224860000022960604183905119015800000015022300000025")); // 11-ago-2020 recargo
    	System.out.println("19.0.223: "+obtenerFechaLimiteDePagoID("905224860000022960604183905119015800000000000000000000")); // 7-jun-2020 voluntaria

    	System.out.println("1223: "+formatearFecha("2011", "223"));
    	
        SimpleDateFormat dateFormatter = new SimpleDateFormat("dd/MM/yyyy");
    	System.out.println("Hoy: " + dateFormatter.format(Calendar.getInstance().getTime()));
    }*/
    
    
    
    
      public static void main(String[] argv) {
    	  long parseLong =  Long.parseLong("1402994664734");   	                                    
    	  Date tst = new Date(parseLong);
    	  SimpleDateFormat dateFormatter = new SimpleDateFormat("dd/MM/yyyy hh:mm:ss");
    	  String fecha = dateFormatter.format(tst);
    	  System.out.println(fecha);
    	  
      }
    
    
    
    

}
