package p12f.exe.pasarelapagos.utils;

import org.bouncycastle.util.encoders.Hex;

import p12f.exe.pasarelapagos.exceptions.ConfigLoadException;
import p12f.exe.pasarelapagos.objects.AccountData;
import p12f.exe.pasarelapagos.objects.config.BasicAdminConfig;
import p12f.exe.pasarelapagos.security.crypto.TripleDESEBCDICCryptoHelper;

import com.ejie.r01f.util.StringUtils;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;

public class PayOnAccountDataUtil {
	/** Cifrar el dato a enviar a la entidad financiera
	 * @throws Exception */
	public static AccountData encrypt (AccountData accountData, BasicAdminConfig  admin) throws Exception{
		
		/**
		 * 	public String timeStamp;
			public EncData encData; 
			public Map aditionalDataList;
			public String accountNumberFormat
		 */
			
		AccountData encryptedAccountData = AccountData.getObject(accountData.toXML());
		
		try {
						
			String clave1 = (String)admin.claves.get("accountPaymentKey1");
			String clave2 = (String)admin.claves.get("accountPaymentKey2");
			
			if ((clave1 == null) ||  (clave2 == null) )
			    throw new Exception ("No se han definido las claves accountPaymentKey1 y accountPaymentKey2 para la Administracion: " + admin.oid );
						
			encryptedAccountData.encData.value = _encryptAccountData(clave1,clave2,accountData);				
			
		    return encryptedAccountData;
		
		} catch (XOMarshallerException e) {		
			throw new Exception(e);
		} catch (ConfigLoadException e) {
			throw new Exception(e);
		}
				
	}
	

	
	/** Descrifrar el dato a enviar a la administracin
	 * @throws Exception */
	public static AccountData decrypt (AccountData encryptedAccountData, BasicAdminConfig admin) throws Exception{
			
		AccountData decryptedAccountData = AccountData.getObject(encryptedAccountData.toXML());

		try {
						
			String clave1 = (String)admin.claves.get("accountPaymentKey1");
			String clave2 = (String)admin.claves.get("accountPaymentKey2");
			
			if ((clave1 == null) ||  (clave2 == null) )
			    throw new Exception ("No se han definido las claves accountPaymentKey1 y  accountPaymentKey2 para la Administracion: " + admin.oid );
		
			decryptedAccountData.encData.value = _decryptData(clave1,clave2,encryptedAccountData).trim();				
		    return decryptedAccountData;
		
		} catch (XOMarshallerException e) {		
			throw new Exception(e);
		} catch (ConfigLoadException e) {
			throw new Exception(e);
		}
				
	}
	
	private static String _encryptAccountData (String clave1, String clave2 ,AccountData accountData) throws Exception{
		
		
		System.out.println("[P12F] PayOnAccountDataUtil._encryptAccountData(...): ENCRIPTAR");
		System.out.println("[P12F] ========================================================");
		System.out.println("[P12F] >>>>> Semiclaves de entrada:");
		System.out.println("[P12F]      .Semiclave1: [" + clave1.substring(0,clave1.length()/2) + "...]");
		System.out.println("[P12F]      .Semiclave2: [" + clave2.substring(0,clave2.length()/2) + "...]");

		String encType =  accountData.encData.encType;
		String data = 	  accountData.encData.value;
		
	    if  (accountData.accountNumberFormat.equalsIgnoreCase(AccountData.ACCOUNT_NUMBER_FORMAT_CC)){
	    	
	    	data = StringUtils.rPadWithChar(data,' ',32);
	    	
	    } else {
	    	data = StringUtils.rPadWithChar(data,' ',32);
	    }
		
	    System.out.println("[P12F] >>>>> Trama final a encriptar (rellenar a 32 espacios por la derecha para lograr paridad x 8:"+ data  + "data.length()=" + data.length());
	 
        String claveFinal  = new String(Hex.encode(TripleDESEBCDICCryptoHelper.doXOR(clave1, clave2))).toUpperCase();
        System.out.println("[P12F] Clave final xor: [" + claveFinal.substring(0,claveFinal.length()/2) + "...]");
        TripleDESEBCDICCryptoHelper encrypter = new TripleDESEBCDICCryptoHelper();	
        if ((encType!=null)&&(encType.equalsIgnoreCase("hex")))
        	return encrypter.encryptHEX(claveFinal,data);
        else return encrypter.encrypt(claveFinal,data); 
	}

	private static String _decryptData (String clave1, String clave2 ,AccountData accountData) throws Exception {
		
		
		String encType =  accountData.encData.encType;
		String data = 	  accountData.encData.value;
		
		String claveFinal  = new String(Hex.encode(TripleDESEBCDICCryptoHelper.doXOR(clave1, clave2))).toUpperCase();
		TripleDESEBCDICCryptoHelper encrypter = new TripleDESEBCDICCryptoHelper();
		
		if ((encType!=null)&&(encType.equalsIgnoreCase("hex")))
        	return encrypter.decryptHEX(claveFinal,data);
        else return encrypter.decrypt(claveFinal,data); 
	}
}
