package p12f.exe.pasarelapagos.utils;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.ejie.r01f.log.R01FLog;
import com.ejie.r01f.reflection.ReflectionException;
import com.ejie.r01f.reflection.ReflectionUtils;

/**
 * Comparador de objetos.
 * @author ie00191m
 *
 */
public class PaymentComparator {
	
	/**
	 * Compara dos objectos y devuelve una lista con las posibles diferencias.
	 * @param object1
	 * @param object2
	 * @return List de diferencias. Si la lista est vacia no existen diferencias.
	 */
	public static List compare (Object object1, Object object2){
		return PaymentComparator._compareObjects(object1.getClass().getName(),object1,object2);
	}
	
	/**
	 * 
	 * @param fieldName   : Nombre del campo que se est comprando. Este parametro es usado para formatear el mensaje de comparacin.
	 * @param thisObject  : Valor del campo 1 
	 * @param otherObject : Valor del campo 2.
	 * @return Lista con mensajes descriptivos de la comparacin.
	 */
	private static List _compareObjects (String fieldName, Object thisObject, Object otherObject) {
		
		List mensajes = new ArrayList();
		try {
			
			if ((thisObject == null ) && (otherObject == null)) {
				return mensajes;
			} else if (( (thisObject == null ) && ! (otherObject == null) ) || ( !(thisObject == null ) && (otherObject == null)) ) {
				
				mensajes.add(new String (".Uno de los campos es nulo, el otro no."));
			} else {
				// Si los objetos son de la misma clase, comparar los miembros
				if ( thisObject.getClass().equals(otherObject.getClass()) ) {
					
					if (ReflectionUtils.isFinalInmutable(thisObject.getClass())){
						if ( !thisObject.equals(otherObject) ) {
							mensajes.add(new String (".Los objetos difieren en el campo " + fieldName + "\n" + " obj1 :=" + thisObject + "\n" + " obj2 :=" + otherObject  ));
						}
					} else {
						Field fields[] = ReflectionUtils.getAllFieldsArray(thisObject.getClass());	// Lista de miembros
						Field currentField = null;	// Miembro actual
						Object thisFValue = null;	// Valor del miembro actual en esta clase
						Object otherFValue = null;	// Valor del miembro actual en la otra clase
						
						for (int i=0; i < fields.length; i++) {
							currentField = fields[i];
							thisFValue =  ReflectionUtils.getMemberValue(thisObject,currentField.getName(),false);
							otherFValue = ReflectionUtils.getMemberValue(otherObject,currentField.getName(),false);
							
							// Antes de comprobar los objetos entre si, comprobar si son Map o List
							// En estos casos, si su longitud es 0, se convierten a null
							// Primer objeto de la comparacion
							if(thisFValue != null){
								if(ReflectionUtils.isMap(thisFValue.getClass())){
									if(((Map)thisFValue).size()==0) thisFValue = null;
								} else if(ReflectionUtils.isList(thisFValue.getClass())){
									if(((List)thisFValue).size()==0) thisFValue = null;
								}
							}
							// Segundo objeto de la comparacion
							if(otherFValue != null){
								if(ReflectionUtils.isMap(otherFValue.getClass())){
									if(((Map)otherFValue).size()==0) {
										otherFValue = null;
									}
								} else if(ReflectionUtils.isList(otherFValue.getClass())){
									if(((List)otherFValue).size()==0) {
										otherFValue = null;
									}
								}
							}
							
							if ( thisFValue != null && otherFValue != null) {
								if ( ReflectionUtils.isMap( thisFValue.getClass())){
									mensajes.addAll( PaymentComparator._compareMaps(currentField.getName(),(Map)thisFValue,(Map)otherFValue)); 
								} else  if ( ReflectionUtils.isList( thisFValue.getClass())){
									mensajes.addAll( PaymentComparator._compareLists(currentField.getName(),(List)thisFValue,(List)otherFValue)); 
								} else
									mensajes.addAll( PaymentComparator._compareObjects(currentField.getName(),thisFValue,otherFValue));
							} else {
								if ( (thisFValue == null) && (otherFValue != null)){
									mensajes.add(new String (".Los objetos difieren en el campo " + fieldName +"."+ currentField.getName()+ "\n" + " obj1 :=" + thisFValue + "\n" + " obj2 :=" + otherFValue ));
								}else {
									if ((thisFValue != null) && (otherFValue == null)){
										mensajes.add(new String (".Los objetos difieren en el campo " + fieldName +"."+ currentField.getName()+ "\n" + " obj1 :=" + thisFValue + "\n" + " obj2 :=" + otherFValue ));
									}
								}
							}
						} //for
					}
				}
			}
			
		} catch (ReflectionException reflex){
			R01FLog.to("p12f.pago").finest("Excepcion :" + reflex.getDetailedMessage());
		}
		
		return mensajes;  // Son objetos de distinta clase
	}
	
	/**
	 * @param fieldName   : Nombre del campo que se est comprando. Este parametro es usado para formatear el mensaje de comparacin.
	 * @param thisObject  : Valor del campo 1 
	 * @param otherObject : Valor del campo 2.
	 * @return Lista con mesnajes descriptivos de la comparacin.
	 */ 
	private static List _compareMaps (String fieldName, Map mapa1, Map mapa2){
		
		List mensajes = new ArrayList();
		Object key = null;
		Object objValue1 = null;
		Object objValue2 = null;
		if ( mapa1.size() != mapa2.size() ){
			mensajes.add(new String ("\n\n  Los objetos " + fieldName + ", tipo Map , no son iguales : no contienen el mismo numero de elementos"));
		} else {
			for ( java.util.Iterator it = mapa1.keySet().iterator(); it.hasNext(); ){
				key = (Object) it.next();
				objValue1 =  mapa1.get(key);
				objValue2 =  mapa2.get(key);
				if ( objValue2  == null ){
					R01FLog.to("p12f.pago").finest(" \n\n  No existe objeto de clave = " + key + " en el objeto");
					mensajes.add(new String ("\n\n + No existe objeto de clave = " + key + " en el objeto"));
				} else {
					mensajes.addAll( PaymentComparator._compareObjects(fieldName + "." + key ,objValue1,objValue2)); 
				}
			}
		}
		return mensajes;
	}
	
	/**
	 * @param fieldName   : Nombre del campo que se est comprando. Este parametro es usado para formatear el mensaje de comparacin.
	 * @param thisObject  : Valor del campo 1 
	 * @param otherObject : Valor del campo 2.
	 * @return Lista con mesnajes descriptivos de la comparacin.
	 */ 
	private static List _compareLists (String fieldName, List list1, List list2){
		List mensajes = new ArrayList();
		Object obj1 = null;
		Object obj2 = null;
		if ( list1.size() != list2.size() ){
			mensajes.add(new String (" \n\n Los objetos " + fieldName + " , tipo List , no son iguales : no contienen el mismo numero de elementos"));
		} else { 
			for ( int i = 0 ; i < list1.size() ; i++){
				obj1 = list1.get(i);
				List currentMess  = null;
				boolean found = false;
				for ( int j = 0 ; j < list2.size() ; j++) {
					obj2 = list2.get(j);
					currentMess = PaymentComparator._compareObjects(printObjectInfo("==",obj1),obj1,obj2);
					if (currentMess.size() == 0 ){
						found = true;
						break;
					}
				}
				if ( !found ) {
					String mensaje = new String (" \n\nEL OBJETO : " + printObjectInfo("ELEMENTO DE LISTA 1, TIPO : "+obj1.getClass().getName(),obj1) + "\nNO SE ENCUENTRA EN LA LISTA" );
					mensaje += printListInfo(list2);
					mensajes.add(mensaje);
				}
			}
		}
		return mensajes;
	}
	
	/**
	 * Muestra info de un objecto, campos, tipos, etc.
	 * @return String con la informacion 
	 */
	private static String printObjectInfo(String detalle ,Object thisObject){
		String info = new String(detalle) + "\n";
		for (int t=0;t< detalle.length();t++) {
			info += "-";
		}
		info += "\n";
		try {
			if ( thisObject != null ) {
				Field fields[] = ReflectionUtils.getAllFieldsArray(thisObject.getClass());  // Lista de miembros
				Field currentField = null;	// Miembro actual
				Object thisFValue = null;	// Valor del miembro actual en esta clase
				for (int i=0; i < fields.length; i++) {
					currentField = fields[i];
					thisFValue = ReflectionUtils.getMemberValue(thisObject,currentField.getName(),false);
					info += currentField.getName() + "=" + thisFValue + "\n";
				}
			} else {
				info += "Object null !!";
			}
		} catch (Throwable reflex){
			reflex.printStackTrace();
		}
		return info;
	}
	
	/**
	 * Muestra info de un objecto tipo list.
	 * @return
	 */
	private static String printListInfo(List lista){
		String mensaje = new String();
		for(int k = 0 ; k < lista.size(); k++){
			mensaje += printObjectInfo("\n==>elem(" + k + ") de la lista 2 " ,lista.get(k));
		}
		return mensaje;
	}
	
}
