package p12f.exe.pasarelapagos.utils;

import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import p12f.exe.pasarelapagos.objects.DatosPago;
import p12f.exe.pasarelapagos.objects.PeriodoPago;
import p12f.exe.pasarelapagos.paymentrequest.PeticionPago;
import p12f.exe.pasarelapagos.utils.exceptions.CSBFormatException;
import p12f.exe.pasarelapagos.utils.exceptions.PaymentDataHelpperException;


/*
 *  Clase de ayuda que dispone de metodos para conformar los OIDS Y 
 *  los cdigos de las tramas csb.
 */
public class PaymentDataHelper {
	
    
    /* Variables Estaticas */
    
     public static final String PAYMENTTYPE_ADMIN = "PAYMENTTYPE_ADMIN";
     public static final String PAYMENTTYPE_TYPE = "PAYMENTTYPE_TYPE";
     public static final String PAYMENTTYPE_FORMAT = "PAYMENTTYPE_FORMAT";    
    
    
    
    
	/**
	 * Obtener el oid.
	 * @param peticionPago
	 * @return oid de la Peticion de Pago.
	 * @throws PaymentDataHelpperException 
	 */
	
	public static String getOID(PeticionPago peticionPago) throws PaymentDataHelpperException{
		// Actualemente el OID es igual al CSBCode.
		// Este mtodo se crea con la intencin de futuras modificaciones.
		return getCSBCode(peticionPago);
	}
	
	/**
	 * Obtener el CSB Code.
	 * @param peticionPago
	 * @return oid de la Peticion de Pago.
	 * @throws PaymentDataHelpperException 
	 */
	
	public static String getCSBCode( PeticionPago peticionPago ) throws PaymentDataHelpperException {
		
      	int formato = Integer.parseInt(peticionPago.datosPago.formato);
		switch ( formato ){
			case 507 :
				return getCSB507Code(peticionPago);			
			case 502 :
				return getCSB502Code(peticionPago);		
			case 508 :
				return getCSB508Code(peticionPago);
			case 521 :
				return getCSB521Code(peticionPago);				
			case 522 :
				return getCSB522Code(peticionPago);
		}	
		return null;
	}
	
	/**
	 * Construye un codigo CSB07;
	 * @param peticionPago
	 * @return
	 * @throws PaymentDataHelpperException 
	 */
	public static String getCSB507Code (PeticionPago peticionPago ) throws PaymentDataHelpperException{
		
		String csbCode = CSBConstants.CSB507.ID_APP.defaultValue + CSBConstants.CSB507.csbCode;
		//Descomponer emisor : "0483300-501"
		String emisorCSB = null;
		String sufijoCSB = null;
		try {
			StringTokenizer st = new StringTokenizer(peticionPago.datosPago.emisor,"-");
			emisorCSB = st.nextToken();
			sufijoCSB = st.nextToken(); 
		} catch (NullPointerException nulEx ){
			throw new PaymentDataHelpperException("El cdigo del Emisor no sigue un formato correcto : cif_sin_letras - sufijo , p.e.: 04833001-501 ");
		} catch (NoSuchElementException nulEx ){
			throw new PaymentDataHelpperException("El cdigo del Emisor no sigue un formato correcto : cif_sin_letras - sufijo , p.e.: 04833001-501 ");
		}
		
		/* Obtener Objeto Periodo de Pago */
		
		PeriodoPago periodoPago = null;
		if  ( (peticionPago.datosPago == null ) || (peticionPago.datosPago.periodosPago == null ) || ( peticionPago.datosPago.periodosPago.values().isEmpty() ) ){
			throw new  PaymentDataHelpperException("PeticionPago.datosPago es nulo o no tiene ningun Periodo de Pago");
		}
		periodoPago = (PeriodoPago) peticionPago.datosPago.periodosPago.values().iterator().next();		
		if ( periodoPago == null ) {
			throw new PaymentDataHelpperException("PeticionPago.datosPago.periodosPago no contiene ningn periodo de Pago");
	    }
		 
		 
		String importe = Long.toString(periodoPago.importe);
		while(importe.length()<10)
            importe = "0" + importe;
		
		/* Construir el CSB */		 
		 csbCode +=   emisorCSB + sufijoCSB  
		             + peticionPago.datosPago.referencia 
		             + periodoPago.identificacion 
		             + importe 
		             + CSBConstants.CSB507.DIGITO_PARIDAD.defaultValue;
		return  csbCode;
	}
	   
	
	public static String getCSB502Code (PeticionPago peticionPago ) throws PaymentDataHelpperException {
		
	    String csbCode = CSBConstants.CSB502.ID_APP.defaultValue + CSBConstants.CSB502.csbCode;
		
		/* Obtener Objeto Periodo de Pago */
		
		PeriodoPago periodoPago = null;
		if  ( (peticionPago.datosPago == null ) || (peticionPago.datosPago.periodosPago == null ) || ( peticionPago.datosPago.periodosPago.values().isEmpty() ) ){
			throw new  PaymentDataHelpperException("PeticionPago.datosPago es nulo o no tiene ningun Periodo de Pago");
		}
		periodoPago = (PeriodoPago) peticionPago.datosPago.periodosPago.values().iterator().next();		
		if ( periodoPago == null ) {
			throw new PaymentDataHelpperException("PeticionPago.datosPago.periodosPago no contiene ningn periodo de Pago");
	    }		 	
		 
		String importe = Long.toString(periodoPago.importe);
		while(importe.length()<8)
            importe = "0" + importe;
		
		/* Construir el CSB */		 
		 csbCode +=  peticionPago.datosPago.emisor   
		             + peticionPago.datosPago.referencia 
		             + periodoPago.identificacion 
		             + importe;
		return  csbCode;
	}
	
	
	public static String getCSB508Code (PeticionPago peticionPago ) throws PaymentDataHelpperException {
		
	       
        String csbCode = CSBConstants.CSB508.ID_APP.defaultValue + CSBConstants.CSB508.csbCode;
	
		String emisorCSB = null;
		String tesoreroCSB = null;
		/* En la primera versin no se dispona de un campo separado llamado "EntidadTesorera" que slo
         * se utiliza para el CSB 508.
         * try {
			StringTokenizer st = new StringTokenizer(peticionPago.datosPago.emisor,"-");
			emisorCSB = (String)st.nextToken();
			tesoreroCSB = (String)st.nextToken(); 
		} catch (NullPointerException nulEx ){
			throw new PaymentDataHelpperException("El cdigo del Emisor no sigue un formato correcto : cif_sin_letras - tesorero , p.e.: 04833001-673332 ");
		} catch (NoSuchElementException nulEx ){
			throw new PaymentDataHelpperException("El cdigo del Emisor no sigue un formato correcto : cif_sin_letras - tesorero , p.e.: 04833001-673332 ");
		}*/
        
         //* Cambio 16 de Septiembre.2005. Entidad Tesorera viene como campo separado.
         emisorCSB = peticionPago.datosPago.emisor;
         tesoreroCSB = peticionPago.emisor.entidadTesorera;
        		
		// Obtenemos la fecha limite de pago.
		String fechaLimiteCSB = ((PeriodoPago)peticionPago.datosPago.periodosPago.get("periodoNormal")).fechaFin;
		
		/* Obtener Objeto Periodo de Pago */
		
		PeriodoPago periodoPago = null;
		if  ( (peticionPago.datosPago == null ) || (peticionPago.datosPago.periodosPago == null ) || ( peticionPago.datosPago.periodosPago.values().isEmpty() ) ){
			throw new  PaymentDataHelpperException("PeticionPago.datosPago es nulo o no tiene ningun Periodo de Pago");
		}
		periodoPago = (PeriodoPago) peticionPago.datosPago.periodosPago.values().iterator().next();		
		if ( periodoPago == null ) {
			throw new PaymentDataHelpperException("PeticionPago.datosPago.periodosPago no contiene ningn periodo de Pago");
	    }	
		 
		String importe = Long.toString(periodoPago.importe);
		while(importe.length()<8)
            importe = "0" + importe;
		
		/* Construir el CSB */		 
		 csbCode +=   tesoreroCSB
		 			 + fechaLimiteCSB
		 			 + emisorCSB   
		             + peticionPago.datosPago.referencia 
		             + periodoPago.identificacion 
		             + importe;
		return  csbCode;
		
	}

	public static String getCSB521Code (PeticionPago peticionPago ) throws PaymentDataHelpperException {		
	    
	    String csbCode = CSBConstants.CSB521.ID_APP.defaultValue + CSBConstants.CSB521.csbCode;
		//No hace falta descomponer el emisor
		String emisorCSB = peticionPago.datosPago.emisor;
		
		/* Obtener Objeto Periodo de Pago */
		
		PeriodoPago periodoPago = null;
		if  ( (peticionPago.datosPago == null ) || (peticionPago.datosPago.periodosPago == null ) || ( peticionPago.datosPago.periodosPago.values().isEmpty() ) ){
			throw new  PaymentDataHelpperException("PeticionPago.datosPago es nulo o no tiene ningun Periodo de Pago");
		}
		periodoPago = (PeriodoPago) peticionPago.datosPago.periodosPago.values().iterator().next();		
		if ( periodoPago == null ) {
			throw new PaymentDataHelpperException("PeticionPago.datosPago.periodosPago no contiene ningn periodo de Pago");
	    }		 
		 
		String importe = Long.toString(periodoPago.importe);
		while(importe.length()<8)
            importe = "0" + importe;
		
		/* Construir el CSB */		 
		 csbCode +=   emisorCSB  
		             + peticionPago.datosPago.referencia 
		             + periodoPago.identificacion 
		             + importe 
		             + CSBConstants.CSB521.DIGITO_PARIDAD.defaultValue;
		return  csbCode;
	}
	
	
	public static String getCSB522Code (PeticionPago peticionPago ) throws PaymentDataHelpperException {		

	    String csbCode = CSBConstants.CSB522.ID_APP.defaultValue + CSBConstants.CSB522.csbCode;
		//Descomponer emisor : "0483300-501"
		String emisorCSB = peticionPago.datosPago.emisor;
		
		/* Obtener Objeto Periodo de Pago */
		
		PeriodoPago periodoPagoVoluntario = (PeriodoPago)peticionPago.datosPago.periodosPago.get("periodoVoluntario");
		PeriodoPago periodoPagoConRecargo = (PeriodoPago)peticionPago.datosPago.periodosPago.get("periodoConRecargo");

		String importeVoluntario = Long.toString(periodoPagoVoluntario.importe);
		while(importeVoluntario.length()<8)
            importeVoluntario = "0" + importeVoluntario;
		String importeRecargo = Long.toString(periodoPagoConRecargo.importe);
		while(importeRecargo.length()<8)
            importeRecargo = "0" + importeRecargo;
		
        String identificacionVoluntario = periodoPagoVoluntario.identificacion;
        String identificacionConRecargo = periodoPagoConRecargo.identificacion;
        String tributo = identificacionVoluntario.substring(1,4);
        String ejercicio = identificacionVoluntario.substring(4,6);
        String fechaVol = identificacionVoluntario.substring(6,10);
        String fechaRec = identificacionConRecargo.substring(6,10);
        
		/* Construir el CSB */		 
		 csbCode +=   emisorCSB  
		             + peticionPago.datosPago.referencia 
		             + tributo
                     + ejercicio
                     + fechaVol
		             + importeVoluntario
		             + fechaRec
		             + importeRecargo;
		return  csbCode;
	}
	
	/*
	 *  Metodo prueba clase;
	 */
	public static void main(String args[]) throws PaymentDataHelpperException {
		
		/*****************  DATOS PETICION PAGO ************************* */
		
		DatosPago datosPeticionPago = new DatosPago();
		
		datosPeticionPago.formato = "507";
		datosPeticionPago.emisor = "04833001-501"; 
		datosPeticionPago.cpr     = "900507"; // Codigo procedimiento recaudatorio		
		datosPeticionPago.validar = DatosPago.NO_VALIDAR;
		datosPeticionPago.referencia = "0005026673427";
		datosPeticionPago.periodosPago = new HashMap();	
		
		PeriodoPago periodoPago = new PeriodoPago();
		periodoPago.id = "Periodo de Pago Actual";
		periodoPago.fechaFin =    "010505";
		periodoPago.fechaInicio = "010605";
		periodoPago.importe = 123456789; 
		periodoPago.identificacion = "123456"; // Identificacin del pago en este periodo
	
		datosPeticionPago.periodosPago.put(periodoPago.id,periodoPago);		
		
		PeticionPago peticionPago = new PeticionPago();		
	
		peticionPago.datosPago = datosPeticionPago;
	}
	
    /**
     * Funcion para obtener los parmetros de entrada para la busqueda de una configuracion (PaymentTypeConfig)
     * Su uso es el siguiente :
     *    
     *     Map attributtes = PaymentDataHelper.getPaymentTypeConfigAttributes(peticionPago)
     *     admin = (String)attributtes.get(PaymentDataHelper.PAYMENTTYPE_ADMIN) 
     *  
     * 
     * @param peticionPago
     * @return
     * @throws PaymentDataHelpperException
     */
	
    public static Map getPaymentTypeConfigAttributes (PeticionPago peticionPago) throws PaymentDataHelpperException{
        
        Map data = new HashMap();
        String cdbCode = peticionPago.datosPago.codigo;
        String cifEmisor = null;
        String tipo = null;
        String identificacion = null;
        
        try {
            if ( cdbCode == null ) throw new PaymentDataHelpperException ("El codigo CSB es nulo (peticionPago.datosPago.codigo ");
            int formato = Integer.parseInt(peticionPago.datosPago.formato);
            switch ( formato ){
                case 507 :
                    cifEmisor = (String)CSBHelpper.decode507(cdbCode).get(CSBConstants.CSB507.EMISOR);
                    tipo = (String)CSBHelpper.decode507(cdbCode).get(CSBConstants.CSB507.SUFIJO);
                    break;
                case 502 :
                    cifEmisor = (String)CSBHelpper.decode502(cdbCode).get(CSBConstants.CSB502.EMISOR);
                    identificacion = (String)CSBHelpper.decode502(cdbCode).get(CSBConstants.CSB502.IDENTIFICACION);
                    tipo = (String)CSBHelpper.decode502(identificacion,CSBConstants.CSB502.IDENTIFICACION).get(CSBConstants.CSB502.TRIBUTO);
                    break;
                case 508 :
                    cifEmisor = (String)CSBHelpper.decode508(cdbCode).get(CSBConstants.CSB508.EMISOR);
                    identificacion = (String)CSBHelpper.decode508(cdbCode).get(CSBConstants.CSB508.IDENTIFICACION);
                    tipo = (String)CSBHelpper.decode508(identificacion,CSBConstants.CSB508.IDENTIFICACION).get(CSBConstants.CSB508.TRIBUTO);
                    break;
                case 521 :
                    cifEmisor = (String)CSBHelpper.decode521(cdbCode).get(CSBConstants.CSB521.EMISOR);
                    identificacion = (String)CSBHelpper.decode521(cdbCode).get(CSBConstants.CSB521.IDENTIFICACION);
                    tipo = (String)CSBHelpper.decode521(identificacion,CSBConstants.CSB521.IDENTIFICACION).get(CSBConstants.CSB521.TRIBUTO);
                    break;
                case 522 :
                    cifEmisor = (String)CSBHelpper.decode522(cdbCode).get(CSBConstants.CSB522.EMISOR);
                    // Se presupone que el tipo(tributo) sera el mismo para el Periodo con Recargo que si Recargo.
                    identificacion = (String)CSBHelpper.decode522(cdbCode).get(CSBConstants.CSB522.IDENTIFICACION_SIN_RECARGO);
                    tipo = (String)CSBHelpper.decode522(cdbCode).get(CSBConstants.CSB522.TRIBUTO);
                    break;
              }
            
            data.put(PAYMENTTYPE_ADMIN,cifEmisor);
            data.put(PAYMENTTYPE_TYPE,tipo);
            data.put(PAYMENTTYPE_FORMAT,peticionPago.datosPago.formato);
        
        } catch (CSBFormatException e) {           
            throw new PaymentDataHelpperException(e);
        }
        return data;
        
    }
    
}
