package p12f.exe.pasarelapagos.utils;

import java.util.HashMap;

import com.ejie.r01f.xmlproperties.XMLProperties;

import p12f.exe.pasarelapagos.objects.security.SecurityContext;
import p12f.exe.pasarelapagos.security.app.SecurityContextGeneratorFactory;
import p12f.exe.pasarelapagos.security.app.SecurityContextGeneratorInterface;
import p12f.exe.pasarelapagos.security.app.SecurityContextParam;
import p12f.exe.pasarelapagos.security.app.custom.DefaultSecurityContextGenerator;
import p12f.exe.pasarelapagos.security.app.manager.custom.DefaultSecurityManager;

/**
 * Clase de utilidades para manejar los contextos de seguridad.
 * @author dbenitof
 */
public class SecurityContextUtils {
	
	public final static String SERVICE_CONFIG = "configService";
	public final static String SERVICE_SIPCA = "sipcaService";
	public final static String SERVICE_FILE = "fileService";
	public final static String SERVICE_DATA = "dataService";
	public final static String SERVICE_OTP = "otpService";
	public final static String SERVICE_IKUS = "ikusService";
	public final static String SERVICE_SMS = "smsService";
	public final static String SERVICE_MAIL = "mailService";
	
	/**
	 * Constructor privado para evitar que se instancie.
	 */
	private SecurityContextUtils() {
		
	}
	
	public static SecurityContext getSecurityContext(String codAplic, String serviceName) throws SecurityException {
		SecurityContextParam appClientID = null;
		SecurityContextParam appClientKey = null;
		try {
			//Lectura de claves cli.
			String id = XMLProperties.getPropertyNode(codAplic,"defaultSecurityClientDefinition/services/service[@id='"+serviceName+"']/app/@id").getNodeValue();
			String key = XMLProperties.get(codAplic,"defaultSecurityClientDefinition/services/service[@id='"+serviceName+"']/app[@id='"+ id + "']/appClientKey");
			//Crear parmetros de contexto.
			appClientID = new SecurityContextParam(DefaultSecurityContextGenerator.APP_CLIENT_ID, id);
			appClientKey = new SecurityContextParam(DefaultSecurityContextGenerator.APP_CLIENT_KEY, key);
		}catch (Exception ex) {
			throw new SecurityException("No se ha podido obtener el par DefaultSecurityContextGenerator.APP_CLIENT_ID / APP_CLIENT_KEY en el cliente del servicio: " + ex);
		}
		SecurityContextParam serviceNameCtxParam = new SecurityContextParam(DefaultSecurityContextGenerator.APP_SERVICE_NAME, serviceName);
		
		HashMap inputdataList = new HashMap ();
		inputdataList.put(appClientID.getID(),appClientID );
		inputdataList.put(appClientKey.getID(),appClientKey );
		inputdataList.put(serviceNameCtxParam.getID(),serviceNameCtxParam );
		//Obtiene un SecuriyConext por Defecto.
		SecurityContextGeneratorInterface scxGenerator = SecurityContextGeneratorFactory.getInstance(SecurityContext.CUSTOM);
		SecurityContext ctx = scxGenerator.generateSecurityContext(inputdataList);
		return ctx;
	}
	
	public static SecurityContext getSecurityContext(String codAplic) throws SecurityException {
		// por defecto se devuelve el contexto de seguridad del servicio de configuraciones
		return getSecurityContext(codAplic, "configService");
	}
	
	public static boolean checkSecurityContext(String codAplic, SecurityContext ctx) throws SecurityException {
		DefaultSecurityManager mng = new DefaultSecurityManager(codAplic);
		if ( mng.hasPermission(ctx)) {
			return true;
		} else {
			throw new SecurityException("Security Context No Valido");
		}
		
	}
	
}
