/*
 * p12/exe/listados/p12CListado.java
 * 
 */
package p12f.exe.pasarelapagos.utils.fop;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;

import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.avalon.framework.logger.ConsoleLogger;
import org.apache.avalon.framework.logger.Logger;
import org.apache.fop.apps.Driver;
import org.apache.fop.apps.FOPException;
import org.apache.fop.apps.InputHandler;
import org.apache.fop.image.FopImageFactory;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.NodeIterator;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import com.ejie.r01f.log.R01FLog;
import com.ejie.r01f.util.StringUtils;
import com.ejie.r01f.xmlproperties.XMLProperties;


/**
 * Clase que representa a un listado
 */
public class XMLTransformer {
	
	private static int logLevel;
	
    static {
    	logLevel = Integer.parseInt( XMLProperties.get("p12f","pdfLogLevel"));
    }
        
    /**
     * Genera un listado
     */ 
    public static byte[] getPDF(Reader xmlReader, File xsltFile) throws XMLTransformerException {

		R01FLog.to("p12f.transformer").info("[P12F][FOP] Generando PDF... XMLTransformer.getPDF(...)");
		R01FLog.to("p12f.transformer").info("[P12F][FOP] Plantilla: " + xsltFile.getPath());

		ByteArrayOutputStream baosResult = new ByteArrayOutputStream(); // contenido PDF resultante
		
		try {
			FopImageFactory.resetCache();
		} catch (Exception e) {
			R01FLog.to("p12f.transformer").warning(e.getMessage());
		} catch (Throwable t) {
			R01FLog.to("p12f.transformer").warning(t.getMessage());
		}
		

        try {
        	String xsltCharset = XMLProperties.get("p12f","xslTemplateConfig/xsltCharset"); // UTF-8
        	
        	if (xsltCharset!=null && !"".equals(xsltCharset)) {

	            Map mapVars = new HashMap(); // valores de las variables $nombreVariable$ a sustituir en la plantilla
	
	   			R01FLog.to("p12f.transformer").info("[P12F][FOP] Mapeo de $variables$ a sustituir en plantilla");
	            NodeIterator nodeIt = XMLProperties.getPropertyNodeIterator("p12f","xslTemplateConfig/varList/var");
	        	for (Node n=nodeIt.nextNode(); n!=null; n=nodeIt.nextNode()) {
	       			String strValue = n.getFirstChild().getNodeValue(); 
	       			String strName  = n.getAttributes().getNamedItem("name").getNodeValue(); 
	
	       			mapVars.put(strName, strValue);
	
	       			R01FLog.to("p12f.transformer").info("[P12F][FOP] - $" + strName + "$ <== " + strValue);
	        	}
	
	            // Obtenemos el contenido de la plantilla XSLT...
	            FileInputStream inStr = new FileInputStream(xsltFile);
	            byte fileContent[] = new byte[(int)xsltFile.length()];
	            inStr.read(fileContent);
	            String strPlantillaXSLContent = new String(fileContent, xsltCharset); // contenido plantilla xslt
	
	            // Sustituimos las variables rootSrv/rootDatos/... por los valores definidos en properties...
	            strPlantillaXSLContent = StringUtils.replaceVariableValues(strPlantillaXSLContent, "$", mapVars).trim();
	
	            // Al coger la plantilla del fichero a veces mete carcteres raros al principio, con esto nos aseguramos de quitarlos
	            int indiceComienzoXML = strPlantillaXSLContent.indexOf("<?xml");
	            strPlantillaXSLContent = strPlantillaXSLContent.substring(indiceComienzoXML);
	            
	            // Ahora hay que usar ese String modificado, que contiene el nuevo
	            // contenido de la plantilla, en vez de utilizar el fichero...
	            StringReader strReader       = new StringReader(strPlantillaXSLContent); //new StringReader("...<xsl:stylesheet ...</xsl:stylesheet>");
	            StringWriter strWriter       = new StringWriter();
	            StreamSource xslStreamSource = new StreamSource(strReader);
	            StreamResult xmlStreamResult = new StreamResult(strWriter);
	
	            // Transformer con la plantilla modificada (rootSrv, rootDatos, ...)
	            TransformerFactory tFactory = TransformerFactory.newInstance();
	            Templates   templates   = tFactory.newTemplates(xslStreamSource); //tFactory.newTemplates(new StreamSource("recibo_pdf.xsl"));
	            Transformer transformer = templates.newTransformer();
	
	            StreamSource xmlSource = new StreamSource(xmlReader);
	            transformer.transform(xmlSource, xmlStreamResult);
	
	            StringWriter sw = (StringWriter) xmlStreamResult.getWriter();
	            StringBuffer sb = sw.getBuffer();
	            String strXmlResult = sb.toString();
	            // TRAZA: Resultado del transform con la plantilla modificada: OK --- 
	            //R01FLog.to("p12f.transformer").info("[FOP][P12F] - Resultado: " + strXmlResult);
	
	            InputStream iStream = new ByteArrayInputStream(strXmlResult.getBytes(xsltCharset)); //UTF-8
	            InputSource iSource = new InputSource(iStream);
	            iSource.setEncoding(xsltCharset); //UTF-8
	
	            R01FLog.to("p12f.transformer").info("[P12F][FOP] Driver...");
	            Driver driver = new Driver();  
	            driver.setLogger(new ConsoleLogger(logLevel)); // Generar el log con el nivel que se quiera
	            driver.setRenderer(Driver.RENDER_PDF);
	            driver.setInputSource(iSource);
	            driver.setOutputStream(baosResult);
	            R01FLog.to("p12f.transformer").info("[P12F][FOP] - Encoding: " + iSource.getEncoding());
	            R01FLog.to("p12f.transformer").info("[P12F][FOP] - Renderizando PDF: driver.run() ...");
	            driver.run();
	
	            return baosResult.toByteArray();
            
        	} else {
        		
        		//Using a Configuration File To access an external configuration: 
				//File userConfigFile = new File("/aplic/p01/html/datos/userconfig.xml");
				//Options options = new Options(userConfigFile);
	            // Obtener un parser del XML utilizando el fichero XSLT		
	            InputHandler xsltInputHandler = new XSLTInputHandler(xmlReader,xsltFile);
	            XMLReader parser = xsltInputHandler.getParser();
	            //System.out.println(parser.getClass().getName());
	            if (parser == null) throw new XMLTransformerException("No se ha podido obtener el parser (filtro XSLT al XML de entrada");
	                        
	            // Aplicar fop aplicando al parser (filtro) el xml de entrada
	            Driver driver = new Driver();  
	            
	            // Generar el log con el nivel que se quiera
	            Logger log = new ConsoleLogger(logLevel);
	            driver.setLogger(log);
	            
	            driver.setRenderer(Driver.RENDER_PDF);      
	            driver.setOutputStream(baosResult);
	            driver.render( xsltInputHandler.getParser(), new InputSource(xmlReader) );
	          
	            return baosResult.toByteArray();
        	}
            
        } catch (FOPException fopEx) {
        	fopEx.printStackTrace();
            throw new XMLTransformerException(fopEx.toString());
        } catch (Exception fopEx) {
        	fopEx.printStackTrace();
            throw new XMLTransformerException(fopEx.toString());
        }
    }

    public static void getHTML(OutputStream out,Reader xmlReader,File xsltFile) throws XMLTransformerException {
        try {
            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer( new StreamSource(xsltFile) );
            transformer.transform( new StreamSource(xmlReader),new StreamResult(out) );
        } catch (Exception ex) {
            throw new XMLTransformerException(ex.toString());
        }
    }
   
}