/*
 * p12/exe/listados/XSLTInputHandler.java
 */
package p12f.exe.pasarelapagos.utils.fop;

// Imported TraX classes
import java.io.File;
import java.io.Reader;

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.stream.StreamSource;

import org.apache.fop.apps.Driver;
import org.apache.fop.apps.FOPException;
import org.apache.fop.apps.InputHandler;
import org.apache.fop.messaging.MessageHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLFilter;
import org.xml.sax.XMLReader;


/**
 * XSLTInputHandler basically takes an xmlfile and transforms it with an xsltFile
 * and the resulting xsl:fo document is input for Fop.
 */
public class XSLTInputHandler extends InputHandler {
    Reader xmlReader;   // XML de entrada
    File xsltFile;      // Fichero XSLT de transformacion
    
    public XSLTInputHandler(Reader newXMLReader, File newXSLTFile ) {
        this.xmlReader = newXMLReader;
        this.xsltFile = newXSLTFile;
    }
    
    /**
     *  overwrites the method of the super class to return the xmlfile
     */
    public InputSource getInputSource() {
        return new InputSource(xmlReader);
    }
    
    /**
     *  overwrites this method of the super class and returns an XMLFilter instead of a
     *  simple XMLReader which allows chaining of transformations
     */
    public XMLReader getParser() {
        return getXMLFilter(xmlReader,xsltFile);
    }
    
    /**
     * Creates from the transformer an instance of an XMLFilter which
     * then can be used in a chain with the XMLReader passed to Driver. This way
     * during the conversion of the xmlReader file + xslt stylesheet the resulting
     * data is fed into Fop. This should help to avoid memory problems
     * @param xmlfile The xmlfile containing the text data
     * @param xsltFile An xslt stylesheet
     * @return XMLFilter an XMLFilter which can be chained together with other XMLReaders or XMLFilters
     */
    public static XMLFilter getXMLFilter(Reader xmlReader,File xsltFile) {
        try {
            // ============= Crear el filtro xslt a partir de una factoria de Transformers
            // ============= a este filtro asociarle un parser del xml de entrada
            TransformerFactory tFactory = TransformerFactory.newInstance();
            // Determine whether the TransformerFactory supports The use uf SAXSource and SAXResult
            if (tFactory.getFeature(SAXSource.FEATURE) && tFactory.getFeature(SAXResult.FEATURE)) {
                // Create an XMLFilter for the stylesheet.                
                SAXTransformerFactory saxTFactory = ((SAXTransformerFactory)tFactory);
                XMLFilter xmlfilter = saxTFactory.newXMLFilter( new StreamSource(xsltFile) );
                
                // Create an XMLReader (instancia de un xml parser)
                XMLReader parser = null;
                parser = createParser();		// Metodo de InputHandler
                /*
                // -- V. Alx no funciona --         
                javax.xml.parsers.SAXParserFactory spf = javax.xml.parsers.SAXParserFactory.newInstance();
                //spf.setValidating(validation);
                javax.xml.parsers.SAXParser saxParser = spf.newSAXParser();
                parser = saxParser.getXMLReader(); 
                */                           
                if (parser == null) {
                    MessageHandler.errorln("ERROR: Unable to create SAX parser");
                    System.exit(1);
                }
                
                // xmlFilter uses the XMLReader as its reader.
                xmlfilter.setParent(parser);
                return xmlfilter;
            } else {
                MessageHandler.errorln("Your parser doesn't support the features SAXSource and SAXResult." +
                                       "\nMake sure you are using a xsl parser which supports TrAX");
                System.exit(1);
                return null;
            }
        } catch (Exception ex) {
            //            ex.printStackTrace();
            MessageHandler.errorln(ex.toString());
            return null;
        }
    }

	public void run(Driver arg0) throws FOPException {
		// TODO Auto-generated method stub
		
	}

}

