package p12f.exe.pasarelapagos.utils.pif;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.util.List;

import org.w3c.dom.Document;

import p12f.exe.pasarelapagos.helpers.SessionTokenHelper;

import com.ejie.r01f.log.R01FLog;
import com.ejie.r01f.xml.utils.XMLParserHelper;
import com.ejie.r01f.xmlproperties.XMLProperties;
import com.ejie.y31.exception.Y31JanoServiceFileNotFoundException;
import com.ejie.y31.exception.Y31JanoServiceGenericException;
import com.ejie.y31.factory.Y31JanoServiceAbstractFactory;
import com.ejie.y31.service.Y31JanoService;
import com.ejie.y31.vo.Y31AttachmentBean;

public class PIFHelper {

	private String _sessionToken = null;

	public PIFHelper() {
		SessionTokenHelper tokenHelper = new SessionTokenHelper();

		if (tokenHelper.setSessionToken( // Hace falta el token de sesin para poder usar el PIF...
				XMLProperties.get("p12f","pif/sessionLoginAppId"), // P12G // X43F
				XMLProperties.get("p12f","pif/simulateXLNets").trim().equalsIgnoreCase("true"))
			) {
			setSessionToken(tokenHelper.getSessionToken());
		}
		else {
			R01FLog.to("p12f.pif").warning("[P12F] PIFHelper() no ha podido obtener token");
		}
	}

	/**
	 * Constructor que utiliza token que ya tengamos (no pide un nuevo token)
	 * Inicializar previamente con SessionTokenHelper...
	 *     SessionTokenHelper tokenHelper = new SessionTokenHelper();
	 *     if (tokenHelper.setSessionToken("P12G",false))
	 *         SignHelper signHelper = new SignHelper(tokenHelper.getSessionToken());
	 * @param strToken
	 */
	public PIFHelper(String strToken) {
		_sessionToken = strToken;
	}

	/**
	 * getter _sessionToken
	 * @return
	 */
	public String getSessionToken() {
		return _sessionToken;
	}

	/**
	 * setter _sessionToken
	 * @param strToken
	 */
	public void setSessionToken(String strToken) {
		_sessionToken = strToken;
	}

	/**
	 * Generar un fichero de liquidaciones para SIPCA a partir de los registros en base de datos 
	 * 
	 * @param filePath    /p12/pif/samples/
	 * @param fileName    test00001.xml
	 * @param fileContent Contenido del fichero a escribir
	 */
	public void writeContentToFile(String filePath, String fileName, String fileContent) {
		writeContentToFile(filePath + fileName, fileContent);
	}

	/**
	 * 
	 * @param filePath
	 * @param fileName
	 * @param fileContent
	 * @param expirationSecs
	 */
	public void writeContentToFile(String filePath, String fileName, String fileContent, Long expirationSecs) {
		writeContentToFile(filePath + fileName, fileContent, expirationSecs);
	}

	/**
	 * 
	 * @param fileNameWithPath
	 * @param fileContent
	 */
	public void writeContentToFile(String fileNameWithPath, String fileContent) {
		Long t = null;
		writeContentToFile(fileNameWithPath, fileContent, t);
	}

	/**
	 * 
	 * @param fileNameWithPath
	 * @param fileContent
	 * @param expirationSecs
	 */
	public void writeContentToFile(String fileNameWithPath, String fileContent, Long expirationSecs) {
		R01FLog.to("p12f.pif").info("[P12F][PIFHelper] Generacin de fichero " + fileNameWithPath + " en PIF...");
		
		Long defaultExpirationSecs;
		
		try {
			Y31JanoService service = Y31JanoServiceAbstractFactory.getInstance();
			
			Document xlnets = XMLParserHelper.parseXML(new ByteArrayInputStream(_sessionToken.getBytes()), false);
			
			InputStream is = new ByteArrayInputStream(fileContent.getBytes());
			
			String to = fileNameWithPath.replace('\\', '/');
			
			boolean preserveName = true;
			String strSegundos = XMLProperties.get("p12f","pif/file/expirationSecs");
			if (expirationSecs==null)
				defaultExpirationSecs = new Long(strSegundos==null ? 7200 : Long.parseLong(strSegundos)); // por defecto: 2h >> 60*60*2=7200
			else
				defaultExpirationSecs = expirationSecs;
			
			R01FLog.to("p12f.pif").info("[P12F][PIFHelper] PIF borrar el fichero " + fileNameWithPath + " pasados " + defaultExpirationSecs + " segundos...");
			
			// invocando el metodo
			Y31AttachmentBean resul = service.put(xlnets, is, to, preserveName, defaultExpirationSecs);
			
			_traza("writeContentToFile/PUT", resul);
			
		} catch (Y31JanoServiceGenericException e1) {
			e1.printStackTrace();
		} catch (Exception e ) {
			e.printStackTrace();
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}
	
	/**
	 * 
	 * @param filePath
	 * @param fileName
	 * @return
	 */
	public String readContentFromFile(String filePath, String fileName) {
		return readContentFromFile(filePath + fileName);
	}
	
	/**
	 * 
	 * @param filePath    /p12/pif/samples/
	 * @param fileName    test00001.xml
	 * @return            contenido del fichero leido de PIF
	 */
	public String readContentFromFile(String fileNameWithPath) {
		StringBuffer fileContent = new StringBuffer(""); //"<test>ejemplo</test>";
		
		BufferedInputStream bis = null;
		
		// implementar lectura del fichero por PIF...
		R01FLog.to("p12f.pif").info("[P12F][PIFHelper] Lectura de fichero " + fileNameWithPath + " de PIF...");
		try {
			Y31JanoService service = Y31JanoServiceAbstractFactory.getInstance();
			
			Document xlnets = XMLParserHelper.parseXML(new ByteArrayInputStream(_sessionToken.getBytes()), false);
			
			// invocando el metodo
			bis = new BufferedInputStream(service.get(xlnets, fileNameWithPath));
			
			// ahora tocara leer el contenido del fichero
			byte[] bytesFileContent = new byte[1024];
			
			int bytesRead=0;
			String s;
			
			while ((bytesRead = bis.read(bytesFileContent)) != -1) {
				s = new String(bytesFileContent, 0, bytesRead);
				fileContent.append(s);
			}
			
		} catch (Y31JanoServiceGenericException e1) {
			e1.printStackTrace();
		} catch (Exception e ) {
			e.printStackTrace();
		} finally {
			try {
				// ES IMPORTANTE QUE AL FINAL SE HAGA UN CLOSE DEL INPUTSTREAM
				bis.close();
			} catch (Exception e2) {
				e2.printStackTrace();
			}
		}
		R01FLog.to("p12f.pif").info("[P12F][PIFHelper] Longitud de fichero: " + fileContent.length());
		return fileContent.toString();
	}
	
	/**
	 * Lista de ficheros en un directorio PIF (objetos Y31AttachmentBean)
	 * 
	 * @param filePath
	 * @return
	 */
	public List fileList(String filePath) {
		R01FLog.to("p12f.pif").info("[P12F][PIFHelper] fileList: Listando todos los ficheros de " + filePath + " en PIF");
		
		List fList = null;
		
		try {
			Y31JanoService service = Y31JanoServiceAbstractFactory.getInstance();
			
			Document xlnets = XMLParserHelper.parseXML(new ByteArrayInputStream(_sessionToken.getBytes()), false);
			
			fList = service.list(xlnets, filePath);
			R01FLog.to("p12f.pif").info("[P12F][PIFHelper] listar "+fList.size()+" ficheros");
			
//			Y31AttachmentBean resul = null;
//
//			for (int i=0; i<fList.size(); i++) {
//				resul=(Y31AttachmentBean)fList.get(i);
//
//				_traza("fileList/LIST", resul);
//				//10/02/2014 [04:11:56] [P12F][PIFHelper] file name    --> test_1392031041799.xml
//				//10/02/2014 [04:11:56] [P12F][PIFHelper] file path    --> /p12g/localhost/testservlet/test_1392031041799.xml
//				//10/02/2014 [04:11:56] [P12F][PIFHelper] size         --> 71
//				//10/02/2014 [04:11:56] [P12F][PIFHelper] content type --> text/xml
//				//10/02/2014 [04:11:56] [P12F][PIFHelper] file name    --> test_1392045086333.xml
//				//10/02/2014 [04:11:56] [P12F][PIFHelper] file path    --> /p12g/localhost/testservlet/test_1392045086333.xml
//				//10/02/2014 [04:11:56] [P12F][PIFHelper] size         --> 90673
//			}
		} catch (Y31JanoServiceFileNotFoundException e) {
			// recorriendo directorios que pueden no existir... (apps que an no han dejado ficheros en PIF)
			R01FLog.to("p12f.pif").warning("[P12F][PIFHelper] LIST - FILE NOT FOUND - " + filePath + " (Y31JanoServiceFileNotFoundException)");
			//e.printStackTrace();
			return fList;
		} catch (Exception e ) {
			e.printStackTrace();
		}
		return fList;
	}
	
	/**
	 * 
	 * @param filePath
	 * @param fileName
	 */
	public void deleteFile(String filePath, String fileName) {
		deleteFile(filePath + fileName);
	}
	
	/**
	 * 
	 * @param fileNameWithPath
	 */
	public void deleteFile(String fileNameWithPath) {
		R01FLog.to("p12f.pif").info("[P12F][PIFHelper] deleteFile: Eliminando fichero " + fileNameWithPath + " de PIF");
		try {
			Y31JanoService service = Y31JanoServiceAbstractFactory.getInstance();
			
			Document xlnets = XMLParserHelper.parseXML(new ByteArrayInputStream(_sessionToken.getBytes()), false);
			
			service.delete(xlnets, fileNameWithPath);
			
		} catch (Exception e ) {
			e.printStackTrace();
		}	
	}
	
	/**
	 * 
	 * @param filesPath
	 */
	public void deleteAllFiles(String filesPath) {
		R01FLog.to("p12f.pif").info("[P12F][PIFHelper] deleteAllFiles: Eliminando todos los ficheros de " + filesPath + " en PIF");
		List fList = fileList(filesPath);
		
		Y31AttachmentBean resul = null;
		
		for (int i=0; i<fList.size(); i++) {
			resul=(Y31AttachmentBean)fList.get(i);
			
			_traza("deleteAllFiles/DELETE ["+resul.getFileName()+" de "+filesPath+"]", resul);
			
			String fileNameWithPath = (filesPath+"/"+resul.getFileName()).replaceAll("//", "/");
			
			deleteFile(fileNameWithPath);
		}
	}
	
	/**
	 * Copia un fichero (identificado con su ruta completa) a un directorio PIF
	 * @param fileNameWithPath /p12g/holdercert/z99/dl000001.xml
	 * @param targetPath       /p12g/holdercert/z99/data/
	 */
	public boolean copyFile(String fileNameWithPath, String targetPath) {
		String fileName   = new File(fileNameWithPath).getName();
		String sourcePath = fileNameWithPath.replaceFirst(fileName+"$", "");
		
		return copyFile(fileName, sourcePath, targetPath);
	}

	/**
	 * Copia un fichero de un directorio PIF a otro
	 * @param fileName
	 * @param sourcePath
	 * @param targetPath
	 */
	public boolean copyFile(String fileName, String sourcePath, String targetPath) {
		R01FLog.to("p12f.pif").info("[P12F][PIFHelper] copyFile: Copiando fichero " + fileName + " de " + sourcePath + " a " + targetPath + " en PIF");
		try {
			Y31JanoService service = Y31JanoServiceAbstractFactory.getInstance();
			
			Document xlnets = XMLParserHelper.parseXML(new ByteArrayInputStream(_sessionToken.getBytes()), false);
			
			String origen  = (sourcePath+"/").replaceAll("//", "/")+fileName; // /p12g/holdercert/z99/dl000001.xml
			String destino = (targetPath+"/").replaceAll("//", "/")+fileName; // /p12g/holdercert/z99/hcdata/dl000001.xml
			
			R01FLog.to("p12f.pif").info("[P12F][PIFHelper] copyFile: origen  --> " + origen);
			R01FLog.to("p12f.pif").info("[P12F][PIFHelper] copyFile: destino --> " + destino);
			Y31AttachmentBean resul = service.copy(xlnets, origen, destino, true); // true => preserveName
			
			_traza("copyFile/COPY", resul);
			
			return true;
			
		} catch (Exception e ) {
			e.printStackTrace();
			return false;
		}
	}
	
	/**
	 * Mueve un fichero (identificado con su ruta completa) a un directorio PIF
	 * @param fileNameWithPath /p12g/holdercert/z99/dl000001.xml
	 * @param targetPath       /p12g/holdercert/z99/hcdata/
	 */
	public void moveFile(String fileNameWithPath, String targetPath) {
		String fileName   = new File(fileNameWithPath).getName();
		String sourcePath = fileNameWithPath.replaceFirst(fileName+"$", "");
		
		moveFile(fileName, sourcePath, targetPath);
	}
	
	/**
	 * Mueve un fichero de un directorio PIF a otro (NO PUEDE LLAMARSE "data")
	 * 
	 * @param fileName   dl000001.xml
	 * @param sourcePath /p12g/holdercert/z99/
	 * @param targetPath /p12g/holdercert/z99/hcdata/
	 */
	public void moveFile(String fileName, String sourcePath, String targetPath) {
		R01FLog.to("p12f.pif").info("[P12F][PIFHelper] moveFile: Moviendo fichero " + fileName + " de " + sourcePath + " a " + targetPath + " en PIF");
		try {
			Y31JanoService service = Y31JanoServiceAbstractFactory.getInstance();
			
			Document xlnets = XMLParserHelper.parseXML(new ByteArrayInputStream(_sessionToken.getBytes()), false);
			
			String origen  = (sourcePath+"/").replaceAll("//", "/")+fileName; // /p12g/holdercert/z99_dl000001.xml
			String destino = (targetPath+"/").replaceAll("//", "/")+fileName; // /p12g/holdercert/hcdata/z99_dl000001.xml
			
			R01FLog.to("p12f.pif").info("[P12F][PIFHelper] moveFile: origen  --> " + origen);
			R01FLog.to("p12f.pif").info("[P12F][PIFHelper] moveFile: destino --> " + destino);
			
			Y31AttachmentBean resul = service.move(xlnets, origen, destino, true); // true => preserveName
			
			_traza("moveFile/MOVE", resul);
			
		} catch (Exception e ) {
			e.printStackTrace();
		}	
	}
	
	/**
	 * 
	 * @param methodName
	 * @param resul
	 */
	private void _traza(String methodName, Y31AttachmentBean resul) {
		R01FLog.to("p12f.pif").info("[P12F][PIFHelper] "+methodName+": getContentType --> "+resul.getContentType());
		R01FLog.to("p12f.pif").info("[P12F][PIFHelper] "+methodName+": getFileName    --> "+resul.getFileName());
		R01FLog.to("p12f.pif").info("[P12F][PIFHelper] "+methodName+": getFilePath    --> "+resul.getFilePath());
		R01FLog.to("p12f.pif").info("[P12F][PIFHelper] "+methodName+": getSize        --> "+resul.getSize());
		
	}
	
//	public static void main(String[] args) {
//		String fullPath   = "/p12g/holdercert/z99/dl000001.xml";
//		String fileName   = new File(fullPath).getName();
//		String sourcePath = fullPath.replaceFirst(fileName+"$", "");
//		System.out.println(fullPath+"\n"+sourcePath+"\n"+fileName+"\n");
//		String app = "z99_dl0123456789.xml".replaceAll("_.*$",""); //"z99";
//		System.out.println("app="+app+"\n");
//		String min = "3";
//		String max = "4";
//		String app = "z99";   if (app.matches("[a-zA-Z0-9]{"+min+","+max+"}")) System.out.println("app "+app+" OK\n");  
//		       app = "pycw";  if (app.matches("[a-zA-Z0-9]{"+min+","+max+"}")) System.out.println("app "+app+" OK\n");  
//		       app = "";      if (app.matches("[a-zA-Z0-9]{"+min+","+max+"}")) System.out.println("app "+app+" OK\n");  
//		       app = "as_df"; if (app.matches("[a-zA-Z0-9]{"+min+","+max+"}")) System.out.println("app "+app+" OK\n");  
//		       app = "12";    if (app.matches("[a-zA-Z0-9]{"+min+","+max+"}")) System.out.println("app "+app+" OK\n");  
//		       app = "123";   if (app.matches("[a-zA-Z0-9]{"+min+","+max+"}")) System.out.println("app "+app+" OK\n");  
//		       app = "1234";  if (app.matches("[a-zA-Z0-9]{"+min+","+max+"}")) System.out.println("app "+app+" OK\n");  
//		       app = "12345"; if (app.matches("[a-zA-Z0-9]{"+min+","+max+"}")) System.out.println("app "+app+" OK\n");  
//
//		String fileNameWithPath = "/p12g/holdercert/z99/dl000001.xml";
//		String filePath = fileNameWithPath.replaceAll("[^/]*$", "");
//		String appCode  = filePath.replaceAll("/$", "").replaceAll("^/.*/", "");
//		System.out.println("path="+filePath+"\napp="+appCode+"\n");
//
//	}
}
