package p12f.exe.pasarelapagos.utils.sign;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.rpc.ServiceException;
import javax.xml.rpc.handler.HandlerInfo;
import javax.xml.rpc.handler.HandlerRegistry;

import p12f.exe.pasarelapagos.exceptions.ConfigLoadException;
import p12f.exe.pasarelapagos.helpers.SessionTokenHelper;
import p12f.exe.pasarelapagos.objects.BaseAuthData;
import p12f.exe.pasarelapagos.objects.MetadataItem;
import p12f.exe.pasarelapagos.objects.SignedAuth;
import p12f.exe.pasarelapagos.objects.SignedAuthData;
import p12f.exe.pasarelapagos.objects.config.BasicAdminConfig;
import p12f.exe.pasarelapagos.objects.config.FinantialOrgConfig;
import p12f.exe.pasarelapagos.security.crypto.TripleDESEBCDICCryptoHelper;
import p12f.exe.pasarelapagos.utils.sign.client.NSHFTokenHandler;
import x43f.ejie.com.X43FNSHF.Body;
import x43f.ejie.com.X43FNSHF.CreateAdESSignature;
import x43f.ejie.com.X43FNSHF.CreateAdESSignatureResponse;
import x43f.ejie.com.X43FNSHF.EjgvDocument;
import x43f.ejie.com.X43FNSHF.EjgvDocumentType;
import x43f.ejie.com.X43FNSHF.Header;
import x43f.ejie.com.X43FNSHF.SignaturePlacementAnonType4;
import x43f.ejie.com.X43FNSHF.VerifyAdESSignature;
import x43f.ejie.com.X43FNSHF.VerifyAdESSignatureResponse;

import com.ejie.nshf.client.X43FNSHF;
import com.ejie.nshf.client.X43FNSHF_Impl;
import com.ejie.nshf.client.X43FNSHF_PortType;
import com.ejie.r01f.log.R01FLog;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

public class SignHelper {
	public final static String defaultEncType =  TripleDESEBCDICCryptoHelper.ENC_TYPE_BASE64;

	private X43FNSHF nshf = null; 

	private String _sessionToken = null;

	Map template = new HashMap();

	public SignHelper() {
		SessionTokenHelper tokenHelper = new SessionTokenHelper();

		if (tokenHelper.setSessionToken( // Hace falta el token de sesin para poder usar el NSHF...
				XMLProperties.get("p12f","nshf/sessionLoginAppId"), // P12G // X43F
				XMLProperties.get("p12f","nshf/simulateXLNets").trim().equalsIgnoreCase("true"))
			) {
			setSessionToken(tokenHelper.getSessionToken());
		}
		else {
			R01FLog.to("p12j.sign").warning("[P12F] SignHelper() no ha podido obtener token");
		}
	}

	/**
	 * Constructor que utiliza token que ya tengamos (no pide un nuevo token)
	 * Inicializar previamente con SessionTokenHelper...
	 *     SessionTokenHelper tokenHelper = new SessionTokenHelper();
	 *     if (tokenHelper.setSessionToken("P12G",false))
	 *         SignHelper signHelper = new SignHelper(tokenHelper.getSessionToken());
	 * @param strToken
	 */
	public SignHelper(String strToken) {
		_sessionToken = strToken;
	}

	/*
	 * Se ha podido acceder al NSHF??? (o ha dado timeout?)
	 */
	public boolean isValidNSHF() {
		return this.nshf!=null;
	}
	
	/**
	 * getter _sessionToken
	 * @return
	 */
	public String getSessionToken() {
		return _sessionToken;
	}

	/**
	 * setter _sessionToken
	 * @param strToken
	 */
	public void setSessionToken(String strToken) {
		_sessionToken = strToken;
	}

	/**
	 * Verifica una autorizacin firmada usando NSHF
	 * @param signedAuth autorizacin firmada (documento original y firma)
	 * @return true si es vlida
	 */ 
	public Boolean verifySignedAuth(SignedAuth signedAuth) {
		
		Boolean bIsValid = Boolean.FALSE;
		
		// La firma nos llega detached y XAdES-T (firma avanzada con sello
		// de tiempo). Vamos a componer un EjgvDocumentType...
		// A peticin: nos mandan la firma como PKCS7
		String docOriginal = signedAuth.baseAuthContent.document;
		String resultFirma = signedAuth.signedAuthContent.signature;
		
		try {
			// Inicializando a valores por defecto (xades, detached...)
		    //<nshf>
	        //<defaultReceive>
	        //    <ejgvDocument>
	        //        <header>
	        //            <type>SIMPLE_SIGNATURE</type><!-- PKCS7 Notarios = SIMPLE_SIGNATURE -->
	        //            <placement>detached</placement>
	        //            <format>ES-T</format>
			String headerType      = XMLProperties.get("p12f","nshf/defaultReceive/ejgvDocument/header/type");
			String headerPlacement = XMLProperties.get("p12f","nshf/defaultReceive/ejgvDocument/header/placement");
			String headerFormat    = XMLProperties.get("p12f","nshf/defaultReceive/ejgvDocument/header/format");

			for (Iterator it = signedAuth.signedAuthContent.metadataList.iterator(); it.hasNext();) {
				MetadataItem meta = (MetadataItem) it.next();
				if (meta.idValue.equalsIgnoreCase("signType")) { // xades/SIMPLE_SIGNATURE/...
					headerType = meta.value;
				} else if (meta.idValue.equalsIgnoreCase("signLocation")) { // detached
					headerPlacement = meta.value;
				} // else ... encType
				R01FLog.to("p12j.sign").info("[P12F][NSHF][METADATOS] verifySignedAuth - signedAuth.signedAuthContent.metadataList: " + meta.idValue + "=" + meta.value);
			}
			
			R01FLog.to("p12j.sign").info("[P12F][NSHF] ******************************************************************************");
			R01FLog.to("p12j.sign").info("[P12F][NSHF] Verificacin de autorizacin firmada...");
			R01FLog.to("p12j.sign").info("[P12F][NSHF] - documento: " + docOriginal.replaceAll("[0-9]", "x"));
			R01FLog.to("p12j.sign").info("[P12F][NSHF] - firma: " + resultFirma);
			R01FLog.to("p12j.sign").info("[P12F][NSHF] Creamos un EjgvDocumentType - "+headerType+"/"+headerPlacement+"/"+headerFormat); // XAdES/detached/ES-T
			R01FLog.to("p12j.sign").info("[P12F][NSHF] Header... --------------------------------------------------------------------");
			Header header = new Header();
			//header.setType(XMLProperties.get("p12f","nshf/ejgvDocument/header/type")); // "xades"
			header.setType(headerType);           // "SIMPLE_SIGNATURE" - PKCS#7
			header.setPlacement(headerPlacement); // "detached"
			header.setFormat(headerFormat);       // "ES-T"
			
			R01FLog.to("p12j.sign").info("[P12F][NSHF] Body... ----------------------------------------------------------------------");
			Body body = new Body();
			if (headerType.equalsIgnoreCase("xades")) {
				body.setSign(resultFirma); // para XaDES
				//body.setSign(resultFirma64);
			} else { // SIMPLE_SIGNATURE
				body.setSimpleSignature(resultFirma); // para PKCS7
			}
			
			R01FLog.to("p12j.sign").info("[P12F][NSHF] EjgvDocument... (h.type:" + headerType + ") ----------------------------------");
			EjgvDocument ejgvDocument = new EjgvDocument();
			ejgvDocument.setHeader(header);
			ejgvDocument.setBody(body);
			
			EjgvDocumentType ejgvDocumentType = new EjgvDocumentType();
			ejgvDocumentType.setEjgvDocument(ejgvDocument);
			
			// ---------------------------------------------------------------
			// Utilizar el NSHF para verificar la firma recibida...
			nshf = getNSHF();

			if (nshf==null) {
				R01FLog.to("p12j.sign").warning("[P12F][NSHF] SignHelper.verifySignedAuth: Error al instanciar X43FNSHF, no se puede comprobar la firma, se da por NO VALIDA...");
				return Boolean.FALSE;
			} else {
				// ---------------------------------------------------------------
				// TOKEN DE SESION - Todos los mtodos del servicio del NSHF tienen
				// como requerimiento para su invocacin el token de sesin de la
				// aplicacin invocante includo en la cabecera del mensaje SOAP.
				Iterator i;
				i = nshf.getPorts();
			
				//R01FLog.to("p12j.validatePaymentData").info("[P12F][NSHF] NSHF necesita el token de sesin; obtenerlo...");
				HandlerRegistry registry = nshf.getHandlerRegistry();
			
				//HandlerInfo hi = null;
				String sSessionToken = _sessionToken; //(String)hi.getHandlerConfig().get("sessionToken");
				R01FLog.to("p12j.sign").info("[P12F][NSHF] EjgvDocument...");
			
				Map map = new HashMap();
				map.put("sessionToken", sSessionToken);
			
				List handlerList = new ArrayList();
				handlerList.add(new HandlerInfo(NSHFTokenHandler.class, map, null));
				registry.setHandlerChain((QName)i.next(), handlerList);
			
				//R01FLog.to("p12j.validatePaymentData").info("[P12F][NSHF] ##### TOKEN DE SESION: " + sSessionToken);
			
				// ---------------------------------------------------------------
				// Vamos a invocar el servicio VerifyAdESSignature...
				// Necesitamos una instancia de VerifyAdESSignature que pasaremos luego como parmetro...
				R01FLog.to("p12j.sign").info("[P12F][NSHF] ##########################################################");
				R01FLog.to("p12j.sign").info("[P12F][NSHF] ##### Vamos a invocar el servicio VerifyAdESSignature...");
				R01FLog.to("p12j.sign").info("[P12F][NSHF] ##########################################################");
				VerifyAdESSignature verifySignature = new VerifyAdESSignature();
				verifySignature.setSignature(ejgvDocumentType);
				verifySignature.setDocument_B64(docOriginal.getBytes());
			
				// NSHF - verifyAdESSignature ------------------------------------
				// Invocamos verifyAdESSignature... obtenemos resultado en resp...
				X43FNSHF_PortType port = nshf.getX43FNSHF_PortType();
				VerifyAdESSignatureResponse resp = port.verifyAdESSignature(verifySignature);
			
				String strFirmaVerificada = resp.getVerifyAdESSignatureResult().getVerificationResult().getIsSuccessResult();
			
				R01FLog.to("p12j.sign").info("[P12F][NSHF] ##### verify...RESPONSE(getIsSuccessResult): "+strFirmaVerificada);
				R01FLog.to("p12j.sign").info("[P12F][NSHF] ##### verify...RESPONSE(getResultMinor): "+resp.getVerifyAdESSignatureResult().getVerificationResult().getResultMinor());
				R01FLog.to("p12j.sign").info("[P12F][NSHF] ##### verify...RESPONSE(getResultMessage): "+resp.getVerifyAdESSignatureResult().getVerificationResult().getResultMessage());
			
				bIsValid = strFirmaVerificada.equalsIgnoreCase("true") ? Boolean.TRUE : Boolean.FALSE;
			}
			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ServiceException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		} catch (Throwable t){
			t.printStackTrace();
		}

		return bIsValid;
	}
	
	/**
	 * Crear un Objeto de Firma a partir de un input dado.
	 * @param input
	 * @return
	 */
	public SignRequestOutputData createSign(SignRequestInputData input){
		
		SignRequestOutputData signResult = new SignRequestOutputData();
		
		String docOriginal = input.getString();
		
		try {
			// ---------------------------------------------------------------
			// Utilizar el NSHF...
			nshf = getNSHF();

			if (nshf==null) {
				R01FLog.to("p12j.sign").warning("[P12F][NSHF] SignHelper.createSign: Error al instanciar X43FNSHF, no se puede generar firma de la pasarela...");
				return signResult;
			} else {
				// ---------------------------------------------------------------
				// TOKEN DE SESION - Todos los mtodos del servicio del NSHF tienen
				// como requerimiento para su invocacin el token de sesin de la
				// aplicacin invocante includo en la cabecera del mensaje SOAP.
				Iterator i;
				i = nshf.getPorts();
			
				HandlerRegistry registry = nshf.getHandlerRegistry();
			
				//HandlerInfo hi = null;
				String sSessionToken = _sessionToken; //(String)hi.getHandlerConfig().get("sessionToken");
			
				Map map = new HashMap();
				map.put("sessionToken", sSessionToken);
			
				List handlerList = new ArrayList();
				handlerList.add(new HandlerInfo(NSHFTokenHandler.class, map, null));
				registry.setHandlerChain((QName)i.next(), handlerList);
			
				//R01FLog.to("p12j.sign").info("[P12F][NSHF] ##### TOKEN DE SESION: " + sSessionToken);
			
				// ---------------------------------------------------------------
				// Vamos a invocar el servicio CreateAdESSignature...
				// Necesitamos una instancia de CreateAdESSignature que pasaremos luego como parmetro...

				// NSHF - CreateAdESSignature ------------------------------------
				// Invocamos CreateAdESSignature...
				R01FLog.to("p12j.sign").info("[P12F][NSHF] ##########################################################");
				R01FLog.to("p12j.sign").info("[P12F][NSHF] ##### Vamos a invocar el servicio CreateAdESSignature...");
				R01FLog.to("p12j.sign").info("[P12F][NSHF] ##########################################################");
			
				X43FNSHF_PortType port = nshf.getX43FNSHF_PortType();
			
				CreateAdESSignature cr = new CreateAdESSignature();
				cr.setCertificateId(XMLProperties.get("p12f","nshf/certificateId")); // "0035"
				cr.setDocument_B64(docOriginal.getBytes());
				cr.setSignaturePlacement(SignaturePlacementAnonType4.detached);//detached o enveloped, en nuestro caso detached
			
				CreateAdESSignatureResponse resp = port.createAdESSignature(cr);
			
				signResult.setResult(resp.getCreateAdESSignatureResult().getEjgvDocument().getBody().getSign());
			
				R01FLog.to("p12d.directPayOnAccount").info("[P12F][NSHF] ##### create...RESPONSE: "+resp.getCreateAdESSignatureResult().getEjgvDocument().getHeader().getType());
				R01FLog.to("p12d.directPayOnAccount").info("[P12F][NSHF] ##### create...RESPONSE (substring): " +
					resp.getCreateAdESSignatureResult().getEjgvDocument().getBody().getSign().substring(0, Math.min(60, resp.getCreateAdESSignatureResult().getEjgvDocument().getBody().getSign().length())) + "...\n");
			}			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ServiceException e) {
			e.printStackTrace();
		}
		return signResult;
	}

	
	//public SignTemplate getTemplateForPaymentOnAccount(PaymentData paymentData, ProtocolData protocolData, AccountData accountData, SignedAuth signedAuth) {
	/**
	 * @param bValidAuth
	 * @return
	 */
	public SignTemplate getTemplateForPaymentOnAccount(boolean bValidAuth) {
		SignTemplate t = new SignTemplate();

		if (bValidAuth) {
			t.setText(XMLProperties.get("p12f","nshf/templates/paymentOnAccountValidation"));
		}
		else {
			t.setText(XMLProperties.get("p12f","nshf/templates/paymentOnAccountInvalidSignature"));
		}

		
		return t;
	}

	// ###################################################################################
	// # * * *  ENCRIPTAR/DESENCRIPTAR DOCUMENTO ORIGINAL EN AUTORIZACIN FIRMADA  * * * #
	// # El documento original de la autorizacin firmada puede contener datos como el   #
	// # nmero de cuenta, por lo que llegar cifrado por la administracin y se enviar #
	// # cifrado a la entidad financiera.                                                #
	// ###################################################################################
	/**
	 * Cifrar documento de la autorizacin firmada recibida de la administracin
	 * @throws Exception */
	public static SignedAuth encryptDocument(SignedAuth signedAuth, BasicAdminConfig adminConfig) throws Exception {
		String encType = defaultEncType;

		if (signedAuth.baseAuthContent!=null) {
			for (Iterator it = signedAuth.baseAuthContent.metadataList.iterator(); it.hasNext();) {
				MetadataItem meta = (MetadataItem) it.next();
				if (meta.idValue.equalsIgnoreCase("encType")) encType = meta.value; // hex/base64/...
				R01FLog.to("p12j.sign").info("[P12F][NSHF][METADATOS] encryptDocument - signedAuth.baseAuthContent.metadataList: " + meta.idValue + "=" + meta.value);
			}
		}
		else R01FLog.to("p12j.sign").warning("[P12F][NSHF][METADATOS] encryptDocument - encryptedSignedAuth.baseAuthContent==null; usando encType por defecto: " + encType);

		return SignHelper.encryptDocument(signedAuth, adminConfig, encType);
	}
	
	/**
	 * Cifrar documento de la autorizacin firmada recibida de la administracin
	 * @throws Exception */
	public static SignedAuth encryptDocument(SignedAuth signedAuth, BasicAdminConfig adminConfig, String encType) throws Exception {
		
		SignedAuth encryptedSignedAuth = SignedAuth.getObject(signedAuth.toXML());
		
		try {
			String clave1 = (String)adminConfig.claves.get("accountPaymentKey1");
			String clave2 = (String)adminConfig.claves.get("accountPaymentKey2");
			
			if ((clave1 == null) ||  (clave2 == null) ) {
				throw new Exception ("[P12F] SignHelper.encryptDocument(...,ADMIN) - No se han definido las claves accountPaymentKey1 y accountPaymentKey2 para la Administracion: " + adminConfig.oid);
			}
			encryptedSignedAuth.baseAuthContent.document = _encryptSignedAuthDocument(clave1,clave2,signedAuth,encType);	
			return encryptedSignedAuth;
			
		} catch (XOMarshallerException e) {
			throw new Exception(e);
		} catch (ConfigLoadException e) {
			throw new Exception(e);
		}
	}
	
	/**
	 * Cifrar documento de la autorizacin firmada a enviar a la entidad financiera
	 * @throws Exception */
	public static SignedAuth encryptDocument(SignedAuth signedAuth, FinantialOrgConfig efConfig) throws Exception {
		String encType = defaultEncType;

		if (signedAuth.baseAuthContent!=null) {
			for (Iterator it = signedAuth.baseAuthContent.metadataList.iterator(); it.hasNext();) {
				MetadataItem meta = (MetadataItem) it.next();
				if (meta.idValue.equalsIgnoreCase("encType")) encType = meta.value; // hex/base64/...
				R01FLog.to("p12j.sign").info("[P12F][NSHF][METADATOS] encryptDocument para EF - signedAuth.baseAuthContent.metadataList: " + meta.idValue + "=" + meta.value);
			}
		}
		else R01FLog.to("p12j.sign").warning("[P12F][NSHF][METADATOS] encryptDocument para EF - encryptedSignedAuth.baseAuthContent==null; usando encType por defecto: " + encType);

		return SignHelper.encryptDocument(signedAuth, efConfig, encType);
	}
	
	/**
	 * Cifrar documento de la autorizacin firmada a enviar a la entidad financiera
	 * @throws Exception */
	public static SignedAuth encryptDocument(SignedAuth signedAuth, FinantialOrgConfig efConfig, String encType) throws Exception {
		SignedAuth encryptedSignedAuth = SignedAuth.getObject(signedAuth.toXML());
		
		try {
			String clave1 = (String)efConfig.claves.get("accountPaymentKey1");
			String clave2 = (String)efConfig.claves.get("accountPaymentKey2");
			
			if ((clave1 == null) ||  (clave2 == null) ) {
				throw new Exception ("[P12F] SignHelper.encryptDocument(...,EF) - No se han definido las claves accountPaymentKey1 y accountPaymentKey2 para la Entidad Financiera: " + efConfig.oid);
			}
			encryptedSignedAuth.baseAuthContent.document = _encryptSignedAuthDocument(clave1,clave2,signedAuth,(encType!=null ? encType : defaultEncType));
			return encryptedSignedAuth;
			
		} catch (XOMarshallerException e) {
			throw new Exception(e);
		} catch (ConfigLoadException e) {
			throw new Exception(e);
		}
	}

	/**
	 * Descifrar el documento de la autorizacin firmada recibido de la administracin
	 * @throws Exception */
	public static SignedAuth decryptDocument(SignedAuth encryptedSignedAuth, BasicAdminConfig adminConfig) throws Exception{
		String encType = defaultEncType;

		if (encryptedSignedAuth.baseAuthContent!=null) {
			for (Iterator it = encryptedSignedAuth.baseAuthContent.metadataList.iterator(); it.hasNext();) {
				MetadataItem meta = (MetadataItem) it.next();
				if (meta.idValue.equalsIgnoreCase("encType")) encType = meta.value; // hex/base64/...
				R01FLog.to("p12j.sign").info("[P12F][NSHF][METADATOS] decryptDocument - signedAuth.baseAuthContent.metadataList: " + meta.idValue + "=" + meta.value);
			}
		}
		else R01FLog.to("p12j.sign").warning("[P12F][NSHF][METADATOS] decryptDocument - encryptedSignedAuth.baseAuthContent==null; usando encType por defecto: " + encType);

		return SignHelper.decryptDocument(encryptedSignedAuth, adminConfig, encType);
	}
	
	/**
	 * Descifrar el documento de la autorizacin firmada recibido de la administracin
	 * @throws Exception */
	public static SignedAuth decryptDocument(SignedAuth encryptedSignedAuth, BasicAdminConfig adminConfig, String encType) throws Exception{
		
		SignedAuth decryptedSignedAuth = SignedAuth.getObject(encryptedSignedAuth.toXML());
		
		try {
			String clave1 = (String)adminConfig.claves.get("accountPaymentKey1");
			String clave2 = (String)adminConfig.claves.get("accountPaymentKey2");
			
			if ((clave1 == null) ||  (clave2 == null) ) {
				throw new Exception ("[P12F] SignHelper.decryptDocument(...) - No se han definido las claves accountPaymentKey1 y  accountPaymentKey2 para la Administracion: " + adminConfig.oid );
			}
			decryptedSignedAuth.baseAuthContent.document = _decryptSignedAuthDocument(clave1,clave2,encryptedSignedAuth,encType).trim();
			return decryptedSignedAuth;
		
		} catch (XOMarshallerException e) {
			throw new Exception(e);
		} catch (ConfigLoadException e) {
			throw new Exception(e);
		}
	}
	
	/**
	 * Descifrar el documento de la autorizacin firmada que se enva a la entidad financiera
	 * @throws Exception */
	public static SignedAuth decryptDocument(SignedAuth encryptedSignedAuth, FinantialOrgConfig efConfig) throws Exception{
		String encType = defaultEncType;

		if (encryptedSignedAuth.baseAuthContent!=null) {
			for (Iterator it = encryptedSignedAuth.baseAuthContent.metadataList.iterator(); it.hasNext();) {
				MetadataItem meta = (MetadataItem) it.next();
				if (meta.idValue.equalsIgnoreCase("encType")) encType = meta.value; // hex/base64/...
				R01FLog.to("p12j.sign").info("[P12F][NSHF][METADATOS] decryptDocument de EF - signedAuth.baseAuthContent.metadataList: " + meta.idValue + "=" + meta.value);
			}
		}
		else R01FLog.to("p12j.sign").warning("[P12F][NSHF][METADATOS] decryptDocument de EF - encryptedSignedAuth.baseAuthContent==null; usando encType por defecto: " + encType);

		return SignHelper.decryptDocument(encryptedSignedAuth, efConfig, encType);
	}

	/**
	 * Descifrar el documento de la autorizacin firmada que se enva a la entidad financiera
	 * @throws Exception */
	public static SignedAuth decryptDocument(SignedAuth encryptedSignedAuth, FinantialOrgConfig efConfig, String encType) throws Exception{
		
		SignedAuth decryptedSignedAuth = SignedAuth.getObject(encryptedSignedAuth.toXML());
		
		try {
			String clave1 = (String)efConfig.claves.get("accountPaymentKey1");
			String clave2 = (String)efConfig.claves.get("accountPaymentKey2");
			
			if ((clave1 == null) ||  (clave2 == null) ) {
				throw new Exception ("[P12F] SignHelper.decryptDocument(...) - No se han definido las claves accountPaymentKey1 y accountPaymentKey2 para la Entidad Fianciera: " + efConfig.oid );
			}
			decryptedSignedAuth.baseAuthContent.document = _decryptSignedAuthDocument(clave1,clave2,encryptedSignedAuth, (encType!=null ? encType : defaultEncType)).trim();
			return decryptedSignedAuth;
		
		} catch (XOMarshallerException e) {
			throw new Exception(e);
		} catch (ConfigLoadException e) {
			throw new Exception(e);
		}
	}
	
	private static String _encryptSignedAuthDocument (String clave1, String clave2, SignedAuth signedAuth, String encType) throws Exception{
		R01FLog.to("p12f.sign").info("[P12F] ********************************************************");
		R01FLog.to("p12f.sign").info("[P12F] ******************  E N C R Y P T  *********************");
		R01FLog.to("p12f.sign").info("[P12F] ********************************************************");
		R01FLog.to("p12f.sign").info("[P12F] SignHelper._encryptSignedAuthDocument(...,"+encType+"): ENCRYPT");
		R01FLog.to("p12f.sign").info("[P12F] ========================================================");
		R01FLog.to("p12f.sign").info("[P12F] >>>>> Semiclaves de entrada:");
		R01FLog.to("p12f.sign").info("[P12F]      .Semiclave1: [" + clave1.substring(0,clave1.length()/2) + "...]");
		R01FLog.to("p12f.sign").info("[P12F]      .Semiclave2: [" + clave2.substring(0,clave2.length()/2) + "...]");
		//String claveFinal  = new String(Hex.encode(TripleDESEBCDICCryptoHelper.doXOR(clave1, clave2))).toUpperCase();
		//System.out.println("[P12F] Clave final xor: [" + claveFinal.substring(0,clave1.length()/2) + "...]");
		
		TripleDESEBCDICCryptoHelper encrypter = new TripleDESEBCDICCryptoHelper();	
		return encrypter.encrypt(clave1,clave2,encType,TripleDESEBCDICCryptoHelper.CIPHER_3DES_CBC_SpacePADDING,signedAuth.baseAuthContent.document);
	}
	
	private static String _decryptSignedAuthDocument (String clave1, String clave2 ,SignedAuth signedAuth, String encType) throws Exception {
		R01FLog.to("p12f.sign").info("[P12F] ********************************************************");
		R01FLog.to("p12f.sign").info("[P12F] ******************  D E C R Y P T  *********************");
		R01FLog.to("p12f.sign").info("[P12F] ********************************************************");
		R01FLog.to("p12f.sign").info("[P12F] SignHelper._decryptSignedAuthDocument(...,"+encType+"): DECRYPT");
		R01FLog.to("p12f.sign").info("[P12F] ========================================================");
		R01FLog.to("p12f.sign").info("[P12F] >>>>> Semiclaves de entrada:");
		R01FLog.to("p12f.sign").info("[P12F]      .Semiclave1: [" + clave1.substring(0,clave1.length()/2) + "...]");
		R01FLog.to("p12f.sign").info("[P12F]      .Semiclave2: [" + clave2.substring(0,clave2.length()/2) + "...]");
		//String claveFinal = new String(Hex.encode(TripleDESEBCDICCryptoHelper.doXOR(clave1, clave2))).toUpperCase();
		//System.out.println("[P12F] Clave final xor: [" + claveFinal.substring(0,clave1.length()/2) + "...]");
		
		//R01FLog.to("p12f.sign").info("[P12F] Doc.original CIFRADO:"+signedAuth.document);
		//R01FLog.to("p12f.sign").info("[P12F] ********************************************************");
		
		TripleDESEBCDICCryptoHelper encrypter = new TripleDESEBCDICCryptoHelper();
		String resul = encrypter.decrypt(clave1,clave2,encType,TripleDESEBCDICCryptoHelper.CIPHER_3DES_CBC_SpacePADDING,signedAuth.baseAuthContent.document); 

		//R01FLog.to("p12f.sign").info("[P12F] Doc.original DESCIFRADO:"+resul);
		//R01FLog.to("p12f.sign").info("[P12F] ********************************************************");
		return resul;
	}

	/**
	 * 
	 * @return
	 */
	public static X43FNSHF getNSHF() {
	
		String wsdlUrl = XMLProperties.get("p12f","nshf/wsdl-url"); // "http://svc.intra.integracion.jakina.ejiedes.net/ctxapp/X43FNSHF2?WSDL"
		X43FNSHF nshf = null;
		try {
			R01FLog.to("p12j.sign").info("[NSHF] URL WSDL: " + wsdlUrl);
			nshf = new X43FNSHF_Impl(wsdlUrl);
		} catch (Exception e) {
			R01FLog.to("p12j.sign").warning("[P12F][NSHF] ERROR instanciando X43FNSHF");
			e.printStackTrace();
			return null;
		}
		return nshf;
	}
	
	// ************************************************************************
	// ******* METODOS PARA MANTENER COMPATIBILIDAD ATRIBUTOS DEPRECATED ******
	// ************************************************************************
	// DEPRECATED, mantener provisionalmente --- COMPATIBILIDAD LIBRERIAS EF
	public static SignedAuth iniDeprecatedFromNew(SignedAuth signedAuth) { // provisional... aplicar a recibido de Notario
		String initializeDeprecatedAttributes = XMLProperties.get("p12f","nshf/initializeDeprecatedAttributes"); 
		if (initializeDeprecatedAttributes!=null && initializeDeprecatedAttributes.equalsIgnoreCase("true")) {
			R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] *********************************************************************************");
			R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] *** SignHelper.iniDeprecatedFromNew(signedAuth)                               ***");
			R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] *********************************************************************************");

			if (signedAuth.document==null) {
				try {
					R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] signedAuth a tratar: " + signedAuth.toXML());
				} catch (XOMarshallerException e) {
					e.printStackTrace();
				}
				R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] Inicializacin SignedAuth: DOCUMENT...");
				signedAuth.document = signedAuth.baseAuthContent.document;
			}
			if (signedAuth.signedAuthData==null) {
				R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] Inicializacin SignedAuth: SIGNEDAUTHDATA...");
				signedAuth.signedAuthData = signedAuth.signedAuthContent.signature;
			}
			// estos atributos no existen en la versin vieja, los ponemos a null...
			signedAuth.baseAuthContent   = null;
			signedAuth.signedAuthContent = null;
			try {
				R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] signedAuth convertido a deprecated (para EF): " + signedAuth.toXML());
			} catch (XOMarshallerException e) {
				e.printStackTrace();
			}
		}
		return signedAuth;
	}
	public static SignedAuth iniNewFromDeprecated(SignedAuth signedAuth, String encType, String signType, String signLocation) { // provisional... aplicar a recibido de EF con libs viejas
		String initializeDeprecatedAttributes = XMLProperties.get("p12f","nshf/initializeDeprecatedAttributes"); 
		if (initializeDeprecatedAttributes!=null && initializeDeprecatedAttributes.equalsIgnoreCase("true")) {
			R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] *********************************************************************************");
			R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] *** SignHelper.iniNewFromDeprecated(signedAuth,encType,signType,signLocation) ***");
			R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] *********************************************************************************");
			R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] params: signedAuth, encType="+encType+", signType="+signType+", signLocation="+signLocation);

			MetadataItem mdItemEnc = new MetadataItem();
			mdItemEnc.idValue = "encType";
			mdItemEnc.value   = encType;//"base64";

			// si se ha nulificado previamente...
			if (signedAuth.baseAuthContent==null)   signedAuth.baseAuthContent   = new BaseAuthData();
			if (signedAuth.signedAuthContent==null) signedAuth.signedAuthContent = new SignedAuthData();

			if (signedAuth.baseAuthContent.document==null) {
				R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] Inicializacin SignedAuth: DOCUMENT; encType="+encType);

				//signedAuth.baseAuthContent = new BaseAuthData();
				signedAuth.baseAuthContent.document = signedAuth.document;
				signedAuth.baseAuthContent.metadataList.add(mdItemEnc);

				signedAuth.document = null;
			}
			if (signedAuth.signedAuthContent.signature==null) {
				R01FLog.to("p12f.sign").info("[P12F][COMPATIBILIDAD DEPRECATED] Inicializacin SignedAuth: SIGNEDAUTHDATA; encType/signType/signLocation="+encType+"/"+signType+"/"+signLocation);

				MetadataItem mdItemSignType = new MetadataItem();
				mdItemSignType.idValue = "signType";
				mdItemSignType.value   = signType;//"xades";
			
				MetadataItem mdItemSignLocation = new MetadataItem();
				mdItemSignLocation.idValue = "signLocation";
				mdItemSignLocation.value   = signLocation;//"detached";

				//signedAuth.signedAuthContent = new SignedAuthData();
				signedAuth.signedAuthContent.signature = signedAuth.signedAuthData;
				signedAuth.signedAuthContent.metadataList.add(mdItemEnc);
				signedAuth.signedAuthContent.metadataList.add(mdItemSignType);
				signedAuth.signedAuthContent.metadataList.add(mdItemSignLocation);

				signedAuth.signedAuthData = null;
			}
		}
		return signedAuth;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		SignedAuth signedAuth = new SignedAuth();

		SignedAuthData signData = new SignedAuthData();
		BaseAuthData docData = new BaseAuthData();
		
		MetadataItem mdItemEnc = new MetadataItem();
		mdItemEnc.idValue = "encType";
		mdItemEnc.value   = "base64";

		MetadataItem mdItemSignType = new MetadataItem();
		mdItemSignType.idValue = "signType";
		mdItemSignType.value   = "xades";
		
		MetadataItem mdItemSignLocation = new MetadataItem();
		mdItemSignLocation.idValue = "signLocation";
		mdItemSignLocation.value   = "detached";

		signData.signature = "qwerty";
		signData.metadataList.add(mdItemEnc);
		signData.metadataList.add(mdItemSignType);
		signData.metadataList.add(mdItemSignLocation);

		docData.document = "NIF 12345678Z autorizo a que se cargue el importe 000045000,50 euros correspondiente al pago 130000000000017G en la cuenta 11111111111111111111";
		docData.metadataList.add(mdItemEnc);

		signedAuth.timeStamp         = "99999999999";
		signedAuth.signedAuthContent = signData;
		signedAuth.baseAuthContent   = docData;
		
		// compatibilidad...
		signedAuth = SignHelper.iniDeprecatedFromNew(signedAuth);

		try {
			//System.out.println(mdItem.toXML());
			//System.out.println(signedAuth.toXML());
			R01FLog.to("p12f.sign").info(signedAuth.toXML());
		} catch (XOMarshallerException e) {
			e.printStackTrace();
		}

		// prueba desde deprecated...
		signedAuth.baseAuthContent   = null;
		signedAuth.signedAuthContent = null;

		// compatibilidad...
		signedAuth = SignHelper.iniNewFromDeprecated(signedAuth, "base64", "xades", "detached");

		try {
			//System.out.println(signedAuth.toXML());
			R01FLog.to("p12f.sign").info(signedAuth.toXML());
		} catch (XOMarshallerException e) {
			e.printStackTrace();
		}
	}
	// ----------------------------------------- COMPATIBILIDAD LIBRERIAS EF
}
