package p12f.exe.pasarelapagos.utils.sms.latinia.client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import javax.xml.namespace.QName;
import javax.xml.rpc.JAXRPCException;
import javax.xml.rpc.handler.HandlerInfo;
import javax.xml.rpc.handler.MessageContext;
import javax.xml.rpc.handler.soap.SOAPMessageContext;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPMessage;

import com.ejie.r01f.log.R01FLog;

import weblogic.webservice.GenericHandler; // OJO!! referencia a weblogic...

public class LatiniaTokenHandler extends GenericHandler {

    HandlerInfo hi;

    public void init(HandlerInfo info) {
        hi = info;
    }

    public QName[] getHeaders() {
        return hi.getHeaders();
    }

    /**
     * 
     * sessionToken de Latinia:
     * 		<?xml version="1.0" encoding="UTF-8"?>
     * 		<authenticationLatinia>
     * 		    <userLatinia>innovus.superusuario</userLatinia>
     * 		    <passwordLatinia>MARKSTAT</passwordLatinia>
     * 		    <refProduct>P12G</refProduct>
     * 		    <idContract>2051</idContract>
     * 		    <loginEnterprise>INNOVUS</loginEnterprise>
     * 		    <password>P12G</password>
     * 		</authenticationLatinia>
     */
    public boolean handleRequest(MessageContext m)  {
         try {
            SOAPMessageContext ctx = (SOAPMessageContext) m;
            SOAPMessage message = ctx.getMessage();

            String sSessionToken = (String)hi.getHandlerConfig().get("sessionToken");

            // recuperar cabecera SOAP
            SOAPHeader soapHeader = message.getSOAPPart().getEnvelope().getHeader();
            //System.out.println("[P12F][LatiniaTokenHandler] handleRequest(m) - soapHeader:\n" + soapHeader.toString());

            if (sSessionToken.startsWith("<?xml version=\"1.0\"")) {
                int pos = sSessionToken.indexOf(">") + 1;
                sSessionToken = sSessionToken.substring(pos).trim();
            }

            R01FLog.to("p12g.smsService").info("[P12F][LatiniaTokenHandler] handleRequest(m) - sessionToken:\n" + sSessionToken);

            // incluir token en la cabecera creada
            soapHeader.addTextNode(sSessionToken);

            ByteArrayOutputStream myByteArrayOutputString = new ByteArrayOutputStream();
            message.writeTo(myByteArrayOutputString);

            String sSoapMessage = new String(myByteArrayOutputString.toByteArray(), "UTF-8");
            sSoapMessage = processHeaderScape(sSoapMessage);

            R01FLog.to("p12g.smsService").info("[P12F][LatiniaTokenHandler] handleRequest(m) - sSoapMessage:\n" + sSoapMessage);

            ByteArrayInputStream in = new ByteArrayInputStream(sSoapMessage.getBytes());

            MessageFactory factory = MessageFactory.newInstance();
            message =  factory.createMessage(null, in);
            ctx.setMessage(message);
        } catch (Exception e) {
            e.printStackTrace();
            throw new JAXRPCException(e);
        }
        return true;
    }

    /**
     * 
     * @param sSoapMessage
     * @return
     */
    private String processHeaderScape(String sSoapMessage) {
        int posEndIni = sSoapMessage.indexOf("Header>") + "Header>".length();
        int posEndEnd = sSoapMessage.indexOf("Header>", posEndIni) + "Header>".length() - 1;
        int posIniEnd = sSoapMessage.lastIndexOf("<", posEndEnd);

        StringBuffer sb = new StringBuffer();
        sb.append(sSoapMessage.substring(0, posEndIni));

        StringBuffer headerContentSb = new StringBuffer();
        String headerContent = posEndIni<=posIniEnd ? sSoapMessage.substring(posEndIni, posIniEnd) : "";
        headerContent = headerContent.replaceAll("&lt;", "<").replaceAll("&gt;", ">");
        headerContentSb.append(headerContent);
        sb.append(headerContentSb);
        sb.append(sSoapMessage.substring(posIniEnd));
        String ret = sb.toString();

        return ret;
    }

    public boolean handleResponse(MessageContext context) {
        return true;
    }

    public boolean handleResponse(MessageContext m, String error) {
        return true;
    }
}
