package p12f.exe.pasarelapagos.utils.sms.latinia.client;

import java.io.StringWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

//import com.ejie.r01f.log.R01FLog;
//import com.ejie.r01f.xml.marshalling.XOMarshallerException;

import p12f.exe.pasarelapagos.objects.sms.latinia.AuthenticationLatinia;

public class LatiniaTokenHelper {
	private AuthenticationLatinia authLatinia;
	private Document token;
	
	public LatiniaTokenHelper() {
		_iniHelper();
	}

	public LatiniaTokenHelper(AuthenticationLatinia authLatinia) {
		_iniHelper(authLatinia);
	}
	
	/**
	 * 
	 */
	private void _iniHelper() {
		this._iniHelper(new AuthenticationLatinia());
	}

	/**
	 * 
	 * @param authLatinia
	 */
	private void _iniHelper(AuthenticationLatinia authLatinia) {
		this.authLatinia = authLatinia;

//		try {
//			System.out.println("AUTH LATINIA: " + authLatinia.toXML());
//		} catch (XOMarshallerException e1) {
//			e1.printStackTrace();
//		}

		try {
			this.token = this.tokenAuthenticationLatinia(authLatinia);
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
	}

	/**
	 * 
	 * @return
	 */
	public AuthenticationLatinia getAuthenticationLatinia() {
		return this.authLatinia;
	}

	/**
	 * 
	 * @return
	 */
	public Document getToken() {
		return this.token;
	}


	/**
	 * Metodo que genera  el XML para poder acceder a Latinia desde aplicaciones 
	 * que no tienen Token de Sesion N38
	 * 
	 * XML que llega para poder acceder a Latinia desde aplicaciones
	 * que no tienen Token de Sesion N38:
	 * <authenticationLatinia>
	 *  	<userLatinia>a</userLatinia>
	 * 		<passwordLatinia>b</passwordLatinia>
	 * 		<refProduct>c</refProduct>
	 * 		<idContract>d</idContract>
	 * 		<loginEnterprise>e</loginEnterprise>
	 * 		<password>f</password>
	 * </authenticationLatinia>
	 * 
	 * @param authLatinia
	 * @return
	 * @throws ParserConfigurationException
	 */
	private Document tokenAuthenticationLatinia(AuthenticationLatinia authLatinia) throws ParserConfigurationException {
		return tokenAuthenticationLatinia(
						authLatinia.userLatinia,
						authLatinia.passwordLatinia,
						authLatinia.refProduct,
						authLatinia.idContract,
						authLatinia.loginEnterprise,
						authLatinia.password
						);
	}

	/**
	 * Metodo que genera  el XML para poder acceder a Latinia desde aplicaciones 
	 * que no tienen Token de Sesion N38
	 * 
	 * @param userLatinia
	 * @param passwordLatinia
	 * @param refProduct
	 * @param idContract
	 * @param loginEnterprise
	 * @param password
	 * @return Document con el formato del Usuario Latinia
	 * @throws ParserConfigurationException
	 */
	private Document tokenAuthenticationLatinia(String userLatinia, String passwordLatinia, String refProduct, String idContract, String loginEnterprise, String password) throws ParserConfigurationException {

		//R01FLog.to("p12g.smsService").info("[P12F][LatiniaTokenHelper] userLatinia=" + userLatinia +", passwordLatinia=" + passwordLatinia +
		//		", refProduct=" + refProduct + ", idContract=" + idContract + ", loginEnterprise=" + loginEnterprise + ", password=" + password);

		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder    = dbf.newDocumentBuilder();
		Document tokenLatinia      = builder.newDocument();

		Element authentication = tokenLatinia.createElement("authenticationLatinia");

		Element elUserLatinia = tokenLatinia.createElement("userLatinia");
		elUserLatinia.appendChild(tokenLatinia.createTextNode(userLatinia));
		authentication.appendChild(elUserLatinia);

		Element elPasswordLatinia = tokenLatinia.createElement("passwordLatinia");
		elPasswordLatinia.appendChild(tokenLatinia.createTextNode(passwordLatinia));
		authentication.appendChild(elPasswordLatinia);  

		Element elRefProduct = tokenLatinia.createElement("refProduct");
		elRefProduct.appendChild(tokenLatinia.createTextNode(refProduct));
		authentication.appendChild(elRefProduct); 

		Element elIdContract = tokenLatinia.createElement("idContract");
		elIdContract.appendChild(tokenLatinia.createTextNode(idContract));
		authentication.appendChild(elIdContract); 

		Element ellLoginEnterprise = tokenLatinia.createElement("loginEnterprise");
		ellLoginEnterprise.appendChild(tokenLatinia.createTextNode(loginEnterprise));
		authentication.appendChild(ellLoginEnterprise); 

		Element elPassword = tokenLatinia.createElement("password");
		elPassword.appendChild(tokenLatinia.createTextNode(password));
		authentication.appendChild(elPassword); 

		Node root = (Node)authentication;
		tokenLatinia.appendChild(root);

		return tokenLatinia;
	}

	/**
	 * 
	 * @return
	 */
	public String latiniaTokenToString() {
		return latiniaTokenToString(this.token);
	}

	/**
	 * 
	 * @param latiniaToken
	 * @return
	 */
	public static String latiniaTokenToString(Document latiniaToken) {
		
		String resul = "";
		
		try {
			Document doc = latiniaToken;
			DOMSource domSource = new DOMSource(doc);
			StringWriter writer = new StringWriter();
			StreamResult result = new StreamResult(writer);
			TransformerFactory tf = TransformerFactory.newInstance();
			Transformer transformer = tf.newTransformer();
			transformer.transform(domSource, result);
			resul = writer.toString(); //System.out.println("token: " + resul);
		} catch (TransformerConfigurationException e) {
			e.printStackTrace();
		} catch (TransformerException e) {
			e.printStackTrace();
		}
		
		return resul;
	}

//	/**
//	 * @param args
//	 */
//	public static void main(String[] args) {
////		AuthenticationLatinia authLatinia = new AuthenticationLatinia();
////		LatiniaTokenHelper lth = new LatiniaTokenHelper(authLatinia);
//
//		LatiniaTokenHelper lth = new LatiniaTokenHelper();
//
//		try {
//			System.out.println("[P12F][LatiniaTokenHelper] AUTH LATINIA: "+lth.getAuthenticationLatinia().toXML());
//			System.out.println("[P12F][LatiniaTokenHelper] TOKEN LATINIA: "+lth.latiniaTokenToString());
//		} catch (XOMarshallerException e) {
//			e.printStackTrace();
//		}
//
//	}

}
