package p12f.exe.pasarelapagos.v1.objects;

/**
 * Clase abstracta que define las funcionalidades comunes de los objetos
 * de pago
 *
 * Todos los objetos de pago, deben heredar de esta clase abstracta para
 * que el sistema funcione correctamente.
 *
 * Este objeto implementa las siguientes funcionalidades comunes:
 * FUNCIONALIDAD 1
 * ---------------
 *      - Se inicializan los miembros de la instancia para crear
 *        de antemano los objetos.
 *        Esto se hace para evitar que el usuario tenga que crearlos:
 *        Ej de lo que se evita:
 *              Pago pago = new Pago();
 *              pago.conceptos = new ArrayList();
 *              pago.conceptos.put("hola");
 *        Ahora simplemente hay que hacer:
 *              Pago pago = new Pago();
 *              pago.conceptos.put("hola");     // No hay que llamar al constructor
 *
 * FUNCIONALIDAD 2
 * ---------------
 * Compara el objeto para ver si es igual que otro del mismo tipo
 *
 * FUNCIONALIDAD 3
 * ---------------
 * Envia el objeto pago en formato XML a la pasarela de pagos, para lo cual
 * establece una conexion directa con la pasarela a travs de http y envia
 * una llamada RPC del tipo:
 * <rpcCall module='ia'>    // Llamada al Interfaz de aplicaciones
 *      <token>____</token>
 *      <function name='newPeticionPago'>   // Funciona a llamar con el XML
 *          <param name='peticionPagoXML' type='XML'><![CDATA[ ______ ]]</param>
 *      </function>
 * </rpcCall>
 * La plataforma recibe la llamada y la dirige automaticamente al interfaz
 *
 * de aplicaciones
 * FUNCIONALIDAD 3
 * ---------------
 * Se encarga de la transformacin XML del objeto
 *
 *
 * IMPORTANTE!!!!!!!!!!!!!!!!!!!!
 * ------------------------------
 * Las clases de pago son SERIALIZABLES para permitir ser pasadas a EJBs, sin embargo
 * para aumentar el rendimiento el MARSHALLER NO SE SERIALIZA, lo que hace que
 * si se llama a el metodo toXML() en un EJB se vuelva a cargar el marshaller...
 * Por lo tanto, EVITAR LLAMAR A toXML() DESDE LOS EJB
 */


import java.util.Iterator;
import java.util.List;





public abstract class AbstractPagoObj implements java.io.Serializable {   
    private List _brokenRules = null;       // Contiene las reglas rotas del objeto actual
    
    
    /**
     * Devuelve una lista con los errores encontrados en el OBJETO ACTUAL
     * Normalmente este metodo sera sobrecargado por las subclases para comprobar
     * la validez de sus datos.
     */
    List validateThisObject() {
        return null;
    }
    /**
     * Devuelve la lista de reglas rotas como una cadena 
     */
    public static String getBrokenRulesAsString(List brokenRules) {
        StringBuffer sb = new StringBuffer();
        if (brokenRules != null && brokenRules.size() > 0) {
            for (Iterator it = brokenRules.iterator(); it.hasNext(); ) {
                sb.append( it.next() + "\r\n" );                
            }
        }
        return sb.toString();
    }
}
