package p12f.exe.pasarelapagos.v1.objects;



public class Emisor extends AbstractPagoObj {
    public String codigo;    
    public String cif;
    public String nombreEuskera;
    public String nombreCastellano;
    public String calle;
    public String municipio;
    public String territorio;	
    public String codigoPostal;
    
    
    /** Constructores */
    public Emisor() {
    }  
    public Emisor(String inCodigo) {
        if (!_validateCodigoEmisor(inCodigo)) throw new IllegalArgumentException("El codigo del emisor no es valido, debe tener el formato: '99999999-999'");        
        this.codigo = inCodigo;
        // Obtener el cif a partir del codigo de emisor
        this.cif = this.codigo.substring(0,8);  // El texto anterior al guion
    }
    public Emisor(String inCodigo,String inNombreEuskera,String inNombreCastellano) {
        this(inCodigo);
        this.nombreCastellano = inNombreCastellano;
        this.nombreEuskera = inNombreEuskera;
    }
    public Emisor(String inCif,String inSufijo,String inNombreEuskera,String inNombreCastellano) {
        this.cif = inCif;      
        this.codigo = inCif + "-" + inSufijo; // Obtener el codigo a partir de cif y sufijo
        this.nombreCastellano = inNombreCastellano;
        this.nombreEuskera = inNombreEuskera;
    }
//____________________________________________________________________________\\
    /**
     * Obtiene el sufijo de la entidad emisora a partir del codigo de emisor
     */
    public String getSufijo() {
        if (_validateCodigoEmisor(this.codigo)) return this.codigo.substring(9); // A partir del - a la derecha          
        return "999";       // Emisor no valido    
    }
    /**
     * Obtiene el codigo de la entidad emisor a partir del codigo del emisor
     */
    public String getEmisor() {
		return cif;
    }
//____________________________________________________________________________\\        
    /**
     * Metodo estatico que devuelve el sufijo a partir de un codigo de emisor
     */
    public static String getSufijo(String inCodigoEmisor) {
        Emisor tempEmisor = new Emisor(inCodigoEmisor);
        return tempEmisor.getSufijo();
    }
    /**
     * Metodo esttico que devuelve el codigo del emisor (cif) a partir de un
     * codigo de emisor en formato cif-sufijo
     */
    public static String getEmisor(String inCodigoEmisor) {
    	Emisor tempEmisor = new Emisor(inCodigoEmisor);
    	return tempEmisor.getEmisor();
    }
//____________________________________________________________________________\\    
    
    /** 
     * Valida el codigo del emisor 
     */
    private boolean _validateCodigoEmisor(String inCodigo) {
        if (inCodigo == null || inCodigo.length() == 0 || inCodigo.length() != 12) return false;
        if ( !(inCodigo.substring(8,9).equals("-")) ) return false;        
        // Obtener el sufijo a partir del emisor y verificar que es valido
        try {
            String sufijo = inCodigo.substring(9);  // A partir del - a la derecha
            Integer.parseInt(sufijo);
        } catch (NumberFormatException nfEx) {
            return false;
        }        
        return true;
    }
    
}
