package p12f.exe.pasarelapagos.v1.objects;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.ejie.r01f.util.DateUtils;
import com.ejie.r01f.util.MoneyUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;


/**
 * Clase que define un pago
 */
public class Pago extends AbstractPagoObj {
    public Emisor emisor;               // Datos del emisor
    public Descripcion descripcion;     // Descripcion del pago
    public String referencia;           // Referencia (CSB57)
    public String identificacion;       // Identificacin del pago (CSB57)
    public String fechaPago;      		 // Fecha pago
    public String fechaLimitePago;      // Fecha lmite de pago (CSB57)
    public String importe;              // Importe (CSB57 - CENTEUROS)
    public String cuentaCargo;          // Cuenta en la que se abona el pago
    public String cuentaAbono;          // Cuenta en la que se carga el pago    
    public List conceptosPago;          // Conceptos (desglose de importes)
    public String idioma;				 // Idioma por omision
    public Map mensajes;                // Mensajes a incluir en el justificante
    public Map imagenes;                // Imagenes a incluir en el justificante
    public List datosAuxiliares;        // Datos auxiliares
    public Protocolo protocolo;         // Datos de protocolo
    
    /** Constructores */
    public Pago() {
    }
    
    public Pago(PeticionPago inPeticionPago) {        
        // Copiar los datos de la peticion de pago al pago...
        this.emisor = inPeticionPago.emisor;
        this.descripcion = inPeticionPago.descripcion;
        this.referencia = inPeticionPago.referencia;
        this.identificacion = inPeticionPago.identificacion;
        this.cuentaCargo = inPeticionPago.cuentaAbono;
        this.idioma = inPeticionPago.presentacion.idioma;
        this.fechaPago =  DateUtils.getDateFormated( new Date(),XMLProperties.get("p12f","formatoFechasViejo") );
        this.fechaLimitePago = inPeticionPago.fechaLimitePago;
        this.importe = String.valueOf(MoneyUtils.getCenteuros(inPeticionPago.importe));
        // Copiar los conceptos del pago
        ConceptoPago currConceptoPago = null;      
        if(inPeticionPago.conceptosPeticionPago!=null && inPeticionPago.conceptosPeticionPago.size()>0){
            this.conceptosPago = new ArrayList();
            for (Iterator it = inPeticionPago.conceptosPeticionPago.iterator(); it.hasNext(); ) {
	            ConceptoPeticionPago currConceptoPeticionPago = (ConceptoPeticionPago) it.next();
	            currConceptoPago = new ConceptoPago();
	            currConceptoPago.euskera = currConceptoPeticionPago.euskera;
	            currConceptoPago.castellano = currConceptoPeticionPago.castellano;
	            currConceptoPago.importe = String.valueOf(MoneyUtils.getCenteuros(currConceptoPeticionPago.importe));
	            this.conceptosPago.add(currConceptoPago);
	        }        
        }
        // Por un error, se defini en el xml los mensajes sin id,
        // los bancos han desarrollado su parte con los mensajes sin id, as que
        // hay que hacer un apao
        if (inPeticionPago.mensajes != null) {
        	this.mensajes = new HashMap();
        	for (Iterator it = inPeticionPago.mensajes.values().iterator(); it.hasNext(); ) {
        		Mensaje currMsg = (Mensaje)it.next();
        		// OJO!!! El incluir el id en el mensaje puede fastidiar a los de BBK
        		//this.mensajes.put(currMsg.id,new Mensaje(currMsg.euskera,currMsg.castellano));
				this.mensajes.put(currMsg.id,new Mensaje(currMsg.id,currMsg.euskera,currMsg.castellano));        		
        	}
        }
        this.imagenes = inPeticionPago.imagenes;
        this.datosAuxiliares = inPeticionPago.datosAuxiliares;
        this.protocolo = inPeticionPago.protocolo;
    }
    
    /**
	 * Convierte un Objeto Pago a XML.
	 * @return
	 * @throws XOMarshallerException
	 */
    public String toXML() throws XOMarshallerException {   
        return XOManager.getXML(XMLProperties.get("p12f","objectMapPathV1"),this);
    }
    
    /**
	 * Convierte un XML en un objeto Pago.
	 * @return
	 * @throws XOMarshallerException
	 */
	public static Pago getObject(String xml) throws XOMarshallerException {    
	    return (Pago)XOManager.getObject(XMLProperties.get("p12f","objectMapPathV1"), xml);
	}
}
