package p12f.exe.pasarelapagos.v1.objects;

/**
 * Clase que define un pago
 */


import java.util.List;
import java.util.Map;

import p12f.exe.pasarelapagos.v1.objects.config.ConfigSufijo;

import com.ejie.r01f.util.MoneyUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;




public class PeticionPago extends AbstractPagoObj {    
// Codigos de modos de pago
    public static final String MODO_PAGO_NO_APLICABLE = "0";
    public static final String MODO_PAGO_FRACCIONADO = "1";
    public static final String MODO_PAGO_CONTRA_DEPOSITO = "2";
    public static final String MODO_PAGO_TPV = "3";
    
    public Emisor emisor;               // Datos del emisor
    public Descripcion descripcion;     // Descripcion del pago
    public String referencia;           // Referencia (CSB57)
    public String identificacion;       // Identificacin del pago (CSB57)
    public double importe;              // Importe (CSB57)
    public String fechaLimitePago;      // Fecha lmite de pago (CSB57)
    public String cuentaCargo;          // Cuenta en la que se abona el pago
    public String cuentaAbono;          // Cuenta en la que se carga el pago
    public boolean informarSIPCA;	// Indica
    public String entidadSIPCA;         // Entidad emisora de dos posiciones (de momento siempre es 01)
    public boolean exentoPago;          // SIPCA
    public boolean gastoGV;             // SIPCA
    public String modoPago;             // Pago Fraccionado / Pago contra deposito / Pago por TPV
    public String numeroReferenciaAnterior;     // Referencia del recibo anterior para pagos fraccionados
    public String numeroReferenciaDeposito;     // Referencia del deposito para pagos contra deposito
    public String numeroAutorizacionTPV;        // Numero de autorizacion del TPV virtual
    public String ejercicioContable;            // Ejercicio contable de la aplicacion
    public String expedienteAsociado;           // Expediente asociado en ATEA
    public List conceptosPeticionPago;      // Conceptos (desglose de importes)
    public Tercero tercero;                 // Datos del tercero
    public Map imagenes;                    // Imgenes auxiliares
    public Map mensajes;                    // Mensajes que aparecen en la liquidacion
    public List datosAdicionales;           // Datos adicionales
    public List datosAuxiliares;            // Datos auxiliares
    public Aplicacion aplicacion;           // Datos de la aplicacin informatica origen..
    public Presentacion presentacion;       // Datos de presentacion en la pasarela de pagos
    public Liquidacion liquidacion;	     // Datos de la liquidacion
    public Protocolo protocolo;             // Datos de protocolo
    
    /** Constructores */
    public PeticionPago() {
        this.entidadSIPCA = "01";   // A pelo de momento,...        
    }
    /**
     * Constructor a partir de la configuracion de un pago por CSB
     * (cuando se paga utilizando un formulario de pago por CSB, los datos que
     *  faltan se cargan de base de datos en config)
     */
    public PeticionPago(ConfigSufijo config) {
        // Construir el objeto de peticin de pago a partir de los datos que se han 
        // recibido y los obtenidos de la base de datos
        this.emisor = config.emisor;
        this.descripcion = config.descripcion;
        this.informarSIPCA = config.informarSIPCA;    // Indica si hay que meter el pago en SIPCA
        this.entidadSIPCA = config.entidadSIPCA;
        this.exentoPago = config.exentoPago;
        this.ejercicioContable = config.ejercicioContable;
        this.gastoGV = config.gastoGV;
        this.modoPago = config.modoPago;
        this.expedienteAsociado = config.expedienteAsociado; 
        this.conceptosPeticionPago = config.conceptosPeticionPago;
        this.datosAdicionales = config.datosAdicionales;
        this.datosAuxiliares = config.datosAuxiliares;
        this.presentacion = config.presentacion;
        this.mensajes = config.mensajes;
        this.aplicacion = config.aplicacion;
        this.liquidacion = config.liquidacion;
        this.presentacion = config.presentacion; 
        this.imagenes = config.imagenes;
    }
//____________________________________________________________________________\\    
    /**
     * Metodo estatico para obtener el id de pago 
     */
    public static String getPagoOID(String inCIF,String inSufijo,String inReferencia,String inIdentificador,double inImporte) {
        return inCIF + inSufijo + inReferencia + inIdentificador + String.valueOf(MoneyUtils.getCenteuros(inImporte));        
    }
    public static String getPagoOID(String inCIF,String inSufijo,String inReferencia,String inIdentificador,String inImporteCenteuros) {
        return inCIF + inSufijo + inReferencia + inIdentificador + inImporteCenteuros;                
    }
    public static String getPagoOID(String inEmisor,String inReferencia,String inIdentificador,double inImporte) {
        if (inEmisor.indexOf('-') > 0) {
            return inEmisor.substring(0,8) + inEmisor.substring(9) + inReferencia + inIdentificador + String.valueOf(MoneyUtils.getCenteuros(inImporte));
        }        
        return inEmisor + inReferencia + inIdentificador + String.valueOf(MoneyUtils.getCenteuros(inImporte));                
    }
    public static String getPagoOID(String inEmisor,String inReferencia,String inIdentificador,String inImporteCenteuros) {
        if (inEmisor.indexOf('-') > 0) {
            return inEmisor.substring(0,8) + inEmisor.substring(9) + inReferencia + inIdentificador + inImporteCenteuros;  
        }
        return inEmisor + inReferencia + inIdentificador + inImporteCenteuros;                
    }   
    public static String getPagoOID(String inEmisor,String inReferencia,String inIdentificador,long inImporteCenteuros) {
        if (inEmisor.indexOf('-') > 0) {
            return inEmisor.substring(0,8) + inEmisor.substring(9) + inReferencia + inIdentificador + inImporteCenteuros;  
        }
        return inEmisor + inReferencia + inIdentificador + inImporteCenteuros;                
    }
    public String toXML() throws XOMarshallerException {   
        return XOManager.getXML(XMLProperties.get("p12f","objectMapPathV1"),this);
   }
   
 
    /**
     * Convierte un XML en un objeto Pago.
     * @return
     * @throws XOMarshallerException
     */
    public static PeticionPago getObject(String xml) throws XOMarshallerException {    
        return (PeticionPago)XOManager.getObject(XMLProperties.get("p12f","objectMapPathV1"), xml);
    }
    
    

}
