package p12f.exe.pasarelapagos.v1.objects.config;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import p12f.exe.pasarelapagos.v1.objects.Descripcion;

/**
 * Datos de configuracion de la pasarela de pagos
 */

public class ConfigInstanciaPasarela {
    public String oid;              // Identificador de la pasarela
    public boolean activa = true; // Est activa la instancia?
    public boolean esInstanciaPorDefecto = false;	// Es la psarela por defecto?
    public Descripcion titulo;      // Titulo de la pasarela
    public String idioma;           // Idioma por defecto
    public Map imagenes;            // Imagenes de la pasarela
    public String tipoLetra;        // Tipo de letra a utlizar 
    public Map colores;             // Colores de la pasarela
    public String copyRight;        // Texto del copyRight 
    public Map urls;                // URL de la pasarela
    public List headerPath;         // Path para llegar a la pasarela que se muestra en la cabecera
    public Map configEntidades;	 	// Configuracion de los emisores admitidos en la pasarela
    								// (a su vez es un mapa con los distintos CSB admitidos)
    
    /** Constructores */
    public ConfigInstanciaPasarela() {       
    }   
    
    
//____________________________________________________________________________\\
    public String getColor(String colorID) {
        if (this.colores == null) return "#FFFFFF";
        Color color = (Color)this.colores.get(colorID);
        if (color != null) {
            return color.code;
        } else {
            return "#FFFFFF";
        }       
    }
//____________________________________________________________________________\\
	/**
	 * Devuelve la configuracin de pago de un sufijo correspondiente
	 * a un determinado emisor dentro de una instancia de la pasarela
	 */
	public ConfigSufijo getConfiguracionSufijo(String entidad,String sufijo) {				
		if (this.configEntidades == null) return null;
		ConfigEntidad cfgEntidad = (ConfigEntidad)this.configEntidades.get(entidad);
		if (cfgEntidad != null && cfgEntidad.configSufijos != null) {			
			return (ConfigSufijo)cfgEntidad.configSufijos.get(sufijo);
		} else {
			return null;
		}
	}   
//____________________________________________________________________________\\ 

	/**
	 * Obtiene la instancia de la pasarela por defecto, para ello recorre 
	 * la lista de pasarelas y devuelve aquella que tenga la propiedad
	 * esInstanciaPorDefecto a true
	 * @param (Map)instancias
	 * @return ConfigInstanciaPasarela: La instancia por defecto
	 */
	public static ConfigInstanciaPasarela getDefaultInstance(Map instances) {
		if (instances == null) return null;
		ConfigInstanciaPasarela currInstance = null;
		for (Iterator it = instances.values().iterator(); it.hasNext(); ) {
			currInstance = (ConfigInstanciaPasarela)it.next();
			if (currInstance != null && currInstance.esInstanciaPorDefecto) return currInstance;			
		}
		return null;	// No hay pasarela por defecto
	}   
}
