/*
 * Created on May 29, 2007
 * 
 * @author ie00191m - Alex Lara
 * (c) 2007 EJIE: Eusko Jaurlaritzako Informatika Elkartea
 */
package p12f.exe.search;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import p12f.exe.pasarelapagos.objects.Estado;
import p12f.exe.pasarelapagos.paymentrequest.PaymentMode;
import p12f.exe.search.parameters.EXPSearchedParam;
import p12f.exe.search.parameters.FinantialOrgSearchedParam;
import p12f.exe.search.parameters.PaymentDataSearchedParam;
import p12f.exe.search.parameters.PaymentModeSearchedParam;
import p12f.exe.search.parameters.PaymentStateSearchedParam;
import p12f.exe.search.parameters.ThirdPartySearchedParam;
import p12f.exe.search.sqlutils.PaymentQuerySQLComposer;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 *  Clase que implementa los diferentes grupos de parmetros de definicin de bsquedas de la BBDD de la Pasarela.
 */
public class PaymentQuery implements Serializable, Initializable{
    
    protected List thirdPartyList;            // Lista  Parmetros de Bsqueda por Tercero :::: ThrirdPartySearchedParam
    protected List expList ;                  // Lista Parmetros Parmetros de Bsqueda por Expediente  ::: EXPSearchedParam
    protected List stateParameterList;        // Parmetros de Bsqueda por Estado.PaymentStateSearchedParam    
    protected List paymentDataParameterList;  // Parmetros de Busqueda por datos de pago (csb, oid, tributo, etc...)PaymentDataSearchedParam
    protected List finantialOrgParameterList; // Parmetros de Busqueda por entidad financiera (3035, 2095, etc...)FinantialOrgSearchedParam
    protected List paymentModeParameterList;  // Parmetros de Busqueda por modo de pago (ventanilla, movil, etc...)PaymentModeSearchedParam


    /**
     * Contructor de busquedas.
     *
     */
    public PaymentQuery() {        
        thirdPartyList =  new ArrayList();
        expList = new ArrayList();        
        stateParameterList = new ArrayList();      
        paymentDataParameterList = new ArrayList();      
        finantialOrgParameterList = new ArrayList();      
        paymentModeParameterList = new ArrayList();      
    }
       
    /************************************************************************************************
        FUNCIONES PARA REALIZAR BUSQUEDAS A TRAVES DE TERCEROS.        
    /************************************************************************************************
    
    /**
     * Aade un nuevo objeto de tercero con sus datos.
     * @param dni     DNI del tercero
     * @param razonSocial Razon Social del Tercero. 
     */
    public void addThirdPartyData(String dni, String razonSocial) {
        
        thirdPartyList.add(new ThirdPartySearchedParam(dni,razonSocial));
    }
    
    /**
     * Aade el dni de tercero como parmetro de busqueda. 
     * @param dni
     */
    public void addThirdPartyDNIData(String dni) {        
        thirdPartyList.add(new ThirdPartySearchedParam(dni,null));
    }
  
    /**
     * Aade la razon social del tercero como parmetro de busqueda. 
     * @param dni
     */
    public void addThirdPartyRazonSocialData(String razonSocial) {        
        thirdPartyList.add(new ThirdPartySearchedParam(null,razonSocial));
    }
    
    
    /************************************************************************************************
                FUNCIONES PARA REALIZAR BUSQUEDAS A TRAVES DE EXPEDIENTES        
    *************************************************************************************************/
    
    
    /**
     * Aade un nuevo objeto expediente con sus datos.
     * @param codExp
     * @param codFamilia
     */    
    public void addEXPData (String codExp, String codFamilia) {
         expList.add( new EXPSearchedParam( codExp,codFamilia));        
    }
    
    /**
     * Aade el cdigo de expediente como parmetro de busqueda.
     * @param codExp
     */
        
    public void addEXPCodeData (String codExp) {
        expList.add( new EXPSearchedParam( codExp,null));        
    }
   
    
    /**
     * Aade el cdigo de expediente como parmetro de busqueda.
     * @param codExp
     */
        
    public void addEXPFamilyData (String codFamily) {
        expList.add( new EXPSearchedParam(null,codFamily));        
    }
   
   
    
    /************************************************************************************************
        FUNCIONES PARA REALIZAR BUSQUEDAS A TRAVES DE DATOS DEL ESTADO.
    *************************************************************************************************/
    
    
    /**
     * Parmetros de busqueda por Datos Estado.
     * @param code Cdigo de Estado del Pago.
     * @param nrc
     */
    
    public void addPaymentStateData(String code, String nrc ) {        
        stateParameterList.add( new PaymentStateSearchedParam(code,nrc));              
    }
    
    /**
     * Parmetros de busqueda por Datos Estado.
     * @param code Cdigo de Estado del Pago.
     * @param nrc
     */    
    public void addPaymentStateData(String code, String nrc,Date dateFrom, Date dateTo ) {        
        stateParameterList.add( new PaymentStateSearchedParam(code, nrc, dateFrom, dateTo));              
    }
    
    /**
     * Parmetros de busqueda por Datos Estado.
     * @param code Cdigo de Estado del Pago.
     * @param nrc
     */    
    public void addPaymentStateData(String code, String nrc, Date dateFrom, Date dateTo, String oidEntidad) {        
        stateParameterList.add( new PaymentStateSearchedParam(code, nrc, dateFrom, dateTo, oidEntidad));              
    }
    
    
    /**
     * Parmetros de busqueda por Datos Estado.
     * @param code
     * @param nrc
     */
    
    public void addPaymentStateCodeData(String stateCode) {        
        stateParameterList.add( new PaymentStateSearchedParam(stateCode,null));    
    }
    
    /**
     * Parmetros de busqueda por Datos Estado.
     * @param code
     * @param nrc
     */
    
    public void addPaymentStateNRCData(String nrc) {        
        stateParameterList.add( new PaymentStateSearchedParam(null,nrc));        
    }
    
    public void addPaymentStatePaymentMode(String paymentMode) {
    	PaymentStateSearchedParam pm = new PaymentStateSearchedParam();
    	pm.paymentMode = paymentMode;
    	stateParameterList.add(pm);
    }
    
    public void addPaymentStateEstadoBackend(String backendStatus) {
    	PaymentStateSearchedParam bs = new PaymentStateSearchedParam();
    	bs.backendStatus = backendStatus;
    	stateParameterList.add(bs);
    }

    
    /************************************************************************************************
    FUNCIONES PARA REALIZAR BUSQUEDAS A TRAVES DATOS INHERENTES AL PAGO (ADMIN, SUFIJO, FORMATO, )      
    /************************************************************************************************/
    
    /**
     *  Aade una bloque de parmetros relativos a datos de pago (Administracion - Tributo - Formato)
     */
    public void addPaymentDataSearchedParam(String admin, String typo, String format) {
        paymentDataParameterList.add(new PaymentDataSearchedParam(admin,typo,format));
    }
    
    /**
     *  Aade una bloque de parmetros relativos a datos de pago (Administracion - Tributo - Formato)
     */
    public void addPaymentDataSearchedParam(String admin, String typo, String format, String dateFrom, String dateTo) {
        paymentDataParameterList.add(new PaymentDataSearchedParam(admin,typo,format));
    }
    
    /**
     *  Aade una bloque de parmetros relativos a datos de pago (OID - Administracion - Tributo - Formato)
     */
    public void addPaymentDataSearchedParam(String oid, String admin, String typo, String format) {
        paymentDataParameterList.add(new PaymentDataSearchedParam(oid,admin,typo,format));
    }  
    
    /**
     *  Aade una bloque de parmetros relativos a datos de pago (OID - Administracion - Tributo - Formato)
     */
    public void addPaymentDataSearchedParam(String oid, String admin, String typo, String format, String reference, String identification) {
        paymentDataParameterList.add(new PaymentDataSearchedParam(oid, admin, typo, format, reference, identification));
    }  
    
    /**
     *  Aade una bloque de parmetros relativos a datos de pago (OID - Administracion - Tributo - Formato)
     */
    public void addPaymentDataSearchedParam(String oid, String admin, String typo, String format, String reference, String identification, String importe) {
        paymentDataParameterList.add(new PaymentDataSearchedParam(oid, admin, typo, format, reference, identification, importe));
    }  

    public void addPaymentDataSearchedParam(String admin) {
        paymentDataParameterList.add(new PaymentDataSearchedParam(admin,null,null));
    }

    /************************************************************************************************
    FUNCIONES PARA REALIZAR BUSQUEDAS A TRAVES DE ENTIDADES FINANCIERAS.        
	/************************************************************************************************

	/**
     * Aade un nuevo objeto de entidad financiera con sus datos.
     * @param finantialOrg     Entidad financiera 3035, 2095, ...
     */
    public void addFinantialOrgData(String finantialOrg) {        
    	finantialOrgParameterList.add(new FinantialOrgSearchedParam(finantialOrg));
    }

    /************************************************************************************************
    FUNCIONES PARA REALIZAR BUSQUEDAS A TRAVES DE MODOS DE PAGO.        
	/************************************************************************************************

	/**
     * Aade un nuevo objeto de modo de pago con sus datos.
     * @param paymentMode     Modo de pago 01, 02, ...
     */
    public void addPaymentModeData(String paymentMode) {        
    	paymentModeParameterList.add(new PaymentModeSearchedParam(paymentMode));
    }

    
    /**
     * Convierte un Objeto PaymentData a XML.
     * @return String que contiene el XML.
     * @throws XOMarshallerException
     */
    public String toXML() throws XOMarshallerException {  
        return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
    }

    
    /*****************************************************************************************/
    /*                      METODOS GET                                                     */
    /****************************************************************************************/
    
    
    
    public List getThirdPartyList() {
        
        return this.thirdPartyList;    // Lista  Parmetros de Bsqueda por Tercero :::: ThrirdPartySearchedParam
    }
    public List getExpList() {
        
        return this.expList;          // Lista Parmetros Parmetros de Bsqueda por Expediente  ::: EXPSearchedParam
    }
    public List getStateParameterList() {
        
        return this.stateParameterList;  // Parmetros de Bsqueda por Estado.PaymentStateSearchedParam    
    }
    public List getpaymentDataParameterList() {
        
        return this.paymentDataParameterList; 
    }
    public List getFinantialOrgParameterList() {
        return this.finantialOrgParameterList;  // Parmetros de Bsqueda por entidad financiera.FinantialOrgSearchedParam    
    }
    public List getPaymentModeParameterList() {
        return this.paymentModeParameterList;  // Parmetros de Bsqueda por modo de pago.PaymentModeSearchedParam    
    }
    
    
    
    
    
    
    
    /***********************************************************************************
     *                                  Test
     **********************************************************************************/
    
    
    public static void main (String[] argv) throws XOMarshallerException {
        
        PaymentQuery paymentQuery  = new PaymentQuery();
        
        /** Buscar del tributo 504 del Gobierno Vasco todos los recibos pagados */
        paymentQuery.addPaymentDataSearchedParam("04833001","504","507");
        paymentQuery.addPaymentDataSearchedParam("486000","502","502");
        paymentQuery.addPaymentStateCodeData(Estado.PAGADO);
        paymentQuery.addPaymentModeData(PaymentMode.MOVIL);

        //System.out.println(XOConstants.xmlHeaderDirective + paymentQuery.toXML());
        //System.out.println("\n\n");
        
        /** Buscar todos los recibos del Gobierno pagados por dni 7888252257 , 741111111 */
        
        // test -------------------------------------------------------------
        paymentQuery  = new PaymentQuery();

        paymentQuery.addPaymentDataSearchedParam("04833001","504","507");
        //paymentQuery.addThirdPartyDNIData("7888252578T");
        //paymentQuery.addThirdPartyDNIData("000000008T");
        //paymentQuery.addThirdPartyDNIData("22702849D");
        paymentQuery.addThirdPartyDNIData("B7596113");

        // p.ej. pagados (04) o backend sipca 06... --------------------------
        //paymentQuery.addPaymentStateCodeData(Estado.PAGADO);
        //paymentQuery.addPaymentStateEstadoBackend("06"); // 06 --> BaseRegistroSIPCA.TIPO_OPERACION_COBROS_OK
        
        // p.ej. pendientes (no 04 && no sipca 06... -------------------------
        paymentQuery.addPaymentStateCodeData(PaymentStateSearchedParam.ALL_NOT_PAID_WITH_BACKEND_ACK);

        PaymentQuerySQLComposer sqlHelper1 = new PaymentQuerySQLComposer();
        String strFinalQuery1 = sqlHelper1.composeQuery(paymentQuery);
        System.out.println("TRAZA: SQL: "+strFinalQuery1);

        //System.out.println(XOConstants.xmlHeaderDirective  +  paymentQuery.toXML());
        System.out.println("TRAZA: "+paymentQuery.toXML());
        // test -------------------------------------------------------------
        
        //PaymentQuery query = (PaymentQuery)  XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), "<paymentQuery><paymentStateSearchedParamList><paymentStateSearchedParam><finantialOrg>3035</finantialOrg><dateFrom>15/06/2001</dateFrom><dateTo>02/11/2010</dateTo></paymentStateSearchedParam></paymentStateSearchedParamList><paymentDataSearchedParamList><paymentDataSearchedParam><admin>04833001</admin></paymentDataSearchedParam></paymentDataSearchedParamList></paymentQuery>");
//        PaymentQuery query = new PaymentQuery();
//        //query.addEXPCodeData("kkkk");
//        query.addEXPData("kk", "codFamily");
//        
//        PaymentQuerySQLComposer sqlHelper = new PaymentQuerySQLComposer();
//
//        String strFinalQuery  = sqlHelper.composeQuery(query);
//        System.out.println(strFinalQuery);
//
//        
//        PaymentQuery query2 = (PaymentQuery)  XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), query.toXML());
//        System.out.println(query.toXML());
//        System.out.println(query2.toXML());
    }
    
    
    

}
