/*
 * Created on May 29, 2007
 * 
 * @author ie00191m - Alex Lara
 * (c) 2007 EJIE: Eusko Jaurlaritzako Informatika Elkartea
 */
package p12f.exe.search.parameters;

import java.io.Serializable;
import java.util.Date;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

/**
 *  Parmetros de Busqueda por Estado.
 */
public class PaymentStateSearchedParam implements Serializable, Initializable {

    public String stateCode ; // Codigo del Estado.
    public String nrc ; // Cdigo del NRC.
    public Date dateFrom;
    public Date dateTo;
    public String finantialOrg;
    public String paymentMode; // "01", "02", "03", "05" (PaymentMode.LIQUIDACION...)
    public String backendStatus; // Estado backend (sipca)
   
    
   // Constantes
    
    public static final String ALL_NOT_PAID = "99";
    public static final String ALL_NOT_PAID_WITH_BACKEND_ACK = "98";         // Pendientes pago... ^06 && ^04 && ^03 && BE is NULL
    public static final String ALL_NOT_PAID_WITH_BACKEND_ACK_ON_DATE = "97";
    
    
    /* 
     * Contructor vacio.
     */
    public PaymentStateSearchedParam () {
        
    }
    
    
    /**
     * Parmetros de Bsqueda por Estado.
     * @param stateCode
     * @param nrc
     */
    public PaymentStateSearchedParam (String stateCode  , String nrc ) {
        this.nrc = nrc;
        this.stateCode = stateCode;       
    }

    /**
     * Parmetros de Bsqueda por Estado.
     * @param stateCode
     * @param nrc
     */
//    public PaymentStateSearchedParam (String stateCode, String backendStatus) {
//        this.nrc = nrc;
//        this.stateCode = stateCode;       
//        this.backendStatus = backendStatus;       
//    }

    /**
     * Parmetros de Bsqueda por Estado.
     * @param stateCode
     * @param nrc
     */
    public PaymentStateSearchedParam (String stateCode, String nrc, Date dateFrom, Date dateTo) {
        this.nrc = nrc;
        this.stateCode = stateCode;
        this.dateFrom = dateFrom;
        this.dateTo = dateTo;
    }
    
    /**
     * Parmetros de Bsqueda por Estado.
     * @param stateCode
     * @param nrc
     */
    public PaymentStateSearchedParam (String stateCode, String nrc, Date dateFrom, Date dateTo, String oidEntidad) {
        this.nrc = nrc;
        this.stateCode = stateCode;
        this.dateFrom = dateFrom;
        this.dateTo = dateTo;
        this.finantialOrg = oidEntidad;
    }

    /**
     * Parmetros de Bsqueda por Modo de pago.
     * @param paymentMode 
     */
    public PaymentStateSearchedParam(String paymentMode) {
        this.paymentMode = paymentMode;
    }
    
    /**
     * Convierte un Objeto PaymentData a XML.
     * @return String que contiene el XML.
     * @throws XOMarshallerException
     */
    public String toXML() throws XOMarshallerException {  
        return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
    }
  
    
    
}
