/*
 * Created on May 31, 2007
 * 
 * @author ie00191m - Alex Lara
 * (c) 2007 EJIE: Eusko Jaurlaritzako Informatika Elkartea
 */
package p12f.exe.search.sqlutils;


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import p12f.exe.pasarelapagos.paymentrequest.PaymentMode;
import p12f.exe.search.PaymentQuery;
import p12f.exe.search.XMLSearchProperties;
import p12f.exe.search.parameters.EXPSearchedParam;
import p12f.exe.search.parameters.FinantialOrgSearchedParam;
import p12f.exe.search.parameters.PaymentDataSearchedParam;
import p12f.exe.search.parameters.PaymentModeSearchedParam;
import p12f.exe.search.parameters.PaymentStateSearchedParam;
import p12f.exe.search.parameters.ThirdPartySearchedParam;

import com.ejie.r01f.xml.marshalling.XOMarshallerException;

/**
 * Convierte un XML de consulta en una SQL contra la BBDD de La Pasarela.
 */
public class PaymentQuerySQLComposer implements PaymentQuerySQLComposerInterface {
        
   
    private PaymentSQL type507PaymentSQL  = null;   // Tipo de Sentencia 507
    private PaymentSQL type502PaymentSQL  = null;   // Tipo de Sentencia 502
    private PaymentSQL type521PaymentSQL  = null;   // Tipo de Sentencia 521
    private PaymentSQL type522PaymentSQL  = null;   // Tipo de Sentencia 522
    
    
    public  PaymentQuerySQLComposer() {
    }
    
    /*
     *  Clase de lectura de Configuracin de Properties : p12x.searcher.properties.xml
     */
    private XMLSearchProperties  searchProperties  = XMLSearchProperties.getInstance();
    private PaymentQuerySQLComposerHelper sqlHelper = new PaymentQuerySQLComposerHelper();
    
    /*
     * Compone la query final.
     */
    public String composeQuery( PaymentQuery paymentQuery ) {
        
        _createSingleQueries (paymentQuery );      
        
        String finalQuery = null;      
           
        if (type507PaymentSQL != null )        
            finalQuery = this._addSingleQuery(finalQuery,type507PaymentSQL.toString());        
        if (type502PaymentSQL != null )
            finalQuery = this._addSingleQuery(finalQuery,type502PaymentSQL.toString());        
        if (type521PaymentSQL != null )        
            finalQuery = this._addSingleQuery(finalQuery,type521PaymentSQL.toString());        
        if (type522PaymentSQL != null )        
            finalQuery = this._addSingleQuery(finalQuery,type522PaymentSQL.toString());       
        
        return finalQuery;
    }
    
    /*
     * Compone la query final.
     */
    public String composeHistoricQuery( String pagoId ) {        
        String finalQuery = null;      
           
        finalQuery = sqlHelper.composeHistoricQuery(pagoId);

        return finalQuery;
    }
    
    public String composeHistoricQueryOrderByDesc( String pagoId) {        
        String finalQuery = null;      
           
        finalQuery = sqlHelper.composeHistoricQueryOrderByDesc(pagoId);

        return finalQuery;
    }
    /****************************************************************************************************
     *  Metodos privados para componer la SQL -
     * **************************************************************************************************/
    
    private void _createSingleQueries ( PaymentQuery paymentQuery ) {  
        
        /* Se inicializan los posibles tipos de Query. Lo normal es que que NO todas se incialicen, es decir
         * que sigan a NULL. Si se inicializan todas es seal del que el usuario no ha introducido el tipo de 
         * formato
         */
        this.type507PaymentSQL = (PaymentSQL)sqlHelper.doInitializeQueryTypes(paymentQuery).get("507");
        this.type502PaymentSQL = (PaymentSQL)sqlHelper.doInitializeQueryTypes(paymentQuery).get("502");
        this.type521PaymentSQL = (PaymentSQL)sqlHelper.doInitializeQueryTypes(paymentQuery).get("521");
        this.type522PaymentSQL = (PaymentSQL)sqlHelper.doInitializeQueryTypes(paymentQuery).get("522");
        
      
        List parameterCSBDataList = paymentQuery.getpaymentDataParameterList();
     
        /* Bloque de datos referentes al CSB : ADMIN, TYPE .....*/
        for (Iterator it= parameterCSBDataList.iterator(); it.hasNext() ;) {             
            PaymentDataSearchedParam dataSearch= (PaymentDataSearchedParam) it.next();               
            type507PaymentSQL = sqlHelper.addPaymentDataBlockClause(dataSearch,type507PaymentSQL);                      
            type502PaymentSQL = sqlHelper.addPaymentDataBlockClause(dataSearch,type502PaymentSQL); 
            type521PaymentSQL = sqlHelper.addPaymentDataBlockClause(dataSearch,type521PaymentSQL);
            type522PaymentSQL = sqlHelper.addPaymentDataBlockClause(dataSearch,type522PaymentSQL); 
                  
        }
        
        /* Bloque de datos referentes al Tercero : DNI, RAZON SOCIAL, ETC... .....*/        
        List parameterThirdPartyList = paymentQuery.getThirdPartyList();
        for (Iterator it= parameterThirdPartyList.iterator(); it.hasNext() ;) {             
         
            ThirdPartySearchedParam third = (ThirdPartySearchedParam) it.next(); 
            
            type507PaymentSQL = sqlHelper.addThridPartyBlockClause(third,type507PaymentSQL);   
            type502PaymentSQL = sqlHelper.addThridPartyBlockClause(third,type502PaymentSQL);  
            type521PaymentSQL = sqlHelper.addThridPartyBlockClause(third,type521PaymentSQL);
            type522PaymentSQL = sqlHelper.addThridPartyBlockClause(third,type522PaymentSQL); 
           
        }        
        /* Bloque de datos referentes al Expediente : Cdigo, Familia .....*/
        
        List expList = paymentQuery.getExpList();
        for (Iterator it= expList.iterator(); it.hasNext() ;) {
            EXPSearchedParam exp = (EXPSearchedParam) it.next();             
            type507PaymentSQL = sqlHelper.addExpBlockClause(exp,type507PaymentSQL);   
            type502PaymentSQL = sqlHelper.addExpBlockClause(exp,type502PaymentSQL);  
            type521PaymentSQL = sqlHelper.addExpBlockClause(exp,type521PaymentSQL);
            type522PaymentSQL = sqlHelper.addExpBlockClause(exp,type522PaymentSQL); 
        }
        
        /* Bloque de datos referentes al Estado : Cdigo, Familia .....*/
        
        List statusList = paymentQuery.getStateParameterList();
        for (Iterator it= statusList.iterator(); it.hasNext() ;) {
            PaymentStateSearchedParam status = (PaymentStateSearchedParam) it.next();            
            type507PaymentSQL = sqlHelper.addStatusBlockClause(status,type507PaymentSQL);   
            type502PaymentSQL = sqlHelper.addStatusBlockClause(status,type502PaymentSQL);  
            type521PaymentSQL = sqlHelper.addStatusBlockClause(status,type521PaymentSQL);
            type522PaymentSQL = sqlHelper.addStatusBlockClause(status,type522PaymentSQL); 
        }    
        
        List modeList = paymentQuery.getPaymentModeParameterList();
        for (Iterator it= modeList.iterator(); it.hasNext() ;) {
        	PaymentModeSearchedParam mode = (PaymentModeSearchedParam) it.next();            
            type507PaymentSQL = sqlHelper.addModeBlockClause(mode,type507PaymentSQL);   
            type502PaymentSQL = sqlHelper.addModeBlockClause(mode,type502PaymentSQL);  
            type521PaymentSQL = sqlHelper.addModeBlockClause(mode,type521PaymentSQL);
            type522PaymentSQL = sqlHelper.addModeBlockClause(mode,type522PaymentSQL); 
        }    
        
        List efList = paymentQuery.getFinantialOrgParameterList();
        for (Iterator it= efList.iterator(); it.hasNext() ;) {
        	FinantialOrgSearchedParam ef = (FinantialOrgSearchedParam) it.next();            
            type507PaymentSQL = sqlHelper.addFinantialOrgBlockClause(ef,type507PaymentSQL);   
            type502PaymentSQL = sqlHelper.addFinantialOrgBlockClause(ef,type502PaymentSQL);  
            type521PaymentSQL = sqlHelper.addFinantialOrgBlockClause(ef,type521PaymentSQL);
            type522PaymentSQL = sqlHelper.addFinantialOrgBlockClause(ef,type522PaymentSQL); 
        }    
        
        type507PaymentSQL = sqlHelper.addJoinClause(type507PaymentSQL);   
        type502PaymentSQL = sqlHelper.addJoinClause(type502PaymentSQL);  
        type521PaymentSQL = sqlHelper.addJoinClause(type521PaymentSQL);
        type522PaymentSQL = sqlHelper.addJoinClause(type522PaymentSQL);       
        
        //System.out.println("507 Type = \n" + type507PaymentSQL + "\n");
        //System.out.println("502 Type = \n" + type502PaymentSQL + "\n");
        //System.out.println("521 Type = \n" + type521PaymentSQL + "\n");
        //System.out.println("552 Type = \n" + type522PaymentSQL + "\n");
                        
    }
    
    
    /**
     * Aade una query simple a la final
     * @param finalQuery
     */
    private String _addSingleQuery(String finalQuery, String singleQuery ) {        
        if (singleQuery == null)
            return finalQuery; 
        if ( finalQuery  == null ) 
            return singleQuery;
        
        return finalQuery + " UNION " + singleQuery;
            
    }
    
    
    
    /**
     * Testeo  de XML a SQL
     * @param arg
     * @throws XOMarshallerException
     */
    public static void main (String[] arg) throws XOMarshallerException { //, ParseException {
        
        PaymentQuery  paymentQuery = new PaymentQuery();
        
        /*Buscar del tributo 504 del Gobierno Vasco todos los recibos pagados */
        //paymentQuery.addPaymentDataSearchedParam("04833001",null,"507");        
        //paymentQuery.addPaymentDataSearchedParam("486000","001","507");
        //paymentQuery.addThirdPartyDNIData("15391987l");        
        paymentQuery.addThirdPartyDNIData("55555555K");        
        paymentQuery.addThirdPartyDNIData("77777777B");        
        paymentQuery.addEXPCodeData("1234567890123456789012345");
        paymentQuery.addEXPCodeData("1234567890123456789012999");
        
         SimpleDateFormat formatter = new SimpleDateFormat("dd/MM/yyyy");
         Date dateTo   = null;
         Date dateFrom = null;
		try {
			dateTo  = formatter.parse("17/11/2010");
	        dateFrom= formatter.parse("12/11/2010");
		} catch (ParseException e) {
			e.printStackTrace();
		}
         String str = null;
         //paymentQuery.addPaymentStateData(PaymentStateSearchedParam.ALL_NOT_PAID,str,dateFrom,dateTo,str); 
         paymentQuery.addPaymentStateData(PaymentStateSearchedParam.ALL_NOT_PAID_WITH_BACKEND_ACK_ON_DATE,str,dateFrom,dateTo,str); 
         paymentQuery.addPaymentModeData(PaymentMode.MOVIL);
         paymentQuery.addPaymentModeData(PaymentMode.BANCA_ONLINE_VENTANILLA);
         paymentQuery.addEXPData("1234567890123456789012777", "familia777");
         paymentQuery.addEXPData("1234567890123456789012888", "familia888");
         paymentQuery.addFinantialOrgData("2095");
         paymentQuery.addFinantialOrgData("3035");
         //paymentQuery.addEXPFamilyData("familia001");
         //paymentQuery.addEXPFamilyData("familia002");
         paymentQuery.addPaymentDataSearchedParam("04833001");
         paymentQuery.addPaymentDataSearchedParam("486000");
         
         
        //System.out.println(XOConstants.xmlHeaderDirective + paymentQuery.toXML());        
        
        PaymentQuerySQLComposer  sqlHelper = new PaymentQuerySQLComposer();        
        
        
        String finalQuery  = sqlHelper.composeQuery(paymentQuery);
     
        
        System.out.println("============================" );
        System.out.println("paymentQuery XML: " + paymentQuery.toXML());        
        System.out.println("============================" );
        System.out.println("FINAL QUERY =\n" + finalQuery  );
        System.out.println("============================" );
        
    }
    
   
   
    
    
}


