/*
 * Created on Jun 6, 2007
 * 
 * @author ie00191m - Alex Lara
 * (c) 2007 EJIE: Eusko Jaurlaritzako Informatika Elkartea
 */
package p12f.exe.search.sqlutils;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import p12f.exe.pasarelapagos.objects.Estado;
import p12f.exe.search.PaymentQuery;
import p12f.exe.search.XMLSearchProperties;
import p12f.exe.search.parameters.EXPSearchedParam;
import p12f.exe.search.parameters.FinantialOrgSearchedParam;
import p12f.exe.search.parameters.PaymentDataSearchedParam;
import p12f.exe.search.parameters.PaymentModeSearchedParam;
import p12f.exe.search.parameters.PaymentStateSearchedParam;
import p12f.exe.search.parameters.ThirdPartySearchedParam;

import com.ejie.r01f.util.StringUtils;


/**
 * PaymentQuerySQLManagerHelper.
 */
public class PaymentQuerySQLComposerHelper {       
    
    /*
     *  Clase de lectura de Configuracin de Properties : p12x.searcher.properties.xml     */
    private XMLSearchProperties  searchProperties  = XMLSearchProperties.getInstance();
    
    private static String TIPO_OPERACION_COBROS_OK = "06";
    private static String TIPO_OPERACION_DEVENGO_COBRO_FUERA_PLAZO = "08";
    
    
    public PaymentQuerySQLComposerHelper() {
        
    }
    
    /**
     * Devuelve un MAP de Objetos PaymentQuery inicializados., con los tipos necesarios. 
     * Si no viene el campo dataSearch.format, se deben inicializar todos los tipos. 
     * @param paymentQuery
     * @return
     */
    public HashMap doInitializeQueryTypes( PaymentQuery paymentQuery ) {
            HashMap mapQuery = new HashMap();
            
            if (( paymentQuery.getpaymentDataParameterList() != null ) 
                    &&  (  ! paymentQuery.getpaymentDataParameterList().isEmpty() )){
            
                for (Iterator it= paymentQuery.getpaymentDataParameterList().iterator(); it.hasNext() ;) {            
                    PaymentDataSearchedParam dataSearch= (PaymentDataSearchedParam) it.next();   
                    
                    if  ( dataSearch.format != null ) { 
                        
                         if (! mapQuery.containsKey(dataSearch.format))
                            mapQuery.put(dataSearch.format,_initFormatQuery(Integer.parseInt(dataSearch.format)));
                    
                     } else {
                       // Si no viene el formato se deben inicalizar todos los tipos de query,
                       //  ya que puede tratarse de cualquier tipo. Esto es una Opcin Poco Optima : Lanzar un Warning.
                         
                         if (! mapQuery.containsKey("507"))    mapQuery.put("507",_initFormatQuery(Integer.parseInt("507")));
                         if (! mapQuery.containsKey("502"))    mapQuery.put("502",_initFormatQuery(Integer.parseInt("502")));
                         if (! mapQuery.containsKey("521"))    mapQuery.put("521",_initFormatQuery(Integer.parseInt("521")));
                         if (! mapQuery.containsKey("522"))    mapQuery.put("522",_initFormatQuery(Integer.parseInt("522")));
                       
                     }
                }//for  
            
             } else {
                 //System.out.println("NO SEARCH DATA");
                 if (! mapQuery.containsKey("507"))    mapQuery.put("507",_initFormatQuery(Integer.parseInt("507")));
                 if (! mapQuery.containsKey("502"))    mapQuery.put("502",_initFormatQuery(Integer.parseInt("502")));
                 if (! mapQuery.containsKey("521"))    mapQuery.put("521",_initFormatQuery(Integer.parseInt("521")));
                 if (! mapQuery.containsKey("522"))    mapQuery.put("522",_initFormatQuery(Integer.parseInt("522")));
                 
             }
           return mapQuery;
    }
        
    /***
     * Inicializa las querys de los diferentes tipos de pagos indicando las diferentes tablas que se inicialziaran 
     * en el proceso de busqueda : Select * from $PAYMENT_DATA_TABLE$ , $STATUS_TABLE$, $EXP_TABLE$ , $THIRD_TABLE$ where]
     * @param format
     */
    private PaymentSQL  _initFormatQuery(int format) {
     
        PaymentSQL  paymentSql = new PaymentSQL(Integer.toString(format));   
        
        // Query bsica de bsqueda,
        String basicQuery = searchProperties.get("p12f","searchPaymentConfig/basicQueryConfig/query");  
        
        Map variables = new HashMap();        
        
        // Variables comunes.
        variables.put("STATUS_TABLE",searchProperties.get("p12f","searchPaymentConfig/tableList/statusTable/name")); 
        variables.put("EXP_TABLE",searchProperties.get("p12f","searchPaymentConfig/tableList/expTable/name")); 
        variables.put("THIRD_TABLE",searchProperties.get("p12f","searchPaymentConfig/tableList/thirdPartyTable/name"));                
    
        // Tabla dependiente del tipo de Formato.
        
        variables.put("PAYMENT_DATA_TABLE", searchProperties.get("p12f","searchPaymentConfig/tableList/formatTables/FormatTable[@id=" + format + "]//name")); 
        
        // Parametros
        variables.put("PARAMS",getParametersToSelect(Integer.toString(format)));
        
        paymentSql.hasBeeInitialized = true;
    
        // Inicializar la clausalas select.
        paymentSql.selectClause = StringUtils.replaceVariableValues(basicQuery,'$',variables);     
        
        return paymentSql;
    
    }

    /***
     * Aade Condiciones propias del Bloque de Expedientes.
     * @param third
     * @param paymentSQL
     * @return
     */    
    public PaymentSQL addExpBlockClause( EXPSearchedParam expediente ,PaymentSQL paymentSQL ) { 
        
        if ( paymentSQL  ==  null )      
            return null;      
        
        String expCondition = null;
        String blockExpedCod  = null;
        String blocExpFamily  = null; 
        
        String expCodeFieldName = searchProperties.get("p12f","searchPaymentConfig/tableList/expTable/fields/expCode");
        String expFamilyFieldName =  searchProperties.get("p12f","searchPaymentConfig/tableList/expTable/fields/expFamily");
        
        if (expediente.codExp!= null)
            blockExpedCod  =   expCodeFieldName + "='" + expediente.codExp +  "'";        
        if  (expediente.codFamily != null )
            blocExpFamily =   expFamilyFieldName  + " ='" + expediente.codFamily + "'";        
        
        if ( blockExpedCod != null) {
            expCondition = blockExpedCod;
            if (blocExpFamily !=  null) 
                expCondition = "(" + blockExpedCod + " AND " + blocExpFamily + ")";
        }
        
        
        // Aadir a las sucesivas condiciones Third.
        if(expCondition!=null) {        
            if ( paymentSQL.expDATAconditionClause == null)
                paymentSQL.expDATAconditionClause = expCondition;        
            else 
                paymentSQL.expDATAconditionClause +=   " OR " +  expCondition;
        }
        
        return paymentSQL;       
    }

    /***
     * Aade la clausula JOIN para la  Query Dada.
     * @param 
     * @param paymentSQL
     * @return
     */
    
    public PaymentSQL addJoinClause(PaymentSQL paymentSQL ) {       
        
        if ( paymentSQL  ==  null )      
            return null;      
        
        int format = Integer.parseInt(paymentSQL.id);        
       // String joinClause = null;       
    
        String pkCSBTable = searchProperties.get("p12f","searchPaymentConfig/tableList/formatTables/FormatTable[@id=" + format + "]/pkField");
        String pkStatusTable =  searchProperties.get("p12f","searchPaymentConfig/tableList/statusTable/joinField");
        String pkExpTable = searchProperties.get("p12f","searchPaymentConfig/tableList/expTable/joinField");
        String pkThirdTable = searchProperties.get("p12f","searchPaymentConfig/tableList/thirdPartyTable/joinField");        
        
        paymentSQL.joinClause = "";
        
        if(paymentSQL.csbDATAconditionClause!=null || 
                paymentSQL.expDATAconditionClause!=null ||
                paymentSQL.statusDATAconditionClause!=null ||
                paymentSQL.thirdPartyDATAconditionClause!=null) {
            paymentSQL.joinClause += "AND ";
        }         
        
        paymentSQL.joinClause +=  pkCSBTable + " = " +  pkStatusTable  + "(+) \n" +
        "AND " +  pkCSBTable + " = " +  pkExpTable     + "(+)\n" +
        "AND " +  pkCSBTable + " = " +  pkThirdTable   + "(+)\n";
        
        return paymentSQL;        
                
    }

    /**
     * Aade condiciones del Bloque de datos del CSB
     * @param dataSearched Condiciones a aadir
     * @param paymentSQL : SQL a la que se le aadiran las condiciones.
     * @return
     */
    public PaymentSQL addPaymentDataBlockClause(PaymentDataSearchedParam dataSearched, PaymentSQL paymentSQL ) {           
    
        if ( paymentSQL  ==  null )      
            return null;
        
        String paymenDataCondition = null;
        String blockOidPago  = null;
        String blockEmisor  = null;
        String blockTributo  = null; 
        String blockReference  = null; 
        String blockIdentification  = null; 
        String blockImporte  = null; 
        
        String oidPago =  dataSearched.oidPago;
        String emisor =  dataSearched.admin;
        String tributo = dataSearched.typo;
        String format = dataSearched.format;
        String reference = dataSearched.reference;
        String identification = dataSearched.identification;
        String importe = dataSearched.importe;
    
       if (format == null || format.equalsIgnoreCase(paymentSQL.id)) {
            
            String pagoIdFieldName = searchProperties.get("p12f","searchPaymentConfig/tableList/formatTables/FormatTable[@id=" + paymentSQL.id + "]/fields/pagoIdField");
            String adminFieldName = searchProperties.get("p12f","searchPaymentConfig/tableList/formatTables/FormatTable[@id=" + paymentSQL.id+ "]/fields/adminField");
            String typoFieldName =  searchProperties.get("p12f","searchPaymentConfig/tableList/formatTables/FormatTable[@id=" + paymentSQL.id+ "]/fields/typoField");
            String referenceFieldName =  searchProperties.get("p12f","searchPaymentConfig/tableList/formatTables/FormatTable[@id=" + paymentSQL.id+ "]/fields/referenceField");
            String identificationFieldName =  searchProperties.get("p12f","searchPaymentConfig/tableList/formatTables/FormatTable[@id=" + paymentSQL.id+ "]/fields/identificationField");
            String importeFieldName =  searchProperties.get("p12f","searchPaymentConfig/tableList/formatTables/FormatTable[@id=" + paymentSQL.id+ "]/fields/importeField");
            
            if (oidPago!=null) {
                blockOidPago = pagoIdFieldName + "='" + oidPago + "'";
                paymenDataCondition = blockOidPago;
            }            
            if (emisor != null) {
                blockEmisor  =   adminFieldName + "='" + emisor +  "'";
                if(paymenDataCondition==null) {
                    paymenDataCondition = blockEmisor;
                } else {
                    paymenDataCondition = paymenDataCondition + " AND " + blockEmisor;
                }
            }    
            if  (tributo != null ) {
                blockTributo =   typoFieldName  + "='" +  tributo + "'";
                if(paymenDataCondition==null) {
                    paymenDataCondition = blockTributo;
                } else {
                    paymenDataCondition = paymenDataCondition + " AND " + blockTributo;
                }
            }
            if  (reference != null ) {
                blockReference =   referenceFieldName  + "='" +  reference + "'";
                if(paymenDataCondition==null) {
                    paymenDataCondition = blockReference;
                } else {
                    paymenDataCondition = paymenDataCondition + " AND " + blockReference;
                }
            }
            if  (identification != null ) {
                blockIdentification =   identificationFieldName  + "='" +  identification + "'";
                if(paymenDataCondition==null) {
                    paymenDataCondition = blockIdentification;
                } else {
                    paymenDataCondition = paymenDataCondition + " AND " + blockIdentification;
                }
            }
            if  (importe != null ) {
                blockImporte =   importeFieldName  + "='" +  importe + "'";
                if(paymenDataCondition==null) {
                    paymenDataCondition = blockImporte;
                } else {
                    paymenDataCondition = paymenDataCondition + " AND " + blockImporte;
                }
            }
            
            /*if(paymenDataCondition!=null) {
                paymenDataCondition = "(" + paymenDataCondition + ")";
            }*/            
            
            if(paymenDataCondition!=null) {
                if ( paymentSQL.csbDATAconditionClause == null)
                    paymentSQL.csbDATAconditionClause= paymenDataCondition;        
                else {                
                    paymentSQL.csbDATAconditionClause +=   " OR " +  paymenDataCondition; 
                }
            }
       }
        
        return paymentSQL;       
        
    }

    /***
     * Aade Condiciones propias del Bloque de Estado : PAGADO, NO_PAGADO, NRC......
     * @param third
     * @param paymentSQL
     * @return
     */
    
    public PaymentSQL addStatusBlockClause( PaymentStateSearchedParam statusParam ,PaymentSQL paymentSQL ) { 
        
        if ( paymentSQL  ==  null )      
            return null;      
        
        String statusCondition = null;
        String blockStatusCode = null;
        String blockNRC  = null; 
        String blockDate  = null;
        String blockFinantialOrg  = null;
        
        String statusCodeFieldName = searchProperties.get("p12f","searchPaymentConfig/tableList/statusTable/fields/stateCode");
        String nrcCodeFieldName =  searchProperties.get("p12f","searchPaymentConfig/tableList/statusTable/fields/nrcCode");
        String initDateFieldName =  searchProperties.get("p12f","searchPaymentConfig/tableList/statusTable/fields/initDate");
        String finantialOrgFieldName =  searchProperties.get("p12f","searchPaymentConfig/tableList/statusTable/fields/finantialOrg");
        String backendStatusFieldName = searchProperties.get("p12f","searchPaymentConfig/tableList/statusTable/fields/backendStatus");
        String limitChargeabilityDateFieldName = searchProperties.get("p12f","searchPaymentConfig/tableList/statusTable/fields/limitChargeabilityDate"); //"FECHALIMITE20";
        //devengo == chargeability
        
        if ( statusParam.stateCode != null) {

        	if (statusParam.stateCode.equalsIgnoreCase(PaymentStateSearchedParam.ALL_NOT_PAID)){
        		blockStatusCode = statusCodeFieldName + "<>'" + Estado.PAGADO  +  "'";      
 	            statusCondition = blockStatusCode;

        	} else {
            	if (statusParam.stateCode.equalsIgnoreCase(PaymentStateSearchedParam.ALL_NOT_PAID_WITH_BACKEND_ACK)){
            		blockStatusCode  = // ^06 && ^03 && ^04 && BE is NULL
            			statusCodeFieldName + "<>'" + Estado.PAGADO  + "' AND " +
            			statusCodeFieldName + "<>'" + Estado.ANULADO + "' AND " +
            			statusCodeFieldName + "<>'" + Estado.NO_DATA + "' AND " +
            			backendStatusFieldName + " IS NULL";
     	            statusCondition = blockStatusCode;
            	} else {
                	if (statusParam.stateCode.equalsIgnoreCase(PaymentStateSearchedParam.ALL_NOT_PAID_WITH_BACKEND_ACK_ON_DATE)){

                		SimpleDateFormat dateFormatter = new SimpleDateFormat("dd/MM/yyyy");
                		String strHoy = dateFormatter.format(Calendar.getInstance().getTime());

                		blockStatusCode  = // ^06 && ^03 && ^04 && BE is NULL && fecha lmite >= hoy (devengo emisin, en ejecutiva)
                			statusCodeFieldName + "<>'" + Estado.PAGADO  + "' AND " +
                			statusCodeFieldName + "<>'" + Estado.ANULADO + "' AND " +
                			statusCodeFieldName + "<>'" + Estado.NO_DATA + "' AND " +
                			// fecha lmite nula OR fecha lmite igual o posterior a hoy 
                			" ("+ limitChargeabilityDateFieldName + " IS NULL OR " +
                			      limitChargeabilityDateFieldName + " >= " +  "TO_DATE('" + strHoy + " 00:00:00','DD/MM/YYYY HH24:MI:SS') " +
                			" ) AND " +
                			backendStatusFieldName + " IS NULL";
         	            statusCondition = blockStatusCode;
                	} else {
                		blockStatusCode  =   statusCodeFieldName + "='" + statusParam.stateCode  +  "'";      
                		statusCondition = blockStatusCode;
                	}
            	}
        	}
        }

        if (statusParam.backendStatus != null) {
        	//if (statusParam.backendStatus.equalsIgnoreCase(BaseRegistroSIPCA.TIPO_OPERACION_COBROS_OK)) {
        		blockStatusCode = backendStatusFieldName + "='" + statusParam.backendStatus + "'";      
        		statusCondition = blockStatusCode;
        	//}
        }

        if  (statusParam.nrc != null ){
            blockNRC =   nrcCodeFieldName  + " = '" + statusParam.nrc + "'";               
	        if ( statusCondition != null) {
	            if (blockNRC !=  null) 
	                statusCondition += " AND " + blockNRC;
	        } else if (blockNRC!=null) {
	            statusCondition = blockNRC;
	        }
        }
        
        // Gestion de Fechas.

        SimpleDateFormat newFormatter = new SimpleDateFormat("MM/dd/yyyy");

 		// Hay que incluir la hora, si no se pierden ciertos pagos...
        // AND FECHAINI20 BETWEEN to_date('01/09/2008 00:00:00','DD/MM/YYYY HH24:MI:SS')
        //                    AND to_date('30/09/2008 23:59:59','DD/MM/YYYY HH24:MI:SS')

        if (statusParam.dateFrom!=null) {
            String dateFromString = newFormatter.format( statusParam.dateFrom);        
        	//blockDate =   initDateFieldName  + " >= TO_DATE('" + dateFromString+ "','MM/DD/YYYY')";               
        	blockDate =   initDateFieldName  + " >= TO_DATE('" + dateFromString.replaceAll("( [0-9]{2}:[0-9]{2}:[0-9]{2})?$", " 00:00:00") + "','MM/DD/YYYY HH24:MI:SS')";               
	        if ( statusCondition != null) {
	            if (blockDate !=  null) 
	                statusCondition += " AND " + blockDate;
	        } else if (blockDate!=null) {
	            statusCondition = blockDate;
	        }
        }
           
        if (statusParam.dateTo!=null) {
        	String dateToString = newFormatter.format( statusParam.dateTo );   
        	//blockDate =   initDateFieldName + " <= TO_DATE('" + dateToString + "','MM/DD/YYYY')";     
        	blockDate =   initDateFieldName + " <= TO_DATE('" + dateToString.replaceAll("( [0-9]{2}:[0-9]{2}:[0-9]{2})?$", " 23:59:59") + "','MM/DD/YYYY HH24:MI:SS')";     
	        if ( statusCondition != null) {
	            if (blockDate !=  null) 
	                statusCondition += " AND " + blockDate;
	        } else if (blockDate!=null) {
	            statusCondition = blockDate;
	        }
        }
        
        if (statusParam.finantialOrg!=null) {
        	blockFinantialOrg =   finantialOrgFieldName  + " = '" + statusParam.finantialOrg + "'";               
	        if ( statusCondition != null) {
	            if (blockFinantialOrg !=  null) 
	                statusCondition += " AND " + blockFinantialOrg;
	        } else if (blockFinantialOrg!=null) {
	            statusCondition = blockFinantialOrg;
	        }
        }
        
        // Aadir a las sucesivas condiciones Third.
        if(statusCondition!=null) {
            if ( paymentSQL.statusDATAconditionClause == null)
                paymentSQL.statusDATAconditionClause = statusCondition;        
            else 
                paymentSQL.statusDATAconditionClause +=   " OR " +  statusCondition;
        }
        
        return paymentSQL;       
    }

    /**
     * Aade condiciones del Bloque de datos de Tercero.
     * @param third
     * @param paymentSQL
     * @return
     */
    public PaymentSQL  addThridPartyBlockClause( ThirdPartySearchedParam third ,PaymentSQL paymentSQL ) {   
        
        if ( paymentSQL  ==  null )      
            return null;
        
        String dni = third.dniNIF;
        String razonSocial  = third.razonSocial;
               
        String thirdCondition = null;
        String blockDNI  = null;
        String blocRazonSocial  = null; 
        
        String dniFieldName = searchProperties.get("p12f","searchPaymentConfig/tableList/thirdPartyTable/fields/dni");
        String razonSocialFieldName =  searchProperties.get("p12f","searchPaymentConfig/tableList/thirdPartyTable/fields/razonSocial");
        
        if (dni != null)
            blockDNI  =  "UPPER(" + dniFieldName + ")" + "='" + dni.toUpperCase() +  "'";        
        if  (razonSocial != null )
             blocRazonSocial =  "UPPER(" + razonSocialFieldName + ")"   + " LIKE '%" +  razonSocial.toUpperCase() + "%'";        
        
        if ( blockDNI != null) {
            thirdCondition = blockDNI;
            if (blocRazonSocial !=  null) 
                thirdCondition = "(" + thirdCondition + " AND " + blocRazonSocial + ")";
        } else if (blocRazonSocial!=null) {
            thirdCondition = blocRazonSocial;
        }
        
        // Aadir a las sucesivas condiciones Third.
        if(thirdCondition!=null) {
            if ( paymentSQL.thirdPartyDATAconditionClause == null)
                paymentSQL.thirdPartyDATAconditionClause = thirdCondition;        
            else 
                paymentSQL.thirdPartyDATAconditionClause +=   " OR " +  thirdCondition;
        }
        
        return paymentSQL;       
    }

    /***
     * Aade Condiciones propias del Bloque de Modo : Liquidacion, Banca Online......
     * @param third
     * @param paymentSQL
     * @return
     */
    
    public PaymentSQL addModeBlockClause( PaymentModeSearchedParam modeParam ,PaymentSQL paymentSQL ) { 
        
        if ( paymentSQL  ==  null )      
            return null;      
        
        // AND (MODOPAGO='01')
        
        String modeCondition = null;
        String blockModeCode = null;
        
        String modeCodeFieldName = searchProperties.get("p12f","searchPaymentConfig/tableList/statusTable/fields/paymentMode");
        //String modeCodeFieldName = "MODOPAGO20"; 
        
        if ( modeParam.paymentMode != null && !"".equals(modeParam.paymentMode)) {

        	blockModeCode  =   modeCodeFieldName + "='" + modeParam.paymentMode  +  "'";      
            modeCondition = blockModeCode;
        }

        // Aadir a las sucesivas condiciones Third.
        if(modeCondition!=null) {
            if ( paymentSQL.paymentModeDATAconditionClause == null)
                paymentSQL.paymentModeDATAconditionClause = modeCondition;        
            else 
                paymentSQL.paymentModeDATAconditionClause +=   " OR " +  modeCondition;
        }
        
        return paymentSQL;       
    }

    /***
     * Aade Condiciones propias del Bloque de Entidad Financiera
     * @param efParam
     * @param paymentSQL
     * @return
     */
    public PaymentSQL addFinantialOrgBlockClause( FinantialOrgSearchedParam efParam ,PaymentSQL paymentSQL ) { 
        
        if ( paymentSQL  ==  null )      
            return null;      
        
        String efCondition = null;
        String blockEFCode = null;
        
        String efFieldName = searchProperties.get("p12f","searchPaymentConfig/tableList/statusTable/fields/finantialOrg");
        //String modeCodeFieldName = "ENTFIN20"; 
        
        if ( efParam.finantialOrg != null && !"".equals(efParam.finantialOrg)) {

        	blockEFCode  =   efFieldName + "='" + efParam.finantialOrg  +  "'";      
            efCondition = blockEFCode;
        }

        // Aadir a las sucesivas condiciones Third.
        if(efCondition!=null) {
            if ( paymentSQL.finantialOrgDATAconditionClause == null)
                paymentSQL.finantialOrgDATAconditionClause = efCondition;        
            else 
                paymentSQL.finantialOrgDATAconditionClause +=   " OR " +  efCondition;
        }
        
        return paymentSQL;       
    }

    /**
     * Inserta paramtros en la Query de Consulta  : Select ID  X.ID_1 ,  ID Y. ID_2 
     * sto es necesario hacerlo por si se requiere hacer UNION de Tablas Diferentes (al denominarse los campos diferentes=
     */
    public String getParametersToSelect (String id) {
      
       Properties listaThird = searchProperties.getProperties("p12f","searchPaymentConfig/tableList/thirdPartyTable/fields");
       Properties listaStatus = searchProperties.getProperties("p12f","searchPaymentConfig/tableList/statusTable/fields");
       Properties listaExp = searchProperties.getProperties("p12f","searchPaymentConfig/tableList/expTable/fields");
       Properties csbCodeList =  searchProperties.getProperties("p12f","searchPaymentConfig/tableList/formatTables/FormatTable[@id=" + id + "]/fields");
            
       // Parametros CSB
       String csbParams ="";
              
       for  (Enumeration enum = csbCodeList.keys(); enum.hasMoreElements(); ) {       
           String key = (String) enum.nextElement();
           if (csbParams != "") 
               csbParams  +=  "," + csbCodeList.getProperty(key) + " " + key;
           else   csbParams  += csbCodeList.getProperty(key) + " " + key;
        }
              
       // Parametros third
       String thirdParams = "";
       for  (Enumeration enum = listaThird.keys(); enum.hasMoreElements(); ) {       
          String key = (String) enum.nextElement();
          thirdParams  +=  "," + listaThird.getProperty(key) + " " + key;
       }
       //  Parametros Status
       String statusParams = "";
       for  (Enumeration enum = listaStatus.keys(); enum.hasMoreElements(); ) {       
           String key = (String) enum.nextElement();
           statusParams  +=  "," + listaStatus.getProperty(key) + " " + key;
       }
       
       // Parametros Exp.
       String expParams = "";              
       for  (Enumeration enum = listaExp.keys(); enum.hasMoreElements(); ) {       
           String key = (String) enum.nextElement();
           expParams  +=  "," + listaExp.getProperty(key) + " " + key ;
       }
       
       return ( csbParams + thirdParams  + statusParams +  expParams);       
    
    }
    
    public String composeHistoricQuery(String pagoId) {
    	return composeHistoricQuery(pagoId, false);
    }
    
    public String composeHistoricQueryOrderByDesc(String pagoId) {
    	return composeHistoricQuery(pagoId, true);
    }
    
    private String composeHistoricQuery(String pagoId, boolean desc) {    	
        // Select
    	String selectClause = "";
        String selectParams ="";
        Properties listaSelectHistoric = searchProperties.getProperties("p12f","searchPaymentConfig/tableList/historicTable/fields");       
        for  (Enumeration enum = listaSelectHistoric.keys(); enum.hasMoreElements(); ) {       
            String key = (String) enum.nextElement();
            if (selectParams != "") { 
            	selectParams  +=  "," + listaSelectHistoric.getProperty(key) + " " + key;
            } else {
            	selectParams  += listaSelectHistoric.getProperty(key) + " " + key;
            }
        }
        selectClause = "SELECT " + selectParams;
        
        // From
        String fromClause = "";
        String fromField = searchProperties.get("p12f","searchPaymentConfig/tableList/historicTable/name");
        fromClause = "FROM " + fromField;
        
        // Where
        String whereClause = "";
        String whereField = searchProperties.get("p12f","searchPaymentConfig/tableList/historicTable/joinField");
        whereClause = "WHERE " + whereField + " = '" + pagoId + "'";
        
        // Order By
        String orderByClause = "";
        String orderByField = searchProperties.get("p12f","searchPaymentConfig/tableList/historicTable/fields/tst");
        orderByClause = "ORDER BY " + orderByField;
        
        if(desc) orderByClause = orderByClause + " DESC";
        
        
    	return selectClause + " " + fromClause + " " + whereClause + " " + orderByClause;
    }
    
    /*** Testeo de Metodos */    
    public static void main (String[] argv) {
        
        
        PaymentQuerySQLComposerHelper helper= new PaymentQuerySQLComposerHelper();        
        helper.getParametersToSelect("507");
        
    }

}
