package p12f.exe.search.sqlutils;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;

import com.ejie.r01f.xml.marshalling.XOMarshallerException;

import p12f.exe.pasarelapagos.objects.Estado;
import p12f.exe.pasarelapagos.paymentrequest.PaymentMode;
import p12f.exe.search.PaymentQuery;
import p12f.exe.search.parameters.EXPSearchedParam;
import p12f.exe.search.parameters.FinantialOrgSearchedParam;
import p12f.exe.search.parameters.PaymentDataSearchedParam;
import p12f.exe.search.parameters.PaymentModeSearchedParam;
import p12f.exe.search.parameters.PaymentStateSearchedParam;
import p12f.exe.search.parameters.ThirdPartySearchedParam;

public class PaymentQuerySQLFinantialOrgStatsComposer implements PaymentQuerySQLComposerInterface {

	private PaymentSQL viewPaymentSQLFinantial   = null;
	private PaymentSQL viewPaymentSQLIniciados   = null;
	private PaymentSQL viewPaymentSQLCompletados = null;
	private PaymentSQL viewPaymentSQLErroneos    = null;
	private PaymentSQL groupByPaymentSQL         = null;

    public String viewId          = "785"; // left join
    public String viewIdFinantial = "786"; // todas las EFs, para left join
    public String viewIdInic      = "787"; // select iniciados
    public String viewIdComp      = "788"; // select completados
    public String viewIdErr       = "789"; // select erroneos

    private PaymentQuerySQLStatsComposerHelper sqlHelper = new PaymentQuerySQLStatsComposerHelper();

	public String composeQuery(PaymentQuery paymentQuery) {

		// Vamos a obtener los pagos iniciados, los completados, y hacer un left join de modo que
		// finalmente tengamos una select que nos d todos los datos estadsticos que necesitamos
		// en una nica consulta...

		// La sentencia SQL compuesta en finalQuery va a ser similar a esta...
		//  SELECT EF.FINANTIALORG AS ENTIDAD,
		// 	       ESTADO02.NPAGOS AS INICIADOS, ESTADO02.IMPORTE AS IMPORTEINICIADOS,
		// 	       ESTADO04.NPAGOS AS COMPLETADOS, ESTADO04.IMPORTE AS IMPORTECOMPLETADOS,
		// 	       ESTADO05.NPAGOS AS ERRONEOS, ESTADO05.IMPORTE AS IMPORTEERRONEOS
		// 	FROM
		// 	(SELECT DISTINCT FINANTIALORG FROM VALLPAYMENTS) EF, 
		// 	(SELECT FINANTIALORG, COUNT(PAGOIDFIELD) AS NPAGOS, SUM(IMPORTEFIELD) AS IMPORTE
		// 	   FROM VALLPAYMENTS
		// 	  WHERE
		// 	 (STATECODE='02'
		// 	  AND INITDATE >= TO_DATE('01/01/2009 00:00:00','MM/DD/YYYY HH24:MI:SS')
		// 	  AND INITDATE <= TO_DATE('12/31/2009 23:59:59','MM/DD/YYYY HH24:MI:SS')
		// 	  AND ADMINFIELD='04833001'
		// 	  )
		// 	  GROUP BY FINANTIALORG 
		// 	) ESTADO02,
		// 	(SELECT FINANTIALORG, COUNT(PAGOIDFIELD) AS NPAGOS, SUM(IMPORTEFIELD) AS IMPORTE
		// 	   FROM VALLPAYMENTS
		// 	  WHERE
		// 	 (STATECODE='05'
		// 	  AND INITDATE >= TO_DATE('01/01/2009 00:00:00','MM/DD/YYYY HH24:MI:SS')
		// 	  AND INITDATE <= TO_DATE('12/31/2009 23:59:59','MM/DD/YYYY HH24:MI:SS')
		// 	  AND ADMINFIELD='04833001'
		// 	  )
		// 	  GROUP BY FINANTIALORG 
		// 	) ESTADO05,
		// 	(SELECT FINANTIALORG, COUNT(PAGOIDFIELD) AS NPAGOS, SUM(IMPORTEFIELD) AS IMPORTE
		// 	   FROM VALLPAYMENTS
		// 	  WHERE
		// 	 (STATECODE='04'
		// 	  AND INITDATE >= TO_DATE('01/01/2009 00:00:00','MM/DD/YYYY HH24:MI:SS')
		// 	  AND INITDATE <= TO_DATE('12/31/2009 23:59:59','MM/DD/YYYY HH24:MI:SS')
		// 	  AND ADMINFIELD='04833001'
		// 	  )
		// 	  GROUP BY FINANTIALORG 
		// 	) ESTADO04
		// 	WHERE EF.FINANTIALORG IS NOT NULL
		// 	  AND EF.FINANTIALORG=ESTADO02.FINANTIALORG (+)
		// 	  AND EF.FINANTIALORG=ESTADO04.FINANTIALORG (+)
		// 	  AND EF.FINANTIALORG=ESTADO05.FINANTIALORG (+)
		// 	ORDER BY EF.FINANTIALORG

		// La diferencia entre las tres subselects sobre las que vamos a realizar el left join,
		// est solo en los atributos seleccionados; el resto de clausulas deben coincidir.

		// Inicializar la select con los nombres de parmetros de la vista...
		// -------------------------------------------------
		// Entidades financieras: SELECT DISTINCT FINANTIALORG FROM VALLPAYMENTS
		this.viewPaymentSQLFinantial = (PaymentSQL)sqlHelper.doInitializeQueryViewTypes(paymentQuery, viewIdFinantial).get(viewIdFinantial);
		// -------------------------------------------------
		// Completados: SELECT COUNT(PAGOIDFIELD) AS NPAGOIDFIELD, FORMATFIELD, ADMINFIELD, TYPOFIELD, SUM(IMPORTEFIELD) AS SIMPORTEFIELD
		//                FROM VALLPAYMENTS WHERE
		this.viewPaymentSQLCompletados = (PaymentSQL)sqlHelper.doInitializeQueryViewTypes(paymentQuery, viewIdComp).get(viewIdComp);
		// -------------------------------------------------
		// Iniciados: SELECT COUNT(PAGOIDFIELD) AS NPAGOIDFIELD, FORMATFIELD, ADMINFIELD, TYPOFIELD
		//              FROM VALLPAYMENTS WHERE
		this.viewPaymentSQLIniciados   = (PaymentSQL)sqlHelper.doInitializeQueryViewTypes(paymentQuery, viewIdInic).get(viewIdInic);
		// -------------------------------------------------
		// Erroneos: SELECT COUNT(PAGOIDFIELD) AS NPAGOIDFIELD, FORMATFIELD, ADMINFIELD, TYPOFIELD
		//              FROM VALLPAYMENTS WHERE
		this.viewPaymentSQLErroneos    = (PaymentSQL)sqlHelper.doInitializeQueryViewTypes(paymentQuery, viewIdErr).get(viewIdErr);
								// SELECT $PARAMS$ FROM $PAYMENT_DATA_VIEW$ WHERE

		// select "global" (left join de iniciados y completados)
		this.groupByPaymentSQL = (PaymentSQL)sqlHelper.doInitializeQueryViewTypes(paymentQuery, viewId).get(viewId);

		// Para cada subselect, las clausulas de join van implicitas en la vista, pero hay que aadir las de la propia consulta...
		// ... WHERE ((ADMINFIELD='04833001') OR (ADMINFIELD='486000'))

        // Bloque de datos referentes al CSB : ADMIN, TYPE ...
        List parameterCSBDataList = paymentQuery.getpaymentDataParameterList();
        for (Iterator it= parameterCSBDataList.iterator(); it.hasNext() ;) {
            PaymentDataSearchedParam dataSearch = (PaymentDataSearchedParam) it.next();
            viewPaymentSQLCompletados = sqlHelper.addPaymentDataBlockClause(dataSearch, viewPaymentSQLCompletados);
            viewPaymentSQLIniciados   = sqlHelper.addPaymentDataBlockClause(dataSearch, viewPaymentSQLIniciados);
            viewPaymentSQLErroneos    = sqlHelper.addPaymentDataBlockClause(dataSearch, viewPaymentSQLErroneos);
        }
        if (viewPaymentSQLCompletados.csbDATAconditionClause!=null) { // Agrupar entre parntesis: ( () OR ... OR () )
        	viewPaymentSQLCompletados.csbDATAconditionClause = "(" + viewPaymentSQLCompletados.csbDATAconditionClause + ")";
        	viewPaymentSQLIniciados.csbDATAconditionClause   = "(" + viewPaymentSQLIniciados.csbDATAconditionClause + ")";
        	viewPaymentSQLErroneos.csbDATAconditionClause    = "(" + viewPaymentSQLErroneos.csbDATAconditionClause + ")";
        }

        // Bloque de datos referentes al Tercero : DNI, RAZON SOCIAL, ETC...
        List parameterThirdPartyList = paymentQuery.getThirdPartyList();
        for (Iterator it= parameterThirdPartyList.iterator(); it.hasNext() ;) {
            ThirdPartySearchedParam third = (ThirdPartySearchedParam) it.next();
            viewPaymentSQLCompletados = sqlHelper.addThridPartyBlockClause(third,viewPaymentSQLCompletados);
            viewPaymentSQLIniciados   = sqlHelper.addThridPartyBlockClause(third,viewPaymentSQLIniciados);
            viewPaymentSQLErroneos    = sqlHelper.addThridPartyBlockClause(third,viewPaymentSQLErroneos);
        }

        // Bloque de datos referentes al Expediente : Cdigo, Familia ...
        List expList = paymentQuery.getExpList();
        for (Iterator it= expList.iterator(); it.hasNext() ;) {
            EXPSearchedParam exp = (EXPSearchedParam) it.next();
            viewPaymentSQLCompletados = sqlHelper.addExpBlockClause(exp,viewPaymentSQLCompletados);
            viewPaymentSQLIniciados   = sqlHelper.addExpBlockClause(exp,viewPaymentSQLIniciados);
            viewPaymentSQLErroneos    = sqlHelper.addExpBlockClause(exp,viewPaymentSQLErroneos);
        }
        if (viewPaymentSQLCompletados.expDATAconditionClause!=null) { // Agrupar entre parntesis: ( () OR ... OR () )
        	viewPaymentSQLCompletados.expDATAconditionClause = "(" + viewPaymentSQLCompletados.expDATAconditionClause + ")";
        	viewPaymentSQLIniciados.expDATAconditionClause   = "(" + viewPaymentSQLIniciados.expDATAconditionClause + ")";
        	viewPaymentSQLErroneos.expDATAconditionClause    = "(" + viewPaymentSQLErroneos.expDATAconditionClause + ")";
        }

        // Bloque de datos referentes al Estado : Cdigo, Familia...
        // Vamos a forzar estado pagado...
//        paymentQuery.addPaymentStateCodeData(Estado.PAGADO); // forzamos STATECODE='04', a considerar solo para subselect COMPLETADOS
//        List statusList = paymentQuery.getStateParameterList();
//        for (Iterator it= statusList.iterator(); it.hasNext() ;) {
//            PaymentStateSearchedParam status = (PaymentStateSearchedParam) it.next();
//            status.stateCode = Estado.PAGADO;
//            viewPaymentSQLCompletados = sqlHelper.addStatusBlockClause(status,viewPaymentSQLCompletados);
//            status.stateCode = null;
//            viewPaymentSQLIniciados   = sqlHelper.addStatusBlockClause(status,viewPaymentSQLIniciados);
//        }
//        List statusList;
//        statusList = paymentQuery.getStateParameterList();
//        for (Iterator it= statusList.iterator(); it.hasNext() ;) {
//            PaymentStateSearchedParam status = (PaymentStateSearchedParam) it.next();
//            viewPaymentSQLIniciados   = sqlHelper.addStatusBlockClause(status,viewPaymentSQLIniciados);
//        }
//        //paymentQuery.addPaymentStateCodeData(Estado.PAGADO); // forzamos STATECODE='04', a considerar solo para subselect COMPLETADOS
//        statusList = paymentQuery.getStateParameterList();
//        for (Iterator it= statusList.iterator(); it.hasNext() ;) {
//            PaymentStateSearchedParam status = (PaymentStateSearchedParam) it.next();
//            //status.stateCode = Estado.PAGADO;
//            viewPaymentSQLCompletados = sqlHelper.addStatusBlockClause(status,viewPaymentSQLCompletados);
//        }

        List statusList;
        statusList = paymentQuery.getStateParameterList();
        for (Iterator it= statusList.iterator(); it.hasNext() ;) {
            PaymentStateSearchedParam status = (PaymentStateSearchedParam) it.next();
            viewPaymentSQLCompletados = sqlHelper.addStatusBlockClause(status,viewPaymentSQLCompletados);
            viewPaymentSQLIniciados   = sqlHelper.addStatusBlockClause(status,viewPaymentSQLIniciados);
            viewPaymentSQLErroneos    = sqlHelper.addStatusBlockClause(status,viewPaymentSQLErroneos);
        }
        viewPaymentSQLCompletados = sqlHelper.addStatusCodeClause(viewPaymentSQLCompletados, Estado.PAGADO);
        viewPaymentSQLIniciados   = sqlHelper.addStatusCodeClause(viewPaymentSQLIniciados,   Estado.ENVIADO_ENTIDAD);
        viewPaymentSQLErroneos    = sqlHelper.addStatusCodeClause(viewPaymentSQLErroneos,    Estado.ERROR_PAGO);

        // Bloque de datos referentes al modo de pago - - - - - - - - - - -
        List paymentModeList;
        paymentModeList = paymentQuery.getPaymentModeParameterList();
        for (Iterator it= paymentModeList.iterator(); it.hasNext() ;) {
            PaymentModeSearchedParam mpago = (PaymentModeSearchedParam) it.next();
            viewPaymentSQLCompletados = sqlHelper.addPaymentModeBlockClause(mpago,viewPaymentSQLCompletados);
            viewPaymentSQLIniciados   = sqlHelper.addPaymentModeBlockClause(mpago,viewPaymentSQLIniciados);
            viewPaymentSQLErroneos    = sqlHelper.addPaymentModeBlockClause(mpago,viewPaymentSQLErroneos);
        }
        if (viewPaymentSQLCompletados.paymentModeDATAconditionClause!=null) {
        	viewPaymentSQLCompletados.paymentModeDATAconditionClause = "(" + viewPaymentSQLCompletados.paymentModeDATAconditionClause + ")";
        	viewPaymentSQLIniciados.paymentModeDATAconditionClause = "(" + viewPaymentSQLIniciados.paymentModeDATAconditionClause + ")";
        	viewPaymentSQLErroneos.paymentModeDATAconditionClause = "(" + viewPaymentSQLErroneos.paymentModeDATAconditionClause + ")";
        }
        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

        // Bloque de datos referentes a la entidad financiera - - - - - -
        List finantialOrgList;
        finantialOrgList = paymentQuery.getFinantialOrgParameterList();
        for (Iterator it= finantialOrgList.iterator(); it.hasNext() ;) {
            FinantialOrgSearchedParam ef = (FinantialOrgSearchedParam) it.next();
            viewPaymentSQLCompletados = sqlHelper.addFinantialOrgBlockClause(ef,viewPaymentSQLCompletados);
            viewPaymentSQLIniciados   = sqlHelper.addFinantialOrgBlockClause(ef,viewPaymentSQLIniciados);
            viewPaymentSQLErroneos    = sqlHelper.addFinantialOrgBlockClause(ef,viewPaymentSQLErroneos);
        }
        if (viewPaymentSQLCompletados.finantialOrgDATAconditionClause!=null) {
        	viewPaymentSQLCompletados.finantialOrgDATAconditionClause = "(" + viewPaymentSQLCompletados.finantialOrgDATAconditionClause + ")";
        	viewPaymentSQLIniciados.finantialOrgDATAconditionClause = "(" + viewPaymentSQLIniciados.finantialOrgDATAconditionClause + ")";
        	viewPaymentSQLErroneos.finantialOrgDATAconditionClause = "(" + viewPaymentSQLErroneos.finantialOrgDATAconditionClause + ")";
        }
        // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


        // quedara: GROUP BY ADMINFIELD, TYPOFIELD, FORMATFIELD
		this.viewPaymentSQLCompletados = sqlHelper.addGroupByAttributes(viewPaymentSQLCompletados, viewIdComp);
		this.viewPaymentSQLIniciados   = sqlHelper.addGroupByAttributes(viewPaymentSQLIniciados,   viewIdInic);
		this.viewPaymentSQLErroneos    = sqlHelper.addGroupByAttributes(viewPaymentSQLErroneos,    viewIdErr);

        String strFinalQuery = null;

        if (viewPaymentSQLCompletados != null) {
        	//strFinalQuery = viewPaymentSQLCompletados.toString() + " ////// " + viewPaymentSQLIniciados.toString();
        	
        	strFinalQuery =
        		//"SELECT INICIADOS.FORMATFIELD AS FORMATFIELD, INICIADOS.ADMINFIELD AS ADMINFIELD, INICIADOS.TYPOFIELD TYPOFIELD, INICIADOS.NPAGOIDFIELD AS NINICIADOS, COMPLETADOS.NPAGOIDFIELD AS NCOMPLETADOS, COMPLETADOS.SIMPORTEFIELD AS SIMPORTEFIELD FROM "
        		  groupByPaymentSQL.toString()
          		//+ sqlHelper.sqlToAlias(viewPaymentSQLFinantial.toString()+" 1=1", viewIdFinantial) + ", "
        		+ sqlHelper.sqlToAlias(viewPaymentSQLFinantial.toString().replaceAll("WHERE", ""), viewIdFinantial) + ", "
        		+ sqlHelper.sqlToAlias(viewPaymentSQLIniciados.toString(), viewIdInic) + ", "
        		+ sqlHelper.sqlToAlias(viewPaymentSQLCompletados.toString(), viewIdComp) + ", "
        		+ sqlHelper.sqlToAlias(viewPaymentSQLErroneos.toString(), viewIdErr)
        		+ sqlHelper.addFinantialOrgLeftJoinClause();

        }

    	return strFinalQuery;
	}

	/**
	 * @param args
	 */
    public static void main (String[] arg) throws XOMarshallerException {

        PaymentQuery  paymentQuery = new PaymentQuery();

		Calendar cal = new GregorianCalendar();
		cal.clear(); cal.set(2001, 5, 15); // mes: 0-enero
		Date dateFrom = cal.getTime();
		cal.clear(); cal.set(2010, 10, 2);
		Date dateTo = cal.getTime();
        /*Buscar del tributo 504 del Gobierno Vasco todos los recibos pagados */
        //paymentQuery.addPaymentDataSearchedParam("04833001", null, null);
        //paymentQuery.addPaymentDataSearchedParam("486000", null, null);
        paymentQuery.addPaymentDataSearchedParam("04833001");
        paymentQuery.addPaymentDataSearchedParam("486000");
        //paymentQuery.addPaymentDataSearchedParam("04833001","501","507");
//        paymentQuery.addPaymentDataSearchedParam("486000","001","502");
//        paymentQuery.addPaymentDataSearchedParam("486000","047","521");
//        paymentQuery.addPaymentDataSearchedParam("486000","051","522");
//        // -------------------------------------------------------------------
//        paymentQuery.addThirdPartyDNIData("15391987l");
//        paymentQuery.addThirdPartyDNIData("12345678Z");
//        paymentQuery.addThirdPartyRazonSocialData("Enjuto Mojamuto");
//        paymentQuery.addThirdPartyRazonSocialData("Condemor");
//        // -------------------------------------------------------------------
//        paymentQuery.addEXPCodeData("1234567890123456789012345");
//        paymentQuery.addEXPCodeData("1234567890123456789012346");
//        paymentQuery.addEXPFamilyData("99999999");
//        // -------------------------------------------------------------------
        //paymentQuery.addPaymentStateCodeData(Estado.PAGADO);
//        paymentQuery.addPaymentStateCodeData(Estado.NO_DATA);
//        paymentQuery.addPaymentStateNRCData("QWERTY1234567890");
//        paymentQuery.addPaymentStateNRCData("ASDFGH1234567890");
		paymentQuery.addPaymentStateData(null, null, dateFrom, dateTo);
		// entidad financiera y rango de fechas... ---------------------
		paymentQuery.addFinantialOrgData("3035");
		paymentQuery.addFinantialOrgData("2097");
		// modo de pago ------------------------------------------------
        paymentQuery.addPaymentModeData(PaymentMode.BANCA_ONLINE_VENTANILLA);
        paymentQuery.addPaymentModeData(PaymentMode.BANCA_ONLINE);
        paymentQuery.addPaymentModeData(PaymentMode.LIQUIDACION);
        paymentQuery.addPaymentModeData(PaymentMode.MOVIL);

        //System.out.println("TRAZA: paymentQuery: " + paymentQuery.toXML());
        //System.out.println(XOConstants.xmlHeaderDirective + paymentQuery.toXML());

        PaymentQuerySQLFinantialOrgStatsComposer sqlHelper = new PaymentQuerySQLFinantialOrgStatsComposer();

        String strFinalQuery  = sqlHelper.composeQuery(paymentQuery);

        System.out.println("============================" );
        System.out.println("FINAL QUERY =\n" + strFinalQuery);
        System.out.println("============================" );
    }
}
