package p12f.exe.thirdsgateway.objects.config;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

import p12f.exe.thirdsgateway.security.crypto.TripleDESEBCDICCryptoHelper;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

public class ThirdsRegistryConfiguration implements Initializable, Serializable {

	public static final String FIRMA = "FIRMA";
	public static final String CONSENTIMIENTO = "CONSENTIMIENTO";
	
	public static final String FIRMA_CONSENTIMIENTO_REQUEST = "signConsent";
	public static final String FIRMA_TEXT = "singText";
	public static final String FIRMA_CONSENTIMIENTO_REQUEST_CAPICOM = "signConsentCAPICOM";
	public static final String FIRMA_TEXT_CAPICOM = "singTextCAPICOM";
	
	public static final String CONF_DEFECTO = "DEFAULT";
	
	public String id;
	public boolean isEnabled;
	public boolean isSecurityContextEnabled;
	public boolean isHidden;
	public String encType;
	public String variableOidContainer;
	public String variableOid;
	public List variableIds;
	public List emails;
	public Map description;
	public ReportGenerator reportGenerator;
	public String authorizationType;
	public Map requestText;
	public ServiceConfig serviceConfig;
	public EventsConfig eventsConfig;
	public Map renderers;
	public String defaultCss;
	public Map variablesConfig;
	public String urlCallBack;
	
	public ThirdsRegistryConfiguration() {
		ObjectUtils.initialize(this);
		isEnabled = true;
		isHidden = false;
		encType = TripleDESEBCDICCryptoHelper.ENC_TYPE_NONE;
	}
	
	/**
	 * Convierte un Objeto ThirdsRegistryConfiguration a XML.
	 * @return
	 * @throws XOMarshallerException
	 */
    public String toXML() throws XOMarshallerException {   
    	return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
    }
    
    /**
	 * Convierte un XML en un objeto ThirdsRegistryConfiguration.
	 * @return
	 * @throws XOMarshallerException
	 */
	public static ThirdsRegistryConfiguration getObject(String xml) throws XOMarshallerException {    
	    return (ThirdsRegistryConfiguration)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}
	
	public int compareTo(Object o) {
		ThirdsRegistryConfiguration otro = (ThirdsRegistryConfiguration)o;
		if (this.description.containsKey("es")) {
			return ((String)this.description.get("es")).compareToIgnoreCase((String)otro.description.get("es"));
		} else {
			return ((String)this.description.get("eu")).compareToIgnoreCase((String)otro.description.get("eu"));
		}
	}
	
	public int compareTo(Object o, String idioma) {
		ThirdsRegistryConfiguration otro = (ThirdsRegistryConfiguration)o;
		return ((String)this.description.get(idioma)).compareToIgnoreCase((String)otro.description.get(idioma));
	}
	
}
