package p12f.exe.thirdsgateway.objects.config;

import java.io.Serializable;
import java.util.Map;

import com.ejie.r01f.util.Initializable;
import com.ejie.r01f.util.ObjectUtils;
import com.ejie.r01f.xml.marshalling.XOManager;
import com.ejie.r01f.xml.marshalling.XOMarshallerException;
import com.ejie.r01f.xmlproperties.XMLProperties;

public class VariableConfig implements Initializable, Serializable, Comparable {

	public String id;
	public String implementation;
	public String implementationHTML;
	public String implementationJS;
	public String implementationCSS;
	public boolean isGenericVar;
	public Map description;
	public int order;
	public Map validatorsConfig;
	
	
	public VariableConfig() {
		ObjectUtils.initialize(this);
		isGenericVar = false;
	}
	
	/**
	 * Convierte un Objeto JSONVariableConfig a XML.
	 * @return
	 * @throws XOMarshallerException
	 */
    public String toXML() throws XOMarshallerException {   
    	return XOManager.getXML(XMLProperties.get("p12f","objectMapPath"),this);
    }
    
    /**
	 * Convierte un XML en un objeto JSONVariableConfig.
	 * @return
	 * @throws XOMarshallerException
	 */
	public static VariableConfig getObject(String xml) throws XOMarshallerException {    
	    return (VariableConfig)XOManager.getObject(XMLProperties.get("p12f","objectMapPath"), xml);
	}

	public int compareTo(Object o) {
		VariableConfig other = (VariableConfig)o;
		return this.order - other.order;
	}
	
	public String getImplementation(){
		if (implementation!=null && !"".equals(implementation)) {
			return implementation;
		}
		if (!hasDifferentImplementation() && implementationHTML!=null && !"".equals(implementationHTML)) {
			return implementationHTML;
		}
		return "";
	}
	
	public String getImplementationHTML(){
		if (implementationHTML!=null && !"".equals(implementationHTML)) {
			return implementationHTML;
		}
		return implementation;
		
	}
	
	public String getImplementationCSS(){
		if (implementationCSS!=null && !"".equals(implementationCSS)) {
			return implementationCSS;
		}
		return implementation;
	}
	
	public String getImplementationJS(){
		if (implementationJS!=null && !"".equals(implementationJS)) {
			return implementationJS;
		}
		return implementation;
	}
	
	public boolean hasDifferentImplementation()  {
		String implementacionAct = implementation;
		if (implementacionAct == null) {
			implementacionAct = implementationHTML;
		} else {
			if (implementacionAct.equals(implementationHTML)) {
				implementacionAct = implementationHTML;
			} else {
				return true;
			}
		}
		if (implementacionAct == null) {
			implementacionAct = implementationCSS;
		} else {
			if (implementacionAct.equals(implementationCSS)) {
				implementacionAct = implementationCSS;
			} else {
				return true;
			}
		}
		if (implementacionAct == null) {
			implementacionAct = implementationJS;
		} else {
			if (implementacionAct.equals(implementationJS)) {
				implementacionAct = implementationJS;
			} else {
				return true;
			}
		}
		return false;
	}
}
