package p12f.exe.thirdsgateway.security.crypto;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import org.bouncycastle.util.encoders.Hex;

import com.ejie.r01f.util.StringUtils;

/**
 * Clase de ayuda para la encriptacin/desencriptacin simtrica TRIPL DES EBCDIC
 * @author ie00191m
 *
 */
public class TripleDESEBCDICCryptoHelper {
	
	public static final String CIPHER_3DES_CBC_NOPADDING = "DESede/CBC/NoPadding";
	public static final String CIPHER_3DES_ECB_PKCS5Padding = "DESede/ECB/PKCS5Padding";
	public static final String CIPHER_3DES_ECB_NOPADDING  = "DESede/ECB/NoPadding";
	
	private static String ALGORITHM_3DES = "DESede";
	
	public static final String encoding = "CP500";
	
	public static final  String  ENC_TYPE_HEX = "hex";
	public static final  String  ENC_TYPE_BASE64 = "base64";
	public static final  String  ENC_TYPE_NONE= "none";
	
	/**
	 * Constructor TripleDESEBCDICCryptoHelper
	 */
	private TripleDESEBCDICCryptoHelper() {
		
	}
	
	// ********************************** Genrico ********************************** ;
	
	/**
	 * Cifra la cadena dada str con el tipo de difrado encType y con las claves clave1 y clave2.
	 * @param clave1
	 * @param clave2
	 * @param encType
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public static String encrypt(String clave1, String clave2, String encType, String str) throws CryptoHelperException {
		if (ENC_TYPE_NONE.equals(encType)){
			return str;
		} else if (ENC_TYPE_BASE64.equals(encType)){
			return encryptBASE64(clave1, clave2, str);
		} else if (ENC_TYPE_HEX.equals(encType)){
			return encryptHEX(clave1, clave2, str);
		}
		return null;
	}
	
	/**
	 * Cifra la cadena dada str con el tipo de difrado encType y con las claves clave1 y clave2.
	 * @param clave1
	 * @param clave2
	 * @param encType
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public static String encrypt(String clave1, String clave2, String encType, String modeCipherPadding, String str) throws CryptoHelperException {
		if (ENC_TYPE_NONE.equals(encType)){
			return str;
		} else if (ENC_TYPE_BASE64.equals(encType)){
			return encryptBASE64(clave1, clave2, modeCipherPadding, str);
		} else if (ENC_TYPE_HEX.equals(encType)){
			return encryptHEX(clave1, clave2, modeCipherPadding, str);
		}
		return null;
	}
	
	/**
	 * Descifra la cadena dada str con el tipo de difrado encType y con las claves clave1 y clave2.
	 * @param clave1
	 * @param clave2
	 * @param encType
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public static String decrypt(String clave1, String clave2, String encType, String str) throws CryptoHelperException {
		if (ENC_TYPE_NONE.equals(encType)){
			return str;
		} else if (ENC_TYPE_BASE64.equals(encType)){
			return decryptBASE64(clave1, clave2, str);
		} else if (ENC_TYPE_HEX.equals(encType)){
			return decryptHEX(clave1, clave2, str);
		}
		return null;
	}
	
	/**
	 * Descifra la cadena dada str con el tipo de cifrado encType y con las claves clave1 y clave2.
	 * @param clave1
	 * @param clave2
	 * @param encType
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public static String decrypt(String clave1, String clave2, String encType, String modeCipherPadding, String str) throws CryptoHelperException {
		if (ENC_TYPE_NONE.equals(encType)){
			return str;
		} else if (ENC_TYPE_BASE64.equals(encType)){
			return decryptBASE64(clave1, clave2, modeCipherPadding, str);
		} else if (ENC_TYPE_HEX.equals(encType)){
			return decryptHEX(clave1, clave2, modeCipherPadding, str);
		}
		return null;
	}
	
	// ********************************** BASE64 ********************************** ;
	
	/**
	 * Cifra la cadena dada str con el tipo de difrado Base64 y con las claves clave1 y clave2.
	 * @param clave1
	 * @param clave2
	 * @param modeCipherPadding CIPHER_3DES_CBC_NOPADDING
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public static String encryptBASE64(String clave1, String clave2, String str) throws CryptoHelperException {
		return encryptBASE64(clave1,clave2,CIPHER_3DES_CBC_NOPADDING,str);
	}
	
	/**
	 * Cifra la cadena dada str y la devuelve en BAse64, en base a un password y un modo de cifrado.
	 * @param clave1
	 * @param clave2
	 * @param modeCipherPadding
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public static String encryptBASE64(String clave1, String clave2, String modeCipherPadding, String str) throws CryptoHelperException {
		try {
			String passPhrase = new String(Hex.encode(doXOR(clave1, clave2))).toUpperCase();
			byte [] seed_key =  Hex.decode(passPhrase);
			SecretKeySpec keySpec = new SecretKeySpec(seed_key,ALGORITHM_3DES);
			Cipher ecipher =  Cipher.getInstance(modeCipherPadding);
			if (modeCipherPadding.equalsIgnoreCase(CIPHER_3DES_CBC_NOPADDING)){
				byte[] ivBytes = new byte[8]; //EMPTY VECTOR
				IvParameterSpec iv = new IvParameterSpec(ivBytes);
				ecipher.init(Cipher.ENCRYPT_MODE, keySpec,iv);
			} else {
				ecipher.init(Cipher.ENCRYPT_MODE, keySpec);
			}
			// El texto a cifrar debe tener una longitud mltiplo de 8
			String strToEncrypt;
			if (str.length()%8!=0){
				int modulo = str.length()%8;
				strToEncrypt=StringUtils.rPadWithChar(str,' ',str.length()+8-modulo);
			} else {
				strToEncrypt = str;
			}
			// Encodear segn EBCDIC
			byte[] toEBCDIC = strToEncrypt.getBytes(TripleDESEBCDICCryptoHelper.encoding);
			// Encrypt
			byte[] enc = ecipher.doFinal(toEBCDIC);
			return new sun.misc.BASE64Encoder().encode(enc);
		} catch (javax.crypto.BadPaddingException e) {
			throw new CryptoHelperException(e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptoHelperException(e);
		} catch (UnsupportedEncodingException e) {
			throw new CryptoHelperException(e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptoHelperException(e);
		} catch (NoSuchPaddingException e) {
			throw new CryptoHelperException(e);
		} catch (InvalidKeyException e) {
			throw new CryptoHelperException(e);
		} catch (InvalidAlgorithmParameterException e) {
			throw new CryptoHelperException(e);
		} 
	}
	
	/**
	 * Descifra una cadena en Base 64 CIPHER_3DES_CBC_NOPADDING
	 * @param clave1
	 * @param clave2
	 * @param modeCipherPadding: Por defecto : CIPHER_3DES_CBC_NOPADDING
	 * @param encryptedChain
	 * @return
	 * @throws CryptoHelperException
	 */
	public static String decryptBASE64(String clave1, String clave2, String encryptedChain ) throws CryptoHelperException {
		return decryptBASE64(clave1,clave2, CIPHER_3DES_CBC_NOPADDING, encryptedChain );
	}
	
	/**
	 * Descifra una cadena en Base 64.
	 * @param clave1
	 * @param clave2
	 * @param modeCipherPadding
	 * @param encryptedChain
	 * @return
	 * @throws CryptoHelperException
	 */
	public static String decryptBASE64(String clave1, String clave2, String modeCipherPadding, String encryptedChain ) throws CryptoHelperException {
		try {
			String passPhrase = new String(Hex.encode(doXOR(clave1, clave2))).toUpperCase();
			byte [] seed_key =  Hex.decode(passPhrase);
			SecretKeySpec keySpec = new SecretKeySpec(seed_key,ALGORITHM_3DES);
			Cipher  dcipher =  Cipher.getInstance(modeCipherPadding);
			// Instancias de Cipher, una para cifrar, otra para descifrar.
			if (modeCipherPadding.equalsIgnoreCase(CIPHER_3DES_CBC_NOPADDING)){
				byte[] ivBytes = new byte[8]; //EMPTY VECTOR
				IvParameterSpec iv = new IvParameterSpec(ivBytes);
				dcipher.init(Cipher.DECRYPT_MODE, keySpec,iv);	
			} else {
				dcipher.init(Cipher.DECRYPT_MODE, keySpec);
			}
			byte[] dec = new sun.misc.BASE64Decoder().decodeBuffer(encryptedChain); 
			return new String( dcipher.doFinal(dec),TripleDESEBCDICCryptoHelper.encoding);
		} catch (javax.crypto.BadPaddingException e) {
			throw new CryptoHelperException(e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptoHelperException(e);
		} catch (UnsupportedEncodingException e) {
			throw new CryptoHelperException(e);
		} catch (java.io.IOException e) {
			throw new CryptoHelperException(e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptoHelperException(e);
		} catch (NoSuchPaddingException e) {
			throw new CryptoHelperException(e);
		} catch (InvalidKeyException e) {
			throw new CryptoHelperException(e);
		} catch (InvalidAlgorithmParameterException e) {
			throw new CryptoHelperException(e);
		}
	}
	
	
	// ********************************** TripleDES ********************************** ;
	
	/**
	 * Cifra la cadena str en TripleDES a partir dela password passPhrase y el modo de cifrado : CIPHER_3DES_CBC_NOPADDING
	 * @param clave1
	 * @param clave2
	 * @param modeCipherPadding
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public static String encryptHEX (String clave1, String clave2, String str) throws CryptoHelperException {
		return encryptHEX (clave1,clave2,CIPHER_3DES_CBC_NOPADDING, str); 
	}
	
	/**
	 * Cifra la cadena str en TripleDES a partir dela password passPhrase y el modo de cifrado.
	 * @param clave1
	 * @param clave2
	 * @param modeCipherPadding
	 * @param str
	 * @return
	 * @throws CryptoHelperException
	 */
	public static String encryptHEX (String clave1, String clave2, String modeCipherPadding, String str) throws CryptoHelperException{
		try {
			String passPhrase = new String(Hex.encode(doXOR(clave1, clave2))).toUpperCase();
			byte [] seed_key =  Hex.decode(passPhrase);
			SecretKeySpec keySpec = new SecretKeySpec(seed_key,ALGORITHM_3DES);
			Cipher ecipher =  Cipher.getInstance(modeCipherPadding);
			if (modeCipherPadding.equalsIgnoreCase(CIPHER_3DES_CBC_NOPADDING)){
				byte[] ivBytes = new byte[8]; //EMPTY VECTOR
				IvParameterSpec iv = new IvParameterSpec(ivBytes);
				ecipher.init(Cipher.ENCRYPT_MODE, keySpec,iv);
			} else {
				ecipher.init(Cipher.ENCRYPT_MODE, keySpec);
			}
			// El texto a cifrar debe tener una longitud mltiplo de 8
			String strToEncrypt;
			if (str.length()%8!=0){
				int modulo = str.length()%8;
				strToEncrypt=StringUtils.rPadWithChar(str,' ',str.length()+8-modulo);
			} else {
				strToEncrypt = str;
			}
			byte[] toEBCDIC = strToEncrypt.getBytes(TripleDESEBCDICCryptoHelper.encoding);
			byte[] enc = ecipher.doFinal(toEBCDIC);
			return new String(Hex.encode(enc)).toUpperCase();
		} catch (Exception ex ){
			ex.printStackTrace();
			throw new CryptoHelperException (ex);
		}
	}
	
	/**
	 * Descifra una cadena en hexadecimal en base a un passWord y un modo de cifrado por defecto : CIPHER_3DES_CBC_NOPADDING
	 * @param clave1
	 * @param clave2
	 * @param modeCipherPadding CIPHER_3DES_CBC_NOPADDING
	 * @param encryptedChain
	 * @return
	 * @throws CryptoHelperException
	 */
	public static String decryptHEX(String clave1, String clave2, String encryptedChain) throws CryptoHelperException {
		return decryptHEX (clave1,clave2,CIPHER_3DES_CBC_NOPADDING,encryptedChain);
	}
	
	/**
	 * Descifra una cadena en hexadecimal en base a un passWord y un modo de cifrado.
	 * @param clave1
	 * @param clave2
	 * @param modeCipherPadding
	 * @param encryptedChain
	 * @return
	 * @throws CryptoHelperException
	 */
	public static String decryptHEX(String clave1, String clave2, String modeCipherPadding, String encryptedChain) throws CryptoHelperException {
		try {
			String passPhrase = new String(Hex.encode(doXOR(clave1, clave2))).toUpperCase();
			byte [] seed_key =  Hex.decode(passPhrase);
			SecretKeySpec keySpec = new SecretKeySpec(seed_key,ALGORITHM_3DES);
			Cipher dcipher =  Cipher.getInstance(modeCipherPadding);
			// Instancias de Cipher, una para cifrar, otra para descifrar.
			if (modeCipherPadding.equalsIgnoreCase(CIPHER_3DES_CBC_NOPADDING)){
				byte[] ivBytes = new byte[8]; //EMPTY VECTOR
				IvParameterSpec iv = new IvParameterSpec(ivBytes);
				dcipher.init(Cipher.DECRYPT_MODE, keySpec,iv);	
			} else {
				dcipher.init(Cipher.DECRYPT_MODE, keySpec);
			}
			byte[] encData = Hex.decode(encryptedChain);
			return new String( dcipher.doFinal(encData),TripleDESEBCDICCryptoHelper.encoding);
		} catch (javax.crypto.BadPaddingException e) {
			throw new CryptoHelperException(e);
		} catch (IllegalBlockSizeException e) {
			throw new CryptoHelperException(e);
		} catch (UnsupportedEncodingException e) {
			throw new CryptoHelperException(e);
		} catch (NoSuchAlgorithmException e) {
			throw new CryptoHelperException(e);
		} catch (NoSuchPaddingException e) {
			 throw new CryptoHelperException(e);
		} catch (InvalidKeyException e) {
			 throw new CryptoHelperException(e);
		} catch (InvalidAlgorithmParameterException e) {
			 throw new CryptoHelperException(e);
		}
	}
	
	// ********************** FUNCIONES AUXILIARES *************************
	
	/**
	 * Metodo que realiza el XOR de dos arrays de bytes.
	 * @param clave1
	 * @param clave2
	 * @return Array de bytes con el resultado de la operacin.
	 */
	public static byte[] doXOR(String clave1, String clave2) throws CryptoHelperException{
		if(clave1.length()!=clave2.length()){
			throw new CryptoHelperException("Las dos claves deben tener la misma longitud.");
		}
		byte[] keyBytes1 = Hex.decode(clave1);
		byte[] keyBytes2 = Hex.decode(clave2);
		byte[] keyBytesXOR = new byte[keyBytes1.length];
		for(int i=0;i<keyBytesXOR.length;i++){
			int intValue = new Byte(keyBytes1[i]).intValue()^new Byte(keyBytes2[i]).intValue();
			keyBytesXOR[i]=new Integer(intValue).byteValue();
		}
		return keyBytesXOR;
	}
	
	
	public static String getXMLStream() throws IOException{
	
		BufferedReader in = null;
		try {
			in = new BufferedReader(new InputStreamReader(new FileInputStream("D:///datos//registro.xml")));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		String stream = "" ; String inputLine= "";
		while ((inputLine = in.readLine()) != null){
			System.out.println(inputLine);
			stream += inputLine;
		}
		in.close();
		
		return stream;
	}
	
}